@@ -1,6 +1,6 @@
 /**
  *
- * $Id: defaults.c,v 1.2 1996/01/16 12:24:11 miers Exp $
+ * $Id: defaults.c,v 1.9 1996/11/11 03:36:52 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,41 +22,72 @@
  *
  **/
 #include "mwm.h"
+#include "mwmstrings.h"
 
 /*
  * This stuff will get used as I write stuff
  *
- Toggle to Default Behavior?
- Toggle to Custom Behavior?
- confirmDefaultBehavior
- confirmCustomBehavior
- confirmRestart
- confirmQuit
- ISO8859-1
-
  Icons
  "Pack Icons" _P  Shift Alt<Key>F7 f.pack_icons
  XBMLANGPATH
  * function names came from running strings against mwm on Solaris
  */
 void *working_base = NULL;
+ScreenInfo *rscr = NULL;
 
-#if 0
-/* assorted gray bitmaps for decorative borders */
-#define g_width 2
-#define g_height 2
-static char     g_bits[] =
-{0x02, 0x01};
-
-#define l_g_width 4
-#define l_g_height 2
-static char     l_g_bits[] =
-{0x08, 0x02};
-
-#define s_g_width 4
-#define s_g_height 4
-static char     s_g_bits[] =
-{0x01, 0x02, 0x04, 0x08};
+#if 1
+extern void __XmRGB2HSV();
+#else
+#define FLOOR(x) ((int)((x) - 0.5) > (x) ? (x) : (x)-1)
+ 
+/* Convert between RGB and HSV */
+void
+__XmRGB2HSV(unsigned short r,
+            unsigned short g,
+            unsigned short b,
+            double *hr,
+            double *sr,
+            double *vr)
+{
+    double rd, gd, bd, max, min, del;
+    double rc, gc, bc;
+    double v, h, s;
+ 
+    /* convert RGB to HSV */
+    rd = r / 65535.0;            /* rd,gd,bd range 0-1 instead of 0-65535 */
+    gd = g / 65535.0;
+    bd = b / 65535.0;
+ 
+    /* compute maximum of rd,gd,bd */
+    if (rd>=gd) { if (rd>=bd) max = rd;  else max = bd; }
+    else { if (gd>=bd) max = gd;  else max = bd; }
+ 
+    /* compute minimum of rd,gd,bd */
+    if (rd<=gd) { if (rd<=bd) min = rd;  else min = bd; }
+    else { if (gd<=bd) min = gd;  else min = bd; }
+ 
+ 
+    del = max - min;
+    v = max;
+    if (max != 0.0) s = (del) / max;
+    else s = 0.0;
+ 
+    h = -1;
+    if (s != 0.0) {
+        rc = (max - rd) / del;
+        gc = (max - gd) / del;
+        bc = (max - bd) / del;
+ 
+        if      (rd==max) h = bc - gc;
+        else if (gd==max) h = 2 + rc - bc;
+        else if (bd==max) h = 4 + gc - rc;
+ 
+        h = h * 60;
+        if (h<0) h += 360;
+    }
+ 
+    *hr = h;  *sr = s;  *vr = v;
+}
 #endif
 
 void
@@ -64,7 +95,7 @@ _WmMultiClickTimeDefault(Widget widget, int offset, XrmValue * val)
 {
     static int time;
 
-    time = XtGetMultiClickTime(XtDisplayOfObject(toplevel));
+    time = XtGetMultiClickTime(dpy);
     val->addr = (XtPointer)&time;
 }
 
@@ -77,6 +108,15 @@ _WmDefaultBorderWidth(Widget widget, int offset, XrmValue * val)
     val->addr = (XtPointer)&border_width;
 }
 
+void
+_WmDefaultResizeBorderWidth(Widget widget, int offset, XrmValue * val)
+{
+    static Dimension border_width;
+
+    border_width = Mwm.frame_border_width + 3;
+    val->addr = (XtPointer)&border_width;
+}
+
 void
 _WmFocusAutoRaiseDefault(Widget widget, int offset, XrmValue * val)
 {
@@ -96,167 +136,63 @@ _WmFocusAutoRaiseDefault(Widget widget, int offset, XrmValue * val)
 void
 _WmIconImageBDefault(Widget widget, int offset, XrmValue * val)
 {
-    static XColor backgroundColor;
-
-    if (!XParseColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
-		     "LightGrey",
-                     &backgroundColor)) {
-	if (!XParseColor(XtDisplay(toplevel),
-			 DefaultColormapOfScreen(XtScreen(toplevel)),
-			 "#A8A8A8A8A8A8",
-			 &backgroundColor)) {
-	    backgroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
-	}
-	else {
-	    XAllocColor(XtDisplay(toplevel),
-			DefaultColormapOfScreen(XtScreen(toplevel)),
-			&backgroundColor);
-	}
-    }
-    else {
-	XAllocColor(XtDisplay(toplevel),
-		    DefaultColormapOfScreen(XtScreen(toplevel)),
-		    &backgroundColor);
-    }
-
-    val->addr = (XtPointer)&backgroundColor.pixel;
+    val->addr = (XtPointer)&rscr->components[MWM_ICON].background;
 }
 
 void
 _WmIconImageBSCDefault(Widget widget, int offset, XrmValue * val)
 {
-    XColor _widgetBackground;
-    static XColor bottomShadowColor;
-    void *w = working_base;
-
-    _widgetBackground.pixel = ((MwmWindow *)w)->icon_image_background;
-
-    XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
-                &_widgetBackground);
-
-    bottomShadowColor.blue = _widgetBackground.blue * .5;
-    bottomShadowColor.green = _widgetBackground.green * .5;
-    bottomShadowColor.red = _widgetBackground.red * .5;
-
-    if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
-                     &bottomShadowColor))
-        bottomShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
-
-    val->addr = (XtPointer)&bottomShadowColor.pixel;
+    val->addr = (XtPointer)&rscr->components[MWM_ICON].bottom_shadow_color;
 }
 
 void
 _WmIconImageBSPDefault(Widget widget, int offset, XrmValue * val)
 {
-    static Pixmap pix = XmUNSPECIFIED_PIXMAP;
-
-    val->addr = (XtPointer)&pix;
+    val->addr = (XtPointer)&rscr->components[MWM_ICON].bottom_shadow_pixmap;
 }
 
 void
 _WmIconImageFDefault(Widget widget, int offset, XrmValue * val)
 {
-    double h,s,v;
-    XColor _widgetBackground;
-    static XColor foregroundColor;
-    void *w = working_base;
-
-    _widgetBackground.pixel = ((MwmWindow *)w)->icon_image_background;
-
-    XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
-                &_widgetBackground);
-
-    __XmRGB2HSV(_widgetBackground.red,
-                _widgetBackground.green,
-                _widgetBackground.blue,
-                &h,&s,&v);
-
-    if (v<0.5)
-        foregroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
-    else
-        foregroundColor.pixel = BlackPixelOfScreen(XtScreen(toplevel));
-
-    XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
-                &foregroundColor);
-
-    val->addr = (XtPointer)&foregroundColor.pixel;
+    val->addr = (XtPointer)&rscr->components[MWM_ICON].foreground;
 }
 
 void
 _WmIconImageTSCDefault(Widget widget, int offset, XrmValue * val)
 {
-    XColor _widgetBackground;
-    static XColor topShadowColor;
-    void *w = working_base;
-
-    _widgetBackground.pixel = ((MwmWindow *)w)->icon_image_background;
-
-    XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
-                &_widgetBackground);
-
-    if (_widgetBackground.red > 65535/1.5)
-        topShadowColor.red = 65535;
-    else
-        topShadowColor.red = _widgetBackground.red * 1.5;
-
-    if (_widgetBackground.green > 65535/1.5)
-        topShadowColor.green = 65535;
-    else
-        topShadowColor.green = _widgetBackground.green * 1.5;
-
-    if (_widgetBackground.blue > 65535/1.5)
-        topShadowColor.blue = 65535;
-    else
-        topShadowColor.blue = _widgetBackground.blue * 1.5;
-
-
-    if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
-                     &topShadowColor))
-        topShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
-
-    val->addr = (XtPointer)&topShadowColor.pixel;
+    val->addr = (XtPointer)&rscr->components[MWM_ICON].top_shadow_color;
 }
 
 void
 _WmIconImageTSPDefault(Widget widget, int offset, XrmValue * val)
 {
-    static Pixmap pix = XmUNSPECIFIED_PIXMAP;
-
-    val->addr = (XtPointer)&pix;
+    val->addr = (XtPointer)&rscr->components[MWM_ICON].top_shadow_pixmap;
 }
 
 void
 _WmMatteBDefault(Widget widget, int offset, XrmValue * val)
 {
     static XColor backgroundColor;
-    void *w = working_base;
 
     if (!XParseColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
 		     "LightGrey",
                      &backgroundColor)) {
 	if (!XParseColor(XtDisplay(toplevel),
-			 DefaultColormapOfScreen(XtScreen(toplevel)),
+			 DefaultColormap(dpy, rscr->screen),
 			 "#A8A8A8A8A8A8",
 			 &backgroundColor)) {
-	    backgroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+	    backgroundColor.pixel = WhitePixel(dpy, rscr->screen);
 	}
 	else {
 	    XAllocColor(XtDisplay(toplevel),
-			DefaultColormapOfScreen(XtScreen(toplevel)),
+			DefaultColormap(dpy, rscr->screen),
 			&backgroundColor);
 	}
     }
     else {
 	XAllocColor(XtDisplay(toplevel),
-		    DefaultColormapOfScreen(XtScreen(toplevel)),
+		    DefaultColormap(dpy, rscr->screen),
 		    &backgroundColor);
     }
 
@@ -273,7 +209,7 @@ _WmMatteBSCDefault(Widget widget, int offset, XrmValue * val)
     _widgetBackground.pixel = ((MwmWindow *)w)->matte_background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     bottomShadowColor.blue = _widgetBackground.blue * .5;
@@ -281,9 +217,9 @@ _WmMatteBSCDefault(Widget widget, int offset, XrmValue * val)
     bottomShadowColor.red = _widgetBackground.red * .5;
 
     if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
                      &bottomShadowColor))
-        bottomShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        bottomShadowColor.pixel = WhitePixel(dpy, rscr->screen);
 
     val->addr = (XtPointer)&bottomShadowColor.pixel;
 }
@@ -307,7 +243,7 @@ _WmMatteFDefault(Widget widget, int offset, XrmValue * val)
     _widgetBackground.pixel = ((MwmWindow *)w)->matte_background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     __XmRGB2HSV(_widgetBackground.red,
@@ -316,12 +252,12 @@ _WmMatteFDefault(Widget widget, int offset, XrmValue * val)
                 &h,&s,&v);
 
     if (v<0.5)
-        foregroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        foregroundColor.pixel = WhitePixel(dpy, rscr->screen);
     else
-        foregroundColor.pixel = BlackPixelOfScreen(XtScreen(toplevel));
+        foregroundColor.pixel = BlackPixel(dpy, rscr->screen);
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &foregroundColor);
 
     val->addr = (XtPointer)&foregroundColor.pixel;
@@ -337,7 +273,7 @@ _WmMatteTSCDefault(Widget widget, int offset, XrmValue * val)
     _widgetBackground.pixel = ((MwmWindow *)w)->matte_background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     if (_widgetBackground.red > 65535/1.5)
@@ -357,9 +293,9 @@ _WmMatteTSCDefault(Widget widget, int offset, XrmValue * val)
 
 
     if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
                      &topShadowColor))
-        topShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        topShadowColor.pixel = WhitePixel(dpy, rscr->screen);
 
     val->addr = (XtPointer)&topShadowColor.pixel;
 }
@@ -368,7 +304,6 @@ void
 _WmMatteTSPDefault(Widget widget, int offset, XrmValue * val)
 {
     static Pixmap pix = XmUNSPECIFIED_PIXMAP;
-    void *w = working_base;
 
     val->addr = (XtPointer)&pix;
 }
@@ -377,30 +312,72 @@ void
 _WmBackgroundDefault(Widget widget, int offset, XrmValue * val)
 {
     static XColor backgroundColor;
-    void *w = working_base;
+    ComponentInfo *comp = (ComponentInfo *)working_base;
 
-    if (!XParseColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
-		     "LightGrey",
-                     &backgroundColor)) {
+    if (comp->type == MWM_MENU) {
 	if (!XParseColor(XtDisplay(toplevel),
-			 DefaultColormapOfScreen(XtScreen(toplevel)),
-			 "#A8A8A8A8A8A8",
+			 DefaultColormap(dpy, rscr->screen),
+			 "#72729f9fffff",
 			 &backgroundColor)) {
-	    backgroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+	    backgroundColor.pixel = WhitePixel(dpy, rscr->screen);
 	}
 	else {
 	    XAllocColor(XtDisplay(toplevel),
-			DefaultColormapOfScreen(XtScreen(toplevel)),
+			DefaultColormap(dpy, rscr->screen),
 			&backgroundColor);
 	}
+
+	val->addr = (XtPointer)&backgroundColor.pixel;
+    }
+    else if (comp->type == MWM_FEEDBACK) {
+	if (!XParseColor(XtDisplay(toplevel),
+                         DefaultColormap(dpy, rscr->screen),
+		         "Cadet Blue",
+                         &backgroundColor)) {
+	    if (!XParseColor(XtDisplay(toplevel),
+			     DefaultColormap(dpy, rscr->screen),
+			     "#5F5F92929E9E",
+			     &backgroundColor)) {
+		backgroundColor.pixel = WhitePixel(dpy, rscr->screen);
+	    }
+	    else {
+		XAllocColor(XtDisplay(toplevel),
+			    DefaultColormap(dpy, rscr->screen),
+			    &backgroundColor);
+	    }
+	}
+	else {
+	    XAllocColor(XtDisplay(toplevel),
+		        DefaultColormap(dpy, rscr->screen),
+		        &backgroundColor);
+	}
+
+	val->addr = (XtPointer)&backgroundColor.pixel;
     }
     else {
-	XAllocColor(XtDisplay(toplevel),
-		    DefaultColormapOfScreen(XtScreen(toplevel)),
-		    &backgroundColor);
+	if (!XParseColor(XtDisplay(toplevel),
+                         DefaultColormap(dpy, rscr->screen),
+		         "LightGrey",
+                         &backgroundColor)) {
+	    if (!XParseColor(XtDisplay(toplevel),
+			     DefaultColormap(dpy, rscr->screen),
+			     "#A8A8A8A8A8A8",
+			     &backgroundColor)) {
+		backgroundColor.pixel = WhitePixel(dpy, rscr->screen);
+	    }
+	    else {
+		XAllocColor(XtDisplay(toplevel),
+			    DefaultColormap(dpy, rscr->screen),
+			    &backgroundColor);
+	    }
+	}
+	else {
+	    XAllocColor(XtDisplay(toplevel),
+		        DefaultColormap(dpy, rscr->screen),
+		        &backgroundColor);
+	}
+	val->addr = (XtPointer)&backgroundColor.pixel;
     }
-    val->addr = (XtPointer)&backgroundColor.pixel;
 }
 
 void
@@ -421,16 +398,16 @@ _WmBottomShadowColorDefault(Widget widget, int offset, XrmValue * val)
     _widgetBackground.pixel = ((ComponentInfo *)w)->background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     bottomShadowColor.blue = _widgetBackground.blue * .5;
     bottomShadowColor.green = _widgetBackground.green * .5;
     bottomShadowColor.red = _widgetBackground.red * .5;
     if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
                      &bottomShadowColor))
-        bottomShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        bottomShadowColor.pixel = WhitePixel(dpy, rscr->screen);
 
     val->addr = (XtPointer)&bottomShadowColor.pixel;
 }
@@ -454,7 +431,7 @@ _WmForegroundDefault(Widget widget, int offset, XrmValue * val)
     _widgetBackground.pixel = ((ComponentInfo *)w)->background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     __XmRGB2HSV(_widgetBackground.red,
@@ -463,12 +440,12 @@ _WmForegroundDefault(Widget widget, int offset, XrmValue * val)
                 &h,&s,&v);
 
     if (v<0.5)
-        foregroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        foregroundColor.pixel = WhitePixel(dpy, rscr->screen);
     else
-        foregroundColor.pixel = BlackPixelOfScreen(XtScreen(toplevel));
+        foregroundColor.pixel = BlackPixel(dpy, rscr->screen);
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &foregroundColor);
 
     val->addr = (XtPointer)&foregroundColor.pixel;
@@ -484,7 +461,7 @@ _WmTopShadowColorDefault(Widget widget, int offset, XrmValue * val)
     _widgetBackground.pixel = ((ComponentInfo *)w)->background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     if (_widgetBackground.red > 65535/1.5)
@@ -504,9 +481,9 @@ _WmTopShadowColorDefault(Widget widget, int offset, XrmValue * val)
 
 
     if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
                      &topShadowColor))
-        topShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        topShadowColor.pixel = WhitePixel(dpy, rscr->screen);
 
     val->addr = (XtPointer)&topShadowColor.pixel;
 }
@@ -523,27 +500,26 @@ void
 _WmABackgroundDefault(Widget widget, int offset, XrmValue * val)
 {
     static XColor backgroundColor;
-    void *w = working_base;
 
     if (!XParseColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
 		     "Cadet Blue",
                      &backgroundColor)) {
 	if (!XParseColor(XtDisplay(toplevel),
-			 DefaultColormapOfScreen(XtScreen(toplevel)),
+			 DefaultColormap(dpy, rscr->screen),
 			 "#5F5F92929E9E",
 			 &backgroundColor)) {
-	    backgroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+	    backgroundColor.pixel = WhitePixel(dpy, rscr->screen);
 	}
 	else {
 	    XAllocColor(XtDisplay(toplevel),
-			DefaultColormapOfScreen(XtScreen(toplevel)),
+			DefaultColormap(dpy, rscr->screen),
 			&backgroundColor);
 	}
     }
     else {
 	XAllocColor(XtDisplay(toplevel),
-		    DefaultColormapOfScreen(XtScreen(toplevel)),
+		    DefaultColormap(dpy, rscr->screen),
 		    &backgroundColor);
     }
 
@@ -558,10 +534,10 @@ _WmAForegroundDefault(Widget widget, int offset, XrmValue * val)
     static XColor foregroundColor;
     void *w = working_base;
 
-    _widgetBackground.pixel = ((ComponentInfo *)w)->background;
+    _widgetBackground.pixel = ((ComponentInfo *)w)->active_background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     __XmRGB2HSV(_widgetBackground.red,
@@ -570,12 +546,12 @@ _WmAForegroundDefault(Widget widget, int offset, XrmValue * val)
                 &h,&s,&v);
 
     if (v<0.5)
-        foregroundColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        foregroundColor.pixel = WhitePixel(dpy, rscr->screen);
     else
-        foregroundColor.pixel = BlackPixelOfScreen(XtScreen(toplevel));
+        foregroundColor.pixel = BlackPixel(dpy, rscr->screen);
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &foregroundColor);
 
     val->addr = (XtPointer)&foregroundColor.pixel;
@@ -588,10 +564,10 @@ _WmABottomShadowColorDefault(Widget widget, int offset, XrmValue * val)
     static XColor bottomShadowColor;
     void *w = working_base;
 
-    _widgetBackground.pixel = ((ComponentInfo *)w)->background;
+    _widgetBackground.pixel = ((ComponentInfo *)w)->active_background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     bottomShadowColor.blue = _widgetBackground.blue * .5;
@@ -599,9 +575,9 @@ _WmABottomShadowColorDefault(Widget widget, int offset, XrmValue * val)
     bottomShadowColor.red = _widgetBackground.red * .5;
 
     if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
                      &bottomShadowColor))
-        bottomShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        bottomShadowColor.pixel = WhitePixel(dpy, rscr->screen);
 
     val->addr = (XtPointer)&bottomShadowColor.pixel;
 }
@@ -613,10 +589,10 @@ _WmATopShadowColorDefault(Widget widget, int offset, XrmValue * val)
     static XColor topShadowColor;
     void *w = working_base;
 
-    _widgetBackground.pixel = ((ComponentInfo *)w)->background;
+    _widgetBackground.pixel = ((ComponentInfo *)w)->active_background;
 
     XQueryColor(XtDisplay(toplevel),
-                DefaultColormapOfScreen(XtScreen(toplevel)),
+                DefaultColormap(dpy, rscr->screen),
                 &_widgetBackground);
 
     if (_widgetBackground.red > 65535/1.5)
@@ -636,9 +612,9 @@ _WmATopShadowColorDefault(Widget widget, int offset, XrmValue * val)
 
 
     if (!XAllocColor(XtDisplay(toplevel),
-                     DefaultColormapOfScreen(XtScreen(toplevel)),
+                     DefaultColormap(dpy, rscr->screen),
                      &topShadowColor))
-        topShadowColor.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+        topShadowColor.pixel = WhitePixel(dpy, rscr->screen);
 
     val->addr = (XtPointer)&topShadowColor.pixel;
 }
