@@ -1,6 +1,6 @@
 /**
  *
- * $Id: resources.c,v 1.4 1996/01/17 20:25:25 miers Exp $
+ * $Id: resources.c,v 1.10 1996/10/28 01:36:24 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,6 +23,7 @@
  **/
 
 #include "mwm.h"
+#include "mwmstrings.h"
 #include <Xm/Xm.h>
 #include <Xm/XmStrDefs.h>
 
@@ -195,11 +196,26 @@ const char      mwm_strings[] =
 "WmCFlags\0"
 "usePager\0"
 "UsePager\0"
+"edgeScrollX\0"
+"EdgeScrollX\0"
+"edgeScrollY\0"
+"EdgeScrollY\0"
+"pagerX\0"
+"PagerX\0"
+"pagerY\0"
+"PagerY\0"
+"virtualX\0"
+"VirtualX\0"
+"virtualY\0"
+"VirtualY\0"
+"clickTime\0"
+"ClickTime\0"
 "\0";
 
 extern void     _WmMultiClickTimeDefault(Widget w, int offset, XrmValue * val);
 extern void     _WmFocusAutoRaiseDefault(Widget w, int offset, XrmValue * val);
 extern void     _WmDefaultBorderWidth(Widget w, int offset, XrmValue * val);
+extern void	_WmDefaultResizeBorderWidth(Widget widget, int offset, XrmValue * val);
 extern void     _WmIconImageBDefault(Widget w, int offset, XrmValue * val);
 extern void     _WmIconImageBSCDefault(Widget w, int offset, XrmValue * val);
 extern void     _WmIconImageBSPDefault(Widget w, int offset, XrmValue * val);
@@ -252,12 +268,12 @@ static XtResource mwm_resources[] =
     {
 	XmNbuttonBindings, XmCButtonBindings, XmRString,
 	sizeof(String), Offset(button_bindings),
-	XmRString, (XtPointer) DEFAULT_MWM_BUTTON_BINDINGS
+	XmRString, (XtPointer) DEFAULT_BUTTON_BINDING_NAME
     },
     {
 	XmNcleanText, XmCCleanText, XmRBoolean,
 	sizeof(Boolean), Offset(clean_text),
-	XmRImmediate, (XtPointer) True
+	XmRImmediate, (XtPointer) False
     },
     {
 	XmNclientAutoPlace, XmCClientAutoPlace, XmRBoolean,
@@ -279,6 +295,11 @@ static XtResource mwm_resources[] =
 	sizeof(Boolean), Offset(deiconify_key_focus),
 	XmRImmediate, (XtPointer) True
     },
+    {
+	XmNclickTime, XmCClickTime, XmRInt,
+	sizeof(Time), Offset(click_time),
+	XmRImmediate, (XtPointer) 100
+    },
     {
 	XmNdoubleClickTime, XmCDoubleClickTime, XmRInt,
 	sizeof(Time), Offset(double_click_time),
@@ -382,7 +403,7 @@ static XtResource mwm_resources[] =
     {
 	XmNkeyBindings, XmCKeyBindings, XmRString,
 	sizeof(String), Offset(key_bindings),
-	XmRImmediate, (XtPointer) DEFAULT_MWM_KEY_BINDINGS
+	XmRImmediate, (XtPointer) DEFAULT_KEY_BINDING_NAME
     },
     {
 	XmNkeyboardFocusPolicy, XmCKeyboardFocusPolicy, XmRWmKFocus,
@@ -450,9 +471,9 @@ static XtResource mwm_resources[] =
 	XmRImmediate, (XtPointer) False
     },
     {
-	XmNresizeBorderWidth, XmCFrameBorderWidth, XmRWmUsePPosition,
+	XmNresizeBorderWidth, XmCFrameBorderWidth, XmRInt,
 	sizeof(Dimension), Offset(resize_border_width),
-	XmRCallProc, (XtPointer) NULL
+	XmRCallProc, (XtPointer) _WmDefaultResizeBorderWidth
     },
     {
 	XmNresizeCursors, XmCResizeCursors, XmRBoolean,
@@ -494,6 +515,36 @@ static XtResource mwm_resources[] =
 	sizeof(Boolean), Offset(use_pager),
 	XmRImmediate, (XtPointer) True
     },
+    {
+	XmNedgeScrollX, XmCEdgeScrollX, XmRInt,
+	sizeof(int), Offset(edge_scroll_x),
+	XmRImmediate, (XtPointer) XmUNSPECIFIED
+    },
+    {
+	XmNedgeScrollY, XmCEdgeScrollY, XmRInt,
+	sizeof(int), Offset(edge_scroll_y),
+	XmRImmediate, (XtPointer) XmUNSPECIFIED
+    },
+    {
+	XmNpagerX, XmCPagerX, XmRInt,
+	sizeof(int), Offset(pager_x),
+	XmRImmediate, (XtPointer) XmUNSPECIFIED
+    },
+    {
+	XmNpagerY, XmCPagerY, XmRInt,
+	sizeof(int), Offset(pager_y),
+	XmRImmediate, (XtPointer) XmUNSPECIFIED
+    },
+    {
+	XmNvirtualX, XmCVirtualX, XmRInt,
+	sizeof(int), Offset(virtual_x),
+	XmRImmediate, (XtPointer) 3
+    },
+    {
+	XmNvirtualY, XmCVirtualY, XmRInt,
+	sizeof(int), Offset(virtual_y),
+	XmRImmediate, (XtPointer) 3
+    },
     {
 	XmNwMenuButtonClick, XmCWMenuButtonClick, XmRBoolean,
 	sizeof(Boolean), Offset(w_menu_button_click),
@@ -622,7 +673,7 @@ static XtResource mwm_client_resources[] =
     {
 	XmNwindowMenu, XmCWindowMenu, XmRString,
 	sizeof(String), Offset(window_menu),
-	XmRImmediate, (XtPointer) DEFAULT_MWM_WINDOW_MENU
+	XmRImmediate, (XtPointer) DEFAULT_WIN_MENU_NAME
     }
 };
 
@@ -659,8 +710,8 @@ static XtResource mwm_component_resources[] =
     },
     {
 	XmNfontList, XmCFontList, XmRFontList,
-	sizeof(XFontStruct *), Offset(font_list),
-	XmRImmediate, (XtPointer) "fixed"
+	sizeof(XmFontList), Offset(font_list),
+	XmRString, (XtPointer) "fixed"
     },
     {
 	XmNforeground, XmCForeground, XmRPixel,
@@ -709,7 +760,7 @@ static XtResource mwm_component_fi_resources[] =
     {
 	XmNactiveBottomShadowPixmap, XmCBottomShadowPixmap, XmRPixmap,
 	sizeof(Pixmap), Offset(active_bottom_shadow_pixmap),
-	XmRImmediate, (XtPointer) XmUNSPECIFIED
+	XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXMAP
     },
     {
 	XmNactiveTopShadowColor, XmCBackground, XmRPixel,
@@ -797,12 +848,6 @@ show_feedback[] = {
     NULL
 };
 
-static char    *
-sizes[] = {
-    "vertical",
-    "horizontal"
-};
-
 static char    *
 use_p_positions[] = {
     "on",
@@ -814,22 +859,20 @@ use_p_positions[] = {
 static char *
 client_flags[] = {
     "sticky",
-    "ontop",
     "circulate_skip",
     "start_iconic",
     "list_skip",
-    "suppress_icon",
-    "no_icon_title",
     NULL
 };
 
-Boolean
+static Boolean
 WmCvtStringToCDecor(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		    XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
     static long     val;
     char           *ptr;
     int             i;
+    Boolean         reverse = False;
 
     if (*num_args != 0) {
 	XtWarningMsg("wrongParameters", "cvtStringToCDecor",
@@ -841,7 +884,9 @@ WmCvtStringToCDecor(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     ptr = ((char *) (from->addr));
 
     if (strcmp(ptr, "all") == 0) {
-	val = MWM_DECOR_ALL;
+	val = MWM_DECOR_BORDER|MWM_DECOR_RESIZEH|MWM_DECOR_TITLE|
+	      MWM_DECOR_MENU|MWM_DECOR_MAXIMIZE|MWM_DECOR_MINIMIZE;
+
 	if (to->addr == NULL) {
 	    to->addr = (XtPointer)&val;
 	    to->size = sizeof(long);
@@ -878,14 +923,20 @@ WmCvtStringToCDecor(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 	}
     }
 
-    if (*ptr == '-')
-	val = MWM_DECOR_ALL;
+    if (*ptr == '-') {
+	val = MWM_DECOR_BORDER|MWM_DECOR_RESIZEH|MWM_DECOR_TITLE|
+	      MWM_DECOR_MENU|MWM_DECOR_MAXIMIZE|MWM_DECOR_MINIMIZE;
+	reverse = True;
+    }
     else
 	val = 0;
 
     for (i = 0; client_decorations[i] != NULL; i++) {
 	if (strstr(ptr, client_decorations[i]) != NULL) {
-	    val |= (1L << i);
+	    if (reverse)
+		val &= ~(1L << i);
+	    else
+		val |= (1L << i);
 	}
     }
     if (to->addr == NULL) {
@@ -905,7 +956,7 @@ WmCvtStringToCDecor(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     }
 }
 
-Boolean
+static Boolean
 WmCvtStringToCFocus(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		    XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
@@ -946,13 +997,14 @@ WmCvtStringToCFocus(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     return False;
 }
 
-Boolean
+static Boolean
 WmCvtStringToCFunc(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		   XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
     static long     val;
     char           *ptr;
     int             i;
+    Boolean         reverse = False;
 
     if (*num_args != 0) {
 	XtWarningMsg("wrongParameters", "cvtStringToCFunc",
@@ -964,7 +1016,8 @@ WmCvtStringToCFunc(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     ptr = ((char *) (from->addr));
 
     if (strcmp(ptr, "all") == 0) {
-	val = MWM_FUNC_ALL;
+	val = MWM_FUNC_RESIZE|MWM_FUNC_MOVE|MWM_FUNC_MINIMIZE|
+	      MWM_FUNC_MAXIMIZE|MWM_FUNC_CLOSE;
 	if (to->addr == NULL) {
 	    to->addr = (XtPointer)&val;
 	    to->size = sizeof(long);
@@ -1001,14 +1054,20 @@ WmCvtStringToCFunc(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 	}
     }
 
-    if (*ptr == '-')
-	val = MWM_FUNC_ALL;
+    if (*ptr == '-') {
+	val = MWM_FUNC_RESIZE|MWM_FUNC_MOVE|MWM_FUNC_MINIMIZE|
+	      MWM_FUNC_MAXIMIZE|MWM_FUNC_CLOSE;
+	reverse = True;
+    }
     else
 	val = 0;
 
     for (i = 0; client_functions[i] != NULL; i++) {
 	if (strstr(ptr, client_functions[i]) != NULL) {
-	    val |= (1L << i);
+	    if (reverse)
+		val &= ~(1L << i);
+	    else
+		val |= (1L << i);
 	}
     }
     if (to->addr == NULL) {
@@ -1028,7 +1087,7 @@ WmCvtStringToCFunc(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     }
 }
 
-Boolean
+static Boolean
 WmCvtStringToCFlags(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		   XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
@@ -1046,8 +1105,7 @@ WmCvtStringToCFlags(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     ptr = ((char *) (from->addr));
 
     if (strcmp(ptr, "all") == 0) {
-	val = STICKY|ONTOP|CIRCULATESKIP|STARTICONIC|WINDOWLISTSKIP|
-	      SUPPRESSICON|NOICON_TITLE;
+	val = STICKY|CIRCULATESKIP|STARTICONIC|WINDOWLISTSKIP;
 	if (to->addr == NULL) {
 	    to->addr = (XtPointer)&val;
 	    to->size = sizeof(long);
@@ -1106,7 +1164,7 @@ WmCvtStringToCFlags(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     }
 }
 
-Boolean
+static Boolean
 WmCvtStringToIDecor(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		    XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
@@ -1149,7 +1207,7 @@ WmCvtStringToIDecor(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     }
 }
 
-Boolean
+static Boolean
 WmCvtStringToIPlace(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		    XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
@@ -1193,7 +1251,7 @@ WmCvtStringToIPlace(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     }
 }
 
-Boolean
+static Boolean
 WmCvtStringToKFocus(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		    XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
@@ -1234,13 +1292,14 @@ WmCvtStringToKFocus(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     return False;
 }
 
-Boolean
+static Boolean
 WmCvtStringToShowFeedback(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 			  XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
     static long     val;
     char           *ptr;
     int             i;
+    Boolean         reverse = False;
 
     if (*num_args != 0) {
 	XtWarningMsg("wrongParameters", "cvtStringToShowFeedback",
@@ -1252,7 +1311,9 @@ WmCvtStringToShowFeedback(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     ptr = ((char *) (from->addr));
 
     if (strcmp(ptr, "all") == 0) {
-	val = XmFEEDBACK_ALL;
+	val = MWM_FEEDBACK_BEHAVIOR|MWM_FEEDBACK_KILL|MWM_FEEDBACK_MOVE|
+	      MWM_FEEDBACK_PLACEMENT|MWM_FEEDBACK_QUIT|MWM_FEEDBACK_RESIZE|
+	      MWM_FEEDBACK_RESTART;
 	if (to->addr == NULL) {
 	    to->addr = (XtPointer)&val;
 	    to->size = sizeof(long);
@@ -1289,14 +1350,21 @@ WmCvtStringToShowFeedback(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 	}
     }
 
-    if (*ptr == '-')
-	val = XmFEEDBACK_ALL;
+    if (*ptr == '-') {
+	val = MWM_FEEDBACK_BEHAVIOR|MWM_FEEDBACK_KILL|MWM_FEEDBACK_MOVE|
+	      MWM_FEEDBACK_PLACEMENT|MWM_FEEDBACK_QUIT|MWM_FEEDBACK_RESIZE|
+	      MWM_FEEDBACK_RESTART;
+	reverse = True;
+    }
     else
 	val = 0;
 
     for (i = 0; show_feedback[i] != NULL; i++) {
 	if (strstr(ptr, show_feedback[i]) != NULL) {
-	    val |= (1L << i);
+	    if (reverse)
+		val &= ~(1L << i);
+	    else
+		val |= (1L << i);
 	}
     }
     if (to->addr == NULL) {
@@ -1316,7 +1384,7 @@ WmCvtStringToShowFeedback(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     }
 }
 
-Boolean
+static Boolean
 WmCvtStringToSize(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 		  XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
@@ -1354,7 +1422,7 @@ WmCvtStringToSize(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     return False;
 }
 
-Boolean
+static Boolean
 WmCvtStringToUsePPosition(Display * dpy, XrmValuePtr args, Cardinal * num_args,
 			  XrmValuePtr from, XrmValuePtr to, XtPointer * data)
 {
@@ -1395,8 +1463,22 @@ WmCvtStringToUsePPosition(Display * dpy, XrmValuePtr args, Cardinal * num_args,
     return False;
 }
 
+/*
+ * get resources for a client window
+ */
 void
-AddWmResourceConverters()
+RES_GetClientDefaults(ScreenInfo *scr, MwmWindow *tmp_win, char *name, char *class)
+{
+    working_base = tmp_win;
+    rscr = scr;
+    XtGetSubresources(toplevel, (XtPointer)tmp_win,
+		      name, class,
+		      mwm_client_resources,
+		      XtNumber(mwm_client_resources), NULL, 0);
+}
+
+void
+RES_Initialize()
 {
     XmRegisterConverters();
 
@@ -1479,30 +1561,18 @@ AddWmResourceConverters()
 		       0,
 		       XtCacheAll,
 		       NULL);
-}
-
-void
-GetClientDefaults(MwmWindow *tmp_win, char *name, char *class)
-{
-    extern void *working_base;
-
-    working_base = tmp_win;
-    XtGetSubresources(toplevel, (XtPointer)tmp_win,
-		      name, class,
-		      mwm_client_resources,
-		      XtNumber(mwm_client_resources), NULL, 0);
-}
-
-void
-MwmInit()
-{
-    extern void *working_base;
-    Widget client, menu, pager, feedback, icon;
-
-    AddWmResourceConverters();
 
     XtGetApplicationResources(toplevel, (XtPointer)&Mwm,
 			   mwm_resources, XtNumber(mwm_resources), NULL, 0);
+}
+
+/*
+ * get the resources for the components in each screen
+ */
+void
+RES_GetComponentDefaults(ScreenInfo *scr)
+{
+    Widget client;
 
     /*
      * MLM: OK, seems the resource manager won't pay attention to me if
@@ -1510,34 +1580,35 @@ MwmInit()
      * appreciate it if someone would correct the name/class values so
      * I don't have to do this.
      */
-    working_base = &(Mwm.components[MWM_MENU]);
-    XtGetSubresources(toplevel, (XtPointer)&(Mwm.components[MWM_MENU]),
+    rscr = scr;
+    working_base = &(scr->components[MWM_MENU]);
+    scr->components[MWM_MENU].type = MWM_MENU;
+    XtGetSubresources(toplevel, (XtPointer)&(scr->components[MWM_MENU]),
 		      "menu", "menu",
 		      mwm_component_resources,
 		      XtNumber(mwm_component_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_FEEDBACK]);
-    XtGetSubresources(toplevel, (XtPointer)&(Mwm.components[MWM_FEEDBACK]),
+    working_base = &(scr->components[MWM_FEEDBACK]);
+    scr->components[MWM_FEEDBACK].type = MWM_FEEDBACK;
+    XtGetSubresources(toplevel, (XtPointer)&(scr->components[MWM_FEEDBACK]),
 		      "feedback", "feedback",
 		      mwm_component_resources,
 		      XtNumber(mwm_component_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_PAGER]);
-    XtGetSubresources(toplevel, (XtPointer)&(Mwm.components[MWM_PAGER]),
+    working_base = &(scr->components[MWM_PAGER]);
+    scr->components[MWM_PAGER].type = MWM_PAGER;
+    XtGetSubresources(toplevel, (XtPointer)&(scr->components[MWM_PAGER]),
 		      "pager", "pager",
 		      mwm_component_resources,
 		      XtNumber(mwm_component_resources), NULL, 0);
-    XtGetSubresources(toplevel, (XtPointer)&(Mwm.components[MWM_PAGER]),
-		      "pager", "pager",
-		      mwm_component_fi_resources,
-		      XtNumber(mwm_component_fi_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_ICON]);
-    XtGetSubresources(toplevel, (XtPointer)&(Mwm.components[MWM_ICON]),
+    working_base = &(scr->components[MWM_ICON]);
+    scr->components[MWM_ICON].type = MWM_ICON;
+    XtGetSubresources(toplevel, (XtPointer)&(scr->components[MWM_ICON]),
 		      "icon", "icon",
 		      mwm_component_resources,
 		      XtNumber(mwm_component_resources), NULL, 0);
-    XtGetSubresources(toplevel, (XtPointer)&(Mwm.components[MWM_ICON]),
+    XtGetSubresources(toplevel, (XtPointer)&(scr->components[MWM_ICON]),
 		      "icon", "icon",
 		      mwm_component_fi_resources,
 		      XtNumber(mwm_component_fi_resources), NULL, 0);
@@ -1545,7 +1616,8 @@ MwmInit()
     client = XtVaCreateWidget("client", (WidgetClass)&objectClassRec,
 			      toplevel, NULL);
 
-    working_base = &(Mwm.components[MWM_TITLE_AREA]);
+    working_base = &(scr->components[MWM_TITLE_A]);
+    scr->components[MWM_TITLE_A].type = MWM_TITLE_A;
     XtGetSubresources(client, (XtPointer)working_base,
 		      "title", "title",
 		      mwm_component_resources,
@@ -1555,7 +1627,8 @@ MwmInit()
 		      mwm_component_fi_resources,
 		      XtNumber(mwm_component_fi_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_RESIZE_HANDLES]);
+    working_base = &(scr->components[MWM_RESIZE_H]);
+    scr->components[MWM_RESIZE_H].type = MWM_RESIZE_H;
     XtGetSubresources(client, (XtPointer)working_base,
 		      "resizeh", "resizeh",
 		      mwm_component_resources,
@@ -1565,7 +1638,8 @@ MwmInit()
 		      mwm_component_fi_resources,
 		      XtNumber(mwm_component_fi_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_BORDER]);
+    working_base = &(scr->components[MWM_BORDER]);
+    scr->components[MWM_BORDER].type = MWM_BORDER;
     XtGetSubresources(client, (XtPointer)working_base,
 		      "border", "border",
 		      mwm_component_resources,
@@ -1575,7 +1649,8 @@ MwmInit()
 		      mwm_component_fi_resources,
 		      XtNumber(mwm_component_fi_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_MAXIMIZE_BUTTON]);
+    working_base = &(scr->components[MWM_MAXIMIZE_B]);
+    scr->components[MWM_MAXIMIZE_B].type = MWM_MAXIMIZE_B;
     XtGetSubresources(client, (XtPointer)working_base,
 		      "maximizeb", "maximizeb",
 		      mwm_component_resources,
@@ -1585,7 +1660,8 @@ MwmInit()
 		      mwm_component_fi_resources,
 		      XtNumber(mwm_component_fi_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_MINIMIZE_BUTTON]);
+    working_base = &(scr->components[MWM_MINIMIZE_B]);
+    scr->components[MWM_MINIMIZE_B].type = MWM_MINIMIZE_B;
     XtGetSubresources(client, (XtPointer)working_base,
 		      "minimizeb", "minimizeb",
 		      mwm_component_resources,
@@ -1595,7 +1671,8 @@ MwmInit()
 		      mwm_component_fi_resources,
 		      XtNumber(mwm_component_fi_resources), NULL, 0);
 
-    working_base = &(Mwm.components[MWM_MENU_BUTTON]);
+    working_base = &(scr->components[MWM_MENU_B]);
+    scr->components[MWM_MENU_B].type = MWM_MENU_B;
     XtGetSubresources(client, (XtPointer)working_base,
 		      "menub", "menub",
 		      mwm_component_resources,
@@ -1604,5 +1681,5 @@ MwmInit()
 		      "menub", "menub",
 		      mwm_component_fi_resources,
 		      XtNumber(mwm_component_fi_resources), NULL, 0);
-    parse_mwmrc();
 }
+
