@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragIcon.h,v 1.3 1996/01/26 20:10:43 miers Exp $
+ * $Id: DragIcon.h,v 1.2 1996/08/29 00:06:00 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -31,6 +31,19 @@
 extern "C" {
 #endif
 
+enum {
+    XmATTACH_NORTH_WEST,
+    XmATTACH_NORTH,
+    XmATTACH_NORTH_EAST,
+    XmATTACH_EAST,
+    XmATTACH_SOUTH_EAST,
+    XmATTACH_SOUTH,
+    XmATTACH_SOUTH_WEST,
+    XmATTACH_WEST,
+    XmATTACH_CENTER,
+    XmATTACH_HOT
+};
+
 #ifndef XmIsDragIcon
 #define XmIsDragIcon(w) XtIsSubClass(w, xmDragIconObjectClass)
 #endif
@@ -42,7 +55,7 @@ extern WidgetClass xmDragIconObjectClass;
 Widget XmCreateDragIcon (Widget widget, String name, ArgList arglist, Cardinal argcount);
 
 #ifdef __cplusplus
-};
+}
 #endif 
 
 #endif /* XM_DRAGICON_H */
