@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Xm.h,v 1.19 1996/04/08 19:46:43 miers Exp $
+ * $Id: Xm.h,v 1.6 1996/09/01 00:42:49 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -47,16 +47,17 @@ extern "C" {
 /* Version Information */
 
 #define LESSTIF_VERSION 0
-#define LESSTIF_REVISION 36
+#define LESSTIF_REVISION 55
 
 #define LesstifVersion (LESSTIF_VERSION * 1000 + LESSTIF_REVISION)
-#define LesstifVERSION_STRING "GNU Lesstif 0.36"
+#define LesstifVERSION_STRING "GNU Lesstif 0.55"
 
 /* since we're replacing Motif 1.2... */
 #define XmVERSION 1
 #define XmREVISION 2
 #define XmVersion (XmVERSION * 1000 + XmREVISION)
 #define XmVERSION_STRING LesstifVERSION_STRING
+#define XmUPDATE_LEVEL 0
 
 /* pixmap stuff */
 
@@ -115,53 +116,16 @@ enum {
 #define XmSTRING_COMPONENT_USER_END     ((XmStringComponentType) 255)
 
 typedef unsigned char XmStringDirection;
+typedef unsigned char *XmString;
+typedef XmString *XmStringTable;
 typedef char *XmStringCharSet;
-typedef struct __XmString **XmStringTable;
-
-#ifdef	notdef
-typedef struct {
-    char *contents;
-    int length;
-    char *tag;
-    XmStringDirection direction;
-    unsigned char type;             /* XmSTRING_COMPONENT_* */
-} XmStringComponentRec, *XmStringComponentType;
-#else
-typedef struct {
-    char *contents;
-    int length;
-    char *tag;
-    XmStringDirection direction;
-    unsigned char type;             /* XmSTRING_COMPONENT_* */
-} XmStringComponentRec, *XmStringComponent;
 typedef unsigned char XmStringComponentType;
-#endif
 
-typedef struct __XmString {
-    XmStringComponent *segments;
-    int number_of_segments;
-} XmStringRec, *XmString;
-
-typedef struct {
-    XmString string;
-    int current_segment;
-} XmStringContextRec, *XmStringContext;
-
-typedef struct {
-    char *tag;
-    XmFontType type;
-    XtPointer font;
-} XmFontListEntryRec, *XmFontListEntry;
-
-typedef struct {
-    XmFontListEntry *entries;
-    int number_of_font_entries;
-} XmFontListRec, *XmFontList;
-
-typedef struct {
-    XmFontList fontlist;
-    int current_entry;
-} XmFontContextRec, *XmFontContext;
+typedef struct _XmFontListRec *XmFontListEntry, *XmFontList;
+typedef struct __XmStringContextRec *_XmStringContext;
+typedef struct __XmStringRec *_XmString;
+typedef struct _XmtStringContextRec *XmStringContext;
+typedef struct _XmFontListContextRec *XmFontContext;
 
 /*
  * make sure the big three are always defined
@@ -276,8 +240,8 @@ enum {
 };
 
 enum {
-    XmTEAR_OFF_DISABLED,
-    XmTEAR_OFF_ENABLED
+    XmTEAR_OFF_ENABLED,
+    XmTEAR_OFF_DISABLED
 };
 
 enum {
@@ -873,8 +837,8 @@ int  XmCvtToHorizontalPixels(Screen *screen, int from_val, int from_type);
 int  XmCvtToVerticalPixels(Screen *screen, int from_val, int from_type);
 int  XmCvtFromHorizontalPixels(Screen *screen, int from_val, int to_type);
 int  XmCvtFromVerticalPixels(Screen *screen, int from_val, int to_type);
-void XmSetFontUnits();
-void XmSetFontUnit();
+void XmSetFontUnits(Display *display, int h_value, int v_value);
+void XmSetFontUnit(Display *display, int value);
 
 /************************* MenuUtil.c *****************************/
 
@@ -959,8 +923,7 @@ void XmStringDraw(Display *d,
 		  unsigned char alignment,
 		  unsigned char layout_direction,
 		  XRectangle *clip);
-void XmStringDrawImage(Display *d,
-		       Window w,
+void XmStringDrawImage(Display *d, Window w,
 		       XmFontList fontlist,
 		       XmString string,
 		       GC gc,
@@ -970,14 +933,9 @@ void XmStringDrawImage(Display *d,
 		       unsigned char alignment,
 		       unsigned char layout_direction,
 		       XRectangle *clip);
-void XmStringDrawUnderline(Display *d,
-			   Window w,
-			   XmFontList fontlist,
-			   XmString string,
-			   GC gc,
-			   Position x,
-			   Position y,
-			   Dimension width,
+void XmStringDrawUnderline(Display *d, Window w,
+			   XmFontList fontlist, XmString string,
+			   GC gc, Position x, Position y, Dimension width,
 			   unsigned char alignment,
 			   unsigned char layout_direction,
 			   XRectangle *clip,
@@ -1015,14 +973,12 @@ XmString XmStringNConcat(XmString s1, XmString s2, int num_bytes);
 XmString XmStringCopy(XmString s);
 XmString XmStringNCopy(XmString s1, int num_bytes);
 XmStringComponentType XmStringPeekNextComponent(XmStringContext context);
-XmString XmStringSegmentCreate(char *text, 
-			       char *tag, 
+XmString XmStringSegmentCreate(char *text, char *tag, 
 			       XmStringDirection direction,
 			       Boolean separator);
 XmString XmStringSeparatorCreate();
 
-Dimension XmStringWidth(XmFontList fontList,
-			XmString string);
+Dimension XmStringWidth(XmFontList fontList, XmString string);
 
 
 /*************************** FontList.c *****************************/
@@ -1062,7 +1018,7 @@ Widget XmGetDestination(Display *display);
 
 /*************************** Traversal.c *****************************/
 
-Boolean XmIsTraversible(Widget widget);
+Boolean XmIsTraversable(Widget widget);
 XmVisibility XmGetVisibility(Widget widget);
 void XmAddTabGroup(Widget widget);
 void XmRemoveTabGroup(Widget widget);
@@ -1083,14 +1039,8 @@ int XmImMbLookupString(Widget w, XKeyPressedEvent *event, char *buf,
 extern void XmImVaSetFocusValues(Widget w, ...);
 extern void XmImVaSetValues(Widget w, ...);
 
-typedef unsigned char XmInputPolicy;
-
-#define	XmINHERIT_POLICY	0
-#define	XmPER_WIDGET		1
-#define	XmPER_SHELL		2
-
 #ifdef __cplusplus
-};
+}
 #endif
 
 #endif /* XM_H */
