@@ -1,6 +1,6 @@
 /**
  *
- * $Id: XmP.h,v 1.26 1996/04/30 23:59:12 toshok Exp $
+ * $Id: XmP.h,v 1.5 1996/09/03 05:57:04 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -497,7 +497,7 @@ typedef struct _XmGeoMatrixRec {
     Boolean uniform_border;
     Dimension border;
     Dimension max_major;
-    Dimension boxed_minor;
+    Dimension boxes_minor;
     Dimension fill_minor;
     Dimension width;
     Dimension height;
@@ -949,10 +949,14 @@ XmGadget _XmInputForGadget(Widget cw, int x, int y);
 void _XmConfigureObject(Widget g, Position x, Position y,
 			Dimension width, Dimension height,
 			Dimension border_width);
+void XmeConfigureObject(Widget g, Position x, Position y,
+			Dimension width, Dimension height,
+			Dimension border_width);
 void _XmResizeObject(Widget g, Dimension width, Dimension height,
 		     Dimension border_width);
 void _XmMoveObject(Widget g, Position x, Position y);
 void _XmRedisplayGadgets(Widget w, XEvent *event, Region region);
+void XmeRedisplayGadgets(Widget w, XEvent *event, Region region);
 void _XmDispatchGadgetInput(Widget g, XEvent *event, Mask mask);
 Time __XmGetDefaultTime(Widget w, XEvent *event);
 
@@ -973,8 +977,6 @@ Time __XmGetDefaultTime(Widget w, XEvent *event);
     IMAGE->bitmap_bit_order = LSBFirst;\
 }
 
-void _XmSetupImageCache(void);
-
 Boolean _XmInstallImage(XImage *image, char *image_name,
 		        int hot_x, int hot_y);
 Boolean _XmGetImage(Screen *screen, char *image_name, XImage **image);
@@ -988,6 +990,16 @@ Boolean _XmGetPixmapData(Screen *screen,
 			 int *hot_y,
 			 unsigned int *width,
 			 unsigned int *height);
+Boolean XmeGetPixmapData(Screen *screen,
+			 Pixmap pixmap,
+			 char **image_name,
+			 int *depth,
+			 Pixel *foreground,
+			 Pixel *background,
+			 int *hot_x,
+			 int *hot_y,
+			 unsigned int *width,
+			 unsigned int *height);
 Pixmap _XmGetPixmap(Screen *screen,
 		    char *image_name,
 		    int depth,
@@ -1032,8 +1044,9 @@ enum { XmLABEL_FONTLIST = 1,
 
 void _XmRegisterConverters(void);
 void _XmWarning(Widget w, char *message, ...);
-void _XmError(Widget w, char *message, ...);
+void XmeWarning(Widget w, char *message, ...);
 Boolean _XmStringsAreEqual(char *in_str, char *text_str);
+Boolean XmeNamesAreEqual(char *in_str, char *text_str);
 XmFontList _XmGetDefaultFontList(Widget w, unsigned char fontListType);
 char *_XmConvertCSToString(XmString cs);
 Boolean _XmCvtXmStringToCT(XrmValue *from, XrmValue *to);
@@ -1081,15 +1094,27 @@ int _XmConvertUnits(Screen *screen,
 XmImportOperator _XmToHorizontalPixels(Widget widget,
 				       int offset,
 				       XtArgVal *value);
+XmImportOperator XmeToHorizontalPixels(Widget widget,
+				       int offset,
+				       XtArgVal *value);
 XmImportOperator _XmToVerticalPixels(Widget widget,
 				     int offset,
 				     XtArgVal *value);
+XmImportOperator XmeToVerticalPixels(Widget widget,
+				     int offset,
+				     XtArgVal *value);
 void _XmFromHorizontalPixels(Widget widget,
 			     int offset,
 			     XtArgVal *value);
+void XmeFromHorizontalPixels(Widget widget,
+			     int offset,
+			     XtArgVal *value);
 void _XmFromVerticalPixels(Widget widget,
 			   int offset,
 			   XtArgVal *value);
+void XmeFromVerticalPixels(Widget widget,
+			   int offset,
+			   XtArgVal *value);
 void _XmSortResourceList(XrmResource *list[], Cardinal len);
 void _XmUnitTypeDefault(Widget widget,
 		        int offset,
@@ -1160,35 +1185,62 @@ void _XmSelectColorDefault(Widget w, int offset, XrmValue *val);
 
 /**************************** XmString.c **********************************/
 
-/*
- * we don't currently have an internal representation for XmStrings, so
- * we get around it this way -- needed for the MegaButton and NewToggleB widgets
- */
-
-typedef XmString _XmString;
-#define _XmStringCreate(x) XmStringCopy(x)
-#define _XmStringCreateExternal(font,string) (string)
-#define _XmStringFree(x) XmStringFree(x)
-#define _XmStringDraw XmStringDraw
-#define _XmStringExtent XmStringExtent
-#define _XmStringEmpty XmStringEmpty
-void _XmStringDrawMnemonic(Display *d,
-                           Window w,
-                           XmFontList fontlist,
-                           XmString string,
-                           GC gc,
-                           Position x,
-                           Position y,
-                           Dimension width,
+XFontStruct *_XmGetFirstFont(XmFontListEntry entry);
+Boolean _XmFontListGetDefaultFont(XmFontList fontlist,
+				  XFontStruct **font_struct);
+Boolean _XmFontListSearch(XmFontList fontlist, XmStringCharSet charset,
+			  short *indx, XFontStruct **font_struct);
+Boolean _XmStringIsXmString(XmString string);
+Boolean _XmStringInitContext(_XmStringContext *context, _XmString string);
+Boolean _XmStringGetNextSegment(_XmStringContext context,
+				XmStringCharSet *charset,
+				XmStringDirection *direction,
+				char **text,
+				short *char_count,
+				Boolean *separator) ;
+void _XmStringFreeContext(_XmStringContext context) ;
+Dimension _XmStringWidth(XmFontList fontlist, _XmString string) ;
+Dimension _XmStringHeight(XmFontList fontlist, _XmString string) ;
+void _XmStringExtent(XmFontList fontlist, _XmString string,
+		     Dimension *width, Dimension *height) ;
+Boolean _XmStringEmpty(_XmString string);
+void _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
+		   GC gc, Position x, Position y, Dimension width,
+		   unsigned char align, unsigned char lay_dir,
+		   XRectangle *clip);
+void _XmStringDrawImage(Display *d, Window w,
+			XmFontList fontlist, _XmString string,
+			GC gc, Position x, Position y, Dimension width,
+			unsigned char align, unsigned char lay_dir,
+			XRectangle *clip);
+void _XmStringDrawUnderline(Display *d, Window w, XmFontList f, _XmString s,
+			    GC gc, Position x, Position y, Dimension width,
+			    unsigned char align, unsigned char lay_dir,
+			    XRectangle *clip, _XmString u);
+void _XmStringDrawMnemonic(Display *d, Window w,
+                           XmFontList fontlist, _XmString string,
+                           GC gc, Position x, Position y, Dimension width,
                            unsigned char alignment,
                            unsigned char layout_direction,
                            XRectangle *clip,
-                           String mnemonic,
-                           XmStringCharSet charset);
-
-/*
- * missing a boatload of string functions
- */
+                           String mnemonic, XmStringCharSet charset);
+_XmString _XmStringCreate(XmString cs);
+void _XmStringFree(_XmString string);
+char *_XmStringGetCurrentCharset(void) ;
+char *_XmCharsetCanonicalize(String charset);
+void _XmStringUpdate(XmFontList fontlist, _XmString string);
+_XmString _XmStringCopy(_XmString string);
+Boolean _XmStringByteCompare(_XmString a, _XmString b);
+Boolean _XmStringHasSubstring(_XmString string, _XmString substring);
+XmString _XmStringCreateExternal(XmFontList fontlist, _XmString cs);
+Dimension _XmStringBaseline(XmFontList fontlist, _XmString string);
+int _XmStringLineCount(_XmString string);
+char * _XmStringGetTextConcat(XmString string);
+Boolean _XmStringIsCurrentCharset(XmStringCharSet c);
+Boolean _XmStringSingleSegment(XmString str, char **pTextOut,
+			       XmStringCharSet *pCharsetOut);
+void _XmStringUpdateWMShellTitle(XmString xmstr, Widget shell);
+void XmeStringUpdateWMShellTitle(XmString xmstr, Widget shell);
 
 /************************* Traversal.c ********************************/
 
@@ -1211,15 +1263,16 @@ void _XmDestroyFocusData(XmFocusData focusData);
 void _XmSetActiveTabGroup(XmFocusData focusData, Widget tabGroup);
 Widget _XmGetActiveItem(Widget w);
 void _XmNavigInitialize(Widget request, Widget new_wid,
-			ArgList args, Cardinal num_args);
+			ArgList args, Cardinal *num_args);
 Boolean _XmNavigSetValues(Widget current, Widget request, Widget new_wid,
 			  ArgList args, Cardinal *num_args);
 void _XmNavigChangeManaged(Widget wid);
+void XmeNavigChangeManaged(Widget wid);
 void _XmNavigResize(Widget wid);
 void _XmValidateFocus(Widget wid);
 void _XmNavigDestroy(Widget wid);
-void _XmCallFocusMoved(Widget old, Widget new_wid, XEvent *event);
-void _XmMgrTraversal(Widget wid, XmTraversalDirection direction);
+Boolean _XmCallFocusMoved(Widget old, Widget new_wid, XEvent *event);
+Boolean _XmMgrTraversal(Widget wid, XmTraversalDirection direction);
 void _XmClearFocusPath(Widget wid);
 Boolean _XmFocusIsHere(Widget w);
 void _XmProcessTraversal(Widget w, XmTraversalDirection dir, Boolean check);
@@ -1242,31 +1295,42 @@ Widget _XmFindPrevTabGroup(Widget wid);
 void _XmSetInitialOfTabGroup(Widget tab_group, Widget init_focus);
 void _XmResetTravGraph(Widget wid);
 Boolean _XmFocusIsInShell(Widget wid);
+Boolean XmeFocusIsInShell(Widget wid);
 Boolean _XmShellIsExclusive(Widget wid);
 Widget _XmGetFirstFocus(Widget wid);
 				
 /*********************** TravAct.c ******************************/
 
+void _XmTrackShellFocus(Widget wid,
+			XtPointer client_data,
+			XEvent *event,
+			Boolean *dontSwallow);
+
 void _XmManagerEnter(Widget wid,
 		     XEvent *event_in,
 		     String *params,
 		     Cardinal *num_params);
+
 void _XmManagerLeave(Widget wid,
 		     XEvent *event_in,
 		     String *params,
 		     Cardinal *num_params);
+
 void _XmManagerFocusInInternal(Widget wid,
 			       XEvent *event,
 			       String *params,
 			       Cardinal *num_params);
+
 void _XmManagerFocusIn(Widget mw,
 		       XEvent *event,
 		       String *params,
 		       Cardinal *num_params);
+
 void _XmManagerFocusOut(Widget wid,
 		        XEvent *event,
 		        String *params,
 		        Cardinal *num_params);
+
 void _XmManagerUnmap(Widget mw,
 		     XEvent *event,
 		     String *params,
@@ -1454,33 +1518,8 @@ void _XmDragUnderAnimation(Widget w, XtPointer clientData, XtPointer callData);
 #include <Xm/GadgetP.h>
 #include <Xm/BaseClassP.h> 
 
-/*
- * default procs
- *
- * Motif has none of these.  (they should probably move to XmI.h)
- */
-void _XmDefaultButtonShadowThickness(Widget w, int offset, XrmValue *val);
-void _XmDefaultMultiClick(Widget w, int offset, XrmValue *val);
-void _XmBulletinBoardDialogStyleDefault(Widget w, int offset, XrmValue *val);
-void _XmRowColumnEntryClassDefault(Widget w, int offset, XrmValue *val);
-void _XmRowColumnIsHomogeneousDefault(Widget w, int offset, XrmValue *val);
-void _XmRowColumnMarginDefault(Widget w, int offset, XrmValue *val);
-void _XmRowColumnMenuAcceleratorDefault(Widget w, int offset, XrmValue *val);
-void _XmRowColumnPackingDefault(Widget w, int offset, XrmValue *val);
-void _XmRowColumnSpacingDefault(Widget w, int offset, XrmValue *val);
-void _XmCascadePixmapDefault(Widget w, int offset, XrmValue *val);
-void _XmScaleProcessingDirectionDefault(Widget w, int offset, XrmValue *val);
-void _XmScrollBarTroughColorDefault(Widget w, int offset, XrmValue *val);
-void _XmScrollBarProcessingDirectionDefault(Widget w, int offset, XrmValue *val);
-void _XmFrameHorizSpaceDefault(Widget w, int offset, XrmValue *val);
-void _XmFrameShadowTypeDefault(Widget w, int offset, XrmValue *val);
-void _XmFrameShadowThicknessDefault(Widget w, int offset, XrmValue *val);
-void _XmToggleButtonFillOnSelectDefault(Widget w, int offset, XrmValue *val);
-void _XmToggleButtonIndicatorTypeDefault(Widget w, int offset, XrmValue *val);
-void _XmVendorShellVirtualBindingsDefault(Widget w, int offset, XrmValue *val);
-
 #ifdef __cplusplus
-};
+}
 #endif
 
 #endif /* XM_P_H */
