@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ArrowB.c,v 1.19 1996/04/22 22:54:15 miers Exp $
+ * $Id: ArrowB.c,v 1.10 1996/12/18 02:35:18 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,24 +22,26 @@
  *
  **/
 
-static char rcsid[] = "$Id: ArrowB.c,v 1.19 1996/04/22 22:54:15 miers Exp $";
+static char rcsid[] = "$Id: ArrowB.c,v 1.10 1996/12/18 02:35:18 miers Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
 #include <Xm/ArrowBP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/TransltnsP.h>
-#include <stdio.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
 static void expose(Widget w, XEvent *event, Region region);
 static XtGeometryResult query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 
 /*
  * Resources for the ArrowButton class
@@ -77,7 +79,6 @@ static XtResource resources[] = {
 static void Activate(Widget w, XEvent *event, String *params, Cardinal *num_params);
 static void Arm(Widget w, XEvent *event, String *params, Cardinal *num_params);
 static void ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params);
-static void Cancel(Widget w, XEvent *event, String *params, Cardinal *num_params);
 static void Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params);
 static void EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params);
 static void Help(Widget w, XEvent *event, String *params, Cardinal *num_params);
@@ -104,7 +105,6 @@ static XtActionsRec actions[] = {
     {"Activate",                Activate},
     {"Arm",                     Arm},
     {"ArmAndActivate",          ArmAndActivate},
-    {"Cancel",                  Cancel},
     {"Disarm",                  Disarm},
     {"Enter",                   EnterWindow},
     {"Help",                    Help},
@@ -217,33 +217,67 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmARROW_BUTTON_BIT);
 }
 
+static void
+CreateArrowGC(Widget w)
+{
+    XGCValues values;
+    XtGCMask mask;
+
+    mask = GCForeground | GCBackground | GCFillStyle | GCFunction |
+		GCSubwindowMode | GCGraphicsExposures | GCPlaneMask;
+    values.function = GXcopy;
+    values.plane_mask = -1;
+    values.subwindow_mode = ClipByChildren;
+    values.graphics_exposures = False;
+    values.foreground = Prim_Foreground(w);
+    values.background = XtBackground(w);
+    values.fill_style = FillSolid;
+
+    AB_ArrowGC(w) = XtGetGC(w, mask, &values);
+}
+
+static void
+CreateInsensitiveGC(Widget w)
+{
+    XGCValues values;
+    XtGCMask mask;
+
+    mask = GCForeground | GCBackground | GCFillStyle | GCFunction | GCStipple |
+		GCSubwindowMode | GCGraphicsExposures | GCPlaneMask |
+		GCTileStipXOrigin | GCTileStipYOrigin;
+
+    values.function = GXcopy;
+    values.plane_mask = -1;
+    values.subwindow_mode = ClipByChildren;
+    values.graphics_exposures = False;
+    values.foreground = Prim_Foreground(w);
+    values.background = XtBackground(w);
+    values.fill_style = FillStippled;
+    values.ts_x_origin = values.ts_y_origin = 0;
+    values.stipple = XmGetPixmapByDepth(XtScreen(w),
+					XmEVEN_STIPPLE_IMAGE,
+					WhitePixelOfScreen(XtScreen(w)),
+					BlackPixelOfScreen(XtScreen(w)),
+					1);
+
+    AB_InsensitiveGC(w) = XtGetGC(w, mask, &values);
+}
+
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     if (XtWidth(request) == (Dimension)0)
-	XtWidth(new) = 20;
+	XtWidth(new_w) = 20;
     if (XtHeight(request) == (Dimension)0)
-	XtHeight(new) = 20;
+	XtHeight(new_w) = 20;
 
-    AB_Armed(new) = False;
-    AB_ArrowGC(new) = XtAllocateGC(new,
-				   0,0,
-				   NULL,
-				   0,0);
-    AB_InsensitiveGC(new) = XtAllocateGC(new,
-					 0,0,
-					 NULL,
-					 0,0);
-    XSetFillStyle(XtDisplay(new),
-		  AB_InsensitiveGC(new),
-		  FillStippled);
-
-    /* FIXME -- if the parent is a rc in a menu, need to set
-       multiClick to XmMULTICLICK_DISCARD */
+    AB_Armed(new_w) = False;
 
+    CreateArrowGC(new_w);
+    CreateInsensitiveGC(new_w);
 }
 
 static void
@@ -256,14 +290,15 @@ destroy(Widget w)
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XmArrowButtonWidget lw = (XmArrowButtonWidget) new;
+    XmArrowButtonWidget lw = (XmArrowButtonWidget) new_w;
     XmArrowButtonWidget ow = (XmArrowButtonWidget) old;
     Boolean refresh_needed = False;
 
+    XdbDebug(__FILE__, new_w, "ArrowB set_values\n");
     if (lw->arrowbutton.direction != ow->arrowbutton.direction)
 	refresh_needed = True;
 
@@ -293,10 +328,6 @@ expose(Widget w,
     else
 	myGC = AB_InsensitiveGC(w);
 
-    XSetForeground(XtDisplay(w),
-		   myGC,
-		   Prim_Foreground(w));
-
     _XmDrawShadows(XtDisplay(w), 
 		   XtWindow(w), 
 		   Prim_TopShadowGC(w),
@@ -339,10 +370,6 @@ expose(Widget w,
 		     1,
 		     AB_Direction(w));
     }
-
-#define superclass (&xmPrimitiveClassRec)
-    (*superclass->core_class.expose)(w, event, region);
-#undef superclass
 }
 
 static XtGeometryResult 
@@ -378,19 +405,28 @@ Arm(Widget w,
     Cardinal *num_params)
 {
     XmArrowButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     XdbDebug(__FILE__, w, "Arm\n");
 
+    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
+
     AB_Armed(w) = True;
+    AB_ArmTimeStamp(w) = event->xbutton.time;
 
-    cbs.reason = XmCR_ARM;
-    cbs.event = event;
-    cbs.click_count = AB_ClickCount(w);
-    XtCallCallbackList(w,
-		       AB_ArmCallback(w),
-		       (XtPointer)&cbs);
+    (*exp)(w, event, (Region)NULL);
 
-    expose(w, event, (Region)NULL);
+    if (AB_ArmCallback(w)) {
+	cbs.reason = XmCR_ARM;
+	cbs.event = event;
+	cbs.click_count = AB_ClickCount(w);
+
+	XFlush(XtDisplay(w));
+
+	XtCallCallbackList(w,
+			   AB_ArmCallback(w),
+			   (XtPointer)&cbs);
+    }
 }
 
 static void 
@@ -401,17 +437,30 @@ Activate(Widget w,
 {
     XmArrowButtonCallbackStruct cbs;
     XButtonEvent *ev = (XButtonEvent *)event;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
 
-    XdbDebug(__FILE__, w, "Activate\n");
+    XdbDebug(__FILE__, w, "Activate 000\n");
+
+    (*exp)(w, event, NULL);
+
+    XFlush(XtDisplay(w));
+
+    AB_ClickCount(w) = 1;
 
     if ((ev->x >= 0 && ev->x < XtWidth(w)) &&
 	(ev->y >= 0 && ev->y < XtHeight(w))) {
-	cbs.reason = XmCR_ACTIVATE;
-	cbs.event = event;
-	cbs.click_count = AB_ClickCount(w);
-	XtCallCallbackList(w,
-			   AB_ActivateCallback(w),
-			   (XtPointer)&cbs);
+
+	if (AB_ActivateCallback(w)) {
+	    cbs.reason = XmCR_ACTIVATE;
+	    cbs.event = event;
+	    cbs.click_count = AB_ClickCount(w);
+
+	    XFlush(XtDisplay(w));
+
+	    XtCallCallbackList(w,
+			       AB_ActivateCallback(w),
+			       (XtPointer)&cbs);
+	}
     }
 }
 
@@ -422,18 +471,55 @@ Disarm(Widget w,
        Cardinal *num_params)
 {
     XmArrowButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
     
     XdbDebug(__FILE__, w, "Disarm\n");
 
-    cbs.reason = XmCR_DISARM;
-    cbs.event = event;
-    cbs.click_count = AB_ClickCount(w);
-    XtCallCallbackList(w,
-		       AB_DisarmCallback(w),
-		       (XtPointer)&cbs);
+    if (AB_Armed(w)) {
+	AB_Armed(w) = False;
+	(*exp)(w, event, NULL);
+    }
+
+    if (AB_DisarmCallback(w)) {
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.click_count = AB_ClickCount(w);
+
+	XFlush(XtDisplay(w));
+
+	XtCallCallbackList(w,
+			   AB_DisarmCallback(w),
+			   (XtPointer)&cbs);
+    }
+}
+
+static void
+ArmTimeout(XtPointer data, XtIntervalId *id)
+{
+    XmArrowButtonCallbackStruct cbs;
+    Widget w = (Widget)data;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
+
+    if (AB_Armed(w) == False)
+	return;
 
     AB_Armed(w) = False;
-    expose(w, event, (Region)NULL);
+
+    (*exp)(w, NULL, NULL);
+
+    if (AB_ActivateCallback(w)) {
+	cbs.reason = XmCR_ACTIVATE;
+	cbs.event = NULL;
+	cbs.click_count = AB_ClickCount(w);
+
+	XFlush(XtDisplay(w));
+
+	XtCallCallbackList(w,
+			   AB_ActivateCallback(w),
+			   (XtPointer)&cbs);
+    }
+
+    Disarm(w, NULL, NULL, 0);
 }
 
 static void
@@ -443,7 +529,9 @@ ArmAndActivate(Widget w,
 	       Cardinal *num_params)
 {
     Arm(w, event, params, num_params);
-    Activate(w, event, params, num_params);
+
+    AB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
+				  ACTIVATE_DELAY, ArmTimeout, (XtPointer)w);
 }
 
 static void
@@ -456,9 +544,11 @@ EnterWindow(Widget w,
     int xsize = XtWidth(w) - margin * 2;
     int ysize = XtHeight(w) - margin * 2;
 
-    XtCallActionProc(w, "PrimitiveEnter", event, params, *num_params);
+    XdbDebug(__FILE__, w, "EnterWindow\n");
+
+    if (AB_Armed(w)) {
+	_XmPrimitiveLeave(w, event, NULL, NULL);
 
-    if (AB_Armed(w)) 
         _XmDrawArrow(XtDisplay(w),
                      XtWindow(w),
                      Prim_TopShadowGC(w),
@@ -472,6 +562,7 @@ EnterWindow(Widget w,
                       */
                      1,
                      AB_Direction(w));
+    }
 }
 
 static void
@@ -484,9 +575,11 @@ LeaveWindow(Widget w,
     int xsize = XtWidth(w) - margin * 2;
     int ysize = XtHeight(w) - margin * 2;
 
-    XtCallActionProc(w, "PrimitiveLeave", event, params, *num_params);
+    XdbDebug(__FILE__, w, "LeaveWindow\n");
+
+    if (AB_Armed(w)) {
+	_XmPrimitiveLeave(w, event, params, num_params);
 
-    if (AB_Armed(w))
 	_XmDrawArrow(XtDisplay(w),
 		     XtWindow(w),
 		     Prim_BottomShadowGC(w),
@@ -500,6 +593,7 @@ LeaveWindow(Widget w,
 		      */
 		     1,
 		     AB_Direction(w));
+    }
 }
 
 static void
@@ -509,6 +603,9 @@ MultiArm(Widget w,
 	 Cardinal *num_params)
 {
     XdbDebug(__FILE__, w, "MultiArm\n");
+
+    if (AB_MultiClick(w) == XmMULTICLICK_KEEP)
+	Arm(w, event, NULL, NULL);
 }
 
 static void
@@ -517,15 +614,46 @@ MultiActivate(Widget w,
 	      String *params,
 	      Cardinal *num_params)
 {
-    XdbDebug(__FILE__, w, "MultiActivate\n");
-}
+    XButtonEvent *ev = (XButtonEvent *)event;
+    XmPushButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
 
-static void
-Cancel(Widget w,
-       XEvent *event,
-       String *params,
-       Cardinal *num_params)
-{
+    XdbDebug(__FILE__, w, "PushB: MultiClick\n");
+
+    if (AB_MultiClick(w) == XmMULTICLICK_KEEP) {
+	Time mctime = XtGetMultiClickTime(XtDisplay(w));
+
+	if ((event->xbutton.time - AB_ArmTimeStamp(w)) < mctime)
+	    AB_ClickCount(w)++;
+	else
+	    AB_ClickCount(w) = 1;
+
+	AB_Armed(w) = False;
+
+	(*exp)(w, event, NULL);
+
+	if (ev->type == KeyPress || ev->type == KeyRelease
+	    || ((ev->x >= 0 && ev->x < XtWidth(w))
+	    && (ev->y >= 0 && ev->y < XtHeight(w))))
+	{
+	    if (AB_MultiClick(w) == XmMULTICLICK_DISCARD &&
+		AB_ClickCount(w) > 1)
+		return;
+
+	    if (AB_ActivateCallback(w)) {
+		cbs.reason = XmCR_ACTIVATE;
+		cbs.event = event;
+		cbs.click_count = AB_ClickCount(w);
+
+		XFlush(XtDisplay(w));
+
+		XtCallCallbackList(w,
+				   AB_ActivateCallback(w),
+				   (XtPointer)&cbs);
+	    }
+	}
+	Disarm(w, event, params, num_params);
+    }
 }
 
 static void
@@ -534,6 +662,7 @@ Help(Widget w,
        String *params,
        Cardinal *num_params)
 {
+    XtCallActionProc(w, "PrimitiveHelp", event, params, *num_params);
 }
 
 Widget
