@@ -1,6 +1,6 @@
 /**
  *
- * $Id: BaseClass.c,v 1.25 1996/05/02 02:29:51 miers Exp $
+ * $Id: BaseClass.c,v 1.9 1997/01/07 02:35:34 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,11 +22,13 @@
  *
  **/
 
-static char rcsid[] = "$Id: BaseClass.c,v 1.25 1996/05/02 02:29:51 miers Exp $";
+static char rcsid[] = "$Id: BaseClass.c,v 1.9 1997/01/07 02:35:34 miers Exp $";
+
+#include <LTconfig.h>
+#include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
 #include <Xm/BaseClassP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/DisplayP.h>
 #include <Xm/ExtObjectP.h>
 #include <Xm/ScreenP.h>
@@ -38,6 +40,9 @@ static char rcsid[] = "$Id: BaseClass.c,v 1.25 1996/05/02 02:29:51 miers Exp $";
 #include <X11/RectObjP.h>
 #include <X11/Xutil.h>
 #include <X11/Xfuncs.h>
+#include <X11/Xfuncproto.h>
+
+#include <XmI/DebugUtil.h>
 
 XrmQuark XmQmotif = 0;
 int _XmInheritClass;
@@ -87,7 +92,7 @@ _XmIsSlowSubclass(WidgetClass wc,
 Boolean 
 _XmIsStandardMotifWidgetClass(WidgetClass wc)
 {
-    XmBaseClassExt *class, *super;
+    XmBaseClassExt *bclass, *super;
     unsigned char  *flags, *sflags;
     int             i;
 
@@ -95,8 +100,8 @@ _XmIsStandardMotifWidgetClass(WidgetClass wc)
      * If the widget class in question has no extension record it is
      * apparently no real LessTif widget class.
      */
-    class = (XmBaseClassExt *)_XmGetBaseClassExtPtr(wc, XmQmotif);
-    if ((class == NULL) || (*class == NULL))
+    bclass = (XmBaseClassExt *)_XmGetBaseClassExtPtr(wc, XmQmotif);
+    if ((bclass == NULL) || (*bclass == NULL))
         return False;
     super = (XmBaseClassExt *)_XmGetBaseClassExtPtr(wc->core_class.superclass,
 						    XmQmotif);
@@ -109,7 +114,7 @@ _XmIsStandardMotifWidgetClass(WidgetClass wc)
      * flags and not the application defined bits from bit 129 upwards.
      */
     if ((super == NULL) || (*super == NULL) ) {
-	flags = (*class)->flags;
+	flags = (*bclass)->flags;
 	for (i = XmLAST_FAST_SUBCLASS_BIT << 3; i >= 0; i--) {
 	    if (flags[i] != 0)
 		return True;
@@ -122,7 +127,7 @@ _XmIsStandardMotifWidgetClass(WidgetClass wc)
      * We only consider the standard motif flags, not the possible application
      * defined bits from bit 129 on.
      */
-    flags = (*class)->flags;
+    flags = (*bclass)->flags;
     sflags = (*super)->flags;
     for (i = XmLAST_FAST_SUBCLASS_BIT << 3; i >= 0; i--) {
 	if (flags[i] != sflags[i])
@@ -190,7 +195,7 @@ FindAssociatedContext(unsigned int extType){
  * through Composite children!  Whadaya think?
  */
 XmWidgetExtData 
-_XmGetWidgetExtData(Widget widget, unsigned int extType)
+_XmGetWidgetExtData(Widget widget, unsigned char extType)
 {
     XContext c = FindAssociatedContext(extType);
     XmWidgetExtDataStackPtr node;
@@ -213,7 +218,7 @@ _XmFreeWidgetExtData(Widget widget)
 void 
 _XmPushWidgetExtData(Widget widget,
 		     XmWidgetExtData data,
-		     unsigned int extType)
+		     unsigned char extType)
 {
     XContext c = FindAssociatedContext(extType);
     XmWidgetExtDataStackPtr head = NULL, node;
@@ -232,7 +237,7 @@ _XmPushWidgetExtData(Widget widget,
 void 
 _XmPopWidgetExtData(Widget widget,
 		    XmWidgetExtData *dataRtn,
-		    unsigned int extType)
+		    unsigned char extType)
 {
     XContext c = FindAssociatedContext(extType);
     XmWidgetExtDataStackPtr node;
@@ -280,7 +285,7 @@ _XmGetWrapperData(WidgetClass wc)
 	bce->next_extension = NULL;
 	bce->record_type = XmQmotif;
 	bce->version = XmBaseClassExtVersion;
-	bce->record_size = sizeof(XmWrapperData);
+	bce->record_size = sizeof(XmBaseClassExtRec);
 	*ext = bce;
     }
     /*
@@ -355,13 +360,13 @@ _XmPushWrapperData(WidgetClass wc)
      */
     if (bce->wrapperData != NULL) {
 	wrapper = (XmWrapperData)XtMalloc(sizeof(XmWrapperDataRec));
-	memcpy(wrapper, bce->wrapperData, sizeof(XmWrapperDataRec));
+	bcopy(bce->wrapperData, wrapper, sizeof(XmWrapperDataRec));
 	wrapper->widgetClass = wc;
         wrapper->next = bce->wrapperData;
     }
     else {
         wrapper = (XmWrapperData)XtMalloc(sizeof(XmWrapperDataRec));
-        memset(wrapper, 0, sizeof(XmWrapperDataRec));
+        bzero(wrapper, sizeof(XmWrapperDataRec));
     }
 
     bce->wrapperData = wrapper;
@@ -679,11 +684,17 @@ GeomDepth(WidgetClass MyWc)
 static XmBaseClassExtRec DummySuperClassExtension = { 0 };
 
 static void (*ObjectClassPartInit)(WidgetClass c) = NULL;
-static Boolean(*ObjectSetValues)(Widget old, Widget request, Widget new,
+static Boolean(*ObjectSetValues)(Widget old, Widget request, Widget new_w,
 				 ArgList args, Cardinal *num_args) = NULL;
 
-static Boolean
-IsClassOrSubclass(WidgetClass wc, WidgetClass sclass)
+/*
+ * Much the same as XtIsSubclass(). But this time it checks whether a
+ * widget *class* is a subclass of some other class. This function is
+ * useful not only to us, but also to others. That may be the reason why
+ * M*tif 2.0 now exports it.
+ */
+Boolean
+_XmIsSubclassOf(WidgetClass wc, WidgetClass sclass)
 {
     WidgetClass parent = wc;
 
@@ -694,7 +705,7 @@ IsClassOrSubclass(WidgetClass wc, WidgetClass sclass)
 	return True;
 
     return False;
-}
+} /* _XmIsSubclassOf */
 
 /*
  * figure out if this widget class is one of those that needs inheritance
@@ -708,16 +719,11 @@ ResolveWrappers(WidgetClass wc, WidgetClass sclass)
     Wrapper      = _XmGetWrapperData(wc);
     SuperWrapper = _XmGetWrapperData(sclass);
 
-/*
- * MLM: 960312 -- this has to go until I figure out the problem with
- * MessageB/FileSB.
- */
-#if 1
     /*
      * Only the vendor shell class and its subclasses will take part in
      * the resize wrapper game...
      */
-    if (IsClassOrSubclass(wc, vendorShellWidgetClass)) {
+    if (_XmIsSubclassOf(wc, vendorShellWidgetClass)) {
 
         /*
          * If the widget class wants to inherit the realize method from
@@ -737,7 +743,7 @@ ResolveWrappers(WidgetClass wc, WidgetClass sclass)
         wc->core_class.realize = RealizeWrappers[RealizeDepth(wc)];
     }
 
-    if (IsClassOrSubclass(wc, rectObjClass)) {
+    if (_XmIsSubclassOf(wc, rectObjClass)) {
 
         /* FIX ME! vendorShell needs its own resize method... */
         if (wc->core_class.resize == XtInheritResize)
@@ -747,11 +753,10 @@ ResolveWrappers(WidgetClass wc, WidgetClass sclass)
 
         wc->core_class.resize = ResizeWrappers[ResizeDepth(wc)];
     }
-#endif
 
 #define C_C(wc) ((CompositeClassRec *) wc)
-#if 0 /* FIX ME */
-    if (IsClassOrSubclass(wc, compositeWidgetClass)) {
+
+    if (_XmIsSubclassOf(wc, compositeWidgetClass)) {
         if (C_C(wc)->composite_class.geometry_manager ==
 	    XtInheritGeometryManager)
             Wrapper->geometry_manager = SuperWrapper->geometry_manager;
@@ -762,7 +767,6 @@ ResolveWrappers(WidgetClass wc, WidgetClass sclass)
         C_C(wc)->composite_class.geometry_manager = 
 		GeometryWrappers[GeomDepth(wc)];
     }
-#endif
 #undef C_C
 }
 
@@ -782,27 +786,22 @@ BaseClassPartInitialize(WidgetClass wc)
      * superclasses which are  Xt 'kernel' classes. And the Xt intrinsics
      * have no knowledge of the dreaded things we're doing here... Sigh,
      * in former times all things were better!
+     *
+     * --aldi: Testing M*tif's bce records using the wctest programs shows
+     * that the list of widget classes below is more than sufficient. Running
+     * wctest indicates that all these widget classes have their own bce
+     * record anyway. So the test below is overcautious -- at least when
+     * LessTif has reached a certain quality and all those widget classes
+     * have their own bce records attached to their class records.
      */
     if ((wc == (WidgetClass) &vendorShellClassRec) || /* wmShell:: */
         (wc == (WidgetClass) &xmPrimitiveClassRec) || /* core:: */
         (wc == (WidgetClass) &xmGadgetClassRec   ) || /* rectObj:: */
-	/* THE NEXT ONE TOO ? */
-        (wc == (WidgetClass) &xmDragIconClassRec ) || /* rectObj:: */
         (wc == (WidgetClass) &xmManagerClassRec  ) || /* constraint:: */
         (wc == (WidgetClass) &xmDisplayClassRec  ) || /* applicationShell:: */
         (wc == (WidgetClass) &xmScreenClassRec   ) || /* core:: */
         (wc == (WidgetClass) &xmMenuShellClassRec) || /* overrideShell:: */
-        (wc == (WidgetClass) &xmExtClassRec      ) || /* object:: */
-	/* THE NEXT ONE TOO ? */
-        (wc == (WidgetClass) &xmDropTransferClassRec)) /* object:: */ {
-	/*
-	 * FIX ME! xmDropSiteManagerClassRec HAS NO PUBLIC class record but may
-	 * use the base class extension mechanism. Does it really need the
-	 * extension mechanism?
-	 *
-	 * FIX ME! Other widget classes missing which are derived directly
-	 * from the Xt intrinsics?
-	 */
+        (wc == (WidgetClass) &xmExtClassRec      )) { /* object:: */
         need_ext = True;
     }
     else if ((wc == (WidgetClass)&rectObjClassRec) ||
@@ -818,7 +817,12 @@ BaseClassPartInitialize(WidgetClass wc)
     sclass = wc->core_class.superclass;
 
     ext = _XmGetBaseClassExtPtr(wc, XmQmotif);
-    sext = _XmGetBaseClassExtPtr(wc->core_class.superclass, XmQmotif);
+    /*
+     * If someone really should instanciate an Object...
+     */
+    if ( sclass ) {
+        sext = _XmGetBaseClassExtPtr(sclass, XmQmotif);
+    }
 
     XdbDebug(__FILE__, NULL, "BaseClassPartInit on %s with %s base ext rec.\n",
              wc->core_class.class_name,
@@ -913,7 +917,7 @@ BaseClassPartInitialize(WidgetClass wc)
      * Let us be notorious and always clean the flags. The flags have
      * to be set during the class_part_init method of a widget class.
      */
-    memset((*ext)->flags, 0, 32);
+    bzero((*ext)->flags, 32);
 
     /*
      * If the superclass hasn't a base class extension record, use
@@ -940,7 +944,7 @@ BaseClassPartInitialize(WidgetClass wc)
     if (new_ext->initializePrehook == XmInheritInitializePrehook)
 	new_ext->initializePrehook = super_ext->initializePrehook;
     if (new_ext->initializePosthook == XmInheritInitializePosthook)
-	new_ext->initializePosthook = super_ext->initializePrehook;
+	new_ext->initializePosthook = super_ext->initializePosthook;
     if (new_ext->setValuesPrehook == XmInheritSetValuesPrehook)
 	new_ext->setValuesPrehook = super_ext->setValuesPrehook;
     if (new_ext->setValuesPosthook == XmInheritSetValuesPosthook)
@@ -1020,7 +1024,6 @@ RealizeWrapper(Widget w, XtValueMask *ValueMask,
 			   VSEP_RealizeCallback(ext->widget),
 			   NULL);
     }
-    
 }
 
 static void
@@ -1029,6 +1032,7 @@ ResizeWrapper(Widget w, int IntentedWrapperDepth)
     int             WindUp;
     WidgetClass     wc = XtClass(w);
     XmWrapperData   WrapperData;
+    Boolean         need_navig = False;
 
     WindUp = ResizeDepth(wc) - IntentedWrapperDepth;
     if (WindUp > 0) {
@@ -1037,6 +1041,10 @@ ResizeWrapper(Widget w, int IntentedWrapperDepth)
         } while (--WindUp > 0);
     }
 
+    /* _XmNavigResize is only used for shell widgets */
+    if (XtParent(w) != NULL && XtIsShell(XtParent(w)))
+	need_navig = True;
+
     /*
      * For the resize wrapper it is just fine to have no resize method.
      */
@@ -1044,14 +1052,22 @@ ResizeWrapper(Widget w, int IntentedWrapperDepth)
     if ((WrapperData != NULL) && (WrapperData->resize != NULL)) {
         WrapperData->resize(w);
     }
+
+    if (need_navig)
+	_XmNavigResize(w);
 }
 
+/*
+ * fun with GeoUtils showed me why this was broken.  Remember how a 
+ * geometry manager is called with the child requesting the change?
+ * Well, guess which Widget class we should use....  The parent's!
+ */
 static XtGeometryResult
 GeometryHandlerWrapper(Widget w, XtWidgetGeometry *request,
 		       XtWidgetGeometry *reply, int IntentedWrapperDepth)
 {
     int             WindUp;
-    WidgetClass     wc = XtClass(w);
+    WidgetClass     wc = XtClass(XtParent(w));
     XmWrapperData   WrapperData;
 
     WindUp = GeomDepth(wc) - IntentedWrapperDepth;
@@ -1063,7 +1079,7 @@ GeometryHandlerWrapper(Widget w, XtWidgetGeometry *request,
 
     WrapperData = _XmGetWrapperData(wc);
     if ((WrapperData == NULL) || (WrapperData->geometry_manager == NULL)) {
-        _XmError(w,
+        _XmError(XtParent(w),
 	    "PANIC: no geometry_manager procedure specified for this widget.");
     }
 
@@ -1162,10 +1178,10 @@ ClassPartInitRootWrapper(WidgetClass widget_class)
  * the control to the last one before activating the post-hook.
  */
 static void
-InitializeLeafWrapper(Widget request, Widget new,
+InitializeLeafWrapper(Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
-    WidgetClass wc = XtClass(new);
+    WidgetClass wc = XtClass(new_w);
     XmBaseClassExt *bce = NULL;
     XmWrapperData wrap;
 
@@ -1183,8 +1199,8 @@ InitializeLeafWrapper(Widget request, Widget new,
 	 * MLM:  Ahhh.  Somebody said something about this, but I missed the
 	 * real meaning.
 	 */
-	if (!XtIsShell(new) && XtParent(new) && XtIsConstraint(XtParent(new))) {
-	    wc = XtClass(XtParent(new));
+	if (!XtIsShell(new_w) && XtParent(new_w) && XtIsConstraint(XtParent(new_w))) {
+	    wc = XtClass(XtParent(new_w));
 	    wrap = _XmPopWrapperData(wc);
 	    ((ConstraintClassRec *)wc)->constraint_class.initialize =
 		wrap->initializeLeaf;
@@ -1195,9 +1211,9 @@ InitializeLeafWrapper(Widget request, Widget new,
 	}
 
 	if (wrap->initializeLeaf)
-	    (wrap->initializeLeaf)(request, new, args, num_args);
+	    (wrap->initializeLeaf)(request, new_w, args, num_args);
 
-	((*bce)->initializePosthook)(request, new, args, num_args);
+	((*bce)->initializePosthook)(request, new_w, args, num_args);
 
 	_XmFreeWrapperData(wrap);
     }
@@ -1205,13 +1221,13 @@ InitializeLeafWrapper(Widget request, Widget new,
 
 /*
  * The initialze pre-hook is called on any fresh widget instance before
- * any other initialize method has ever seen this new widget.
+ * any other initialize method has ever seen this new_w widget.
  */
 static void
-InitializeRootWrapper(Widget request, Widget new,
+InitializeRootWrapper(Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
-    WidgetClass wc = XtClass(new);
+    WidgetClass wc = XtClass(new_w);
     XmBaseClassExt *bce = NULL;
     XmWrapperData wrap;
 
@@ -1228,11 +1244,11 @@ InitializeRootWrapper(Widget request, Widget new,
      * has been called.
      */
     if ((*bce)->initializePrehook)
-	((*bce)->initializePrehook)(request, new, args, num_args);
+	((*bce)->initializePrehook)(request, new_w, args, num_args);
 
     if ((*bce)->initializePosthook) {
-	if (!XtIsShell(new) && XtParent(new) && XtIsConstraint(XtParent(new))) {
-	    wc = XtClass(XtParent(new));
+	if (!XtIsShell(new_w) && XtParent(new_w) && XtIsConstraint(XtParent(new_w))) {
+	    wc = XtClass(XtParent(new_w));
 	    wrap = _XmPushWrapperData(wc);
 	    wrap->initializeLeaf =
 	    	((ConstraintClassRec *)wc)->constraint_class.initialize;
@@ -1254,10 +1270,10 @@ InitializeRootWrapper(Widget request, Widget new,
  * and the post-hook. 'nuff said.
  */
 static Boolean
-SetValuesLeafWrapper(Widget old, Widget request, Widget new,
+SetValuesLeafWrapper(Widget old, Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
-    WidgetClass wc = XtClass(new);
+    WidgetClass wc = XtClass(new_w);
     XmBaseClassExt *bce = NULL;
     XmWrapperData wrap;
     Boolean refresh = False;
@@ -1269,9 +1285,9 @@ SetValuesLeafWrapper(Widget old, Widget request, Widget new,
 
     if ((*bce)->setValuesPosthook) {
 
-	if (!XtIsShell(new) && XtParent(new) && XtIsConstraint(XtParent(new))) {
-	    wrap = _XmPopWrapperData(XtClass(XtParent(new)));
-	    wc = XtClass(XtParent(new));
+	if (!XtIsShell(new_w) && XtParent(new_w) && XtIsConstraint(XtParent(new_w))) {
+	    wrap = _XmPopWrapperData(XtClass(XtParent(new_w)));
+	    wc = XtClass(XtParent(new_w));
 	    (((ConstraintClassRec *)wc)->constraint_class.set_values)
 		= wrap->setValuesLeaf;
 	}
@@ -1281,9 +1297,9 @@ SetValuesLeafWrapper(Widget old, Widget request, Widget new,
 	}
 
 	if (wrap->setValuesLeaf != NULL)
-	    refresh = wrap->setValuesLeaf(old, request, new, args, num_args);
+	    refresh = wrap->setValuesLeaf(old, request, new_w, args, num_args);
 
-	refresh |= (*bce)->setValuesPosthook(old, request, new,
+	refresh |= (*bce)->setValuesPosthook(old, request, new_w,
 					    args, num_args);
 	_XmFreeWrapperData(wrap);
     }
@@ -1300,7 +1316,7 @@ SetValuesLeafWrapper(Widget old, Widget request, Widget new,
  * set_values handler as well as the post-hook.
  */
 static Boolean
-SetValuesRootWrapper(Widget old, Widget request, Widget new,
+SetValuesRootWrapper(Widget old, Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
     WidgetClass wc = XtClass(request);
@@ -1312,11 +1328,11 @@ SetValuesRootWrapper(Widget old, Widget request, Widget new,
 
     if (!bce || !*bce) {
 	if (ObjectSetValues)
-	    return (ObjectSetValues)(old, request, new, args, num_args);
+	    return (ObjectSetValues)(old, request, new_w, args, num_args);
     }
 
     if ((*bce)->setValuesPrehook)
-	refresh = ((*bce)->setValuesPrehook)(old, request, new, args, num_args);
+	refresh = ((*bce)->setValuesPrehook)(old, request, new_w, args, num_args);
 
     /*
      * If this widget has a constraint resources, then the
@@ -1325,9 +1341,9 @@ SetValuesRootWrapper(Widget old, Widget request, Widget new,
      * case we have to hook in the setValues method of the parent.
      */
     if ((*bce)->setValuesPosthook) {
-	if (!XtIsShell(new) && XtParent(new) && XtIsConstraint(XtParent(new))) {
-	    wrap = _XmPushWrapperData(XtClass(XtParent(new)));
-	    wc = XtClass(XtParent(new));
+	if (!XtIsShell(new_w) && XtParent(new_w) && XtIsConstraint(XtParent(new_w))) {
+	    wrap = _XmPushWrapperData(XtClass(XtParent(new_w)));
+	    wc = XtClass(XtParent(new_w));
 	    wrap->setValuesLeaf
 		= ((ConstraintClassRec *)wc)->constraint_class.set_values;
 	    (((ConstraintClassRec *)wc)->constraint_class.set_values)
@@ -1341,7 +1357,7 @@ SetValuesRootWrapper(Widget old, Widget request, Widget new,
     }
 
     if (ObjectSetValues)
-	refresh |= (ObjectSetValues)(old, request, new, args, num_args);
+	refresh |= (ObjectSetValues)(old, request, new_w, args, num_args);
 
     return refresh;
 }
