@@ -1,6 +1,6 @@
 /**
  *
- * $Id: BulletinBoard.c,v 1.34 1996/04/22 22:54:20 miers Exp $
+ * $Id: BulletinBoard.c,v 1.18 1997/01/11 02:19:41 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -25,10 +25,11 @@
 static char rcsid[] = "$Id";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
 #include <Xm/BaseClassP.h>
 #include <Xm/BulletinBP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/TransltnsP.h>
 #include <Xm/DialogS.h>
 #include <Xm/ArrowB.h>
@@ -37,26 +38,27 @@ static char rcsid[] = "$Id";
 #include <Xm/LabelGP.h>
 #include <Xm/PushB.h>
 #include <Xm/PushBG.h>
-#include <Xm/XmI.h>
-#include <stdio.h>
+#include <Xm/MwmUtil.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
 static XtGeometryResult query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void expose(Widget w, XEvent *event, Region region);
 static XtGeometryResult geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply);
 static void change_managed(Widget w);
-static void ConstraintInitialize(Widget request, Widget new, Arg *args, Cardinal *nargs);
-static Boolean ConstraintSetValues(Widget old, Widget request, Widget new, Arg *args, Cardinal *nargs);
-static void DoResize(Widget w);
+static void ConstraintInitialize(Widget request, Widget new_w, Arg *args, Cardinal *nargs);
+static Boolean ConstraintSetValues(Widget old, Widget request, Widget new_w, Arg *args, Cardinal *nargs);
 static void realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes);
-static void InsertChild(Widget w);
 static void resize(Widget w);
+static void InsertChild(Widget w);
+static void DeleteChild(Widget w);
 static Boolean _XmBBParentProcess(Widget widget, XmParentProcessData data);
 
 /*
@@ -97,17 +99,17 @@ static XtResource resources[] = {
     },
     {
 	XmNfocusCallback, XmCCallback, XmRCallback,
-	sizeof(XtCallbackList), Offset(focus_callback_list),
+	sizeof(XtCallbackList), Offset(focus_callback),
 	XmRImmediate, (XtPointer)NULL
     },
     {
 	XmNmapCallback, XmCCallback, XmRCallback,
-	sizeof(XtCallbackList), Offset(map_callback_list),
+	sizeof(XtCallbackList), Offset(map_callback),
 	XmRImmediate, (XtPointer)NULL
     },
     {
 	XmNunmapCallback, XmCCallback, XmRCallback,
-	sizeof(XtCallbackList), Offset(unmap_callback_list),
+	sizeof(XtCallbackList), Offset(unmap_callback),
 	XmRImmediate, (XtPointer)NULL
     },
     {
@@ -171,7 +173,7 @@ static XmSyntheticResource syn_resources[] = {
     {
 	XmNdialogTitle,
 	sizeof(XmString), Offset(dialog_title),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNmarginWidth,
@@ -201,6 +203,16 @@ char _XmBulletinB_defaultTranslations[] =
      <Key>Return:               ManagerParentActivate()\n\
      <Key>:                     ManagerGadgetKeyInput()";
 
+static char _XmBulletinB_mapTranslations[] =
+    "<Map>:           BulletinBoardMap()\n\
+     <Unmap>:         BulletinBoardMap()";
+
+static XtActionsRec actions[] = {
+    { "BulletinBoardMap", _XmBulletinBoardMap}
+};
+
+static XtTranslations mapTrans;
+
 static XmBaseClassExtRec _XmBulletinBCoreClassExtRec = {
     /* next_extension            */ NULL,
     /* record_type               */ NULLQUARK,                             
@@ -222,8 +234,8 @@ static XmBaseClassExtRec _XmBulletinBCoreClassExtRec = {
     /* compiled_ext_resources    */ NULL, /* FIXME */
     /* num_ext_resources         */ 0, /* FIXME */
     /* use_sub_resources         */ FALSE, /* FIXME */
-    /* widget_navigable          */ NULL, /* FIXME */
-    /* focus_change              */ NULL, /* FIXME */
+    /* widget_navigable          */ XmInheritWidgetNavigable,
+    /* focus_change              */ XmInheritFocusChange,
     /* wrapper_data              */ NULL
 };
 
@@ -259,13 +271,13 @@ XmBulletinBoardClassRec xmBulletinBoardClassRec = {
 	/* initialize            */ initialize,
 	/* initialize_hook       */ NULL,
 	/* realize               */ realize,
-	/* actions               */ NULL,
-	/* num_actions           */ 0,
+	/* actions               */ actions,
+	/* num_actions           */ XtNumber(actions),
 	/* resources             */ resources,
 	/* num_resources         */ XtNumber(resources),
 	/* xrm_class             */ NULLQUARK,
 	/* compress_motion       */ TRUE,
-	/* compress_exposure     */ XtExposeCompressMultiple,
+	/* compress_exposure     */ XtExposeCompressMaximal,
 	/* compress_enterleave   */ TRUE,
 	/* visible_interest      */ FALSE,
 	/* destroy               */ destroy,
@@ -287,8 +299,8 @@ XmBulletinBoardClassRec xmBulletinBoardClassRec = {
     {
 	/* geometry manager */ geometry_manager, 
         /* change_managed   */ change_managed, 
-        /* insert_child     */ InsertChild,
-        /* delete_child     */ XtInheritDeleteChild,
+        /* insert_child     */	InsertChild,
+        /* delete_child     */	DeleteChild,
         /* extension        */ (XtPointer) &bbCompositeExt,
     },
     /* Constraint class part */
@@ -326,6 +338,8 @@ static void
 class_initialize()
 {
     _XmBulletinBCoreClassExtRec.record_type = XmQmotif;
+
+    mapTrans = XtParseTranslationTable(_XmBulletinB_mapTranslations);
 }
 
 static void
@@ -367,65 +381,116 @@ class_part_initialize(WidgetClass widget_class)
 
     if (bbclass->bulletin_board_class.focus_moved_proc ==
 	XmInheritFocusMovedProc)
+    {
 	if (sclass->bulletin_board_class.focus_moved_proc)
 	    bbclass->bulletin_board_class.focus_moved_proc =
 		sclass->bulletin_board_class.focus_moved_proc;
 	else
 	    bbclass->bulletin_board_class.focus_moved_proc =
 		_XmBulletinBoardFocusMoved;
+    }
 
     _XmFastSubclassInit(widget_class, XmBULLETIN_BOARD_BIT);
 }
 
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XmBulletinBoardWidget bb = (XmBulletinBoardWidget)new;
+    XmBulletinBoardWidget bb = (XmBulletinBoardWidget)new_w;
     XmBulletinBoardWidgetClass bbclass;
 
     if (!BB_LabelFontList(bb))
-	BB_LabelFontList(bb) = _XmGetDefaultFontList(new, XmLABEL_FONTLIST);
+	BB_LabelFontList(bb) = _XmGetDefaultFontList(new_w, XmLABEL_FONTLIST);
 
     if (!BB_ButtonFontList(bb))
-	BB_ButtonFontList(bb) = _XmGetDefaultFontList(new, XmBUTTON_FONTLIST);
+	BB_ButtonFontList(bb) = _XmGetDefaultFontList(new_w, XmBUTTON_FONTLIST);
 
     if (!BB_TextFontList(bb))
-	BB_TextFontList(bb) = _XmGetDefaultFontList(new, XmTEXT_FONTLIST);
+	BB_TextFontList(bb) = _XmGetDefaultFontList(new_w, XmTEXT_FONTLIST);
 
-    if (XtIsSubclass(XtParent(new), xmDialogShellWidgetClass)) {
-	/* Shouldn't this be in synthetic resource handlers ? FIX ME */
-	if (bb->bulletin_board.dialog_title) {
+    XtAugmentTranslations(new_w, mapTrans);
+
+    if (XmIsDialogShell(XtParent(new_w))) {
+
+	switch (BB_DialogStyle(new_w)) {
+	case XmDIALOG_MODELESS:
+	    XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			  MWM_INPUT_MODELESS, NULL);
+	    break;
+	case XmDIALOG_PRIMARY_APPLICATION_MODAL:
+	    XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			  MWM_INPUT_PRIMARY_APPLICATION_MODAL, NULL);
+	    break;
+	case XmDIALOG_FULL_APPLICATION_MODAL:
+	    XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			  MWM_INPUT_FULL_APPLICATION_MODAL, NULL);
+	    break;
+	case XmDIALOG_SYSTEM_MODAL:
+	    XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			  MWM_INPUT_SYSTEM_MODAL, NULL);
+	    break;
+	}
+	/*
+	 * --aldi: Better do the realize call of our parent -- which is a
+	 * shell -- first here. Otherwise we have *real trouble* with
+	 * the grab mechanism. This replaces the call to XtRealizeWidget()
+	 * within the initialize_posthook() method of the BulletinBoard
+	 * class. And it does get us rid off that %"&!$ double installed
+	 * grabs when using a dialog shell.
+	 */
+	XtRealizeWidget(XtParent(new_w));
+
+	/*
+	 * Shouldn't this be in synthetic resource handlers?
+	 *
+	 * MLM: No.  The bottom line is that the resource converter can
+	 * bypass the arglist passed to initialize; this means that it
+	 * makes just as much sense to catch this here as it does in a
+	 * Synth resource handler, and duplicating the work here.  This
+	 * little jewel of knowledge came from Danny and I working out the
+	 * rules for internal strings, as applicable for Labels.
+	 */
+	if (BB_DialogTitle(new_w))
+	{
 	    char *p;
 
-	    if (XmStringGetLtoR(bb->bulletin_board.dialog_title,
+	    if (XmStringGetLtoR(BB_DialogTitle(new_w),
 				XmFONTLIST_DEFAULT_TAG, &p)) {
-		XtVaSetValues(XtParent(new), XtNtitle, p, NULL);
+		XtVaSetValues(XtParent(new_w), XtNtitle, p, NULL);
 		XtFree(p);
 	    }
+	    BB_DialogTitle(new_w) = XmStringCopy(BB_DialogTitle(new_w));
 	}
     }
 
-    if (XtIsSubclass(XtParent(new), xmDialogShellWidgetClass) ||
-	XtIsSubclass(XtParent(new), vendorShellWidgetClass)) {
-	if (MGR_ShadowThickness(new) == 0)
-	    MGR_ShadowThickness(new) = 1;
+    if (XtIsSubclass(XtParent(new_w), xmDialogShellWidgetClass) ||
+	XtIsSubclass(XtParent(new_w), vendorShellWidgetClass)) {
+	if (MGR_ShadowThickness(new_w) == 0)
+	    MGR_ShadowThickness(new_w) = 1;
     }
 
-    BB_DynamicDefaultButton(new) = NULL;
-    BB_DynamicCancelButton(new) = NULL;
+    BB_DynamicDefaultButton(new_w) = NULL;
+    BB_DynamicCancelButton(new_w) = NULL;
 
-    bbclass = (XmBulletinBoardWidgetClass)XtClass(new);
+    bbclass = (XmBulletinBoardWidgetClass)XtClass(new_w);
     if (bbclass->bulletin_board_class.focus_moved_proc) {
-	XtAddCallback(new, XmNfocusCallback,
-		      bbclass->bulletin_board_class.focus_moved_proc, NULL);
+	Widget ve;
+
+	if ((ve = _LtFindVendorExt(new_w)) != NULL) {
+	    XtAddCallback(ve, XmNfocusMovedCallback,
+			  bbclass->bulletin_board_class.focus_moved_proc, new_w);
+	}
     }
 
     /* initialize these to values that aren't possible */
-    BB_OldWidth(new) = -1;
-    BB_OldHeight(new) = -1;
+    BB_OldWidth(new_w) = -1;
+    BB_OldHeight(new_w) = -1;
+
+    BB_GeoCache(new_w) = NULL;
+    BB_InSetValues(new_w) = False;
 }
 
 static void
@@ -436,62 +501,115 @@ destroy(Widget w)
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     XmBulletinBoardWidget ow = (XmBulletinBoardWidget) old;
-    XmBulletinBoardWidget nw = (XmBulletinBoardWidget) new;
+    XmBulletinBoardWidget nw = (XmBulletinBoardWidget) new_w;
     Boolean need_refresh = False;
 
-    XdbDebug(__FILE__, new, "SetValues\n");
+    BB_InSetValues(new_w) = True;
+
+    XdbDebug(__FILE__, new_w, "SetValues\n");
     if (BB_DefaultButton(ow) != BB_DefaultButton(nw))
     {
 	/* the default button has changed, so set the old one's showAsDefault
 	   to 0, and make the new one display as the default. */
 
-	_XmBBUpdateDynDefaultButton(new);
+	_XmBBUpdateDynDefaultButton(new_w);
 
 	need_refresh = True; /* do we need to ?*/
     }
 
-    if (XtIsSubclass(XtParent(new), xmDialogShellWidgetClass)) {
-/* Shouldn't this be in synthetic resource handlers ? FIX ME */
-	if (XmStringCompare(ow->bulletin_board.dialog_title, nw->bulletin_board.dialog_title) != 0){
+    if (XmIsDialogShell(XtParent(new_w))) {
+	/*
+	 * Shouldn't this be in synthetic resource handlers ?
+	 * MLM 960524 not according to a dump of the synthetics.  Only an
+	 * export proc is given, to copy the existing title.
+	 */
+	if (!XmStringCompare(BB_DialogTitle(old), BB_DialogTitle(new_w))) {
 	    char	*p;
-	    if (XmStringGetLtoR(nw->bulletin_board.dialog_title, XmFONTLIST_DEFAULT_TAG, &p)) {
-		XtVaSetValues(XtParent(new), XtNtitle, p, NULL);
+
+	    if (XmStringGetLtoR(BB_DialogTitle(new_w),
+				XmFONTLIST_DEFAULT_TAG,
+				&p)) {
+		XtVaSetValues(XtParent(new_w), XtNtitle, p, NULL);
 		XtFree(p);
+		BB_DialogTitle(new_w) = XmStringCopy(BB_DialogTitle(new_w));
 	    }
-	    XmStringFree(ow->bulletin_board.dialog_title);
-	    nw->bulletin_board.dialog_title = XmStringCopy(nw->bulletin_board.dialog_title);
+	    XmStringFree(BB_DialogTitle(old));
 	}
-/*
- * If we're a dialog, refuse to be placed away from 0,0
- */
-	if (XtX(new) != 0) {
-	    XtX(new) = 0;
+
+	/*
+	 * If we're a dialog, refuse to be placed away from 0,0
+	 */
+	if (XtX(new_w) != 0) {
+	    XtX(new_w) = 0;
 	    need_refresh = True;
 	}
 
-	if (XtY(new) != 0) {
-	    XtY(new) = 0;
+	if (XtY(new_w) != 0) {
+	    XtY(new_w) = 0;
 	    need_refresh = True;
 	}
     }
+    if (XmIsDialogShell(XtParent(new_w))) {
+	if (BB_DialogStyle(new_w) != BB_DialogStyle(old)) {
+	    switch (BB_DialogStyle(new_w)) {
+	    case XmDIALOG_MODELESS:
+		XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			      MWM_INPUT_MODELESS, NULL);
+		break;
+	    case XmDIALOG_PRIMARY_APPLICATION_MODAL:
+		XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			      MWM_INPUT_PRIMARY_APPLICATION_MODAL, NULL);
+		break;
+	    case XmDIALOG_FULL_APPLICATION_MODAL:
+		XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			      MWM_INPUT_FULL_APPLICATION_MODAL, NULL);
+		break;
+	    case XmDIALOG_SYSTEM_MODAL:
+		XtVaSetValues(XtParent(new_w), XmNmwmInputMode,
+			      MWM_INPUT_SYSTEM_MODAL, NULL);
+		break;
+	    }
+	}
+    }
 
+    BB_InSetValues(new_w) = False;
+
+    if (XtWidth(new_w) != XtWidth(old) || XtHeight(new_w) != XtHeight(old)) {
+	need_refresh = True;
+    }
+
+    if (need_refresh == True && XtClass(new_w) == xmBulletinBoardWidgetClass)
+    {
+	_XmBulletinBoardSizeUpdate(new_w);
+	return False;
+    }
     return need_refresh;
 }
 
 static XtGeometryResult 
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
-    XdbDebug(__FILE__, w, "QueryGeometry => YES\n");
+    XmBulletinBoardWidgetClass bbc = (XmBulletinBoardWidgetClass)XtClass(w);
+    XtGeometryResult res;
 
-/* FIX ME */
-    if (answer)
-	*answer = *proposed;
-    return XtGeometryYes;
+    XdbDebug(__FILE__, w, "QueryGeometry\n");
+
+    if (bbc->bulletin_board_class.geo_matrix_create) {
+	return _XmHandleQueryGeometry(w, proposed, answer, BB_ResizePolicy(w),
+				      bbc->bulletin_board_class.geo_matrix_create);
+    }
+
+    res = _XmGMHandleQueryGeometry(w, proposed, answer,
+				   BB_MarginWidth(w), BB_MarginHeight(w),
+				   BB_ResizePolicy(w));
+    XdbDebug(__FILE__, w, "BB wants %d %d\n", answer->width, answer->height);
+
+    return res;
 }
 
 static void
@@ -532,7 +650,6 @@ static void
 resize(Widget w)
 {
     XmBulletinBoardClassRec *bb = (XmBulletinBoardClassRec *)XtClass(w);
-    Widget p;
 
     XdbDebug(__FILE__, NULL, "resize\n");
 
@@ -541,20 +658,16 @@ resize(Widget w)
 	return;
     }
 
-    /* some call with args (w, BB_MarginWidth(w), BB_MarginHeight(w)); */
+    _XmGMEnforceMargin(w,
+		       BB_MarginWidth(w), BB_MarginHeight(w),
+		       False);
 
     _XmClearShadowType(w, BB_OldWidth(w), BB_OldHeight(w),
 		       BB_OldShadowThickness(w), 0);
 
     BB_OldShadowThickness(w) = 0;
 
-    if (XtIsComposite(w))
-	p = w;
-    else
-	p = XtParent(w);
-
-    if (XtIsRealized(p) || XtWidth(w) == 0 || XtHeight(w) == 0) {
-
+    if (XtIsRealized(w) || XtWidth(w) == 0 || XtHeight(w) == 0) {
 	_XmGMDoLayout(w, BB_MarginWidth(w), BB_MarginHeight(w),
 		      BB_ResizePolicy(w), True);
     }
@@ -572,221 +685,75 @@ resize(Widget w)
     BB_OldShadowThickness(w) = MGR_ShadowThickness(w);
 }
 
-static void
-CalculateNewSize(Widget w, Dimension *wid, Dimension *hei)
-{
-    XmBulletinBoardWidget	bb = (XmBulletinBoardWidget) w;
-    Widget			child;
-    int				i;
-    Dimension			ww, hh;
-
-    *wid = 0;
-    *hei = 0;
-
-    for (i=0; i<bb->composite.num_children; i++) {
-	child = bb->composite.children[i];
-
-	if (!XtIsManaged(child)) continue;
-
-	ww = XtX(child) + XtWidth(child) +
-		2 * XtBorderWidth(child) +
-		BB_MarginWidth(bb);
-	hh = XtY(child) + XtHeight(child) +
-		2 * XtBorderWidth(child) +
-		BB_MarginHeight(bb);
-
-	if (*wid < ww)
-	    *wid = ww;
-
-	if (*hei < hh)
-	    *hei = hh;
-    }
-}
-
 /*
- * DoResize() : compute the BB's size depending on XmNresizePolicy
- *	and the geometry of its children.
- *
- *	Note : XmNnoResize has nothing to do with this.
+ * handle subclasses that use the GeoCache
  */
-static void
-DoResize(Widget w)
+static XtGeometryResult
+handle_geometry_manager(Widget w,
+			XtWidgetGeometry *desired, XtWidgetGeometry *allowed,
+			XmGeoCreateProc mat_make)
 {
-    XmBulletinBoardWidget	bb = (XmBulletinBoardWidget)w;
-    Dimension	wid, ht;
-    XtWidgetGeometry	request;
-    XtGeometryResult	result;
+    Widget bb = XtParent(w);
+    XtGeometryResult res;
 
-    if (BB_ResizePolicy(bb) == XmRESIZE_NONE) {
-	XdbDebug(__FILE__, w, "DoResize: XmRESIZE_NONE\n", XtName(w));
-        return;
+    XdbDebug2(__FILE__, bb, w,
+	      "handle_geometry_manager\n");
+
+    if (!(desired->request_mode & (CWWidth|CWHeight)))
+	return XtGeometryYes;
+
+    if (BB_OldShadowThickness(bb) != 0 ||
+	BB_ResizePolicy(bb) != XmRESIZE_NONE) {
+	_XmClearShadowType(bb, BB_OldWidth(bb), BB_OldHeight(bb),
+			   BB_OldShadowThickness(bb), 0);
+	BB_OldShadowThickness(bb) = 0;
     }
 
-    request.width = XtWidth(bb);
-    request.height = XtHeight(bb);
+    res = _XmHandleGeometryManager(bb, w, desired, allowed,
+				   BB_ResizePolicy(bb), &BB_GeoCache(bb),
+				   mat_make);
 
-    CalculateNewSize(w, &wid, &ht);
-
-    if (BB_ResizePolicy(bb) == XmRESIZE_ANY) {
-	request.width = wid;
-	request.height = ht;
+    if (!BB_InSetValues(bb) ||
+	XtWidth(bb) > BB_OldWidth(bb) || XtHeight(bb) > BB_OldHeight(bb)) {
+	if (XtIsRealized(bb)) {
+	    _XmDrawShadows(XtDisplay(bb), XtWindow(bb),
+			 MGR_TopShadowGC(bb), MGR_BottomShadowGC(bb),
+			 0, 0, XtWidth(bb), XtHeight(bb),
+			 MGR_ShadowThickness(bb), BB_ShadowType(bb));
+	}
     }
+    BB_OldWidth(bb) = XtWidth(bb);
+    BB_OldHeight(bb) = XtHeight(bb);
 
-    if (BB_ResizePolicy(bb) == XmRESIZE_GROW) {
-        if (wid > XtWidth(bb))
-	    request.width = wid;
-        if (ht > XtHeight(bb))
-	    request.height = ht;
-    }
-
-/* Deal with initial size */
-    if (! XtIsRealized(w)) {
-	if (XtWidth(bb) != 0) request.width = XtWidth(bb);
-	if (XtHeight(bb) != 0) request.height = XtHeight(bb);
-    }
-
-    XdbDebug(__FILE__, w, "DoResize => %d %d\n", request.width, request.height);
-
-/* Now try to resize yourself - if parent permits ... */
-    if (request.width != XtWidth(bb) || request.height != XtHeight(bb)) {
-	request.request_mode = CWWidth | CWHeight;
-	do {
-	    result = XtMakeGeometryRequest(w, &request, &request);
-
-	    XdbDebug(__FILE__, w, "DoResize: XtMakeGeometryRequest => %s\n",
-		XdbGeometryResult2String(result));
-	} while (result == XtGeometryAlmost);
-    }
+    return res;
 }
-
+			
 /*
  * Geometry Manager is always called by a child of BB.
  *	It asks to get a different geometry. We may allow this.
  *	Also we may have to resize ourself because of this.
- *
- * As we are a simplistic widget, we'll allow everything.
- * FIXME : should this thing implement XmNallowOverlap ?
- * 101995 - MLM -- yes.  It should also check for MarginWidth/MarginHeight
  */
 static XtGeometryResult
 geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
 {
-    XmBulletinBoardWidget           bb = (XmBulletinBoardWidget) XtParent(w);
-    Position x, y;
-    Dimension width, height;
+    Widget bb = XtParent(w);
+    XmBulletinBoardWidgetClass bbc = (XmBulletinBoardWidgetClass)XtClass(bb);
 
-#define Wants(x)        (desired->request_mode & x)
-
-    XdbDebug2(__FILE__, (Widget)bb, w, "GeometryManager (desired ");
-    if (Wants(CWX)) XdbDebug0(__FILE__, (Widget)bb, "X %d ", desired->x);
-    if (Wants(CWY)) XdbDebug0(__FILE__, (Widget)bb, "Y %d ", desired->y);
-    if (Wants(CWWidth)) XdbDebug0(__FILE__, (Widget)bb, "W %d ", desired->width);
-    if (Wants(CWHeight)) XdbDebug0(__FILE__, (Widget)bb, "H %d ", desired->height);
-    XdbDebug0(__FILE__, w, ")\n");
-
-    if (Wants(CWX))
-	x = desired->x;
-    else
-	x = XtX(w);
-    if (Wants(CWY))
-	y = desired->y;
-    else
-	y = XtY(w);
-    if (Wants(CWWidth))
-	width = desired->width;
-    else
-	width = XtWidth(w);
-    if (Wants(CWHeight))
-	height = desired->height;
-    else
-	height = XtHeight(w);
-
-    /* if we don't fit in the MarginWidth/MarginHeight bounding box, say no */
-    if (x < BB_MarginWidth(bb) || x + width > XtWidth(bb) - BB_MarginWidth(bb)) {
-	XdbDebug(__FILE__, w, "Not in Margin* bounding box => NO\n");
-	return XtGeometryNo;
+    if (bbc->bulletin_board_class.geo_matrix_create) {
+	return handle_geometry_manager(w, desired, allowed,
+				       bbc->bulletin_board_class.geo_matrix_create);
     }
-    if (y < BB_MarginHeight(bb) || y + height > XtHeight(bb) - BB_MarginHeight(bb)) {
-	XdbDebug(__FILE__, w, "Not in Margin* bounding box => NO\n");
-	return XtGeometryNo;
-    }
-
-    /* check for overlap */
-    if (BB_AllowOverlap(bb) == False) {
-	Boolean overlap = False;
-	int i;
-
-	/* FIXME -- this algorithm only works with rectangles -- is this
-	 * right? */
-	for (i = 0; i < bb->composite.num_children; i++) {
-	    Widget cw = bb->composite.children[i];
-
-	    if (cw == w)
-		continue;
-
-	    if (XtY(cw) + XtHeight(cw) <= y) /* above us */
-		continue; /* then quit */
-
-	    if (XtY(cw) >= y + height) /* below us */
-		continue; /* then quit */
-
-	    if (XtX(cw) + XtWidth(cw) <= x) /* to the left */
-		continue; /* then quit */
-
-	    if (XtX(cw) >= x + width) /* to the right */
-		continue; /* then quit */
-
-	    /* all your edges aren't above, below, left, or right of us;
-	     * you must intersect/overlay/are-contained-by us */
-	    overlap = True;
-	    break;
-	}
-	if (overlap) {
-	    XdbDebug(__FILE__, w, "Overlap => NO\n");
-	    return XtGeometryNo;
-	}
-    }
-
-    if (Wants(XtCWQueryOnly)) {
-	XdbDebug(__FILE__, w, "QueryOnly => YES\n");
-	return XtGeometryYes;
-    }
-
-    XdbDebug(__FILE__, w, "GeometryManager => ");
-    if (Wants(CWWidth)) {
-	XtWidth(w) = desired->width;
-	XdbDebug0(__FILE__, w, "W %d ", desired->width);
-    }
-    if (Wants(CWHeight)) {
-	XtHeight(w) = desired->height;
-	XdbDebug0(__FILE__, (Widget)bb, "H %d ", desired->height);
-    }
-    if (Wants(CWX)) {
-	XtX(w) = desired->x;
-	XdbDebug0(__FILE__, w, "X %d ", desired->x);
-    }
-    if (Wants(CWY)) {
-	XtY(w) = desired->y;
-	XdbDebug0(__FILE__, w, "Y %d ", desired->y);
-    }
-    if (Wants(CWBorderWidth)) {
-	XtBorderWidth(w) = desired->border_width;
-	XdbDebug0(__FILE__, w, "BW %d ", desired->border_width);
-    }
-    XdbDebug0(__FILE__, w, "=> YES\n");
-
-    DoResize((Widget)bb);
-
-    return XtGeometryYes;
-#undef  Wants
+    return _XmGMHandleGeometryManager(bb, w, desired, allowed,
+				      BB_MarginWidth(bb), BB_MarginHeight(bb),
+				      BB_ResizePolicy(bb), BB_AllowOverlap(bb));
 }
 
 static void
 handle_change_managed(Widget w, XmGeoCreateProc mat_make)
 {
-    Dimension wd, ht, retw, reth;
+    Dimension wd, ht;
     XmGeoMatrix geo;
-    XtGeometryResult result;
+    XtWidgetGeometry request;
 
     if (!XtIsRealized(w))
 	wd = ht = 0;
@@ -821,14 +788,13 @@ handle_change_managed(Widget w, XmGeoCreateProc mat_make)
 	return;
     }
 
-    retw = wd;
-    reth = ht;
-    do {
-	result = XtMakeResizeRequest((Widget)w, retw, reth, &retw, &reth);
-    } while (result == XtGeometryAlmost);
+    request.request_mode = (CWWidth|CWHeight);
+    request.width = wd;
+    request.height = ht;
+    _XmMakeGeometryRequest(w, &request);
 
-    if (retw != wd || reth != ht)
-	_XmGeoArrangeBoxes(geo, 0, 0, &retw, &reth);
+    if (request.width != wd || request.height != ht)
+	_XmGeoArrangeBoxes(geo, 0, 0, &request.width, &request.height);
 
     _XmGeoMatrixSet(geo);
 
@@ -854,29 +820,25 @@ handle_change_managed(Widget w, XmGeoCreateProc mat_make)
 static void
 change_managed(Widget w)
 {
-    Widget p;
     XmBulletinBoardClassRec *bb = (XmBulletinBoardClassRec *)XtClass(w);
 
-    XdbDebug(__FILE__, NULL, "ChangeManaged\n");
+    XdbDebug(__FILE__, w, "ChangeManaged\n");
 
     if (bb->bulletin_board_class.geo_matrix_create) {
 	handle_change_managed(w, bb->bulletin_board_class.geo_matrix_create);
 	return;
     }
 
-    /* some call with args (w, BB_MarginWidth(w), BB_MarginHeight(w)); */
+    _XmGMEnforceMargin(w,
+		       BB_MarginWidth(w), BB_MarginHeight(w),
+		       False);
 
     _XmClearShadowType(w, BB_OldWidth(w), BB_OldHeight(w),
 		       BB_OldShadowThickness(w), 0);
 
     BB_OldShadowThickness(w) = 0;
 
-    if (XtIsComposite(w))
-	p = w;
-    else
-	p = XtParent(w);
-
-    if (XtIsRealized(p) || XtWidth(w) == 0 || XtHeight(w) == 0) {
+    if (XtIsRealized(w) || XtWidth(w) == 0 || XtHeight(w) == 0) {
 	_XmGMDoLayout(w, BB_MarginWidth(w), BB_MarginHeight(w),
 		      BB_ResizePolicy(w), False);
     }
@@ -927,17 +889,17 @@ XmCreateBulletinBoardDialog(Widget parent,
 }
 
 static void
-ConstraintInitialize(Widget request, Widget new, Arg *args, Cardinal *nargs)
+ConstraintInitialize(Widget request, Widget new_w, Arg *args, Cardinal *nargs)
 {
-        XdbDebug(__FILE__, new, "ConstraintInitialize\n");
+        XdbDebug(__FILE__, new_w, "ConstraintInitialize\n");
 }
 
 static Boolean
-ConstraintSetValues(Widget old, Widget request, Widget new, Arg *args, Cardinal *nargs)
+ConstraintSetValues(Widget old, Widget request, Widget new_w, Arg *args, Cardinal *nargs)
 {
-        XmBulletinBoardWidget   bb = (XmBulletinBoardWidget) XtParent(new);
+        XmBulletinBoardWidget   bb = (XmBulletinBoardWidget) XtParent(new_w);
 
-        XdbDebug2(__FILE__, (Widget)bb, new, "ConstraintSetValues\n");
+        XdbDebug2(__FILE__, (Widget)bb, new_w, "ConstraintSetValues\n");
 
         return False;
 }
@@ -945,9 +907,9 @@ ConstraintSetValues(Widget old, Widget request, Widget new, Arg *args, Cardinal
 static void
 handle_realize(Widget w, XmGeoCreateProc mat_make)
 {
-    Dimension wd, ht, retw, reth;
+    Dimension wd, ht;
     XmGeoMatrix geo;
-    XtGeometryResult	result;
+    XtWidgetGeometry request;
 
     wd = XtWidth(w);
     ht = XtHeight(w);
@@ -969,18 +931,24 @@ handle_realize(Widget w, XmGeoCreateProc mat_make)
     }
 
     if (wd == XtWidth(w) && ht == XtHeight(w)) {
+#if 0
+/*
+ * this SHOULDN'T be necessary, but somebody isn't playing by the rules
+ * (RowColumn, judging by smartmb/test1, selectionbox/test4)
+ */
+	_XmGeoMatrixSet(geo);
+#endif
 	_XmGeoMatrixFree(geo);
 	return;
     }
 
-    retw = wd;
-    reth = ht;
-    do {
-	result = XtMakeResizeRequest((Widget)w, retw, reth, &retw, &reth);
-    } while (result == XtGeometryAlmost);
+    request.request_mode = (CWWidth|CWHeight);
+    request.width = wd;
+    request.height = ht;
+    _XmMakeGeometryRequest(w, &request);
 
-    if (retw != wd || reth != ht)
-	_XmGeoArrangeBoxes(geo, 0, 0, &retw, &reth);
+    if (request.width != wd || request.height != ht)
+	_XmGeoArrangeBoxes(geo, 0, 0, &request.width, &request.height);
 
     _XmGeoMatrixSet(geo);
 
@@ -1008,7 +976,6 @@ realize(Widget w,
         XSetWindowAttributes *attributes)
 {
     XmBulletinBoardClassRec *bb = (XmBulletinBoardClassRec *)XtClass(w);
-    Widget p;
 
     XdbDebug(__FILE__, w, "Realize\n");
 
@@ -1021,20 +988,19 @@ realize(Widget w,
 	return;
     }
 
-    /* some call with args (w, BB_MarginWidth(w), BB_MarginHeight(w)); */
+    _XmGMEnforceMargin(w,
+		       BB_MarginWidth(w), BB_MarginHeight(w),
+		       False);
 
     _XmClearShadowType(w, BB_OldWidth(w), BB_OldHeight(w),
 		       BB_OldShadowThickness(w), 0);
 
     BB_OldShadowThickness(w) = 0;
 
-    if (XtIsComposite(w))
-	p = w;
-    else
-	p = XtParent(w);
+    _XmGMDoLayout(w, BB_MarginWidth(w), BB_MarginHeight(w),
+		  BB_ResizePolicy(w), False);
 
-    if ((XtWidth(w) < BB_OldWidth(w) || XtHeight(w) < BB_OldHeight(w)) &&
-	XtIsRealized(w)) {
+    if (XtWidth(w) < BB_OldWidth(w) || XtHeight(w) < BB_OldHeight(w)) {
 	_XmDrawShadows(XtDisplay(w), XtWindow(w),
 		       MGR_TopShadowGC(w), MGR_BottomShadowGC(w),
 		       0, 0, XtWidth(w), XtHeight(w),
@@ -1047,94 +1013,100 @@ realize(Widget w,
 }
 
 static void
-expose(Widget w, XEvent *event, Region region) {
-    _XmDrawShadows(XtDisplay(w), XtWindow(w),
-                  MGR_TopShadowGC(w), MGR_BottomShadowGC(w),
-                  0, 0, XtWidth(w), XtHeight(w),
-                  MGR_ShadowThickness(w), BB_ShadowType(w));
-
+expose(Widget w, XEvent *event, Region region)
+{
     _XmRedisplayGadgets(w, event, region);
+
+    if (MGR_ShadowThickness(w)) {
+	_XmDrawShadows(XtDisplay(w), XtWindow(w),
+		       MGR_TopShadowGC(w), MGR_BottomShadowGC(w),
+		       0, 0, XtWidth(w), XtHeight(w),
+		       MGR_ShadowThickness(w), BB_ShadowType(w));
+    }
 }
 
 static void
 _XmBbButton(Widget w, XtPointer client, XtPointer call)
 {
-	XmBulletinBoardWidget bb = (XmBulletinBoardWidget) XtParent(w);
+    XmBulletinBoardWidget bb = (XmBulletinBoardWidget) XtParent(w);
 
-	XdbDebug2(__FILE__, (Widget)bb, w, "XmBbButton\n");
+    XdbDebug2(__FILE__, (Widget)bb, w, "XmBbButton\n");
 
-/*
- * How do we make sure the APPLY or HELP buttons don't
- * trigger unmanaging ??  
- *
- * Somebody already caught this.  See comments in SelectionBox.c (look
- * for auto_unmanage).
- */
-	if (BB_AutoUnmanage(bb)) {
-		if (XtIsSubclass(XtParent((Widget)bb), xmDialogShellWidgetClass)) {
-			XtUnmanageChild((Widget)bb);
-			XdbDebug2(__FILE__, (Widget)bb, w, "AutoUnmanage\n");
-		}
+    /*
+     * How do we make sure the APPLY or HELP buttons don't
+     * trigger unmanaging ??  
+     *
+     * Somebody already caught this.  See comments in SelectionBox.c (look
+     * for auto_unmanage).
+     */
+    if (BB_AutoUnmanage(bb)) {
+	Widget	s = XtParent((Widget)bb);
+
+	if (XtIsSubclass(s, xmDialogShellWidgetClass)) {
+	    XtUnmanageChild((Widget)bb);
+	    XdbDebug2(__FILE__, (Widget)bb, w, "AutoUnmanage\n");
+
+	    /* XtNpopdownCallback */
+	    XtCallCallbackList(s, Shell_PopdownCallback(s), NULL);
 	}
+    }
 }
 
 /*
  * Keep track of button children
- * again, this shouldn't be done, but a call to XmGMEnforceMargins should be
- * made in the relevant places (such as change_managed).
  */
 static void
 InsertChild(Widget w)
 {
-    Widget p = XtParent(w);
-    XmBulletinBoardWidgetClass bbclass;
-
 #define	superclass	(&xmManagerClassRec)
     (*superclass->composite_class.insert_child)(w);
 #undef	superclass
 
-    if (XtX(w) < BB_MarginWidth(p))
-	XtX(w) = BB_MarginWidth(p);
-    if (XtY(w) < BB_MarginHeight(p))
-	XtY(w) = BB_MarginHeight(p);
-
-    _XmConfigureObject(w, XtX(w), XtY(w), XtWidth(w), XtHeight(w),
-		       XtBorderWidth(w));
-
-    /* this can't be right, but it does work */
     if (_XmIsFastSubclass(XtClass(w), XmPUSH_BUTTON_GADGET_BIT) ||
-	_XmIsFastSubclass(XtClass(w), XmPUSH_BUTTON_BIT)) {
+	_XmIsFastSubclass(XtClass(w), XmPUSH_BUTTON_BIT))
+    {
 	XtAddCallback(w, XmNactivateCallback, _XmBbButton, NULL);
-	bbclass = (XmBulletinBoardWidgetClass)XtClass(XtParent(w));
-	if (bbclass->bulletin_board_class.focus_moved_proc)
-	    _XmBulletinBoardSetDefaultShadow(w);
     }
 }
 
-void
-_XmBbMap(Widget w)
+static void
+DeleteChild(Widget w)
 {
-    XdbDebug(__FILE__, w, "Map\n");
-    XtCallCallbackList(w, BB_MapCallback(w), NULL);
-    if (BB_DefaultPosition(w)
-	&& XtIsSubclass(XtParent(w), xmDialogShellWidgetClass)) {
-	Position	px, py;
-	Widget	p;
+    Widget	bb = XtParent(w);
 
-	p = XtParent(XtParent(w));
+    XdbDebug2(__FILE__, bb, w, "DeleteChild\n");
 
-	px = XtX(p) + (XtWidth(p) - XtWidth(w)) / 2;
-	py = XtY(p) + (XtHeight(p) - XtHeight(w)) / 2;
-	XdbDebug(__FILE__, w, "def. pos. %d %d\n", px, py);
-	XtMoveWidget(XtParent(w), px, py);
-    }
+#define	superclass	(&xmManagerClassRec)
+    (*superclass->composite_class.delete_child)(w);
+#undef superclass
+
+    if (w == BB_CancelButton(bb))
+	BB_CancelButton(bb) = NULL;
 }
- 
-void
-_XmBbUnmap(Widget w)
+
+static Boolean
+_XmBBParentProcess(Widget widget, XmParentProcessData data)
 {
-    XdbDebug(__FILE__, w, "Unmap\n");
-    XtCallCallbackList(w, BB_UnmapCallback(w), NULL);
+    if (data->input_action.process_type == XmINPUT_ACTION)
+    {
+	if (data->input_action.action == XmPARENT_ACTIVATE)
+	{
+	    _XmBulletinBoardReturn(widget,
+				   data->input_action.event,
+				   data->input_action.params,
+				   data->input_action.num_params);
+	    return True;
+	}
+	else if (data->input_action.action == XmPARENT_CANCEL)
+	{
+	    _XmBulletinBoardCancel(widget,
+				   data->input_action.event,
+				   data->input_action.params,
+				   data->input_action.num_params);
+	    return True;
+	}
+    }
+    return False;
 }
 
 Widget
@@ -1160,6 +1132,7 @@ _XmBB_CreateLabelG(Widget bb, XmString l_string, char *name)
     Arg args[2];
 
     XtSetArg(args[0], XmNlabelString, l_string);
+
 #ifdef USE_WIDGETS
     label = XmCreateLabel(bb, name, args, 1);
 #else
@@ -1169,32 +1142,129 @@ _XmBB_CreateLabelG(Widget bb, XmString l_string, char *name)
 }
 
 void
-_XmBulletinBoardSizeUpdate(Widget wid)
+_XmBulletinBoardSizeUpdate(Widget w)
 {
+    XmBulletinBoardWidgetClass bbc = (XmBulletinBoardWidgetClass)XtClass(w);
+
+    if (!XtIsRealized(w))
+	return;
+
+    if (bbc->bulletin_board_class.geo_matrix_create == NULL) {
+	BB_OldWidth(w) = XtWidth(w);
+	BB_OldHeight(w) = XtHeight(w);
+	return;
+    }
+    if (!BB_OldShadowThickness(w) && BB_ResizePolicy(w) != XmRESIZE_NONE) {
+	_XmClearShadowType(w, BB_OldWidth(w), BB_OldHeight(w),
+			   BB_OldShadowThickness(w), 0);
+	BB_OldShadowThickness(w) = 0;
+    }
+
+    _XmHandleSizeUpdate(w, BB_ResizePolicy(w),
+			bbc->bulletin_board_class.geo_matrix_create);
+
+    if ((XtWidth(w) < BB_OldWidth(w) || XtHeight(w) < BB_OldHeight(w)) &&
+	XtIsRealized(w)) {
+	_XmDrawShadows(XtDisplay(w), XtWindow(w),
+		       MGR_TopShadowGC(w), MGR_BottomShadowGC(w),
+		       0, 0, XtWidth(w), XtHeight(w),
+		       MGR_ShadowThickness(w), BB_ShadowType(w));
+    }
+
+    BB_OldWidth(w) = XtWidth(w);
+    BB_OldHeight(w) = XtHeight(w);
+    BB_OldShadowThickness(w) = MGR_ShadowThickness(w);
 }
 
 void
 _XmBulletinBoardFocusMoved(Widget wid, XtPointer client_data, XtPointer data)
 {
-    _XmBulletinBoardSetDynDefaultButton(wid, (Widget)data);
+    XmFocusMovedCallback cbs = (XmFocusMovedCallback)data;
+    Widget bb = (Widget)client_data, par;
+    Boolean call = False;
+    XmAnyCallbackStruct cb;
+
+    if (cbs->new_focus == wid)
+	call = True;
+
+    else if (cbs->new_focus) {
+	par = XtParent(cbs->new_focus);
+	for (;;) {
+	    if (par == bb) {
+		call = True;
+		if (!XmIsForm(bb) && XtParent(cbs->new_focus) == bb &&
+		    XtIsShell(XtParent(bb)))
+		    _XmBulletinBoardSetDynDefaultButton(bb, cbs->new_focus);
+		break;
+	    }
+	    if (XtIsShell(par))
+		break;
+	    if (!par)
+		break;
+	    par = XtParent(par);
+	}
+    }
+
+    if (call && BB_FocusCallback(bb)) {
+	cb.reason = XmCR_FOCUS;
+	cb.event = cbs->event;
+	XtCallCallbackList(bb, BB_FocusCallback(bb), data);
+    }
+
+    cbs->cont = True;
 }
 
 void
-_XmBulletinBoardMap(Widget wid, XEvent *event,
+_XmBulletinBoardMap(Widget w, XEvent *event,
 		    String *params, Cardinal *numParams)
 {
+    XMapEvent *mev = (XMapEvent *)event;
+    XUnmapEvent *uev = (XUnmapEvent *)event;
+    XmAnyCallbackStruct cb;
+
+    cb.event = event;
+    if (mev->type == MapNotify)
+    {
+	XdbDebug(__FILE__, w, "Map\n");
+	cb.reason = XmCR_MAP;
+	XtCallCallbackList(w, BB_MapCallback(w), (XtPointer)&cb);
+    }
+    else if (uev->type == UnmapNotify)
+    {
+	XdbDebug(__FILE__, w, "Unmap\n");
+	cb.reason = XmCR_UNMAP;
+	XtCallCallbackList(w, BB_UnmapCallback(w), (XtPointer)&cb);
+    }
 }
 
 void
 _XmBulletinBoardReturn(Widget wid, XEvent *event,
 		       String *params, Cardinal *numParams)
 {
+    /* FIXME -- this needs filling in */
 }
 
 void
 _XmBulletinBoardCancel(Widget wid, XEvent *event,
 		       String *params, Cardinal *numParams)
 {
+    XmPushButtonCallbackStruct cbs;
+
+    cbs.reason = XmCR_ACTIVATE;
+    cbs.event = event;
+    if (BB_CancelButton(wid) && XtIsSensitive(BB_CancelButton(wid)) &&
+	XtIsManaged(BB_CancelButton(wid)) && XtIsRealized(BB_CancelButton(wid)))
+    {
+	XtCallCallbacks(BB_CancelButton(wid), XmNactivateCallback, &cbs);
+    }
+    else if (BB_DynamicCancelButton(wid) &&
+	     XtIsSensitive(BB_DynamicCancelButton(wid)) &&
+	     XtIsManaged(BB_DynamicCancelButton(wid)) &&
+	     XtIsRealized(BB_DynamicCancelButton(wid)))
+    {
+	XtCallCallbacks(BB_DynamicCancelButton(wid), XmNactivateCallback, &cbs);
+    }
+   
 }
 
 void
@@ -1221,8 +1291,6 @@ _XmBulletinBoardSetDynDefaultButton(Widget wid, Widget newDefaultButton)
 {
     Arg argl[2];
 
-    if (BB_DynamicDefaultButton(wid) == newDefaultButton)
-	return;
     if (BB_DynamicDefaultButton(wid) != NULL) {
 	if (!BB_DynamicDefaultButton(wid)->core.being_destroyed) {
 	    XtSetArg(argl[0], XmNshowAsDefault, 0);
@@ -1252,9 +1320,3 @@ _XmBBUpdateDynDefaultButton(Widget bb)
 	_XmBulletinBoardSetDynDefaultButton(bb, BB_DefaultButton(bb));
 }
 
-static Boolean
-_XmBBParentProcess(Widget widget, XmParentProcessData data)
-{
-    return False;
-}
-
