@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DrawingArea.c,v 1.23 1996/04/22 22:54:42 miers Exp $
+ * $Id: DrawingArea.c,v 1.9 1996/12/30 07:36:25 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,28 +22,32 @@
  *
  **/
 
-static char rcsid[] = "$Id: DrawingArea.c,v 1.23 1996/04/22 22:54:42 miers Exp $";
+static char rcsid[] = "$Id: DrawingArea.c,v 1.9 1996/12/30 07:36:25 u27113 Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/DrawingAP.h>
 #include <Xm/TransltnsP.h>
-#include <stdio.h>
+#include <Xm/ScrolledWP.h>
+#include <XmI/MacrosI.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
 static void resize(Widget w);
 static void expose(Widget w, XEvent *event, Region region);
 static XtGeometryResult query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
-static XtGeometryResult geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static XtGeometryResult GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply);
 static void changed_managed(Widget w);
-static void constraint_initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void constraint_initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 
 /*
  * Resources for the Drawing Area class
@@ -157,8 +161,8 @@ static XmBaseClassExtRec _XmDrawingACoreClassExtRec = {
     /* compiled_ext_resources    */ NULL, /* FIXME */
     /* num_ext_resources         */ 0, /* FIXME */
     /* use_sub_resources         */ FALSE, /* FIXME */
-    /* widget_navigable          */ NULL, /* FIXME */
-    /* focus_change              */ NULL, /* FIXME */
+    /* widget_navigable          */ XmInheritWidgetNavigable,
+    /* focus_change              */ XmInheritFocusChange,
     /* wrapper_data              */ NULL
 };
 
@@ -220,7 +224,7 @@ XmDrawingAreaClassRec xmDrawingAreaClassRec = {
     },
     /* Composite class part */
     {
-	/* geometry manager */ geometry_manager, 
+	/* geometry manager */ GeometryManager, 
         /* change_managed   */ changed_managed, 
         /* insert_child     */ XtInheritInsertChild,
         /* delete_child     */ XtInheritDeleteChild,
@@ -291,15 +295,15 @@ class_part_initialize(WidgetClass widget_class)
 
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     if (XtWidth(request) == 0)
-	XtWidth(new) = 2 * DA_MarginWidth(new);
+	XtWidth(new_w) = 2 * DA_MarginWidth(new_w);
 
     if (XtHeight(request) == 0)
-	XtHeight(new) = 2 * DA_MarginHeight(new);
+	XtHeight(new_w) = 2 * DA_MarginHeight(new_w);
 }
 
 static void
@@ -310,15 +314,15 @@ destroy(Widget w)
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     Boolean need_refresh = False;
 
-    XdbDebug(__FILE__, new, "setvalues()\n");
+    XdbDebug(__FILE__, new_w, "setvalues()\n");
 
-#define NE(x)	(x(old) != x(new))
+#define NE(x)	(x(old) != x(new_w))
 
     if (NE(DA_MarginHeight) ||
 	NE(DA_MarginWidth) ||
@@ -361,7 +365,7 @@ resize(Widget w)
 
 static void 
 constraint_initialize(Widget request, 
-		      Widget new, 
+		      Widget new_w, 
 		      ArgList args, 
 		      Cardinal *num_args)
 {
@@ -369,13 +373,13 @@ constraint_initialize(Widget request,
        by changing their X/Y to the margin width/height
        if they are */
 
-    Widget da = XtParent(new);
+    Widget da = XtParent(new_w);
 
-    if (XtX(new) < DA_MarginWidth(da))
-	XtX(new) = DA_MarginWidth(da);
+    if (XtX(new_w) < (Position)DA_MarginWidth(da))
+	XtX(new_w) = DA_MarginWidth(da);
 
-    if (XtY(new) < DA_MarginHeight(da))
-	XtY(new) = DA_MarginHeight(da);
+    if (XtY(new_w) < (Position)DA_MarginHeight(da))
+	XtY(new_w) = DA_MarginHeight(da);
 }
 
 static void
@@ -409,6 +413,10 @@ PreferedSize(Widget w,
 	else
 	    XtQueryGeometry(child, NULL, &child_geometry);
 
+	XdbDebug2(__FILE__, w, child,
+		  "Child wants %d %d\n",
+		  child_geometry.width, child_geometry.height);
+
 	if (XtX(child) + child_geometry.width > *wid)
 	    *wid = XtX(child) + child_geometry.width;
 
@@ -457,17 +465,52 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 }
 
 static XtGeometryResult
-geometry_manager(Widget w,
+GeometryManager(Widget w,
 		 XtWidgetGeometry *request,
 		 XtWidgetGeometry *reply)
 {
-    XmDrawingAreaWidget dw = (XmDrawingAreaWidget) XtParent(w);
+    Widget dw = XtParent(w), sw = XtParent(dw);
     XtGeometryResult result;
-    Dimension reply_height, reply_width;
+    Dimension reply_height = 0, reply_width = 0;
     Dimension wid = 0, hei = 0;
     Dimension child_w = 0, child_h = 0;
+    XtWidgetGeometry req;
 
-    XdbDebug2(__FILE__, (Widget)dw, w, "geometry_manager()\n");
+    XdbDebug2(__FILE__, dw, w, "GeometryManager(%s)\n",
+	(DA_ResizePolicy(dw) == XmRESIZE_NONE) ? "XmRESIZE_NONE" :
+	(DA_ResizePolicy(dw) == XmRESIZE_GROW) ? "XmRESIZE_GROW" :
+	(DA_ResizePolicy(dw) == XmRESIZE_ANY) ? "XmRESIZE_ANY" : "???" );
+
+    if (XtIsSubclass(sw, xmScrolledWindowWidgetClass)
+		&& SW_ScrollPolicy(sw) == XmAUTOMATIC) {
+	XdbDebug2(__FILE__, dw, w,
+		"GeometryManager: in ScrolledWindow (%s) => YES\n",
+		XdbWidgetGeometry2String(request));
+	*reply = *request;
+	reply->request_mode &= (CWWidth|CWHeight);
+
+	if ((reply->request_mode & CWWidth) == 0)
+	    reply->width = XtWidth(w);
+	if ((reply->request_mode & CWHeight) == 0)
+	    reply->height = XtHeight(w);
+	if ((reply->request_mode & CWBorderWidth) == 0)
+	    reply->border_width = XtBorderWidth(w);
+#if 1
+	/* Resize the widget as requested */
+/* FIX ME : ScrolledWindow should do this for us */
+	XtResizeWidget(w, reply->width, reply->height, reply->border_width);
+#endif
+	/* Trigger the ScrolledWindow's geometry algorithms so it can
+	 * handle scrollbars and such more.
+	 *
+	 * Not using _XmMakeGeometryRequest here since this is a special case.
+	 */
+	XdbDebug(__FILE__, dw, "XtMakeResizeRequest (phony) %d %d\n",
+		reply->width, reply->height);
+	XtMakeResizeRequest(dw, reply->width, reply->height, NULL, NULL);
+
+	return XtGeometryYes;
+    }
 
     if (request->request_mode & CWWidth)
     {
@@ -481,7 +524,7 @@ geometry_manager(Widget w,
 	XtHeight(w) = request->height;
     }
 
-    PreferedSize((Widget)dw, &wid, &hei, w);
+    PreferedSize(dw, &wid, &hei, w);
 
     XtWidth(w) = child_w;
     XtHeight(w) = child_h;
@@ -499,10 +542,13 @@ geometry_manager(Widget w,
 	if ((wid > XtWidth(dw)) || (hei > XtHeight(dw)))
 	{
 	    /* attempt to resize */
-	    result = XtMakeResizeRequest((Widget)dw,
-					 wid, hei, 
-					 &reply_width, &reply_height);
-	    
+	    req.request_mode = (CWWidth|CWHeight);
+	    req.width = wid;
+	    req.height = hei;
+	    result = _XmMakeGeometryRequest(w, &req);
+	    reply_width = req.width;
+	    reply_height = req.height;
+
 	    if (result == XtGeometryYes)
 	    {
 		XtWidth(dw) = reply_width;
@@ -518,23 +564,24 @@ geometry_manager(Widget w,
 	    return XtGeometryYes;
 	break;
     case XmRESIZE_ANY:
-	if ((wid != XtWidth(dw)) || (hei != XtHeight(dw)))
-	{
-	    result = XtMakeResizeRequest((Widget)dw,
-					 wid, hei, 
-					 &reply_width, &reply_height);
-	    
-	    if (result == XtGeometryYes)
-	    {
+	if ((wid != XtWidth(dw)) || (hei != XtHeight(dw))) {
+
+	    req.request_mode = (CWWidth|CWHeight);
+	    req.width = wid;
+	    req.height = hei;
+	    result = _XmMakeGeometryRequest(w, &req);
+	    reply_width = req.width;
+	    reply_height = req.height;
+
+	    if (result == XtGeometryYes) {
 		XtWidth(dw) = reply_width;
 		XtHeight(dw) = reply_height;
 		
 		return XtGeometryYes;
-	    }
-	    else if (result == XtGeometryNo)
+	    } else if (result == XtGeometryNo) {
 		return XtGeometryNo;
-	}
-	else
+	    }
+	} else
 	    /* the resize request doesn't require us to alter our geometry, accept it */
 	    return XtGeometryYes;
 	break;
@@ -548,13 +595,21 @@ changed_managed(Widget w)
     Dimension wid, hei;
     Dimension reply_wid, reply_hei;
     XtGeometryResult result;
+    XtWidgetGeometry req;
 
     if (DA_ResizePolicy(w) != XmRESIZE_NONE)
     {
 	PreferedSize(w, &wid, &hei, NULL);
 	if (wid > XtWidth(w) || hei > XtHeight(w))
 	{
-	    result = XtMakeResizeRequest(w, wid, hei, &reply_wid, &reply_hei);
+
+	    req.request_mode = (CWWidth|CWHeight);
+	    req.width = wid;
+	    req.height = hei;
+	    result = _XmMakeGeometryRequest(w, &req);
+	    reply_wid = req.width;
+	    reply_hei = req.height;
+
 	    if (result == XtGeometryYes)
 	    {
 		XtWidth(w) = reply_wid;
@@ -564,7 +619,13 @@ changed_managed(Widget w)
 	/* else we're not getting any bigger... */
 	else if (DA_ResizePolicy(w) == XmRESIZE_ANY)
 	{
-	    result = XtMakeResizeRequest(w, wid, hei, &reply_wid, &reply_hei);
+	    req.request_mode = (CWWidth|CWHeight);
+	    req.width = wid;
+	    req.height = hei;
+	    result = _XmMakeGeometryRequest(w, &req);
+	    reply_wid = req.width;
+	    reply_hei = req.height;
+
 	    if (result == XtGeometryYes)
 	    {
 		XtWidth(w) = reply_wid;
@@ -586,6 +647,8 @@ _XmDrawingAreaInput(Widget w,
     cb.event = event;
     cb.window = XtWindow(w);
 
+    XdbDebug(__FILE__, w, "_XmDrawingAreaInput\n");
+
     XtCallCallbackList(w,
 		       DA_InputCallback(w),
 		       (XtPointer)&cb);
