@@ -1,6 +1,6 @@
 /** 
  *
- * $Id: Frame.c,v 1.20 1996/04/30 01:36:25 miers Exp $
+ * $Id: Frame.c,v 1.13 1997/01/13 07:08:52 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,19 +22,29 @@
  *
  **/
  
-static char rcsid[] = "$Id: Frame.c,v 1.20 1996/04/30 01:36:25 miers Exp $";
+static char rcsid[] = "$Id: Frame.c,v 1.13 1997/01/13 07:08:52 u27113 Exp $";
+
+#define OPTIMISTIC_COMPUTE_SIZE
+#undef	STRICT_GEOMETRY_RULES
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/BaseClassP.h>
 #include <Xm/FrameP.h>
 #include <Xm/TransltnsP.h>
-#include <stdio.h>
+#include <Xm/RepType.h>
+
+#include <XmI/DebugUtil.h>
 
 #define _XmMax(a,b) ((a) > (b) ? (a) : (b))
 #define _XmMin(a,b) ((a) < (b) ? (a) : (b))
 
+#ifndef XmUNSPECIFIED
+#define XmUNSPECIFIED (~0)
+#endif
+
 /*
  * Forward Declarations
  */
@@ -42,7 +52,7 @@ static char rcsid[] = "$Id: Frame.c,v 1.20 1996/04/30 01:36:25 miers Exp $";
 static void class_initialize(void);
 static void class_part_initialize(WidgetClass widget_class);
 static void initialize(Widget request,
-		       Widget new,
+		       Widget new_w,
 		       ArgList args,
 		       Cardinal *num_args);
 static void destroy(Widget w);
@@ -52,7 +62,7 @@ static void realize(Widget w,
 static void resize(Widget w);
 static Boolean set_values(Widget current,
 			  Widget request,
-			  Widget new,
+			  Widget new_w,
 			  ArgList args,
 			  Cardinal *num_args);
 static void expose(Widget w, XEvent *event, Region region);
@@ -66,18 +76,21 @@ static XtGeometryResult geometry_manager(Widget w,
 static void change_managed(Widget w);
 /* constraint */
 static void constraint_initialize(Widget request,
-				  Widget new,
+				  Widget new_w,
 				  ArgList args,
 				  Cardinal *num_args);
 static Boolean constraint_set_values(Widget current, 
 				     Widget request,
-				     Widget new,
+				     Widget new_w,
 				     ArgList args,
 				     Cardinal *num_args);
 /* helper */
-static void _XmFrameLayout(Widget w, Boolean ParentResize,
-			   Widget child, Boolean TestMode,
-			   XtWidgetGeometry *childgeom);
+static void _XmFrameComputeSize(Widget w, Widget instig,
+				XtWidgetGeometry *desired,
+				Dimension *wd, Dimension *ht);
+static void _XmFrameConfigureChildren(Widget w, Widget instig,
+				      XtWidgetGeometry *desired,
+				      Dimension wd, Dimension ht);
 
 /* actions */
 static void Enter(Widget w, XEvent *event, String *params, Cardinal *num_params);
@@ -127,14 +140,12 @@ static XtResource resources[] = {
 	XmNshadowType, XmCShadowType, XmRShadowType,
 	sizeof(unsigned char), Offset(shadow_type),
 	XmRImmediate, (XtPointer)((unsigned char)XmUNSPECIFIED)
-	/* this was XmRCallProc, (XtPointer)_XmFrameShadowTypeDefault */
     },
     /* manager override */
     {
 	XmNshadowThickness, XmCShadowThickness, XmRHorizontalDimension,
 	sizeof(Dimension), XtOffsetOf(XmFrameRec, manager.shadow_thickness),
 	XmRImmediate, (XtPointer)XmINVALID_DIMENSION
-	/* this was XmRCallProc, (XtPointer)_XmFrameShadowThicknessDefault */
     },
 };
 
@@ -177,7 +188,6 @@ static XtResource frameConstraintResources[] = {
 	XmNchildHorizontalSpacing, XmCChildHorizontalSpacing, XmRHorizontalDimension,
 	sizeof(Dimension), Offset(child_h_spacing),
 	XmRImmediate, (XtPointer)XmINVALID_DIMENSION
-	/* this was XmRCallProc, (XtPointer)_XmFrameHorizSpaceDefault */
     }
 };
 
@@ -185,7 +195,7 @@ static XmSyntheticResource constraint_syn_resources[] = {
     {
 	XmNchildHorizontalSpacing,
 	sizeof(Dimension), Offset(child_h_spacing),
-	_XmFromVerticalPixels, _XmToVerticalPixels
+	_XmFromHorizontalPixels, _XmToHorizontalPixels
     }
 };
 
@@ -210,8 +220,8 @@ static XmBaseClassExtRec _XmFrameCoreClassExtRec = {
     /* compiled_ext_resources    */ NULL, /* FIXME */
     /* num_ext_resources         */ 0, /* FIXME */
     /* use_sub_resources         */ FALSE, /* FIXME */
-    /* widget_navigable          */ NULL, /* FIXME */
-    /* focus_change              */ NULL, /* FIXME */
+    /* widget_navigable          */ XmInheritWidgetNavigable,
+    /* focus_change              */ XmInheritFocusChange,
     /* wrapper_data              */ NULL
 };
 
@@ -345,25 +355,39 @@ class_part_initialize(WidgetClass widget_class)
 
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    Frame_TitleArea(new) = NULL;
-    Frame_WorkArea(new) = NULL;
+    Frame_TitleArea(new_w) = NULL;
+    Frame_WorkArea(new_w) = NULL;
 
-    if (Frame_ShadowType(new) == (unsigned char)XmUNSPECIFIED)
+    if (Frame_ShadowType(new_w) == (unsigned char)XmUNSPECIFIED)
     {
-	if (XtIsShell(XtParent(new)))
-	    Frame_ShadowType(new) = XmSHADOW_ETCHED_OUT;
+	if (XtIsShell(XtParent(new_w)))
+	    Frame_ShadowType(new_w) = XmSHADOW_OUT;
 	else
-	    Frame_ShadowType(new) = XmSHADOW_IN;
+	    Frame_ShadowType(new_w) = XmSHADOW_ETCHED_IN;
     }
 
-    if (MGR_ShadowThickness(new) == XmINVALID_DIMENSION)
-	MGR_ShadowThickness(new) = 2;
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRShadowType),
+			     Frame_ShadowType(new_w),
+			     new_w))
+	Frame_ShadowType(new_w) = XmSHADOW_ETCHED_IN;
 
-    _XmFrameLayout(new, True, NULL, False, NULL);
+    if (MGR_ShadowThickness(new_w) == XmINVALID_DIMENSION)
+    {
+	if (XtIsShell(XtParent(new_w)))
+	    MGR_ShadowThickness(new_w) = 1;
+	else
+	    MGR_ShadowThickness(new_w) = 2;
+    }
+
+    Frame_OldShadowX(new_w) = Frame_OldShadowY(new_w) = 0;
+    Frame_OldShadowThickness(new_w) = MGR_ShadowThickness(new_w);
+    Frame_OldWidth(new_w) = XtWidth(new_w);
+    Frame_OldHeight(new_w) = XtHeight(new_w);
+    Frame_ProcessingConstraints(new_w) = False;
 }
 
 static void
@@ -374,19 +398,47 @@ destroy(Widget w)
 static Boolean
 set_values(Widget current,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     Boolean redisplay = False;
+    Dimension wd, ht;
+    XtWidgetGeometry req;
 
-    if (Frame_MarginWidth(new) != Frame_MarginWidth(current) ||
-	Frame_MarginHeight(new) != Frame_MarginHeight(current) ||
-	Frame_ShadowType(new) != Frame_ShadowType(current))
+    if (Frame_MarginWidth(new_w) != Frame_MarginWidth(current) ||
+	Frame_MarginHeight(new_w) != Frame_MarginHeight(current) ||
+	Frame_ShadowType(new_w) != Frame_ShadowType(current))
 	redisplay = True;
 
     if (redisplay)
-	_XmFrameLayout(new, True, NULL, False, NULL);
+    {
+	_XmFrameComputeSize(new_w, NULL, NULL, &wd, &ht);
+	if (wd > Frame_OldWidth(new_w) || ht > Frame_OldHeight(new_w))
+	{
+	    req.request_mode = (CWWidth|CWHeight);
+	    req.width = wd;
+	    req.height = ht;
+	    _XmMakeGeometryRequest(new_w, &req);
+	    wd = req.width;
+	    ht = req.height;
+
+	    _XmFrameConfigureChildren(new_w, NULL, NULL, wd, ht);
+	}
+    }
+
+    Frame_OldShadowX(new_w) = 0;
+    if (Frame_TitleArea(new_w) && XtIsManaged(Frame_TitleArea(new_w)))
+	Frame_OldShadowY(new_w) = XtHeight(Frame_TitleArea(new_w)) +
+				MGR_ShadowThickness(new_w) +
+				Frame_MarginHeight(new_w);
+    else
+	Frame_OldShadowY(new_w) = MGR_ShadowThickness(new_w) +
+				Frame_MarginHeight(new_w);
+
+    Frame_OldShadowThickness(new_w) = MGR_ShadowThickness(current);
+    Frame_OldWidth(new_w) = XtWidth(new_w);
+    Frame_OldHeight(new_w) = XtHeight(new_w);
 
     return redisplay;
 }
@@ -400,7 +452,7 @@ realize(Widget w,
     (*superclass->core_class.realize)(w, value_mask, attributes);
 #undef superclass
 
-    _XmFrameLayout(w, True, NULL, False, NULL);
+    _XmFrameConfigureChildren(w, NULL, NULL, XtWidth(w), XtHeight(w));
 
     XdbDebug(__FILE__, w, "Frame Realize\n");
 }
@@ -410,7 +462,7 @@ resize(Widget w)
 {
     XdbDebug(__FILE__, w, "Frame resize (%d %d)\n", XtWidth(w), XtHeight(w));
 
-    _XmFrameLayout(w, False, NULL, False, NULL);
+    _XmFrameConfigureChildren(w, NULL, NULL, XtWidth(w), XtHeight(w));
 }
 
 static void
@@ -420,7 +472,7 @@ expose(Widget w,
 {
     Position topline_y = 0;
 
-    if (Frame_TitleArea(w))
+    if (Frame_TitleArea(w) && XtIsManaged(Frame_TitleArea(w)))
     {
 	switch (FrameC_ChildVAlignment(Frame_TitleArea(w)))
 	{
@@ -451,62 +503,147 @@ expose(Widget w,
 		   MGR_ShadowThickness(w),
 		   Frame_ShadowType(w));
 
+    if (Frame_TitleArea(w) && XtIsManaged(Frame_TitleArea(w)))
+    {
+	XFillRectangle(XtDisplay(w), XtWindow(w), MGR_BackgroundGC(w),
+		       XtX(Frame_TitleArea(w)), XtY(Frame_TitleArea(w)),
+		       XtWidth(Frame_TitleArea(w)),
+		       XtHeight(Frame_TitleArea(w)));
+    }
+
     /* redisplay our gadget children -- if any */
     _XmRedisplayGadgets(w, event, region);
 }
 
 static XtGeometryResult
-query_geometry(Widget w,
-	       XtWidgetGeometry *request,
-	       XtWidgetGeometry *reply)
+query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 {
-    XtWidgetGeometry want;
+    Dimension		wd, ht;
+    XtGeometryResult	r;
+    XtWidgetGeometry	rr = *request;
 
-    _XmFrameLayout(w, True, w, True, &want);
+    _XmFrameComputeSize(w, w, NULL, &wd, &ht);
 
-    *reply = want;
-    reply->width = want.width;
-    reply->height = want.height;
+    reply->width = wd;
+    reply->height = ht;
     reply->request_mode = CWWidth|CWHeight;
 
     if ((request->request_mode & (CWWidth|CWHeight)) == (CWWidth|CWHeight) &&
-	reply->width == request->width && reply->height == request->height)
-	return XtGeometryYes;
-    else if (((request->request_mode & CWWidth) && (request->width == XtWidth(w))) 
-             && ((request->request_mode & CWHeight) && (request->height == XtHeight(w))))
-	return XtGeometryNo;
-    else
-	return XtGeometryAlmost;
+	reply->width == request->width && reply->height == request->height) {
+	r =  XtGeometryYes;
+    } else if (((request->request_mode & CWWidth) && (request->width == XtWidth(w))) &&
+	     ((request->request_mode & CWHeight) && (request->height == XtHeight(w)))) {
+	r =  XtGeometryNo;
+    } else {
+	r =  XtGeometryAlmost;
+    }
+
+    XdbDebug(__FILE__, w, "QueryGeometry [%s] => %s [%s]\n",
+	XdbWidgetGeometry2String(&rr), XdbGeometryResult2String(r),
+	XdbWidgetGeometry2String(reply));
+
+    return r;
 }
 
+/*
+ * This GeometryManager is a weirdo compared to many others in LessTif.
+ *
+ * Frame can have two children - a title and a work area.
+ * Geometry Manager replies differently depending on which one asks a question.
+ *
+ * Implementation before October 1996 didn't do that :-(
+ */
 static XtGeometryResult
 geometry_manager(Widget w,
 		 XtWidgetGeometry *request,
 		 XtWidgetGeometry *reply)
 {
-    XtWidgetGeometry want;
+    Dimension wd, ht;
+    Widget	fw = XtParent(w);
+
+    XdbDebug2(__FILE__, fw, w, "Frame GeometryManager [%s]\n",
+	XdbWidgetGeometry2String(request));
 
     if (FrameC_ChildType(w) != XmFRAME_WORKAREA_CHILD &&
-	FrameC_ChildType(w) != XmFRAME_TITLE_CHILD)
+	FrameC_ChildType(w) != XmFRAME_TITLE_CHILD) {
+	XdbDebug(__FILE__, w, "Frame GeometryManager => No\n");
 	return XtGeometryNo;
-
-    want = *request;
-
-    /* FIXME -- there is an implicit assumption that the child has set
-     * CWWidth and CWHeight in the request_mode */
-    _XmFrameLayout(XtParent(w), True, w, True, &want);
-
-    if (request->request_mode & (CWX|CWY))
+    }
+#ifdef	STRICT_GEOMETRY_RULES
+    if (request->request_mode & (CWX|CWY)) {
+	XdbDebug2(__FILE__, fw, w, "Frame GeometryManager (asked X or Y) => No\n");
 	return XtGeometryNo;
+    }
+#endif
+    if (request->request_mode & XtCWQueryOnly) {
+	abort();
+    }
 
-    *reply = want;
-    if ((request->request_mode & (CWWidth|CWHeight)) == (CWWidth|CWHeight) &&
-	reply->width == request->width && reply->height == request->height)
-	return XtGeometryYes;
-    else if (request->width == XtWidth(w) && request->height == XtHeight(w))
-	return XtGeometryNo;
+/* Calculate desired Frame geometry in wd/ht from requested child geometry in "request" */
+    _XmFrameComputeSize(fw, w, request, &wd, &ht);
+
+    if (wd != XtWidth(fw) || ht != XtHeight(fw)) {
+	XtWidgetGeometry	geo;
+
+	geo.width = wd;
+	geo.height = ht;
+	geo.request_mode = CWWidth | CWHeight;
+
+	if (_XmMakeGeometryRequest(fw, &geo) == XtGeometryYes) {
+	    XtWidth(fw) = geo.width;
+	    XtHeight(fw) = geo.height;
+	}
+
+	_XmFrameConfigureChildren(fw, w, reply, XtWidth(fw), XtHeight(fw));
+
+/* FIXME : Does _XmFrameConfigureChildren just return a value in "reply" here ? */
+	XdbDebug2(__FILE__, fw, w, "GeometryManager : _XmFrameConfigureChildren => %s\n",
+		XdbWidgetGeometry2String(reply));
+    } else {
+	if (w == Frame_TitleArea(fw)) {
+	    reply->width = XtWidth(fw)
+		- 2 * (MGR_ShadowThickness(fw) + Frame_MarginHeight(fw) + FrameC_ChildHSpacing(w));
+	    reply->height = XtHeight(fw) - 2 * (MGR_ShadowThickness(fw) + Frame_MarginWidth(fw));
+	    reply->request_mode = CWWidth | CWHeight;
+	} else {	/* Work Area */
+	    reply->width = XtWidth(fw) - 2 * (MGR_ShadowThickness(fw) + Frame_MarginWidth(fw));
+	    reply->height = XtHeight(fw) - 2 * (MGR_ShadowThickness(fw) + Frame_MarginHeight(fw));
+	    reply->request_mode = CWWidth | CWHeight;
+	}
+	if (reply->width < 1)
+	    reply->width = 1;
+	if (reply->height < 1)
+	    reply->height = 1;
+
+	XdbDebug2(__FILE__, fw, w, "GeometryManager determined child size at %d [%s]\n",
+		__LINE__, XdbWidgetGeometry2String(reply));
+    }
+
+    Frame_OldShadowX(w) = 0;
+    if (Frame_TitleArea(fw) && XtIsManaged(Frame_TitleArea(fw)))
+	Frame_OldShadowY(fw) = XtHeight(Frame_TitleArea(fw)) +
+				MGR_ShadowThickness(fw) +
+				Frame_MarginHeight(fw);
     else
+	Frame_OldShadowY(fw) = MGR_ShadowThickness(fw) +
+				Frame_MarginHeight(fw);
+
+    Frame_OldWidth(fw) = XtWidth(fw);
+    Frame_OldHeight(fw) = XtHeight(fw);
+
+    if ((request->request_mode & (CWWidth|CWHeight)) == (CWWidth|CWHeight) &&
+	reply->width == request->width && reply->height == request->height) {
+	XdbDebug2(__FILE__, fw, w, "Frame GeometryManager => Yes (%d %d)\n",
+	    reply->width, reply->height);
+	return XtGeometryYes;
+    } else if (request->width == XtWidth(w) && request->height == XtHeight(w)) {
+	XdbDebug2(__FILE__, fw, w, "Frame GeometryManager => No\n");
+	return XtGeometryNo;
+    } else {
+	XdbDebug2(__FILE__, fw, w, "Frame GeometryManager => Almost (%d %d)\n",
+	    reply->width, reply->height);
 	return XtGeometryAlmost;
+    }
 }
 
 static void
@@ -514,6 +651,8 @@ change_managed(Widget w)
 {
     int i;
     Widget child;
+    Dimension wd, ht;
+    XtWidgetGeometry request;
 
     Frame_TitleArea(w) = NULL;
     Frame_WorkArea(w) = NULL;
@@ -542,7 +681,31 @@ change_managed(Widget w)
 	}
     }
  
-    _XmFrameLayout(w, True, NULL, False, NULL);
+    _XmFrameComputeSize(w, NULL, NULL, &wd, &ht);
+
+    if (wd > XtWidth(w) || ht > XtHeight(w))
+    {
+	request.request_mode = (CWWidth|CWHeight);
+	request.width = wd;
+	request.height = ht;
+	_XmMakeGeometryRequest(w, &request);
+	wd = request.width;
+	ht = request.height;
+
+	_XmFrameConfigureChildren(w, NULL, NULL, wd, ht);
+    }
+
+    Frame_OldShadowX(w) = 0;
+    if (Frame_TitleArea(w) && XtIsManaged(Frame_TitleArea(w)))
+	Frame_OldShadowY(w) = XtHeight(Frame_TitleArea(w)) +
+				MGR_ShadowThickness(w) +
+				Frame_MarginHeight(w);
+    else
+	Frame_OldShadowY(w) = MGR_ShadowThickness(w) +
+				Frame_MarginHeight(w);
+
+    Frame_OldWidth(w) = XtWidth(w);
+    Frame_OldHeight(w) = XtHeight(w);
 
     _XmNavigChangeManaged(w);
 }
@@ -551,165 +714,137 @@ change_managed(Widget w)
 
 static void 
 constraint_initialize(Widget request, 
-		      Widget new, 
+		      Widget new_w, 
 		      ArgList args, 
 		      Cardinal *num_args)
 {
-    if (FrameC_ChildHSpacing(new) == XmINVALID_DIMENSION)
-	FrameC_ChildHSpacing(new) = Frame_MarginWidth(XtParent(new));
+    if (FrameC_ChildHSpacing(new_w) == XmINVALID_DIMENSION)
+	FrameC_ChildHSpacing(new_w) = Frame_MarginWidth(XtParent(new_w)) + 10;
 }
 
 static Boolean
 constraint_set_values(Widget current, 
 		      Widget request, 
-		      Widget new, 
+		      Widget new_w, 
 		      ArgList args, 
 		      Cardinal *num_args)
 {
     Boolean redisplay = False;
 
-    if (FrameC_ChildType(current) != FrameC_ChildType(new) ||
-	FrameC_ChildHAlignment(current) != FrameC_ChildHAlignment(new) ||
-	FrameC_ChildHSpacing(current) != FrameC_ChildHSpacing(new) ||
-	FrameC_ChildVAlignment(current) != FrameC_ChildVAlignment(new))
+    if (FrameC_ChildType(current) != FrameC_ChildType(new_w) ||
+	FrameC_ChildHAlignment(current) != FrameC_ChildHAlignment(new_w) ||
+	FrameC_ChildHSpacing(current) != FrameC_ChildHSpacing(new_w) ||
+	FrameC_ChildVAlignment(current) != FrameC_ChildVAlignment(new_w))
 	redisplay = True;
 
-    if (redisplay)
-	_XmFrameLayout(XtParent(new), True, NULL, False, NULL);
-
     return redisplay;
 }
 
+/*
+ * Danny 11/1/1997 - adding some risk to the implementation.
+ *
+ * This function is only called with an instigator and a desired
+ * geometry from GeometryManager.
+ *
+ * The problem is that the result of this function depends highly on
+ * whether you pass an instigator (and geometry) or not.
+ *
+ * Since the only place from which this happens is one where we assume
+ * that we're actually doing this geometry, we will now not go wrong in
+ * setting the instigator widget's geometry to the desired one.
+ *
+ * The above assumption is true because in all of LessTif we're not
+ * implementing QueryOnly in the Geometry Managers.
+ *
+ * You can disable this with #undef OPTIMISTIC_COMPUTE_SIZE above.
+ */
 static void
-_XmFrameLayout(Widget w, Boolean ParentResize,
-	       Widget child, Boolean TestMode, XtWidgetGeometry *childgeom)
+_XmFrameComputeSize(Widget w, Widget instig, XtWidgetGeometry *desired,
+		    Dimension *wd, Dimension *ht)
 {
-    int title_x = 0, title_y = 0;
-    int workarea_x = 0, workarea_y = 0;
     Dimension curw, curh;
-    XtWidgetGeometry geo;
-    XtGeometryResult result;
 
-    XdbDebug(__FILE__, w, "_XmFrameLayout %s %s\n", ParentResize ? "(ParentResize)" : "",
-		TestMode ? "(TestMode)" : "");
+    XdbDebug(__FILE__, w, "_XmFrameComputeSize");
+    if (instig != w && instig)
+	XdbDebug0(__FILE__, w, " Instig %s (desired geo %s)",
+		XtName(instig), XdbWidgetGeometry2String(desired));
+    if (Frame_TitleArea(w))
+	XdbDebug0(__FILE__, w, " Title %s wid %d ht %d",
+		XtName(Frame_TitleArea(w)), XtWidth(Frame_TitleArea(w)),
+		XtHeight(Frame_TitleArea(w)));
+    if (Frame_WorkArea(w))
+	XdbDebug0(__FILE__, w, " Work %s wid %d ht %d",
+		XtName(Frame_WorkArea(w)), XtWidth(Frame_WorkArea(w)),
+		XtHeight(Frame_WorkArea(w)));
+    XdbDebug0(__FILE__, w, "\n");
 
     curw = curh = 0;
 
-    if (Frame_TitleArea(w))
+#ifdef	OPTIMISTIC_COMPUTE_SIZE
+    if (Frame_TitleArea(w) && instig == Frame_TitleArea(w)) {
+	if (desired->request_mode & CWWidth)
+	    XtWidth(Frame_TitleArea(w)) = desired->width;
+	if (desired->request_mode & CWHeight)
+	    XtHeight(Frame_TitleArea(w)) = desired->height;
+    }
+    if (Frame_WorkArea(w) && instig == Frame_WorkArea(w)) {
+	if (desired->request_mode & CWWidth)
+	    XtWidth(Frame_WorkArea(w)) = desired->width;
+	if (desired->request_mode & CWHeight)
+	    XtHeight(Frame_WorkArea(w)) = desired->height;
+    }
+#endif
+
+    if (Frame_TitleArea(w) && XtIsManaged(Frame_TitleArea(w)))
     {
-	if (TestMode && child == Frame_TitleArea(w)) {
-	    curh += childgeom->height;
-	    curw = childgeom->width;
-	}
-	else {
-	    /* FIXME -- when all children answer right, the curh/curw
-	     * assignment should be changed to use the geo results */
-	    XtQueryGeometry(Frame_TitleArea(w), NULL, &geo);
+	if (instig == Frame_TitleArea(w) && desired->request_mode & CWHeight)
+	    curh += desired->height;
+	else
 	    curh += XtHeight(Frame_TitleArea(w));
-	    curw = XtWidth(Frame_TitleArea(w));
-	}
+
+	if (instig == Frame_TitleArea(w) && desired->request_mode & CWWidth)
+	    curw = desired->width + 2 * FrameC_ChildHSpacing(Frame_TitleArea(w));
+	else
+	    curw = XtWidth(Frame_TitleArea(w)) + 2 * FrameC_ChildHSpacing(Frame_TitleArea(w));
     }
 
-    if (Frame_WorkArea(w))
+    if (Frame_WorkArea(w) && XtIsManaged(Frame_WorkArea(w)))
     {
-	if (TestMode && child == Frame_WorkArea(w)) {
-	    curh += childgeom->height;
-	    curw = childgeom->width;
-	}
-	else {
-	    /* FIXME -- when all children answer right, the curh/curw
-	     * assignment should be changed to use the geo results */
-	    XtQueryGeometry(Frame_WorkArea(w), NULL, &geo);
+	if (instig == Frame_WorkArea(w) && desired->request_mode & CWHeight)
+	    curh += desired->height;
+	else
 	    curh += XtHeight(Frame_WorkArea(w));
+
+	if (instig == Frame_WorkArea(w) && desired->request_mode & CWWidth)
+	    curw = desired->width;
+	else
 	    curw = _XmMax(curw, XtWidth(Frame_WorkArea(w)));
-	}
     }
 
     curh += 2 * Frame_MarginHeight(w) + 2 * MGR_ShadowThickness(w);
     curw += 2 * Frame_MarginWidth(w) + 2 * MGR_ShadowThickness(w);
 
-    /* this condition is met in the query_geometry call */
-    if (TestMode && w == child) {
-	childgeom->width = curw;
-	childgeom->height = curh;
-	return;
-    }
+    *wd = curw;
+    *ht = curh;
 
-    /* FIXME -- this needs to be beefed up to properly set child values
-     * when XtGeometryYes */
-    if (TestMode) {
-	geo.width = curw;
-	geo.height = curh;
-	geo.request_mode = XtCWQueryOnly|CWWidth|CWHeight;
-	result = XtMakeGeometryRequest(w, &geo, &geo);
+    XdbDebug2(__FILE__, w, instig, "_XmFrameComputeSize (%s) => %d %d\n",
+	XdbWidgetGeometry2String(desired), curw, curh);
+}
 
-	switch (result) {
-	case XtGeometryNo:
-	    childgeom->width = XtWidth(child);
-	    childgeom->height = XtHeight(child);
-	    /* go back, no layout change */
-	    return;
+static void
+_XmFrameConfigureChildren(Widget w, Widget instig, XtWidgetGeometry *desired,
+			  Dimension curw, Dimension curh)
+{
+    int title_x = 0, title_y = 0;
+    int workarea_x = 0, workarea_y = 0;
 
-	case XtGeometryYes:
-	    /* implicit assumption:  since we don't override the values in
-	     * childgeom, Xt should take care of the layout changes for us.
-	     * So we continue with the layout method, attempting to resize
-	     * ourselves, and setting the fields appropriately */
-	    break;
+    XdbDebug2(__FILE__, w, instig,
+	"_XmFrameConfigureChildren (child wants %s, Frame geo %d %d)\n",
+	XdbWidgetGeometry2String(desired), curw, curh);
 
-	case XtGeometryAlmost:
-	    /* we need to compute the layout based on the size our parent said
-	     * we could attain. */
-	    if (Frame_TitleArea(w) && child == Frame_TitleArea(w))
-	    {
-		childgeom->width =
-			 _XmMin(XtWidth(Frame_TitleArea(w)),
-				curw - 2 * MGR_ShadowThickness(w)
-				- 2 * Frame_MarginWidth(w)
-				- 2 * FrameC_ChildHSpacing(Frame_TitleArea(w)));
-	    }
-	    if (Frame_WorkArea(w) && child == Frame_WorkArea(w))
-	    {
-		childgeom->width = curw - 2 * MGR_ShadowThickness(w)
-				       - 2 * Frame_MarginWidth(w);
-		childgeom->height = curh - XtY(Frame_WorkArea(w))
-				        - Frame_MarginHeight(w)
-				        - MGR_ShadowThickness(w);
-	    }
-	    return;
+    workarea_y = MGR_ShadowThickness(w) + Frame_MarginHeight(w);
 
-	/* shouldn't happen */
-	case XtGeometryDone:
-	default:
-	    _XmWarning(w, "Parent returned unexpected geometry result.");
-	    return;
-	}
-    }
-
-    if (ParentResize) {
-	do {
-	    result = XtMakeResizeRequest(w, curw, curh, &curw, &curh);
-	} while (result == XtGeometryAlmost);
-
-	if (result == XtGeometryYes) {
-	    XtWidth(w) = curw;
-	    XtHeight(w) = curh;
-	}
-	if (result == XtGeometryNo) {
-	    curw = XtWidth(w);
-	    curh = XtHeight(w);
-	}
-	XdbDebug(__FILE__, NULL, "_XmFrameLayout %s (%d %d) => %s\n",
-		 XtName(w), curw, curh, XdbGeometryResult2String(result));
-    }
-    else {
-	curw = XtWidth(w);
-	curh = XtHeight(w);
-    }
-
-    workarea_y = MGR_ShadowThickness(w);
-
-    if (Frame_TitleArea(w))
+    if (Frame_TitleArea(w) && XtIsManaged(Frame_TitleArea(w)))
     {
 	switch (FrameC_ChildVAlignment(Frame_TitleArea(w)))
 	{
@@ -718,7 +853,9 @@ _XmFrameLayout(Widget w, Boolean ParentResize,
 	    workarea_y = XtHeight(Frame_TitleArea(w)) + Frame_MarginHeight(w);
 	    break;
 	case XmALIGNMENT_BASELINE_BOTTOM:
-	    /* fill this out */
+	    _XmWarning(w, "_XmFrameConfigureChildren doesn't implement"
+			"XmALIGNMENT_BASELINE_BOTTOM yet\n");
+	    /* FIX ME - fill this out */
 	    break;
 	case XmALIGNMENT_BASELINE_TOP:
 	    title_y = 0;
@@ -736,31 +873,45 @@ _XmFrameLayout(Widget w, Boolean ParentResize,
 	    break;
 	}
 
-	switch (FrameC_ChildHAlignment(Frame_TitleArea(w)))
-	{
+	title_x = MGR_ShadowThickness(w) +
+		  FrameC_ChildHSpacing(Frame_TitleArea(w));
+
+	switch (FrameC_ChildHAlignment(Frame_TitleArea(w))) {
 	case XmALIGNMENT_BEGINNING:
-	    title_x = 20;
-	    break;
-	case XmALIGNMENT_END:
-	    title_x = curw - XtWidth(Frame_TitleArea(w)) - 20;
+	    if (MGR_StringDirection(w) != XmSTRING_DIRECTION_L_TO_R)
+	    {
+		title_x = XtWidth(w) - XtWidth(Frame_TitleArea(w)) - title_x -
+			  2 * XtBorderWidth(Frame_TitleArea(w));
+	    }
 	    break;
 	case XmALIGNMENT_CENTER:
-	    title_x = (int)((float)(curw/2)
-			  - (float)(XtWidth(Frame_TitleArea(w))/2));
+	    title_x = (curw >> 1) - (XtWidth(Frame_TitleArea(w)) >> 1);
+	    break;
+	case XmALIGNMENT_END:
+	default:
+	    if (MGR_StringDirection(w) == XmSTRING_DIRECTION_L_TO_R)
+	    {
+		title_x = XtWidth(w) - XtWidth(Frame_TitleArea(w)) - title_x -
+			  2 * XtBorderWidth(Frame_TitleArea(w));
+	    }
 	    break;
 	}
 
-	if (TestMode && Frame_TitleArea(w) &&
-	    child == Frame_TitleArea(w)) {
-	    XtX(Frame_TitleArea(w)) = title_x;
-	    XtY(Frame_TitleArea(w)) = title_y;
-	    XtWidth(Frame_TitleArea(w)) =
+	if (Frame_TitleArea(w) == instig)
+	{
+	    desired->request_mode = CWBorderWidth|CWWidth|CWHeight|CWX|CWY;
+	    desired->x = title_x;
+	    desired->y = title_y;
+	    desired->width =
 		 _XmMin(XtWidth(Frame_TitleArea(w)),
 			curw - 2 * MGR_ShadowThickness(w)
 			- 2 * Frame_MarginWidth(w)
 			- 2 * FrameC_ChildHSpacing(Frame_TitleArea(w)));
+	    desired->height = XtHeight(Frame_TitleArea(w));
+	    desired->border_width = XtBorderWidth(Frame_TitleArea(w));
 	}
 	else
+	{
 	    _XmConfigureObject(Frame_TitleArea(w),
 			       title_x, title_y,
 			       _XmMin(XtWidth(Frame_TitleArea(w)),
@@ -769,22 +920,50 @@ _XmFrameLayout(Widget w, Boolean ParentResize,
 					- 2 * FrameC_ChildHSpacing(Frame_TitleArea(w))),
 			       XtHeight(Frame_TitleArea(w)),
 			       XtBorderWidth(Frame_TitleArea(w)));
+	}
     }
 	
-    if (Frame_WorkArea(w))
+    if (Frame_WorkArea(w) && XtIsManaged(Frame_WorkArea(w)))
     {
 	workarea_x = MGR_ShadowThickness(w) + Frame_MarginWidth(w);
 
-	if (TestMode && Frame_WorkArea(w) && child == Frame_WorkArea(w)) {
-	    XtX(Frame_WorkArea(w)) = workarea_x;
-	    XtY(Frame_WorkArea(w)) = workarea_y;
-	    XtWidth(Frame_WorkArea(w)) = curw - 2 * MGR_ShadowThickness(w)
-					       - 2 * Frame_MarginWidth(w);
-	    XtHeight(Frame_WorkArea(w)) = curh - XtY(Frame_WorkArea(w))
-					        - Frame_MarginHeight(w)
-					        - MGR_ShadowThickness(w);
+	if (Frame_WorkArea(w) == instig)
+	{
+	    desired->request_mode = CWBorderWidth|CWWidth|CWHeight|CWX|CWY;
+	    desired->x = workarea_x;
+	    desired->y = workarea_y;
+	    desired->border_width = XtBorderWidth(Frame_WorkArea(w));
+
+/*
+ * This is the place that makes Xinvest-2.1 fail.
+ *
+ * In a call with instig vs. one without it, we get different geometries for the height
+ * of the workarea child.
+ *
+ * Danny 4/10/1996
+ */
+#if 1
+	    desired->width = curw - 2 * MGR_ShadowThickness(w)
+			- 2 * Frame_MarginWidth(w);
+	    desired->height = curh - XtY(Frame_WorkArea(w))
+			- Frame_MarginHeight(w) - MGR_ShadowThickness(w);
+#else
+	    Dimension	wd, ht;
+
+	    wd = curw - 2 * MGR_ShadowThickness(w) - 2 * Frame_MarginWidth(w);
+	    ht = curh - XtY(Frame_WorkArea(w))
+		- Frame_MarginHeight(w) - MGR_ShadowThickness(w);
+
+	    if (wd > desired->width)
+		desired->width = wd;
+	    if (ht > desired->height)
+		desired->height = ht;
+#endif
 	}
 	else
+	{
+#if 0
+/* xmcd breakage here ... */
 	    _XmConfigureObject(Frame_WorkArea(w),
 			       workarea_x, workarea_y,
 			       curw - 2 * MGR_ShadowThickness(w)
@@ -793,6 +972,17 @@ _XmFrameLayout(Widget w, Boolean ParentResize,
 				        - Frame_MarginHeight(w)
 				        - MGR_ShadowThickness(w),
 			       XtBorderWidth(Frame_WorkArea(w)));
+#else
+	    _XmConfigureObject(Frame_WorkArea(w),
+			       workarea_x, workarea_y,
+			       curw - 2 * MGR_ShadowThickness(w)
+				       - 2 * Frame_MarginWidth(w),
+			       curh - workarea_y
+				        - Frame_MarginHeight(w)
+				        - MGR_ShadowThickness(w),
+			       XtBorderWidth(Frame_WorkArea(w)));
+#endif
+	}
     }
 }
 
