@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Gadget.c,v 1.27 1996/04/22 22:54:54 miers Exp $
+ * $Id: Gadget.c,v 1.6 1996/11/28 09:21:17 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,27 +22,29 @@
  *
  **/
 
-static char rcsid[] = "$Id: Gadget.c,v 1.27 1996/04/22 22:54:54 miers Exp $";
+static char rcsid[] = "$Id: Gadget.c,v 1.6 1996/11/28 09:21:17 u27113 Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/GadgetP.h>
 #include <Xm/ExtObjectP.h>
 #include <Xm/RepType.h>
 #include <string.h>
-#include <stdio.h>
+
+#include <XmI/DebugUtil.h>
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 
 static void input_dispatch(Widget w, XEvent *event, Mask event_mask);
 static Boolean visual_change(Widget w, Widget parent, Widget n);
 static void focus_change(Widget w, XmFocusChange fevent);
-static void secondary_object_create(Widget req, Widget new, ArgList args,
+static void secondary_object_create(Widget req, Widget new_w, ArgList args,
 				    Cardinal *num_args);
 static XmNavigability widget_navigable(Widget w);
 
@@ -208,8 +210,8 @@ static XmGadgetClassExtRec _XmGadgetClassExtRec = {
     /* record_type               */ NULLQUARK,
     /* version                   */ XmGadgetClassExtVersion,
     /* size                      */ sizeof(XmGadgetClassExtRec),
-    /* widget_baseline_proc      */ NULL, /* FIXME */
-    /* display_rect_proc         */ NULL, /* FIXME */
+    /* widget_baseline_proc      */ NULL,
+    /* display_rect_proc         */ NULL,
 };
 
 XmGadgetClassRec xmGadgetClassRec = {
@@ -274,18 +276,26 @@ class_initialize()
 static void
 class_part_initialize(WidgetClass widget_class)
 {
-    XmGadgetClass gc;
+    XmGadgetClass gc, sc;
 
     gc = (XmGadgetClass)widget_class;
+    sc = (XmGadgetClass)((XmGadgetClass)widget_class)->rect_class.superclass;
 
     if (gc->gadget_class.border_highlight == XmInheritBorderHighlight)
-	gc->gadget_class.border_highlight = gadget_border_highlight;
+	gc->gadget_class.border_highlight =
+		sc->gadget_class.border_highlight;
     if (gc->gadget_class.border_unhighlight == XmInheritBorderUnhighlight)
-	gc->gadget_class.border_unhighlight = gadget_border_unhighlight;
+	gc->gadget_class.border_unhighlight =
+		sc->gadget_class.border_unhighlight;
     if (gc->gadget_class.visual_change == XmInheritVisualChange)
-	gc->gadget_class.visual_change = visual_change;
+	gc->gadget_class.visual_change =
+		sc->gadget_class.visual_change;
     if (gc->gadget_class.input_dispatch == XmInheritInputDispatch)
-	gc->gadget_class.input_dispatch = input_dispatch;
+	gc->gadget_class.input_dispatch =
+		sc->gadget_class.input_dispatch;
+    if (gc->gadget_class.arm_and_activate == XmInheritArmAndActivate)
+	gc->gadget_class.arm_and_activate =
+		sc->gadget_class.arm_and_activate;
 
     _XmBaseClassPartInitialize(widget_class);
     _XmFastSubclassInit(widget_class, XmGADGET_BIT);
@@ -300,94 +310,107 @@ class_part_initialize(WidgetClass widget_class)
 }
 
 static void
-secondary_object_create(Widget req, Widget new,
+secondary_object_create(Widget req, Widget new_w,
 			ArgList args, Cardinal *num_args)
 {
     XmBaseClassExt *bce;
     Arg argl[1];
     ArgList merged;
 
-    XtSetArg(argl[0], XmNlogicalParent, new);
+    XtSetArg(argl[0], XmNlogicalParent, new_w);
 
     if (*num_args) {
-	merged = XtMergeArgLists(args, *num_args, argl, XtNumber(args));
-	bce = _XmGetBaseClassExtPtr(XtClass(new), XmQmotif);
-	XtCreateWidget(XtName(new), (*bce)->secondaryObjectClass,
-		       XtParent(new)
-			? XtParent(new)
-			: new,
+	merged = XtMergeArgLists(args, *num_args, argl, XtNumber(argl));
+	bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
+	XtCreateWidget(XtName(new_w), (*bce)->secondaryObjectClass,
+		       XtParent(new_w)
+			? XtParent(new_w)
+			: new_w,
 		       merged, *num_args + 1);
 	XtFree((char *)merged);
     }
     else {
-	bce = _XmGetBaseClassExtPtr(XtClass(new), XmQmotif);
-	XtCreateWidget(XtName(new), (*bce)->secondaryObjectClass,
-		       XtParent(new)
-			? XtParent(new)
-			: new,
+	bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
+	XtCreateWidget(XtName(new_w), (*bce)->secondaryObjectClass,
+		       XtParent(new_w)
+			? XtParent(new_w)
+			: new_w,
 		       argl, 1);
     }
 }
 
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     XmBaseClassExt bce;
 
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRNavigationType),
-			     G_NavigationType(new),
-			     new))
-	G_NavigationType(new) = XmNONE; 
+			     G_NavigationType(new_w),
+			     new_w))
+	G_NavigationType(new_w) = XmNONE; 
 
-#if 0
-/* this should be here, but doesn't work */
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRUnitType),
-			     G_UnitType(new),
-			     new))
-	G_UnitType(new) = XmPIXELS;
-#endif
+			     G_UnitType(new_w),
+			     new_w))
+	G_UnitType(new_w) = XmPIXELS;
 
     if (XtWidth(request) == (Dimension)0)
-	XtWidth(new) = (G_HighlightThickness(new) * 2
-			+ G_ShadowThickness(new) * 2);
+	XtWidth(new_w) = (G_HighlightThickness(new_w) * 2
+			+ G_ShadowThickness(new_w) * 2);
     if (XtHeight(request) == (Dimension)0)
-	XtHeight(new) = (G_HighlightThickness(new) * 2
-			 + G_ShadowThickness(new) * 2);
-    XtBorderWidth(new) = 0;
+	XtHeight(new_w) = (G_HighlightThickness(new_w) * 2
+			 + G_ShadowThickness(new_w) * 2);
 
-    _XmGadgetImportArgs(new, args, num_args);
+    _XmGadgetImportArgs(new_w, args, num_args);
     /* BaseClass stuff provides this */
-    _XmGadgetImportSecondaryArgs(new, args, num_args);
+    _XmGadgetImportSecondaryArgs(new_w, args, num_args);
 
-    bce = *(XmBaseClassExt *)_XmGetBaseClassExtPtr(XtClass(new), XmQmotif);
+    bce = *(XmBaseClassExt *)_XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
 
     if (bce && bce->secondaryObjectClass) {
 	if (bce->secondaryObjectCreate)
-	    (bce->secondaryObjectCreate)(request, new, args, num_args);
+	    (bce->secondaryObjectCreate)(request, new_w, args, num_args);
     }
 
-    G_EventMask(new) = 0;
-    G_HaveTraversal(new) = False;
-    G_Highlighted(new) = False;
-    G_HighlightDrawn(new) = False;
+    _XmNavigInitialize(request, new_w, args, num_args);
+
+    G_EventMask(new_w) = 0;
+    G_HaveTraversal(new_w) = False;
+    G_Highlighted(new_w) = False;
+    G_HighlightDrawn(new_w) = False;
 }
 
 static void
 destroy(Widget w)
 {
+    _XmNavigDestroy(w);
 }
 
 static Boolean
 set_values(Widget current,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    _XmGadgetImportArgs(new, args, num_args);
+    Boolean need_refresh = False;
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRUnitType),
+			     G_UnitType(new_w),
+			     new_w))
+	G_UnitType(new_w) = G_UnitType(current);
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRNavigationType),
+			     G_NavigationType(new_w),
+			     new_w))
+	G_NavigationType(new_w) = G_NavigationType(current); 
+
+    _XmGadgetImportArgs(new_w, args, num_args);
+
+    need_refresh = _XmNavigSetValues(current, request, new_w, args, num_args);
 
     return True; /* FIX ME */
 }
@@ -498,14 +521,10 @@ gadget_border_unhighlight(Widget w)
 
     if (XmIsManager(XtParent(w)))
     {
-	XSetForeground(XtDisplay(w), XmParentHighlightGC(w), XmParentBackground(w));
-
 	_XmDrawHighlight(XtDisplayOfObject(w), XtWindowOfObject(w),
-			 XmParentHighlightGC(w),
+			 XmParentBackgroundGC(w),
 			 XtX(w), XtY(w), XtWidth(w), XtHeight(w),
 			 G_HighlightThickness(w), LineSolid);
-
-	XSetForeground(XtDisplay(w), XmParentHighlightGC(w), XmParentHighlightColor(w));
     }
     else
 	_XmClearBorder(XtDisplay(w),
