@@ -1,32 +1,36 @@
+/* $XConsortium: EditresCom.c,v 1.33 94/04/17 20:16:05 rws Exp $ */
+
 /*
- * $Id: LTEditres.c,v 1.8 1996/04/23 04:40:29 toshok Exp $
- *
- * derived from X11R5 lib/Xmu/Editres.c 
- *
- * Copyright (C) 1995 Free Software Foundation, Inc.
- * Copyright 1989 Massachusetts Institute of Technology
- *
- * This file is part of the GNU LessTif Library.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- **/
 
-static char rcsid[] = "$Id: LTEditres.c,v 1.8 1996/04/23 04:40:29 toshok Exp $";
+Copyright (c) 1989  X Consortium
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of the X Consortium shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from the X Consortium.
+
+*/
+
+/*
+ * Author:  Chris D. Peterson, Dave Sternlicht, MIT X Consortium
+ */
 
-#include <LTconfig.h>
 #include <X11/IntrinsicP.h>	/* To get into the composite and core widget
 				   structures. */
 #include <X11/ObjectP.h>	/* For XtIs<Classname> macros. */
@@ -35,47 +39,17 @@ static char rcsid[] = "$Id: LTEditres.c,v 1.8 1996/04/23 04:40:29 toshok Exp $";
 
 #include <X11/Xatom.h>
 #include <X11/Xos.h>		/* for strcpy declaration */
+#if 0
 #include <X11/Xaw/Cardinals.h>
-#define  XK_LATIN1
-#include <X11/keysymdef.h>
-#include <X11/Xmu/CharSet.h>
-#undef XK_LATIN1
-#include <Xm/LTEditresP.h>
+#endif
+#include <XmI/LTEditresP.h>
+#include <X11/Xmd.h>
 
 #include <stdio.h>
 
 #define _LesstifEditResPutBool _LesstifEditResPut8	
 #define _LesstifEditResPutResourceType _LesstifEditResPut8
 
-/*
- * ISO Latin-1 case conversion routine
- */
-
-#if NeedFunctionPrototypes
-void XmuCopyISOLatin1Lowered(char *dst, _Xconst char *src)
-#else
-void XmuCopyISOLatin1Lowered(dst, src)
-    char *dst, *src;
-#endif
-{
-    register unsigned char *dest, *source;
-
-    for (dest = (unsigned char *)dst, source = (unsigned char *)src;
-         *source;
-         source++, dest++)
-    {
-        if ((*source >= XK_A) && (*source <= XK_Z))
-            *dest = *source + (XK_a - XK_A);
-        else if ((*source >= XK_Agrave) && (*source <= XK_Odiaeresis))
-            *dest = *source + (XK_agrave - XK_Agrave);
-        else if ((*source >= XK_Ooblique) && (*source <= XK_Thorn))
-            *dest = *source + (XK_oslash - XK_Ooblique);
-        else
-            *dest = *source;
-    }
-    *dest = '\0';
-}
-
 /************************************************************
  *
  * Local structure definitions.
@@ -101,6 +75,13 @@ typedef struct _SVErrorInfo {
     WidgetInfo * entry;
 } SVErrorInfo;
 
+typedef struct _GetValuesEvent {
+  EditresCommand type;		/* first field must be type */
+  WidgetInfo * widgets;
+  unsigned short num_entries;	/* number of get values requests */
+  char * name;
+} GetValuesEvent;
+
 typedef struct _FindChildEvent {
     EditresCommand type;	/* first field must be type. */
     WidgetInfo * widgets;
@@ -139,9 +120,12 @@ typedef struct _Globals {
     SVErrorInfo error_info;
     ProtocolStream stream;
     ProtocolStream * command_stream; /* command stream. */
+#if defined(LONG64) || defined(WORD64)
+    unsigned long base_address;
+#endif
 } Globals;
 
-#define CURRENT_PROTOCOL_VERSION 4L
+#define CURRENT_PROTOCOL_VERSION 5L
 
 #define streq(a,b) (strcmp( (a), (b) ) == 0)
 
@@ -149,25 +133,56 @@ static Atom res_editor_command, res_editor_protocol, client_value;
 
 static Globals globals;
 
-static void SendCommand(Widget, Atom, ResIdent, EditresCommand, ProtocolStream *);
-static void SendFailure(Widget, Atom, ResIdent, char *);
-static void InsertWidget(ProtocolStream *stream, Widget w);
+static void SendFailure(), SendCommand(), InsertWidget(), ExecuteCommand();
+static void FreeEvent(), ExecuteSetValues(), ExecuteGetGeometry();
+static void ExecuteGetResources();
 
-static void FreeEvent(EditresEvent *);
-static void ExecuteCommand(Widget, Atom, ResIdent, EditresEvent *event); 
-static void ExecuteSetValues(Widget, SetValuesEvent *, WidgetInfo *,
-			     ProtocolStream *, unsigned short *);
-static void ExecuteGetGeometry(Widget, ProtocolStream *);
-static void ExecuteGetResources(Widget, ProtocolStream *);
-static void GetCommand(Widget, XtPointer, Atom *,
-		       Atom *, XtPointer, unsigned long *, int *);
-static void LoadResources(Widget);
-static void DumpChildren(Widget, ProtocolStream *, unsigned short *);
-static char *DumpWidgets(Widget, EditresEvent *, ProtocolStream *);
-static char *DoSetValues(Widget, EditresEvent *, ProtocolStream *);
-static char *DoFindChild(Widget, EditresEvent *, ProtocolStream *);
-static char *DoGetGeometry(Widget, EditresEvent *, ProtocolStream *);
-static char *DoGetResources(Widget, EditresEvent *, ProtocolStream *);
+static void GetCommand();
+static void LoadResources();
+static Boolean IsChild();
+static void DumpChildren();
+static char *DumpWidgets(), *DoSetValues(), *DoFindChild();
+static char *DoGetGeometry(), *DoGetResources(), *DumpValues();
+
+#if XtSpecificationRelease < 6
+static Status
+XInternAtoms(Display *dpy, char **names, int count, Bool ifExists, Atom *aret)
+{
+    int i;
+
+    for (i = 0; i < count; i++) {
+	aret[i] = XInternAtom(dpy, names[i], ifExists);
+    }
+
+    return Success;
+}
+#endif
+
+/*
+ * avoid linking with Xmu
+ */
+#include <X11/keysym.h>
+
+static void
+XmuCopyISOLatin1Lowered(char *dst, _Xconst char *src)
+{
+    register unsigned char *dest, *source;
+
+    for (dest = (unsigned char *)dst, source = (unsigned char *)src;
+	 *source;
+	 source++, dest++)
+    {
+	if ((*source >= XK_A) && (*source <= XK_Z))
+	    *dest = *source + (XK_a - XK_A);
+	else if ((*source >= XK_Agrave) && (*source <= XK_Odiaeresis))
+	    *dest = *source + (XK_agrave - XK_Agrave);
+	else if ((*source >= XK_Ooblique) && (*source <= XK_Thorn))
+	    *dest = *source + (XK_oslash - XK_Ooblique);
+	else
+	    *dest = *source;
+    }
+    *dest = '\0';
+}
 
 /************************************************************
  *
@@ -175,7 +190,7 @@ static char *DoGetResources(Widget, EditresEvent *, ProtocolStream *);
  *
  ************************************************************/
 
-/*	Function Name: _LesstifEditResCheckMessages
+/*	Function Name: _LesstifEditResCheckMessages (_LesstifEditResCheckMessages)
  *	Description: This callback routine is set on all shell widgets,
  *                   and checks to see if a client message event
  *                   has come from the resource editor.
@@ -186,11 +201,13 @@ static char *DoGetResources(Widget, EditresEvent *, ProtocolStream *);
  *	Returns: none.
  */
 
+/* ARGSUSED */
 void
-_LesstifEditResCheckMessages(Widget w, 
-			     XtPointer data, 
-			     XEvent *event, 
-			     Boolean *cont)
+_LesstifEditResCheckMessages(w, data, event, cont)
+Widget w;
+XtPointer data;
+XEvent *event;
+Boolean *cont;
 {
     Time time;
     ResIdent ident;
@@ -203,14 +220,18 @@ _LesstifEditResCheckMessages(Widget w,
 	dpy = XtDisplay(w);
 
 	if (!first_time) {
+	    Atom atoms[4];
+	    static char* names[] = {
+		EDITRES_NAME, EDITRES_COMMAND_ATOM,
+		EDITRES_PROTOCOL_ATOM, EDITRES_CLIENT_VALUE };
+		
 	    first_time = TRUE;
-	    res_editor = XInternAtom(dpy, EDITRES_NAME, False);
-	    res_editor_command = XInternAtom(dpy, EDITRES_COMMAND_ATOM, False);
-	    res_editor_protocol = XInternAtom(dpy, EDITRES_PROTOCOL_ATOM,
-					      False);
-
+	    XInternAtoms(dpy, names, 4, FALSE, atoms);
+	    res_editor = atoms[0];
+	    res_editor_command = atoms[1];
+	    res_editor_protocol = atoms[2];
 	    /* Used in later procedures. */
-	    client_value = XInternAtom(dpy, EDITRES_CLIENT_VALUE, False);
+	    client_value = atoms[3];
 	    LoadResources(w);
 	}
 
@@ -229,8 +250,7 @@ _LesstifEditResCheckMessages(Widget w,
 	}
 
 	XtGetSelectionValue(w, res_comm, res_editor_command,
-			    (XtSelectionCallbackProc)GetCommand, 
-                            (XtPointer) ((unsigned)ident), time);
+			    GetCommand, (XtPointer)((unsigned)ident), time);
     }
 }
 
@@ -248,11 +268,12 @@ _LesstifEditResCheckMessages(Widget w,
 #define ERROR_MESSAGE ("Client: Improperly formatted protocol request")
 
 static EditresEvent *
-BuildEvent(Widget w, 
-	   Atom sel, 
-	   XtPointer data, 
-	   ResIdent ident, 
-	   unsigned long length)
+BuildEvent(w, sel, data, ident, length)
+Widget w;
+Atom sel;
+XtPointer data;
+ResIdent ident;
+unsigned long length;
 {
     EditresEvent * event;
     ProtocolStream alloc_stream, *stream;
@@ -269,7 +290,7 @@ BuildEvent(Widget w,
      */
 
     if (length < HEADER_SIZE) {
-	SendFailure(w, sel, ident, ERROR_MESSAGE);
+	SendFailure(w, sel, ident, Failure, ERROR_MESSAGE);
 	return(NULL);
     }
 
@@ -365,6 +386,18 @@ BuildEvent(Widget w,
 	    }
 	}
 	break;
+
+    case GetValues: 
+        {
+            GetValuesEvent * gv_event = (GetValuesEvent *) event;
+            _LesstifEditResGetString8(stream, &(gv_event->name));
+            _LesstifEditResGet16(stream, &(gv_event->num_entries));
+	    gv_event->widgets = (WidgetInfo *)
+		XtCalloc(sizeof(WidgetInfo), gv_event->num_entries);
+            _LesstifEditResGetWidgetInfo(stream, gv_event->widgets);
+        }
+        break;	
+
     default:
 	{
 	    char buf[BUFSIZ];
@@ -391,7 +424,8 @@ BuildEvent(Widget w,
  */
 
 static void
-FreeEvent(EditresEvent *event)
+FreeEvent(event)
+EditresEvent * event;
 {
     if (event->any_event.widgets != NULL) {
 	XtFree((char *)event->any_event.widgets->ids);
@@ -414,16 +448,16 @@ FreeEvent(EditresEvent *event)
  *	Returns: none.
  */
 
+/* ARGSUSED */
 static void
-GetCommand(Widget w, 
-	   XtPointer data, 
-	   Atom *selection, 
-	   Atom *type, 
-	   XtPointer value, 
-	   unsigned long *length, 
-	   int *format)
+GetCommand(w, data, selection, type, value, length, format)
+Widget w;
+XtPointer data, value;
+Atom *selection, *type;
+unsigned long *length;
+int * format;
 {
-    ResIdent ident = (ResIdent) ((unsigned)data);
+    ResIdent ident = (ResIdent)((unsigned)data);
     EditresEvent * event;
 
     if ( (*type != res_editor_protocol) || (*format != EDITRES_FORMAT) )
@@ -446,13 +480,15 @@ GetCommand(Widget w,
  * NOTES:  munges str
  */
 
+/* ARGSUSED */    
 static void
-ExecuteCommand(Widget w, 
-	       Atom sel, 
-	       ResIdent ident, 
-	       EditresEvent *event)
+ExecuteCommand(w, sel, ident, event)
+Widget w;
+Atom sel;
+ResIdent ident;
+EditresEvent * event;
 {
-    char * (*func)(Widget, EditresEvent *, ProtocolStream *);
+    char * (*func)();
     char * str;
 
     if (globals.block == BlockAll) {
@@ -469,6 +505,9 @@ ExecuteCommand(Widget w,
 
     switch(event->any_event.type) {
     case SendWidgetTree:
+#if defined(LONG64) || defined(WORD64)
+	globals.base_address = (unsigned long)w & 0xFFFFFFFF00000000;
+#endif
 	func = DumpWidgets;
 	break;
     case SetValues:
@@ -483,6 +522,9 @@ ExecuteCommand(Widget w,
     case GetResources:
 	func = DoGetResources;
 	break;
+    case GetValues:
+        func = DumpValues;
+    break;
     default: 
         {
 	    char buf[BUFSIZ];
@@ -512,15 +554,16 @@ ExecuteCommand(Widget w,
  *                 format_ret - the format the selection is in.
  *	Returns: True if conversion was sucessful.
  */
-
+    
+/* ARGSUSED */
 static Boolean
-ConvertReturnCommand(Widget w, 
-		     Atom *selection, 
-		     Atom *target,
-		     Atom *type_ret, 
-		     XtPointer *value_ret, 
-		     unsigned long *length_ret, 
-		     int *format_ret)
+ConvertReturnCommand(w, selection, target,
+		     type_ret, value_ret, length_ret, format_ret)
+Widget w;
+Atom * selection, * target, * type_ret;
+XtPointer *value_ret;
+unsigned long * length_ret;
+int * format_ret;
 {
     /*
      * I assume the intrinsics give me the correct selection back.
@@ -543,10 +586,12 @@ ConvertReturnCommand(Widget w,
  *	Returns: none.
  */
 
+/* ARGSUSED */
 static void
-CommandDone(Widget widget, 
-	    Atom *selection, 
-	    Atom *target)
+CommandDone(widget, selection, target)
+Widget widget;
+Atom *selection;
+Atom *target;    
 {
     /* Keep the toolkit from automaticaly freeing the selection value */
 }
@@ -561,10 +606,11 @@ CommandDone(Widget widget,
  */
 
 static void
-SendFailure(Widget w, 
-	    Atom sel, 
-	    ResIdent ident, 
-	    char *str) 
+SendFailure(w, sel, ident, str) 
+Widget w;
+Atom sel;
+ResIdent ident;
+char * str;
 {
     _LesstifEditResResetStream(&globals.stream);
     _LesstifEditResPutString8(&globals.stream, str);
@@ -580,9 +626,10 @@ SendFailure(Widget w,
  */
 
 static XtPointer
-BuildReturnPacket(ResIdent ident, 
-		  EditresCommand command, 
-		  ProtocolStream *stream)
+BuildReturnPacket(ident, command, stream)
+ResIdent ident;
+EditresCommand command;
+ProtocolStream * stream;
 {
     long old_alloc, old_size;
     unsigned char * old_current;
@@ -607,7 +654,7 @@ BuildReturnPacket(ResIdent ident,
     
     _LesstifEditResPut8(stream, ident);
     _LesstifEditResPut8(stream, (unsigned char) command);
-    _LesstifEditResPut32(stream, (unsigned long) old_size);
+    _LesstifEditResPut32(stream, old_size);
 
     stream->alloc = old_alloc;
     stream->current = old_current;
@@ -627,11 +674,12 @@ BuildReturnPacket(ResIdent ident,
  */
 
 static void
-SendCommand(Widget w, 
-	    Atom sel, 
-	    ResIdent ident, 
-	    EditresCommand command, 
-	    ProtocolStream *stream)
+SendCommand(w, sel, ident, command, stream)
+Widget w;
+Atom sel;
+ResIdent ident;
+EditresCommand command;
+ProtocolStream * stream;
 {
     BuildReturnPacket(ident, command, stream);
     globals.command_stream = stream;	
@@ -663,10 +711,9 @@ SendCommand(Widget w,
  */
 
 static int
-FindChildren(Widget parent, 
-	     Widget **children, 
-	     Boolean normal, 
-	     Boolean popup)
+FindChildren(parent, children, normal, popup)
+Widget parent, **children;
+Boolean normal, popup;
 {
     CompositeWidget cw = (CompositeWidget) parent;
     int i, num_children, current = 0;
@@ -706,9 +753,8 @@ FindChildren(Widget parent,
  */
 
 static Boolean
-IsChild(Widget top, 
-	Widget parent, 
-	Widget child)
+IsChild(top, parent, child)
+Widget top, parent, child;
 {
     int i, num_children;
     Widget * children;
@@ -737,8 +783,9 @@ IsChild(Widget top,
  */
 
 static char * 
-VerifyWidget(Widget w, 
-	     WidgetInfo *info)
+VerifyWidget(w, info)
+Widget w;
+WidgetInfo *info;
 {
     Widget top;
 
@@ -782,9 +829,10 @@ VerifyWidget(Widget w,
  */
 
 static char *
-DoSetValues(Widget w, 
-	    EditresEvent *event, 
-	    ProtocolStream *stream)
+DoSetValues(w, event, stream)
+Widget w;
+EditresEvent * event;
+ProtocolStream * stream;
 {
     char * str;
     register unsigned i;
@@ -824,13 +872,11 @@ DoSetValues(Widget w,
  *	Returns: none.
  */
 
+/* ARGSUSED */
 static void
-HandleToolkitErrors(String name, 
-		    String type, 
-		    String class, 
-		    String msg, 
-		    String *params, 
-		    Cardinal *num_params)
+HandleToolkitErrors(name, type, class, msg, params, num_params)
+String name, type, class, msg, *params;
+Cardinal * num_params;
 {
     SVErrorInfo * info = &globals.error_info;	
     char buf[BUFSIZ];
@@ -872,11 +918,12 @@ HandleToolkitErrors(String name,
  */
 
 static void
-ExecuteSetValues(Widget w, 
-		 SetValuesEvent *sv_event, 
-		 WidgetInfo *entry, 
-		 ProtocolStream *stream, 
-		 unsigned short *count)
+ExecuteSetValues(w, sv_event, entry, stream, count)
+Widget w;
+SetValuesEvent * sv_event;
+WidgetInfo * entry;
+ProtocolStream * stream;
+unsigned short * count;
 {
     XtErrorMsgHandler old;
     
@@ -913,10 +960,14 @@ ExecuteSetValues(Widget w,
  *	Returns: NULL
  */
 
+#define TOOLKIT_TYPE ("Xt")
+
+/* ARGSUSED */
 static char * 
-DumpWidgets(Widget w, 
-	    EditresEvent *event, 
-	    ProtocolStream *stream)
+DumpWidgets(w, event, stream)
+Widget w;
+EditresEvent * event;		/* UNUSED */
+ProtocolStream * stream;
 {
     unsigned short count = 0;
         
@@ -931,6 +982,12 @@ DumpWidgets(Widget w,
 
     DumpChildren(w, stream, &count);
 
+    /*
+     * write out toolkit type (Xt, of course...). 
+     */
+
+    _LesstifEditResPutString8(stream, TOOLKIT_TYPE);
+
     /*
      * Overwrite the first 2 bytes with the real count.
      */
@@ -956,8 +1013,8 @@ DumpWidgets(Widget w,
  * go that far.  Then, we test whether it is an applicationShellWidget
  * class by looking for an explicit class name.  Seems pretty safe.
  */
-static Bool 
-isApplicationShell(Widget w)
+static Bool isApplicationShell(w)
+    Widget w;
 {
     register WidgetClass c;
 
@@ -971,9 +1028,10 @@ isApplicationShell(Widget w)
 }
 
 static void
-DumpChildren(Widget w, 
-	     ProtocolStream *stream, 
-	     unsigned short *count)
+DumpChildren(w, stream, count)
+Widget w;
+ProtocolStream * stream;
+unsigned short *count;
 {
     int i, num_children;
     Widget *children;
@@ -1029,9 +1087,10 @@ DumpChildren(Widget w,
  */
 
 static char *
-DoGetGeometry(Widget w, 
-	      EditresEvent *event, 
-	      ProtocolStream *stream)
+DoGetGeometry(w, event, stream)
+Widget w;
+EditresEvent * event;
+ProtocolStream * stream;
 {
     unsigned i;
     char * str;
@@ -1065,8 +1124,9 @@ DoGetGeometry(Widget w,
  */
 
 static void
-ExecuteGetGeometry(Widget w, 
-		   ProtocolStream *stream)
+ExecuteGetGeometry(w, stream)
+Widget w;
+ProtocolStream * stream;
 {
     int i;
     Boolean mapped_when_man;
@@ -1140,9 +1200,9 @@ ExecuteGetGeometry(Widget w,
  */
 
 static Boolean
-PositionInChild(Widget child, 
-		int x, 
-		int y)
+PositionInChild(child, x, y)
+Widget child;
+int x, y;
 {
     Arg args[6];
     Cardinal num;
@@ -1195,9 +1255,9 @@ PositionInChild(Widget child,
  */
 
 static Widget 
-_FindChild(Widget parent, 
-	   int x, 
-	   int y)
+_FindChild(parent, x, y)
+Widget parent;
+int x, y;
 {
     Widget * children;
     int i = FindChildren(parent, &children, TRUE, FALSE);
@@ -1227,9 +1287,10 @@ _FindChild(Widget parent,
  */
 
 static char *
-DoFindChild(Widget w, 
-	    EditresEvent *event, 
-	    ProtocolStream *stream)
+DoFindChild(w, event, stream)
+Widget w;
+EditresEvent * event;
+ProtocolStream * stream;
 {
     char * str;
     Widget parent, child;
@@ -1266,9 +1327,10 @@ DoFindChild(Widget w,
  */
 
 static char *
-DoGetResources(Widget w, 
-	       EditresEvent *event, 
-	       ProtocolStream *stream)
+DoGetResources(w, event, stream)
+Widget w;
+EditresEvent * event;
+ProtocolStream * stream;
 {
     unsigned int i;
     char * str;
@@ -1303,8 +1365,9 @@ DoGetResources(Widget w,
  */
 
 static void
-ExecuteGetResources(Widget w, 
-		    ProtocolStream *stream)
+ExecuteGetResources(w, stream)
+Widget w;
+ProtocolStream * stream;
 {
     XtResourceList norm_list, cons_list;
     Cardinal num_norm, num_cons;
@@ -1350,6 +1413,37 @@ ExecuteGetResources(Widget w,
     }
 }
 
+/*
+ *	Function Name: DumpValues
+ *	Description: Returns resource values to the resource editor.
+ *	Arguments: event - the event that caused this action.
+ *                 stream - the protocol stream to add.
+ *	Returns: NULL
+ */
+
+static char*
+DumpValues(w, event, stream)	/* ARGSUSED */
+Widget w;
+EditresEvent* event; 
+ProtocolStream* stream;
+{
+  Arg warg[1];
+  String res_value = NULL;
+  GetValuesEvent * gv_event = (GetValuesEvent *)event; 
+
+  /* put the count in the stream. */
+
+  _LesstifEditResPut16(stream, (unsigned int) 1); 
+
+  /* get the resource of the widget asked for by the */
+  /* resource editor and insert it into the stream */
+  XtSetArg(warg[0], gv_event->name, &res_value);
+  /*_XtGetStringValues(gv_event->widgets[0].ids[0], warg, 1);*/
+  if (!res_value) res_value = "NoValue";
+  _LesstifEditResPutString8(stream, res_value);
+  return(NULL);
+}
+
 /************************************************************
  *
  * Code for inserting values into the protocol stream.
@@ -1365,8 +1459,9 @@ ExecuteGetResources(Widget w,
  */
 
 static void
-InsertWidget(ProtocolStream *stream, 
-	     Widget w)
+InsertWidget(stream, w)
+ProtocolStream * stream;
+Widget w;
 {
     Widget temp;
     unsigned long * widget_list;
@@ -1407,8 +1502,9 @@ InsertWidget(ProtocolStream *stream,
  */
 
 void
-_LesstifEditResPutString8(ProtocolStream *stream, 
-			  char *str)
+_LesstifEditResPutString8(stream, str)
+ProtocolStream * stream;
+char * str;
 {
     int i, len = strlen(str);
 
@@ -1425,8 +1521,9 @@ _LesstifEditResPutString8(ProtocolStream *stream,
  */
 
 void
-_LesstifEditResPut8(ProtocolStream *stream, 
-		    unsigned int value)
+_LesstifEditResPut8(stream, value)
+ProtocolStream * stream;
+unsigned int value;
 {
     unsigned char temp;
 
@@ -1452,8 +1549,9 @@ _LesstifEditResPut8(ProtocolStream *stream,
  */
 
 void
-_LesstifEditResPut16(ProtocolStream *stream, 
-		     unsigned short value)
+_LesstifEditResPut16(stream, value)
+ProtocolStream * stream;
+unsigned int value;
 {
     _LesstifEditResPut8(stream, (value >> XER_NBBY) & BYTE_MASK);
     _LesstifEditResPut8(stream, value & BYTE_MASK);
@@ -1467,8 +1565,9 @@ _LesstifEditResPut16(ProtocolStream *stream,
  */
 
 void
-_LesstifEditResPut32(ProtocolStream *stream, 
-		     unsigned long value)
+_LesstifEditResPut32(stream, value)
+ProtocolStream * stream;
+unsigned long value;
 {
     int i;
 
@@ -1484,8 +1583,9 @@ _LesstifEditResPut32(ProtocolStream *stream,
  */
 
 void
-_LesstifEditResPutWidgetInfo(ProtocolStream *stream, 
-			     WidgetInfo *info)
+_LesstifEditResPutWidgetInfo(stream, info)
+ProtocolStream * stream;
+WidgetInfo * info;
 {
     unsigned int i;
 
@@ -1507,7 +1607,8 @@ _LesstifEditResPutWidgetInfo(ProtocolStream *stream,
  */
 
 void
-_LesstifEditResResetStream(ProtocolStream *stream)
+_LesstifEditResResetStream(stream)
+ProtocolStream * stream;
 {
     stream->current = stream->top;
     stream->size = 0;
@@ -1538,8 +1639,9 @@ _LesstifEditResResetStream(ProtocolStream *stream)
  */
 
 Boolean
-_LesstifEditResGet8(ProtocolStream *stream, 
-		    unsigned char *val)
+_LesstifEditResGet8(stream, val)
+ProtocolStream * stream;
+unsigned char * val;
 {
     if (stream->size < (stream->current - stream->top)) 
 	return(FALSE);
@@ -1557,8 +1659,9 @@ _LesstifEditResGet8(ProtocolStream *stream,
  */
 
 Boolean
-_LesstifEditResGet16(ProtocolStream *stream, 
-		     unsigned short *val)
+_LesstifEditResGet16(stream, val)
+ProtocolStream * stream;
+unsigned short * val;
 {
     unsigned char temp1, temp2;
 
@@ -1577,8 +1680,9 @@ _LesstifEditResGet16(ProtocolStream *stream,
  */
 
 Boolean
-_LesstifEditResGetSigned16(ProtocolStream *stream, 
-			   short *val)
+_LesstifEditResGetSigned16(stream, val)
+ProtocolStream * stream;
+short * val;
 {
     unsigned char temp1, temp2;
 
@@ -1605,8 +1709,9 @@ _LesstifEditResGetSigned16(ProtocolStream *stream,
  */
 
 Boolean
-_LesstifEditResGet32(ProtocolStream *stream, 
-		     unsigned long *val)
+_LesstifEditResGet32(stream, val)
+ProtocolStream * stream;
+unsigned long * val;
 {
     unsigned short temp1, temp2;
 
@@ -1626,8 +1731,9 @@ _LesstifEditResGet32(ProtocolStream *stream,
  */
 
 Boolean
-_LesstifEditResGetString8(ProtocolStream *stream, 
-			  char **str)
+_LesstifEditResGetString8(stream, str)
+ProtocolStream * stream;
+char ** str;
 {
     unsigned short len;
     register unsigned i;
@@ -1658,8 +1764,9 @@ _LesstifEditResGetString8(ProtocolStream *stream,
  */
 
 Boolean
-_LesstifEditResGetWidgetInfo(ProtocolStream *stream, 
-			     WidgetInfo *info)
+_LesstifEditResGetWidgetInfo(stream, info)
+ProtocolStream * stream;
+WidgetInfo * info;
 {
     unsigned int i;
 
@@ -1674,6 +1781,9 @@ _LesstifEditResGetWidgetInfo(ProtocolStream *stream,
 	    info->ids = NULL;
 	    return(FALSE);
 	}
+#if defined(LONG64) || defined(WORD64)
+	info->ids[i] |= globals.base_address;
+#endif
     }
     return(TRUE);
 }
@@ -1693,13 +1803,14 @@ _LesstifEditResGetWidgetInfo(ProtocolStream *stream,
  *	Returns: TRUE if conversion was sucessful.
  */
 
+/* ARGSUSED */
 static Boolean
-CvtStringToBlock(Display *dpy, 
-		 XrmValue *args, 
-		 Cardinal *num_args, 
-		 XrmValue *from_val, 
-		 XrmValue *to_val, 
-		 XtPointer *converter_data)
+CvtStringToBlock(dpy, args, num_args, from_val, to_val, converter_data)
+Display * dpy;
+XrmValue * args;
+Cardinal * num_args;
+XrmValue * from_val, * to_val;
+XtPointer * converter_data;
 {
     char ptr[BUFSIZ];
     static EditresBlock block;
@@ -1748,7 +1859,8 @@ CvtStringToBlock(Display *dpy,
  */
 
 static void
-LoadResources(Widget w)
+LoadResources(w)
+Widget w;
 {
     static XtResource resources[] = {
         {"editresBlock", "EditresBlock", XtREditresBlock, sizeof(EditresBlock),
@@ -1764,5 +1876,3 @@ LoadResources(Widget w)
     XtGetApplicationResources( w, (XtPointer) &globals, resources,
 			      XtNumber(resources), NULL, (Cardinal) 0);
 }
-
-    
