@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MainW.c,v 1.22 1996/04/24 19:25:23 miers Exp $
+ * $Id: MainW.c,v 1.10 1997/01/13 07:08:54 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,30 +22,39 @@
  *
  **/
 
-static char rcsid[] = "$Id: MainW.c,v 1.22 1996/04/24 19:25:23 miers Exp $";
+static char rcsid[] = "$Id: MainW.c,v 1.10 1997/01/13 07:08:54 u27113 Exp $";
+
+/*
+ * Be optimistic in GeometryManager
+ */
+#define	OPTIMISTIC
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/ScrollBarP.h>
 #include <Xm/BaseClassP.h>
 #include <Xm/ScrolledWP.h>
 #include <Xm/RowColumnP.h>
+#include <Xm/CommandP.h>
 #include <Xm/MainWP.h>
+#include <Xm/SeparatoG.h>
+#include <Xm/SeparatoGP.h>
 #include <X11/Xfuncs.h>
-#include <stdio.h>
-#include <Xm/XmI.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
 static void resize(Widget w);
 static void realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes);
 static XtGeometryResult QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static XtGeometryResult GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply);
 static void change_managed(Widget w);
 static void InsertChild(Widget w);
@@ -131,8 +140,8 @@ static XmBaseClassExtRec _XmMainWindowCoreClassExtRec = {
     /* compiled_ext_resources    */ NULL, /* FIXME */
     /* num_ext_resources         */ 0, /* FIXME */
     /* use_sub_resources         */ FALSE, /* FIXME */
-    /* widget_navigable          */ NULL, /* FIXME */
-    /* focus_change              */ NULL, /* FIXME */
+    /* widget_navigable          */ XmInheritWidgetNavigable,
+    /* focus_change              */ XmInheritFocusChange,
     /* wrapper_data              */ NULL
 };
 
@@ -234,30 +243,29 @@ class_part_initialize(WidgetClass widget_class)
 
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-#if 0
-    if (XtWidth(new) == 0)
-	XtWidth(new) = DEFAULT_WIDTH;
-    if (XtHeight(new) == 0)
-	XtHeight(new) = DEFAULT_HEIGHT;
-#endif
+/* Revert setting from XmScrolledWindow's initialize */
+    if (XtWidth(request) == 0)
+	XtWidth(new_w) = 0;
+    if (XtHeight(request) == 0)
+	XtHeight(new_w) = 0;
 
-    SW_MarginWidth(new) = MW_MarginWidth(new);
-    SW_MarginHeight(new) = MW_MarginHeight(new);
+    SW_MarginWidth(new_w) = MW_MarginWidth(new_w);
+    SW_MarginHeight(new_w) = MW_MarginHeight(new_w);
 
-    MW_Sep1(new) = XmCreateSeparatorGadget(new, "Separator1", args, *num_args);
-    MW_Sep2(new) = XmCreateSeparatorGadget(new, "Separator2", args, *num_args);
-    MW_Sep3(new) = XmCreateSeparatorGadget(new, "Separator3", args, *num_args);
-    if (MW_ShowSep(new) == True) {
-	XtManageChild(MW_Sep1(new));
-	XtManageChild(MW_Sep2(new));
-	XtManageChild(MW_Sep3(new));
+    MW_Sep1(new_w) = (XmSeparatorGadget)XmCreateSeparatorGadget(new_w, "Separator1", args, *num_args);
+    MW_Sep2(new_w) = (XmSeparatorGadget)XmCreateSeparatorGadget(new_w, "Separator2", args, *num_args);
+    MW_Sep3(new_w) = (XmSeparatorGadget)XmCreateSeparatorGadget(new_w, "Separator3", args, *num_args);
+    if (MW_ShowSep(new_w) == True) {
+	XtManageChild((Widget)MW_Sep1(new_w));
+	XtManageChild((Widget)MW_Sep2(new_w));
+	XtManageChild((Widget)MW_Sep3(new_w));
     }
 
-    _XmMainWindowLayout(new, True, NULL, False, NULL);
+    _XmMainWindowLayout(new_w, True, NULL, False, NULL);
 }
 
 
@@ -269,12 +277,12 @@ destroy(Widget w)
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     Boolean refresh = False;
-#define NE(x)	(x(old) != x(new))
+#define NE(x)	(x(old) != x(new_w))
 
     if (NE(MW_CommandWindow) ||
 	NE(MW_CommandLoc) ||
@@ -284,31 +292,31 @@ set_values(Widget old,
 
 
     if (NE(MW_ShowSep)) {
-	if (MW_ShowSep(new)) {
-	    XtManageChild(MW_Sep1(new));
-	    XtManageChild(MW_Sep2(new));
-	    XtManageChild(MW_Sep3(new));
+	if (MW_ShowSep(new_w)) {
+	    XtManageChild((Widget)MW_Sep1(new_w));
+	    XtManageChild((Widget)MW_Sep2(new_w));
+	    XtManageChild((Widget)MW_Sep3(new_w));
 	}
 	else {
-	    XtUnmanageChild(MW_Sep1(new));
-	    XtUnmanageChild(MW_Sep2(new));
-	    XtUnmanageChild(MW_Sep3(new));
+	    XtUnmanageChild((Widget)MW_Sep1(new_w));
+	    XtUnmanageChild((Widget)MW_Sep2(new_w));
+	    XtUnmanageChild((Widget)MW_Sep3(new_w));
 	}
 
 	refresh = True;
     }
 
     if (NE(MW_MarginHeight)) {
-	SW_MarginHeight(new) = MW_MarginHeight(new);
+	SW_MarginHeight(new_w) = MW_MarginHeight(new_w);
 	refresh = True;
     }
 
     if (NE(MW_MarginWidth)) {
-	SW_MarginWidth(new) = MW_MarginWidth(new);
+	SW_MarginWidth(new_w) = MW_MarginWidth(new_w);
 	refresh = True;
     }
 
-    _XmMainWindowLayout(new, True, NULL, False, NULL);
+    _XmMainWindowLayout(new_w, True, NULL, False, NULL);
 
     return refresh;
 }
@@ -324,13 +332,15 @@ resize(Widget w)
 static void
 realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
 {
-    XdbDebug(__FILE__, w, "Realize\n");
+    XdbDebug(__FILE__, w, "Realize ...\n");
 
 #define superclass (&xmManagerClassRec)
     (*superclass->core_class.realize)(w, value_mask, attributes);
 #undef superclass
 
     _XmMainWindowLayout(w, True, NULL, False, NULL);
+
+    XdbDebug(__FILE__, w, "Realize => size %d %d\n", XtWidth(w), XtHeight(w));
 }
 
 static XtGeometryResult 
@@ -343,7 +353,7 @@ QueryGeometry(Widget w,
 
     dbug = *preferred;
 
-    _XmMainWindowLayout(w, False, w, False, preferred);
+    _XmMainWindowLayout(w, False, w, True, preferred);
 
     if (intended)
 	*intended = *preferred;
@@ -373,7 +383,8 @@ GeometryManager(Widget w,
 {
     XtWidgetGeometry wants;
 
-    XdbDebug2(__FILE__, XtParent(w), w, "GeometryManager\n");
+    XdbDebug2(__FILE__, XtParent(w), w, "GeometryManager request %s\n",
+	XdbWidgetGeometry2String(desired));
 
 #define Wants(flag)     (desired->request_mode & flag)
 
@@ -381,20 +392,33 @@ GeometryManager(Widget w,
 	wants = *desired;
     else
 	bzero((void *)&wants, sizeof(XtWidgetGeometry));
+
+    if (Wants(XtCWQueryOnly)) {
+	XdbDebug(__FILE__, w, "Geometry Mgr Query Only Unimplemented\n");
+	return XtGeometryYes;
+    }
+
     /*
      * We control the XY of all children.  Width/Height is all I care about
      */
     wants.request_mode = CWWidth | CWHeight;
 
-    /* not chained */
+#ifdef	OPTIMISTIC
+    if (Wants(CWWidth))
+	XtWidth(w) = desired->width;
+    if (Wants(CWHeight))
+	XtHeight(w) = desired->height;
+#endif
+
+#if 1
+    /* This sucks - we're not supposed to be in test mode. */
     _XmMainWindowLayout(XtParent(w), False, w, True, &wants);
+#else
+    _XmMainWindowLayout(XtParent(w), True, w, False, &wants);
+#endif
 
     if (allowed)
 	*allowed = wants;
-    if (Wants(XtCWQueryOnly)) {
-	XdbDebug(__FILE__, w, "Geometry Mgr Query Only Unimplemented\n");
-	return XtGeometryYes;
-    }
 
     if (Wants(CWX) && Wants(CWY) && !(Wants(CWWidth) && Wants(CWHeight)))
 	return XtGeometryNo;
@@ -403,18 +427,11 @@ GeometryManager(Widget w,
 	Wants(CWHeight) && wants.height == desired->height)
 	return XtGeometryYes;
 
-#ifdef	notdef
-    if ((Wants(CWWidth) && wants.width < desired->width)
-     || (Wants(CWHeight) && wants.height < desired->height))
-	return XtGeometryAlmost;
-
-    return XtGeometryNo;
-#else
-/*
- * Try this : if we have a difference, just report "Almost"
- */
+    /*
+     * Try this : if we have a difference, just report "Almost"
+     */
     return XtGeometryAlmost;
-#endif
+#undef	Wants
 }
 
 static void
@@ -437,110 +454,497 @@ change_managed(Widget w)
  *	called from geometry_manager. That's why the child parameter is
  *	there !!
  */
+#define	VALID(w)	(w != NULL && XtIsManaged(w))
+
+static void
+_XmMainWLayoutAbove(Widget w, Dimension curw, Dimension curh)
+{
+    Position starty, endy;
+    Position	xx, yy;
+    Dimension	ww, hh;
+
+    XdbDebug(__FILE__, w, "LayoutAbove\n");
+    /*
+     * FIXME -- this layout algorithm is probably wrong.  I don't know which
+     * child should win if the window is smaller than the asked for geometry
+     */
+    starty = MW_MarginHeight(w);
+    endy = curh - MW_MarginHeight(w);
+    if (VALID(MW_MenuBar(w))) {
+	XdbDebug2(__FILE__, w, MW_MenuBar(w),
+		  "MenuBar geometry X %d Y %d W %d H %d\n",
+			   MW_MarginWidth(w), starty,
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_MenuBar(w)));
+	_XmConfigureObject(MW_MenuBar(w),
+			   MW_MarginWidth(w), starty,
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_MenuBar(w)),
+			   XtBorderWidth(MW_MenuBar(w)));
+	starty += XtHeight(MW_MenuBar(w));
+    }
+
+    if (VALID(MW_CommandWindow(w))) {
+	if (MW_ShowSep(w)) {
+	    if (MW_Sep1(w) && XtIsManaged((Widget)MW_Sep1(w))) {
+		XdbDebug2(__FILE__, w, (Widget)MW_Sep1(w),
+			  "Sep1 geometry X %d Y %d W %d Y %d\n",
+					MW_MarginWidth(w), starty,
+					curw - 2 * MW_MarginWidth(w),
+					XtHeight(MW_Sep1(w)));
+
+		_XmConfigureObject((Widget)MW_Sep1(w),
+				   MW_MarginWidth(w), starty,
+				   curw - 2 * MW_MarginWidth(w),
+				   XtHeight(MW_Sep1(w)),
+				   XtBorderWidth((Widget)MW_Sep1(w)));
+		starty += XtHeight(MW_Sep1(w));
+	    }
+	}
+	XdbDebug2(__FILE__, w, MW_CommandWindow(w),
+		  "CommandWindow geometry X %d Y %d W %d Y %d\n",
+			MW_MarginWidth(w), starty,
+			curw - 2 * MW_MarginWidth(w),
+			XtHeight(MW_CommandWindow(w)));
+
+	_XmConfigureObject(MW_CommandWindow(w),
+			   MW_MarginWidth(w), starty,
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_CommandWindow(w)),
+			   XtBorderWidth(MW_CommandWindow(w)));
+	starty += XtHeight(MW_CommandWindow(w));
+    }
+
+    if (VALID(MW_MessageWindow(w))) {
+	endy -= XtHeight(MW_MessageWindow(w));
+
+	XdbDebug2(__FILE__, w, MW_MessageWindow(w),
+		  "MessageWindow geometry X %d Y %d W %d Y %d\n",
+			MW_MarginWidth(w), endy,
+			curw - 2 * MW_MarginWidth(w),
+			endy);
+
+	_XmConfigureObject(MW_MessageWindow(w),
+			   MW_MarginWidth(w), endy,
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_MessageWindow(w)),
+			   XtBorderWidth(MW_MessageWindow(w)));
+	if (MW_ShowSep(w)) {
+	    if (VALID(MW_Sep3(w))) {
+		endy -= XtHeight((Widget)MW_Sep3(w));
+
+		XdbDebug2(__FILE__, w, (Widget)MW_Sep3(w),
+			  "Sep3 geometry X %d Y %d W %d Y %d\n",
+			MW_MarginWidth(w),
+			MW_MarginHeight(w),
+			curw - 2 * MW_MarginWidth(w),
+			XtHeight((Widget)MW_Sep3(w)));
+
+		_XmConfigureObject((Widget)MW_Sep3(w),
+				   MW_MarginWidth(w), endy,
+				   curw - 2 * MW_MarginWidth(w),
+				   XtHeight((Widget)MW_Sep3(w)),
+				   XtBorderWidth((Widget)MW_Sep3(w)));
+	    }
+	}
+    }
+
+    if (VALID(SW_WorkWindow(w))) {
+	if (MW_ShowSep(w)) {
+	    if (VALID(MW_Sep2(w))) {
+		XdbDebug2(__FILE__, w, (Widget)MW_Sep2(w),
+			  "Sep2 geometry X %d Y %d W %d Y %d\n",
+				MW_MarginWidth(w),
+				MW_MarginHeight(w),
+				curw - 2 * MW_MarginWidth(w),
+				XtHeight((Widget)MW_Sep2(w)));
+
+		_XmConfigureObject((Widget)MW_Sep2(w),
+				   MW_MarginWidth(w), starty,
+				   curw - 2 * MW_MarginWidth(w),
+				   XtHeight((Widget)MW_Sep2(w)),
+				   XtBorderWidth((Widget)MW_Sep2(w)));
+		starty += XtHeight(MW_Sep2(w));
+	    }
+	}
+
+	/* if we're treating this as a -ConfigureObject call, this
+	 * should always be True. */
+	SW_FromResize(w) = True;
+
+	xx = MW_MarginWidth(w);
+	yy = starty;
+	ww = curw - 2 * MW_MarginWidth(w);
+	hh = endy - starty;
+
+	if (curw < 2 * MW_MarginWidth(w)) {
+		XdbDebug2(__FILE__, w, SW_WorkWindow(w),
+			"curw %d < 2 * MarginWidth = %d\n",
+			curw, 2 * MW_MarginWidth(w));
+	}
+	if (endy < starty) {
+		XdbDebug2(__FILE__, w, SW_WorkWindow(w),
+			"endy %d < starty %d\n", endy, starty);
+	}
+
+	XdbDebug2(__FILE__, w, SW_WorkWindow(w),
+		  "WorkWindow geometry X %d Y %d W %d H %d\n",
+		  xx, yy, ww, hh);
+
+/* For lack of better treatment, don't call this with negative args */
+/* FIX ME */
+	if (curw >= 2 * MW_MarginWidth(w) && endy >= starty)
+		_XmScrolledWindowLayout(w, False, NULL, False, NULL,
+				xx, yy, ww, hh);
+
+	SW_FromResize(w) = False;
+    }
+}
+
+static void
+_XmMainWLayoutBelow(Widget w, Dimension curw, Dimension curh)
+{
+    Position	starty, endy;
+    Position	xx, yy;
+    Dimension	ww, hh;
+
+    XdbDebug(__FILE__, w, "LayoutBelow\n");
+    /*
+     * FIXME -- this layout algorithm is probably wrong.  I don't know which
+     * child should win if the window is smaller than the asked for geometry
+     */
+    starty = MW_MarginHeight(w);
+    endy = curh - MW_MarginHeight(w);
+    if (VALID(MW_MenuBar(w))) {
+	XdbDebug2(__FILE__, w, MW_MenuBar(w),
+		  "MenuBar geometry X %d Y %d W %d Y %d\n",
+			   MW_MarginWidth(w), MW_MarginHeight(w),
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_MenuBar(w)));
+
+	_XmConfigureObject(MW_MenuBar(w),
+			   MW_MarginWidth(w), starty,
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_MenuBar(w)),
+			   XtBorderWidth(MW_MenuBar(w)));
+	starty += XtHeight(MW_MenuBar(w));
+    }
+
+
+    if (VALID(MW_MessageWindow(w))) {
+	endy -= XtHeight(MW_MessageWindow(w));
+	XdbDebug2(__FILE__, w, MW_MessageWindow(w),
+		  "MessageWindow geometry X %d Y %d W %d Y %d\n",
+			MW_MarginWidth(w),
+			endy,
+			curw - 2 * MW_MarginWidth(w),
+			XtHeight(MW_MessageWindow(w)));
+
+	_XmConfigureObject(MW_MessageWindow(w),
+			   MW_MarginWidth(w), endy,
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_MessageWindow(w)),
+			   XtBorderWidth(MW_MessageWindow(w)));
+	if (MW_ShowSep(w)) {
+	    if (VALID(MW_Sep3(w))) {
+		endy -= XtHeight((Widget)MW_Sep3(w));
+
+		XdbDebug2(__FILE__, w, (Widget)MW_Sep3(w),
+			  "Sep3 geometry X %d Y %d W %d Y %d\n",
+				MW_MarginWidth(w),
+				endy,
+				curw - 2 * MW_MarginWidth(w),
+				XtHeight((Widget)MW_Sep3(w)));
+
+		_XmConfigureObject((Widget)MW_Sep3(w),
+				   MW_MarginWidth(w), endy,
+				   curw - 2 * MW_MarginWidth(w),
+				   XtHeight((Widget)MW_Sep3(w)),
+				   XtBorderWidth((Widget)MW_Sep3(w)));
+	    }
+	}
+    }
+
+    if (VALID(MW_CommandWindow(w))) {
+	endy -= XtHeight(MW_CommandWindow(w));
+	XdbDebug2(__FILE__, w, MW_CommandWindow(w),
+		  "CommandWindow geometry X %d Y %d W %d Y %d\n",
+			MW_MarginWidth(w),
+			endy,
+			curw - 2 * MW_MarginWidth(w),
+			XtHeight(MW_CommandWindow(w)));
+
+	_XmConfigureObject(MW_CommandWindow(w),
+			   MW_MarginWidth(w), endy,
+			   curw - 2 * MW_MarginWidth(w),
+			   XtHeight(MW_CommandWindow(w)),
+			   XtBorderWidth(MW_CommandWindow(w)));
+	if (MW_ShowSep(w)) {
+	    if (VALID(MW_Sep2(w))) {
+		endy -= XtHeight(MW_Sep2(w));
+		XdbDebug2(__FILE__, w, (Widget)MW_Sep2(w),
+			  "Sep2 geometry X %d Y %d W %d Y %d\n",
+				       MW_MarginWidth(w),
+				       endy,
+				       curw - 2 * MW_MarginWidth(w),
+				       XtHeight(MW_Sep2(w)));
+
+		_XmConfigureObject((Widget)MW_Sep2(w),
+				   MW_MarginWidth(w), endy,
+				   curw - 2 * MW_MarginWidth(w),
+				   XtHeight(MW_Sep2(w)),
+				   XtBorderWidth(MW_Sep2(w)));
+	    }
+	}
+    }
+
+    if (VALID(SW_WorkWindow(w))) {
+	if (MW_ShowSep(w)) {
+	    if (VALID(MW_Sep1(w))) {
+		XdbDebug2(__FILE__, w, (Widget)MW_Sep1(w),
+			  "Sep1 geometry X %d Y %d W %d Y %d\n",
+			  MW_MarginWidth(w),
+			  starty,
+			  curw - 2 * MW_MarginWidth(w),
+			  XtHeight((Widget)MW_Sep1(w)));
+
+		_XmConfigureObject((Widget)MW_Sep1(w),
+				   MW_MarginWidth(w), starty,
+				   curw - 2 * MW_MarginWidth(w),
+				   XtHeight(MW_Sep1(w)),
+				   XtBorderWidth(MW_Sep1(w)));
+		starty += XtHeight(MW_Sep2(w));
+	    }
+	}
+
+	/* if we're treating this as a -ConfigureObject call, this
+	 * should always be True. */
+	SW_FromResize(w) = True;
+
+	xx = MW_MarginWidth(w);
+	yy = starty;
+	ww = curw - 2 * MW_MarginWidth(w);
+	hh = endy - starty;
+
+	if (curw < 2 * MW_MarginWidth(w)) {
+		XdbDebug2(__FILE__, w, SW_WorkWindow(w),
+			"curw %d < 2 * MarginWidth = %d\n",
+			curw, 2 * MW_MarginWidth(w));
+	}
+	if (endy < starty) {
+		XdbDebug2(__FILE__, w, SW_WorkWindow(w),
+			"endy %d < starty %d\n", endy, starty);
+	}
+
+	XdbDebug2(__FILE__, w, SW_WorkWindow(w),
+		  "WorkWindow geometry X %d Y %d W %d H %d\n",
+		  xx, yy, ww, hh);
+
+/* For lack of better treatment, don't call this with negative args */
+/* FIX ME */
+	if (curw >= 2 * MW_MarginWidth(w) && endy >= starty)
+	    _XmScrolledWindowLayout(w, False, NULL, False, NULL,
+				xx, yy, ww, hh);
+
+	SW_FromResize(w) = False;
+    }
+}
+
+static void
+_XmMainWLayoutChildAnswer(Widget w, Widget child, XtWidgetGeometry *childgeom,
+			  Dimension curw, Dimension curh)
+{
+    Position starty, endy;
+
+    childgeom->request_mode &= (CWWidth|CWHeight);
+
+    XdbDebug(__FILE__, w, "Anwer child's geom request\n");
+
+    /*
+     * FIXME -- this layout algorithm is probably wrong.  I don't know which
+     * child should win if the window is smaller than the asked for geometry
+     */
+    starty = MW_MarginHeight(w);
+    endy = curh - MW_MarginHeight(w);
+    if (VALID(MW_MenuBar(w))) {
+	if (child == MW_MenuBar(w)) {
+	    childgeom->width = curw - 2 * MW_MarginWidth(w);
+	    childgeom->height = XtHeight(child);
+	    return;
+	}
+	starty += XtHeight(MW_MenuBar(w));
+    }
+
+    if (VALID(MW_CommandWindow(w))) {
+	if (MW_ShowSep(w) && MW_Sep1(w) && XtIsManaged(MW_Sep1(w))) {
+	    if (child == (Widget)MW_Sep1(w)) {
+		childgeom->width = curw - 2 * MW_MarginWidth(w);
+		childgeom->height = XtHeight(child);
+		return;
+	    }
+	    starty += XtHeight(MW_Sep1(w));
+	}
+	if (child == MW_CommandWindow(w)) {
+	    childgeom->width = curw - 2 * MW_MarginWidth(w);
+	    childgeom->height = XtHeight(child);
+	    return;
+	}
+	starty += XtHeight(MW_CommandWindow(w));
+    }
+
+    if (VALID(MW_MessageWindow(w))) {
+	endy -= XtHeight(MW_MessageWindow(w));
+	if (child == MW_MessageWindow(w)) {
+	    childgeom->width = curw - 2 * MW_MarginWidth(w);
+	    childgeom->height = XtHeight(child);
+	    return;
+	}
+	if (MW_ShowSep(w) && VALID(MW_Sep3(w))) {
+	    endy -= XtHeight(MW_Sep3(w));
+
+	    if (child == (Widget)MW_Sep3(w)) {
+		childgeom->width = curw - 2 * MW_MarginWidth(w);
+		childgeom->height = XtHeight(child);
+		return;
+	    }
+	}
+    }
+
+    if (VALID(SW_WorkWindow(w))) {
+	if (MW_ShowSep(w) && VALID(MW_Sep2(w))) {
+	    if (child == (Widget)MW_Sep2(w)) {
+		childgeom->width = curw - 2 * MW_MarginWidth(w);
+		childgeom->height = XtHeight(child);
+		return;
+	    }
+	    starty += XtHeight(MW_Sep2(w));
+	}
+	if (child == SW_WorkWindow(w)) {
+	    childgeom->width = curw - 2 * MW_MarginWidth(w);
+	    childgeom->height = endy - starty;
+	    return;
+	}
+    }
+}
+
 static void
 _XmMainWindowLayout(Widget w, Boolean ParentResize,
 		    Widget child, Boolean TestMode, XtWidgetGeometry *childgeom)
 {
-    Dimension curw, curh;
-    Position starty, endy;
-    XtWidgetGeometry wants, ask;
+    Dimension curw, curh, fixedh;
+    XtWidgetGeometry wants;
     XtGeometryResult result;
-    int			num = 0;;
+    XtWidgetGeometry request;
 
     curw = 2 * MW_MarginWidth(w);
     curh = 2 * MW_MarginHeight(w);
-
-#define	VALID(w)	(w != NULL && XtIsManaged(w))
+    fixedh = 2 * MW_MarginHeight(w);
 
     if (TestMode && (child == NULL || childgeom == NULL)) {
-	XdbDebug(__FILE__, w, "_XmMainWindowLayout TestMode with invalid parameters\n");
+	XdbDebug(__FILE__, w,
+		 "_XmMainWindowLayout TestMode with invalid parameters\n");
 	return;
     }
 
     if (TestMode)
 	ParentResize = False;
 
+    XdbDebug(__FILE__, w, "_XmMainWindowLayout(%s%s) Child %s geo %s\n",
+	ParentResize ? "ParentResize" : "",
+	TestMode ? (ParentResize ? " TestMode" : "TestMode") : "",
+	child ? XtName(child) : "(none)",
+	XdbWidgetGeometry2String(childgeom));
+
+#define	Wants(x)	((childgeom->request_mode & (x)) == (x))
+
     /* first, find out how our scrolledWindow portion wants to look,
      * as it will be the controlling factor for our width (if it's managed) */
     if (VALID(SW_WorkWindow(w))) {
-	_XmScrolledWindowLayout(w, False, w, True, &wants,
+	wants.request_mode = 0;
+	if (child == SW_WorkWindow(w) && Wants(CWWidth) && Wants(CWHeight)) {
+	    wants = *childgeom;
+	} else {
+	    _XmScrolledWindowLayout(w, False, w, True, &wants,
 				XtX(w), XtY(w), 0, 0);
-	curw = wants.width;
-	curh = wants.height;
+	}
+	if (child == SW_WorkWindow(w) && Wants(CWWidth))
+	    wants.width = childgeom->width;
+	if (child == SW_WorkWindow(w) && Wants(CWHeight))
+	    wants.height = childgeom->height;
 
-	XdbDebug2(__FILE__, w, SW_WorkWindow(w), "_XmMainWindowLayout: work window wants %d %d\n",
-		curw, curh);
+	if (wants.height == 0)
+	    wants.height = 1;
 
-	num++;
+	curw = wants.width + 2 * MW_MarginWidth(w);
+	curh = wants.height + 2 * MW_MarginHeight(w);
+
+	XdbDebug2(__FILE__, w, SW_WorkWindow(w),
+		  "_XmMainWindowLayout: work window wants %d %d\n",
+		  curw, curh);
+	if (MW_ShowSep(w) && VALID(MW_Sep2(w)))
+	    curh += XtHeight(MW_Sep2(w));
     }
 
     /* add the menubar.  FIXME -- should this affect our width? */
     if (VALID(MW_MenuBar(w))) {
-	ask.request_mode = CWWidth | CWHeight;
-	ask.width = ask.height = 0;
-	(void) XtQueryGeometry(MW_MenuBar(w), &ask, &wants);
-	if (wants.request_mode & CWWidth && wants.width > curw)
-	    curw = wants.width;
-	if (wants.request_mode & CWHeight)
-	    curh += wants.height;
-
-	num++;
+	if ((XtWidth(MW_MenuBar(w)) + 2 * MW_MarginWidth(w)) > curw)
+		curw = XtWidth(MW_MenuBar(w)) + 2 * MW_MarginWidth(w);
+	curh += XtHeight(MW_MenuBar(w));
+	fixedh += XtHeight(MW_MenuBar(w));
     }
 
     /* command window.  FIXME -- should this affect our width? */
     if (VALID(MW_CommandWindow(w))) {
-	ask.request_mode = CWWidth | CWHeight;
-	ask.width = ask.height = 0;
-	(void) XtQueryGeometry(MW_CommandWindow(w), &ask, &wants);
-	if (wants.request_mode & CWWidth && wants.width > curw)
-	    curw = wants.width;
-	if (wants.request_mode & CWHeight)
-	    curh += wants.height;
-
-	num++;
+	if ((XtWidth(MW_CommandWindow(w)) + 2 * MW_MarginWidth(w)) > curw)
+		curw = XtWidth(MW_CommandWindow(w)) + 2 * MW_MarginWidth(w);
+	curh += XtHeight(MW_CommandWindow(w));
+	fixedh += XtHeight(MW_CommandWindow(w));
+	if (MW_ShowSep(w) && VALID(MW_Sep1(w))) {
+	    curh += XtHeight(MW_Sep1(w));
+	    fixedh += XtHeight(MW_Sep1(w));
+	}
     }
 
     /* message box.  FIXME -- should this affect our width? */
     if (VALID(MW_MessageWindow(w))) {
-	ask.request_mode = CWWidth | CWHeight;
-	ask.width = ask.height = 0;
-	(void) XtQueryGeometry(MW_MessageWindow(w), &ask, &wants);
-	if (wants.request_mode & CWWidth && wants.width > curw)
-	    curw = wants.width;
-	if (wants.request_mode & CWHeight)
-	    curh += wants.height;
-
-	num++;
-    }
-
-    /* add any separators */
-    if (MW_ShowSep(w)) {
-	if (VALID(MW_Sep1(w))) {
-	    curh += XtHeight(MW_Sep1(w));
-	}
-	if (VALID(MW_Sep2(w))) {
-	    curh += XtHeight(MW_Sep2(w));
-	}
-	if (VALID(MW_Sep3(w))) {
+	if ((XtWidth(MW_MessageWindow(w)) + 2 * MW_MarginWidth(w)) > curw)
+		curw = XtWidth(MW_MessageWindow(w)) + 2 * MW_MarginWidth(w);
+	curh += XtHeight(MW_MessageWindow(w));
+	fixedh += XtHeight(MW_MessageWindow(w));
+	if (MW_ShowSep(w) && VALID(MW_Sep3(w))) {
 	    curh += XtHeight(MW_Sep3(w));
+	    fixedh += XtHeight(MW_Sep3(w));
 	}
     }
 
-    if (num)
-	curh += (num - 1) * MW_MarginHeight(w);		/* FIX ME ??? */
-
-    if (TestMode) {
-	/* We need to return the geometry of the child widget now */
-	/* FIX ME */
-
+    /*
+     * if we're coming from query_geometry
+     */
+    if (TestMode && w == child) {
+	childgeom->width = curw;
+	childgeom->height = curh;
 	return;
     }
 
-    /* OK, we know how big we want to be.  See if our parent will let us. */
+    /*
+     * OK, we know how big we want to be.  See if our parent will let us.
+     */
     if (ParentResize && SW_VisualPolicy(w) != XmCONSTANT) {
-	do {
-	    result = XtMakeResizeRequest(w, curw, curh, &curw, &curh);
-	} while (result == XtGeometryAlmost);
+	if (curw < XtWidth(w))
+	    curw = XtWidth(w);
+	if (curh < XtHeight(w))
+	    curh = XtHeight(w);
+	if (curw == 0)
+	    curw = 1;
+	if (curh == 0)
+	    curh = 1;
+
+	request.request_mode = (CWWidth|CWHeight);
+	request.width = curw;
+	request.height = curh;
+	result = _XmMakeGeometryRequest(w, &request);
+	curw = request.width;
+	curh = request.height;
 
 	if (result == XtGeometryYes) {
 	    XtWidth(w) = curw;
@@ -558,216 +962,20 @@ _XmMainWindowLayout(Widget w, Boolean ParentResize,
 	curh = XtHeight(w);
     }
 
-    /* FIXME -- this layout algorithm is probably wrong.  I don't know which
-     * child should win if the window is smaller than the asked for geometry */
-    starty = MW_MarginHeight(w);
-    endy = curh - MW_MarginHeight(w);
-    if (VALID(MW_MenuBar(w)) && child != MW_MenuBar(w) && !TestMode) {
-	XdbDebug2(__FILE__, w, MW_MenuBar(w), "Child geometry X %d Y %d W %d Y %d\n",
-			   MW_MarginWidth(w),		MW_MarginHeight(w),
-			   curw - 2 * MW_MarginWidth(w), XtHeight(MW_MenuBar(w)));
+    /* FIXME -- this is the most broken part.  I gotta do some
+     * M*tif testing to fix this */
+    if (curh < (fixedh + 4 + MW_MarginHeight(w) * 2))
+	curh = fixedh + 4 + MW_MarginHeight(w) * 2;
 
-	_XmConfigureObject(MW_MenuBar(w),
-			   MW_MarginWidth(w), MW_MarginHeight(w),
-			   curw - 2 * MW_MarginWidth(w),
-			   XtHeight(MW_MenuBar(w)),
-			   XtBorderWidth(MW_MenuBar(w)));
-	starty += XtHeight(MW_MenuBar(w));
+    if (TestMode) {
+	_XmMainWLayoutChildAnswer(w, child, childgeom, curw, curh);
+	return;
     }
 
-    if (! TestMode) {
-      if (MW_CommandLoc(w) == XmCOMMAND_ABOVE_WORKSPACE) {
-	if (VALID(MW_CommandWindow(w)) && child != MW_CommandWindow(w)) {
-	    if (MW_ShowSep(w)) {
-		if (MW_Sep1(w) && XtIsManaged(MW_Sep1(w))) {
-		    XdbDebug2(__FILE__, w, MW_Sep1(w), "Child geometry X %d Y %d W %d Y %d\n",
-					MW_MarginWidth(w),
-					starty,
-					curw - 2 * MW_MarginWidth(w),
-					XtHeight(MW_Sep1(w)));
-
-		    _XmConfigureObject(MW_Sep1(w),
-				       MW_MarginWidth(w), starty,
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep1(w)),
-				       XtBorderWidth(MW_Sep1(w)));
-		    starty += XtHeight(MW_Sep1(w));
-		}
-	    }
-	    XdbDebug2(__FILE__, w, MW_CommandWindow(w), "Child geometry X %d Y %d W %d Y %d\n",
-			MW_MarginWidth(w),
-			starty,
-			curw - 2 * MW_MarginWidth(w),
-			XtHeight(MW_CommandWindow(w)));
-
-	    _XmConfigureObject(MW_CommandWindow(w),
-			       MW_MarginWidth(w), starty,
-			       curw - 2 * MW_MarginWidth(w),
-			       XtHeight(MW_CommandWindow(w)),
-			       XtBorderWidth(MW_CommandWindow(w)));
-	    starty += XtHeight(MW_CommandWindow(w));
-	}
-
-	if (VALID(MW_MessageWindow(w)) && MW_MessageWindow(w) != child) {
-	    endy -= XtHeight(MW_MessageWindow(w));
-
-	    XdbDebug2(__FILE__, w, MW_MessageWindow(w), "Child geometry X %d Y %d W %d Y %d\n",
-			MW_MarginWidth(w),
-			curh - XtHeight(MW_MessageWindow(w)),
-			curw - 2 * MW_MarginWidth(w),
-			endy);
-
-	    _XmConfigureObject(MW_MessageWindow(w),
-			       MW_MarginWidth(w),
-			       curh - XtHeight(MW_MessageWindow(w)),
-			       curw - 2 * MW_MarginWidth(w), endy,
-			       XtBorderWidth(MW_MessageWindow(w)));
-	    if (MW_ShowSep(w)) {
-		if (VALID(MW_Sep3(w)) && child != MW_Sep3(w)) {
-		    endy -= XtHeight(MW_Sep3(w));
-
-		    XdbDebug2(__FILE__, w, MW_Sep3(w), "Child geometry X %d Y %d W %d Y %d\n",
-			MW_MarginWidth(w),
-			MW_MarginHeight(w),
-			curw - 2 * MW_MarginWidth(w),
-			XtHeight(MW_Sep3(w)));
-
-		    _XmConfigureObject(MW_Sep3(w),
-				       MW_MarginWidth(w), MW_MarginHeight(w),
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep3(w)),
-				       XtBorderWidth(MW_Sep3(w)));
-		}
-	    }
-	}
-
-	if (VALID(SW_WorkWindow(w)) && child != SW_WorkWindow(w)) {
-	    if (MW_ShowSep(w)) {
-		if (VALID(MW_Sep2(w)) && child != MW_Sep2(w)) {
-		    XdbDebug2(__FILE__, w, MW_Sep2(w), "Child geometry X %d Y %d W %d Y %d\n",
-				MW_MarginWidth(w),
-				MW_MarginHeight(w),
-				curw - 2 * MW_MarginWidth(w),
-				XtHeight(MW_Sep2(w)));
-
-		    _XmConfigureObject(MW_Sep2(w),
-				       MW_MarginWidth(w), MW_MarginHeight(w),
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep2(w)),
-				       XtBorderWidth(MW_Sep2(w)));
-		    starty += XtHeight(MW_Sep2(w));
-		}
-	    }
-
-	    /* FIXME -- this is the most broken part.  I gotta do some
-	     * M*tif testing to fix this */
-	    if (endy < starty + DEFAULT_HEIGHT)
-		endy = starty + DEFAULT_HEIGHT;
-
-	    /* if we're treating this as a -ConfigureObject call, this
-	     * should always be True. */
-	    SW_FromResize(w) = True;
-	    XdbDebug2(__FILE__, w, SW_WorkWindow(w), "Work Window geo X %d Y %d W %d H %d\n",
-				    0, starty, curw, endy - starty);
-	    _XmScrolledWindowLayout(w, False, NULL, False, NULL,
-				    0, starty, curw, endy - starty);
-	    SW_FromResize(w) = False;
-	}
-      } else {
-	if (child != MW_MessageWindow(w) && VALID(MW_MessageWindow(w))) {
-	    endy -= XtHeight(MW_MessageWindow(w));
-	    XdbDebug2(__FILE__, w, MW_MessageWindow(w), "Child geometry X %d Y %d W %d Y %d\n",
-			MW_MarginWidth(w),
-			endy,
-			curw - 2 * MW_MarginWidth(w),
-			XtHeight(MW_MessageWindow(w)));
-
-	    _XmConfigureObject(MW_MessageWindow(w),
-			       MW_MarginWidth(w), endy,
-			       curw - 2 * MW_MarginWidth(w),
-			       XtHeight(MW_MessageWindow(w)),
-			       XtBorderWidth(MW_MessageWindow(w)));
-	    if (MW_ShowSep(w)) {
-		if (child != MW_Sep3(w) && VALID(MW_Sep3(w))) {
-		    endy -= XtHeight(MW_Sep3(w));
-
-		    XdbDebug2(__FILE__, w, MW_Sep3(w), "Child geometry X %d Y %d W %d Y %d\n",
-				MW_MarginWidth(w),
-				endy,
-				curw - 2 * MW_MarginWidth(w),
-				XtHeight(MW_Sep3(w)));
-
-		    _XmConfigureObject(MW_Sep3(w),
-				       MW_MarginWidth(w), endy,
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep3(w)),
-				       XtBorderWidth(MW_Sep3(w)));
-		}
-	    }
-	}
-	if (child != MW_CommandWindow(w) && VALID(MW_CommandWindow(w))) {
-	    endy -= XtHeight(MW_CommandWindow(w));
-	    XdbDebug2(__FILE__, w, MW_CommandWindow(w), "Child geometry X %d Y %d W %d Y %d\n",
-			MW_MarginWidth(w),
-			endy,
-			curw - 2 * MW_MarginWidth(w),
-			XtHeight(MW_CommandWindow(w)));
-
-	    _XmConfigureObject(MW_CommandWindow(w),
-			       MW_MarginWidth(w), endy,
-			       curw - 2 * MW_MarginWidth(w),
-			       XtHeight(MW_CommandWindow(w)),
-			       XtBorderWidth(MW_CommandWindow(w)));
-	    if (MW_ShowSep(w)) {
-		if (child != MW_Sep2(w) && VALID(MW_Sep2(w))) {
-		    endy -= XtHeight(MW_Sep2(w));
-		    XdbDebug2(__FILE__, w, MW_Sep2(w), "Child geometry X %d Y %d W %d Y %d\n",
-				       MW_MarginWidth(w),
-				       endy,
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep2(w)));
-
-		    _XmConfigureObject(MW_Sep2(w),
-				       MW_MarginWidth(w), endy,
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep2(w)),
-				       XtBorderWidth(MW_Sep2(w)));
-		}
-	    }
-	}
-	if (child != SW_WorkWindow(w) && VALID(SW_WorkWindow(w))) {
-	    if (MW_ShowSep(w)) {
-		if (child != MW_Sep1(w) && VALID(MW_Sep1(w))) {
-		    XdbDebug2(__FILE__, w, MW_Sep1(w), "Child geometry X %d Y %d W %d Y %d\n",
-				       MW_MarginWidth(w),
-				       starty,
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep1(w)));
-
-		    _XmConfigureObject(MW_Sep1(w),
-				       MW_MarginWidth(w), starty,
-				       curw - 2 * MW_MarginWidth(w),
-				       XtHeight(MW_Sep1(w)),
-				       XtBorderWidth(MW_Sep1(w)));
-		    starty += XtHeight(MW_Sep2(w));
-		}
-	    }
-	    /* FIXME -- this is the most broken part.  I gotta do some
-	     * M*tif testing to fix this */
-	    if (endy < starty + DEFAULT_HEIGHT)
-		endy = starty + DEFAULT_HEIGHT;
-
-	    /* if we're treating this as a -ConfigureObject call, this
-	     * should always be True. */
-	    SW_FromResize(w) = True;
-	    XdbDebug2(__FILE__, w, SW_WorkWindow(w), "Work Window geo X %d Y %d W %d H %d\n",
-				    0, starty, curw, starty - endy);
-	    _XmScrolledWindowLayout(w, False, w, False, NULL,
-				    0, starty, curw, starty - endy);
-	    SW_FromResize(w) = False;
-	}
-      }
-    }
+    if (MW_CommandLoc(w) == XmCOMMAND_ABOVE_WORKSPACE)
+	_XmMainWLayoutAbove(w, curw, curh);
+    else
+	_XmMainWLayoutBelow(w, curw, curh);
 }
 
 Widget
@@ -786,19 +994,19 @@ XmCreateMainWindow(Widget parent,
 Widget 
 XmMainWindowSep1(Widget widget)
 {
-    return MW_Sep1(widget);
+    return (Widget)MW_Sep1(widget);
 }
 
 Widget 
 XmMainWindowSep2(Widget widget)
 {
-    return MW_Sep2(widget);
+    return (Widget)MW_Sep2(widget);
 }
 
 Widget 
 XmMainWindowSep3(Widget widget)
 {
-    return MW_Sep3(widget);
+    return (Widget)MW_Sep3(widget);
 }
 
 void 
@@ -823,50 +1031,44 @@ XmMainWindowSetAreas(Widget widget,
     P(work_region, " WorkRegion %s");
     XdbDebug0(__FILE__, widget, "\n");
 
-    XtVaSetValues(widget,
-		  XmNmenuBar, menu_bar,
-		  XmNcommandWindow, command_window,
-		  XmNworkWindow, work_region,
-		  XmNhorizontalScrollBar, horizontal_scrollbar,
-		  XmNverticalScrollBar, vertical_scrollbar,
-		  NULL);
+    MW_MenuBar(widget) = menu_bar;
+    MW_CommandWindow(widget) = command_window;
+    SW_WorkWindow(widget) = work_region;
+    SW_HSB(widget) = (XmScrollBarWidget)horizontal_scrollbar;
+    SW_VSB(widget) = (XmScrollBarWidget)vertical_scrollbar;
+
+    _XmMainWindowLayout(widget, True, NULL, False, NULL);
 }
 
 static void
 InsertChild(Widget w)
 {
-	Widget	p = XtParent(w);
-
+    Widget p = XtParent(w);
+    
+    if ((XmIsRowColumn(w) && RC_Type(w) == XmMENU_BAR) ||
+        XmIsCommand(w) || XmIsSeparator(w) || XmIsSeparatorGadget(w)) {
+#define	superclass	(&xmManagerClassRec)
+	(*superclass->composite_class.insert_child)(w);
+#undef	superclass
+	if (XmIsRowColumn(w) && RC_Type(w) == XmMENU_BAR) {
+	    XdbDebug2(__FILE__, p, w, "InsertChild : this is the menu bar\n");
+	    XtVaSetValues(p, XmNmenuBar, w, NULL);
+	}
+	else if (XmIsCommand(w)) {
+	    XdbDebug2(__FILE__, p, w, "InsertChild : this is the command window\n");
+	    XtVaSetValues(p, XmNcommandWindow, w, NULL);
+	}
+	else if (XmIsSeparator(w) || XmIsSeparatorGadget(w)) {
+	    XdbDebug2(__FILE__, p, w, "InsertChild : this is the seperator\n");
+	}
+    }
+    else {
+/* 
+ * Now we have a scrollbar or a workwindow - let scrolledwindow handle this
+ */
 #define	superclass	(&xmScrolledWindowClassRec)
 	(*superclass->composite_class.insert_child)(w);
 #undef	superclass
-
-	if (XmIsRowColumn(w) && RC_Type(w) == XmMENU_BAR) {
-		XtVaSetValues(p, XmNmenuBar, w, NULL);
-		XdbDebug2(__FILE__, p, w, "InsertChild : this is the menu bar\n");
-	} else if (XmIsScrollBar(w)) {
-	    XdbDebug2(__FILE__, p, w, "InsertChild (some scrollbar)\n");
-
-	    if (SW_HSB(p) == NULL && SW_VSB(p) != NULL) {
-		XtVaSetValues(p, XmNhorizontalScrollBar, w, NULL);
-		XtVaSetValues(w, XmNorientation, XmHORIZONTAL, NULL);
-	    } else if (SW_HSB(p) != NULL && SW_VSB(p) == NULL) {
-		XtVaSetValues(p, XmNverticalScrollBar, w, NULL);
-		XtVaSetValues(w, XmNorientation, XmVERTICAL, NULL);
-	    } else if (SW_HSB(p) == NULL && SCB_Orientation(w) == XmHORIZONTAL) {
-		XtVaSetValues(p, XmNhorizontalScrollBar, w, NULL);
-	    } else if (SW_VSB(p) == NULL && SCB_Orientation(w) == XmVERTICAL) {
-		XtVaSetValues(p, XmNverticalScrollBar, w, NULL);
-	    }
-	} else if (SW_WorkWindow(p) == NULL
-/*
- * These widget classes should NOT be the work window
- */
-		&& (! XmIsSeparator(w))
-		&& (! XmIsSeparatorGadget(w))
-		) {
-		XtVaSetValues(p, XmNworkWindow, w, NULL);
-		XdbDebug2(__FILE__, p, w, "InsertChild : this is the work window\n");
-	} else
-		XdbDebug2(__FILE__, p, w, "InsertChild\n");
+    } 
 }
+
