@@ -1,8 +1,8 @@
 /**
  *
- * $Id: MessageB.c,v 1.35 1996/04/30 00:35:54 miers Exp $
+ * $Id: MessageB.c,v 1.22 1997/01/11 02:19:48 miers Exp $
  *
- * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1996 Free Software Foundation, Inc.
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,40 +22,40 @@
  *
  **/
 
-static char rcsid[] = "$Id: MessageB.c,v 1.35 1996/04/30 00:35:54 miers Exp $";
+static char rcsid[] = "$Id: MessageB.c,v 1.22 1997/01/11 02:19:48 miers Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <X11/Xfuncs.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/XmP.h>
-#include <Xm/XmI.h>
 #include <Xm/MessageBP.h>
 #ifdef USE_WIDGETS
 #include <Xm/SeparatorP.h>
 #else
 #include <Xm/SeparatoGP.h>
 #endif
-
 #include <Xm/RowColumn.h>
 #include <Xm/RowColumnP.h>
-
 #include <Xm/DialogS.h>
-#include <stdio.h>
+#include <Xm/LabelGP.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
-static XtGeometryResult QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
-static XtGeometryResult geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void _XmMbButton(Widget w, XtPointer client, XtPointer call);
-static Boolean XmMbSetSubresources(Widget new, Widget request, ArgList args, Cardinal *nargs);
+static Boolean XmMbSetSubresources(Widget new_w, Widget request, ArgList args, Cardinal *nargs);
+static Pixmap _XmMessageBoxPixmap(XmMessageBoxWidget w, unsigned char dt);
 
 XmGeoMatrix _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref);
 Boolean _XmMessageBoxNoGeoRequest(XmGeoMatrix _geoSpec);
+static void DeleteChild(Widget w);
 
 /*
  * Resources for the Message Box class
@@ -116,14 +116,40 @@ static XtResource resources[] = {
 	XmNhelpLabelString, XmCXmString, XmRXmString,
 	sizeof(XmString), Offset(help_label_string),
 	XmRXmString, (XtPointer)NULL
-    }
+    },
+/* Initialize children widgets to NULL */
+    {
+	".", ".", XmRWidget,
+	sizeof(Widget), Offset(message_wid),
+	XmRImmediate, (XtPointer)NULL
+    },
+    {
+	".", ".", XmRWidget,
+	sizeof(Widget), Offset(symbol_wid),
+	XmRImmediate, (XtPointer)NULL
+    },
+    {
+	".", ".", XmRWidget,
+	sizeof(Widget), Offset(ok_button),
+	XmRImmediate, (XtPointer)NULL
+    },
+    {
+	".", ".", XmRWidget,
+	sizeof(Widget), Offset(help_button),
+	XmRImmediate, (XtPointer)NULL
+    },
+    {
+	".", ".", XmRWidget,
+	sizeof(Widget), Offset(separator),
+	XmRImmediate, (XtPointer)NULL
+    },
 };
 
 static XmSyntheticResource syn_resources[] = {
     {
 	XmNmessageString,
 	sizeof(XmString), Offset(message_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNsymbolPixmap,
@@ -133,17 +159,17 @@ static XmSyntheticResource syn_resources[] = {
     {
 	XmNokLabelString,
 	sizeof(XmString), Offset(ok_label_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNcancelLabelString,
 	sizeof(XmString), Offset(cancel_label_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNhelpLabelString,
 	sizeof(XmString), Offset(help_label_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     }
 };
 
@@ -168,8 +194,8 @@ static XmBaseClassExtRec _XmMessageBCoreClassExtRec = {
     /* compiled_ext_resources    */ NULL, /* FIXME */
     /* num_ext_resources         */ 0, /* FIXME */
     /* use_sub_resources         */ FALSE, /* FIXME */
-    /* widget_navigable          */ NULL, /* FIXME */
-    /* focus_change              */ NULL, /* FIXME */
+    /* widget_navigable          */ XmInheritWidgetNavigable,
+    /* focus_change              */ XmInheritFocusChange,
     /* wrapper_data              */ NULL
 };
 
@@ -199,7 +225,7 @@ XmMessageBoxClassRec xmMessageBoxClassRec = {
 	/* num_resources         */ XtNumber(resources),
 	/* xrm_class             */ NULLQUARK,
 	/* compress_motion       */ TRUE,
-	/* compress_exposure     */ XtExposeCompressMultiple,
+	/* compress_exposure     */ XtExposeCompressMaximal,
 	/* compress_enterleave   */ TRUE,
 	/* visible_interest      */ FALSE,
 	/* destroy               */ destroy,
@@ -213,16 +239,16 @@ XmMessageBoxClassRec xmMessageBoxClassRec = {
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ XtInheritTranslations,
-	/* query_geometry        */ QueryGeometry,
+	/* query_geometry        */ XtInheritQueryGeometry,
 	/* display_accelerator   */ NULL,
 	/* extension             */ (XtPointer)&_XmMessageBCoreClassExtRec
     },
     /* Composite class part */
     {
-	/* geometry manager */ geometry_manager, 
+	/* geometry manager */ XtInheritGeometryManager, 
         /* change_managed   */ XtInheritChangeManaged, 
         /* insert_child     */ XtInheritInsertChild,
-        /* delete_child     */ XtInheritDeleteChild,
+        /* delete_child     */	DeleteChild,
         /* extension        */ NULL,
     },
     /* Constraint class part */
@@ -274,34 +300,39 @@ class_part_initialize(WidgetClass widget_class)
 
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XmMessageBoxWidget mb = (XmMessageBoxWidget)new;
+    XmMessageBoxWidget mb = (XmMessageBoxWidget)new_w;
     Widget default_button;
-    Arg	al[2];
+    Arg	al[3];
 
     /* core dump avoidance */
     MB_Message(mb) = NULL;
     MB_Symbol(mb) = NULL;
 
-    MB_OKButton(mb) = _XmBB_CreateButtonG(new, NULL, "OK");
-    BB_CancelButton(mb) = _XmBB_CreateButtonG(new, NULL, "Cancel");
-    MB_HelpButton(mb) = _XmBB_CreateButtonG(new, NULL, "Help"); 
+    MB_OKButton(mb) = _XmBB_CreateButtonG(new_w, MB_OKLabelString(new_w), "OK");
+    BB_CancelButton(mb) = _XmBB_CreateButtonG(new_w, MB_CancelLabelString(new_w), "Cancel");
+    MB_HelpButton(mb) = _XmBB_CreateButtonG(new_w, MB_HelpLabelString(new_w), "Help"); 
 
 #ifdef USE_WIDGETS
-    MB_Separator(mb) = XmCreateSeparator(new, "Separator", NULL,0);
+    MB_Separator(mb) = XmCreateSeparator(new_w, "Separator", NULL,0);
 #else
-    MB_Separator(mb) = XmCreateSeparatorGadget(new, "Separator", NULL,0);
+    MB_Separator(mb) = XmCreateSeparatorGadget(new_w, "Separator", NULL,0);
 #endif
 
-    MB_Symbol(mb) = _XmBB_CreateLabelG(new, NULL, "Symbol");
-    XtSetArg(al[0], XmNalignment, MB_MessageAlignment(new));
-    XtSetValues(MB_Symbol(mb), al, 1);
+    MB_Symbol(mb) = _XmBB_CreateLabelG(new_w, NULL, "Symbol");
+    if (MB_SymbolPixmap(mb) == XmUNSPECIFIED_PIXMAP)
+	MB_SymbolPixmap(mb) = _XmMessageBoxPixmap(mb, MB_DialogType(mb));
 
-    MB_Message(mb) = _XmBB_CreateLabelG(new, NULL, "Message");
-    XtSetArg(al[0], XmNalignment, MB_MessageAlignment(new));
+    XtSetArg(al[0], XmNlabelPixmap, MB_SymbolPixmap(mb));
+    XtSetArg(al[1], XmNlabelType, XmPIXMAP);
+    XtSetValues(MB_Symbol(mb), al, 2);
+
+    MB_Message(mb) = _XmBB_CreateLabelG(new_w, MB_MessageString(new_w),
+					"Message");
+    XtSetArg(al[0], XmNalignment, MB_MessageAlignment(new_w));
     XtSetValues(MB_Message(mb), al, 1);
 
     /* Remove auto_unmanage */
@@ -316,11 +347,12 @@ initialize(Widget request,
     XtManageChild(BB_CancelButton(mb));
     XtManageChild(MB_HelpButton(mb));
     XtManageChild(MB_Separator(mb));
-    XtManageChild(MB_Symbol(mb));
-    XtManageChild(MB_Message(mb));
 
-/* Deal with all the subresources that SetValues does too */
-    (void) XmMbSetSubresources(new, NULL, args, num_args);
+    if (MB_DialogType(mb) != XmDIALOG_MESSAGE)
+	XtManageChild(MB_Symbol(mb));
+
+    if (MB_MessageString(mb))
+	XtManageChild(MB_Message(mb));
 
     switch (MB_DefaultType(mb))
     {
@@ -343,7 +375,7 @@ initialize(Widget request,
  * By putting this assignment last, we can check in insert_child
  * whether the newly inserted widget is one created in Initialize.
  */
-    XtVaSetValues(new, XmNdefaultButton, default_button, NULL);
+    XtVaSetValues(new_w, XmNdefaultButton, default_button, NULL);
 /* Don't add anything after this !! */
 }
 
@@ -352,6 +384,10 @@ destroy(Widget w)
 {
 }
 
+/*
+ * The ORIG_BITMAPS are created by the LessTif authors.
+ * The others (ORIG_BITMAPS == 0) look an awful lot like the ones used in M*tif.
+ */
 #define ORIG_BITMAPS	0
 
 /*
@@ -362,7 +398,7 @@ destroy(Widget w)
 #if ORIG_BITMAPS
 #define warning_width 9
 #define warning_height 22
-static char warning_bits[] = {
+static unsigned char warning_bits[] = {
    0x00, 0x00, 0x10, 0x00, 0x38, 0x00, 0x6c, 0x00, 0xee, 0x00, 0xc6, 0x00,
    0xc6, 0x00, 0x6c, 0x00, 0x6c, 0x00, 0x6c, 0x00, 0x7c, 0x00, 0x38, 0x00,
    0x38, 0x00, 0x38, 0x00, 0x38, 0x00, 0x10, 0x00, 0x10, 0x00, 0x00, 0x00,
@@ -381,7 +417,7 @@ static unsigned char warning_bits[] = {
 #if ORIG_BITMAPS
 #define question_width 23
 #define question_height 22
-  static char question_bits[] = {
+  static unsigned char question_bits[] = {
    0x00, 0x18, 0x00, 0x00, 0x18, 0x00, 0x7e, 0x18, 0x00, 0x78, 0x18, 0x0f,
    0x78, 0x98, 0x1f, 0x78, 0xd8, 0x39, 0x40, 0xd8, 0x30, 0x40, 0x18, 0x30,
    0x00, 0x18, 0x30, 0x00, 0x18, 0x38, 0x00, 0x18, 0x1c, 0x00, 0x18, 0x0e,
@@ -409,7 +445,7 @@ static unsigned char question_bits[] = {
 #if ORIG_BITMAPS
 #define info_width 8
 #define info_height 15
-static char info_bits[] = {
+static unsigned char info_bits[] = {
    0x08, 0x1c, 0x08, 0x00, 0x1e, 0x1e, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,
    0x18, 0xff, 0xff};
 #else
@@ -426,7 +462,7 @@ static unsigned char info_bits[] = {
 #if ORIG_BITMAPS
 #define error_width 16
 #define error_height 16
-static char error_bits[] = {
+static unsigned char error_bits[] = {
    0x00, 0x00, 0xc0, 0x01, 0xf0, 0x07, 0x1c, 0x1c, 0x0c, 0x18, 0x1e, 0x30,
    0x32, 0x20, 0x63, 0x60, 0xc3, 0x61, 0x83, 0x63, 0x02, 0x26, 0x06, 0x3c,
    0x0c, 0x18, 0x1c, 0x1c, 0xf0, 0x07, 0xc0, 0x01};
@@ -445,7 +481,7 @@ static unsigned char error_bits[] = {
 #if ORIG_BITMAPS
 #define working_width 16
 #define working_height 16
-static char working_bits[] = {
+static unsigned char working_bits[] = {
    0xff, 0xff, 0xf2, 0x4f, 0xf2, 0x4f, 0xe2, 0x47, 0xe2, 0x47, 0xc2, 0x43,
    0xc2, 0x43, 0x82, 0x41, 0x82, 0x41, 0xc2, 0x43, 0xc2, 0x43, 0xe2, 0x47,
    0xe2, 0x47, 0xf2, 0x4f, 0xf2, 0x4f, 0xff, 0xff};
@@ -529,26 +565,28 @@ _XmMessageBoxPixmap(XmMessageBoxWidget w, unsigned char dt)
 }
 
 static Boolean 
-XmMbSetSubresources(Widget new, Widget old, ArgList args, Cardinal *nargs)
+XmMbSetSubresources(Widget new_w, Widget old, ArgList args, Cardinal *nargs)
 {
-    Widget		x = NULL;
-    XmMessageBoxWidget	mb = (XmMessageBoxWidget)new;
+    XmMessageBoxWidget	mb = (XmMessageBoxWidget)new_w;
     XmMessageBoxWidget	ob = (XmMessageBoxWidget)old;
     Boolean		refresh_needed = False;
     
     /* Make sure you can compare */
     if (old == NULL) {
- 	x = old = (Widget) XtMalloc(sizeof(*ob));
- 	ob = (XmMessageBoxWidget)old;
-	
- 	bzero(ob, sizeof(*ob));
+ 	old = new_w;
+	ob = (XmMessageBoxWidget)old;
     }
 
-
     if (MB_Message(mb) &&
+	MB_MessageString(mb) != MB_MessageString(ob) &&
 	!XmStringCompare(MB_MessageString(mb), MB_MessageString(ob))) {
 	Arg	al[2];
 
+	if (MB_MessageString(mb) != NULL && !XtIsManaged(MB_Message(mb)))
+	    XtManageChild(MB_Message(mb));
+	else if (MB_MessageString(mb) == NULL && XtIsManaged(MB_Message(mb)))
+	    XtUnmanageChild(MB_Message(mb));
+	
 	XtSetArg(al[0], XmNlabelString, MB_MessageString(mb));
 	XtSetArg(al[1], XmNlabelType, XmSTRING);
 	XtSetValues(MB_Message(mb), al, 2);
@@ -560,90 +598,106 @@ XmMbSetSubresources(Widget new, Widget old, ArgList args, Cardinal *nargs)
 	MB_DialogType(ob) != MB_DialogType(mb)) {
 	Arg	al[2];
 
-	MB_SymbolPixmap(mb) = _XmMessageBoxPixmap(mb, MB_DialogType(mb));
+	if (MB_SymbolPixmap(mb) == XmUNSPECIFIED_PIXMAP)
+	    MB_SymbolPixmap(mb) = _XmMessageBoxPixmap(mb, MB_DialogType(mb));
 
 	XtSetArg(al[0], XmNlabelPixmap, MB_SymbolPixmap(mb));
 	XtSetArg(al[1], XmNlabelType, XmPIXMAP);
 	XtSetValues(MB_Symbol(mb), al, 2);
 
+	if (MB_DialogType(mb) == XmDIALOG_MESSAGE && XtIsManaged(MB_Symbol(mb)))
+	    XtUnmanageChild(MB_Symbol(mb));
+	else if (MB_DialogType(mb) != XmDIALOG_MESSAGE &&
+		 !XtIsManaged(MB_Symbol(mb)))
+	    XtUnmanageChild(MB_Symbol(mb));
+
 	refresh_needed = True;
     }
 
-    XtFree((XtPointer)x);
-    
+    if (MB_Symbol(mb) &&
+	MB_SymbolPixmap(ob) != MB_SymbolPixmap(mb)) {
+	Arg	al[2];
+
+	if (MB_SymbolPixmap(mb) == XmUNSPECIFIED_PIXMAP) {
+	    XtSetArg(al[1], XmNlabelType, XmSTRING);
+	    XtSetValues(MB_Symbol(mb), al, 1);
+	}
+	else {
+	    XtSetArg(al[0], XmNlabelPixmap, MB_SymbolPixmap(mb));
+	    XtSetArg(al[1], XmNlabelType, XmPIXMAP);
+	    XtSetValues(MB_Symbol(mb), al, 2);
+	}
+
+	refresh_needed = True;
+    }
+
+    if (MB_HelpButton(mb) &&
+	!XmStringCompare(MB_HelpLabelString(mb), MB_HelpLabelString(ob))) {
+	Arg	al[2];
+
+	XtSetArg(al[0], XmNlabelString, MB_HelpLabelString(mb));
+	XtSetArg(al[1], XmNlabelType, XmSTRING);
+	XtSetValues(MB_HelpButton(mb), al, 2);
+
+	refresh_needed = True;
+    }    
+
+    if (BB_CancelButton(mb) &&
+	!XmStringCompare(MB_CancelLabelString(mb), MB_CancelLabelString(ob))) {
+	Arg	al[2];
+
+	XtSetArg(al[0], XmNlabelString, MB_CancelLabelString(mb));
+	XtSetArg(al[1], XmNlabelType, XmSTRING);
+	XtSetValues(BB_CancelButton(mb), al, 2);
+
+	refresh_needed = True;
+    }    
+
+    if (MB_OKButton(mb) &&
+	!XmStringCompare(MB_OKLabelString(mb), MB_OKLabelString(ob))) {
+	Arg	al[2];
+
+	XtSetArg(al[0], XmNlabelString, MB_OKLabelString(mb));
+	XtSetArg(al[1], XmNlabelType, XmSTRING);
+	XtSetValues(MB_OKButton(mb), al, 2);
+
+	refresh_needed = True;
+    }    
+
     return refresh_needed;
 }
 
 static Boolean
 set_values(Widget old,
  	   Widget request,
- 	   Widget new,
+ 	   Widget new_w,
  	   ArgList args,
  	   Cardinal *num_args)
 {
     Boolean		refresh_needed;
-    XmMessageBoxWidget	mb = (XmMessageBoxWidget)new;
+    XmMessageBoxWidget	mb = (XmMessageBoxWidget)new_w;
     XmMessageBoxWidget	ob = (XmMessageBoxWidget)old;
     
+    BB_InSetValues(new_w) = True;
+
     /* Must do this call first */
-    refresh_needed = XmMbSetSubresources(new, old, args, num_args);
+    refresh_needed = XmMbSetSubresources(new_w, old, args, num_args);
     
     if (refresh_needed || XtWidth(ob) != XtWidth(mb) ||
 	XtHeight(ob) != XtHeight(mb))
 	refresh_needed = True;
     
+    BB_InSetValues(new_w) = False;
+
+    if (refresh_needed && (XtClass(new_w) == xmMessageBoxWidgetClass))
+    {
+	_XmBulletinBoardSizeUpdate(new_w);
+	return False;
+    }
+
     return refresh_needed;
 }
 
-static XtGeometryResult 
-QueryGeometry(Widget w, 
-	       XtWidgetGeometry *proposed, 
-	       XtWidgetGeometry *answer)
-{
-	XdbDebug(__FILE__, w, "QueryGeometry\n");
-
- 	if (proposed->request_mode == 0) {
-#if 0
-	    _XmMessageBoxLayout(mb);
-#endif
-	    answer->request_mode = CWWidth | CWHeight;
-	    answer->width = XtWidth(w);
-	    answer->height = XtHeight(w);
-	    
-	    return XtGeometryAlmost;		/* FIX ME */
- 	}
-	
-/* FIX ME */
-	if (answer) {
-		*answer = *proposed;
-		answer->request_mode &= CWWidth | CWHeight;
-	}
-
-	return XtGeometryYes;
-}
-
-static XtGeometryResult
-geometry_manager(Widget w,
-		 XtWidgetGeometry *request,
-		 XtWidgetGeometry *reply)
-{
-	XmMessageBoxWidget	mb = (XmMessageBoxWidget) XtParent(w);
-	XtGeometryResult	r = XtGeometryYes;
-	XtWidgetGeometry        R = *request;
-	if (reply) {
-		*reply = *request;
-		reply->request_mode &= CWWidth | CWHeight;
-	}
-
-#define	Wants(x)	(request->request_mode & x)
-
-	XdbDebug2(__FILE__, (Widget)mb, w, "GeometryManager [%d %d => %d %d, %s]\n",
-		R.width, R.height, reply->width, reply->height,
-		XdbGeometryResult2String(r));
-
-	return r;
-}
-
 XmGeoMatrix
 _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 {
@@ -651,16 +705,64 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     register XmGeoRowLayout	layoutPtr;
     register XmKidGeometry boxPtr;
     Cardinal numKids;
-    Boolean newRow;
-    int nrows;
+    Boolean newRow, tookChild;
+    int nrows, i, nextras;
+    Widget *extras;
+
+    numKids = MGR_NumChildren(_w);
+
+    nextras = 0;
+    extras = NULL;
+    for (i = 0; i < numKids; i++)
+    {
+	if (MGR_Children(_w)[i] != MB_Symbol(_w) &&
+	    MGR_Children(_w)[i] != MB_Message(_w) &&
+	    MGR_Children(_w)[i] != MB_Separator(_w) &&
+	    MGR_Children(_w)[i] != MB_OKButton(_w) &&
+	    MGR_Children(_w)[i] != MB_HelpButton(_w) &&
+	    MGR_Children(_w)[i] != BB_CancelButton(_w))
+	{
+	    nextras++;
+	}
+    }
+
+    if (nextras)
+	extras = (Widget *)XtMalloc(sizeof(Widget) * nextras);
+
+    nextras = 0;
+    for (i = 0; i < numKids; i++)
+    {
+	if (MGR_Children(_w)[i] != MB_Symbol(_w) &&
+	    MGR_Children(_w)[i] != MB_Message(_w) &&
+	    MGR_Children(_w)[i] != MB_Separator(_w) &&
+	    MGR_Children(_w)[i] != MB_OKButton(_w) &&
+	    MGR_Children(_w)[i] != MB_HelpButton(_w) &&
+	    MGR_Children(_w)[i] != BB_CancelButton(_w))
+	{
+	    extras[nextras] = MGR_Children(_w)[i];
+	    nextras++;
+	}
+    }
 
     nrows = 0;
 
     numKids = MGR_NumChildren(_w);
 
+    /* note the starting from one.  The zero'th child is the "work area" */
+    if (nextras > 0) {
+	for (i = 1; i < nextras; i++) {
+	    if (XmIsMenuBar(extras[i]) && XtIsManaged(extras[i]))
+		nrows++;
+	}
+	if (extras[0] && XtIsManaged(extras[0]))
+	    nrows++;
+    }
+
     if ((MB_Message(_w) && XtIsManaged(MB_Message(_w))) ||
 	(MB_Symbol(_w) && XtIsManaged(MB_Symbol(_w))))
 	nrows++;
+    else if (nextras)
+	nrows++;
 
     if (MB_Separator(_w) && XtIsManaged(MB_Separator(_w)))
 	nrows++;
@@ -669,6 +771,16 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
         (MB_OKButton(_w)     && XtIsManaged(MB_OKButton(_w))) ||
         (MB_HelpButton(_w)   && XtIsManaged(MB_HelpButton(_w))))
 	nrows++;
+    else {
+	for (i = i; i < nextras; i++) {
+	    if (extras[i] && XtIsManaged(extras[i]) &&
+		(XmIsPushButton(extras[i]) || XmIsPushButtonGadget(extras[i])))
+	    {
+		nrows++;
+		break;
+	    }
+	}
+    }
 
     geoSpec = _XmGeoMatrixAlloc(nrows, numKids, 0);
     geoSpec->composite = (Widget)_w;
@@ -682,13 +794,24 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     layoutPtr = &(geoSpec->layouts->row);
     boxPtr = geoSpec->boxes;
 
+    for (i = 1; i < nextras; i++) {
+	if (XmIsMenuBar(extras[i]) && XtIsManaged(extras[i]))
+	{
+	    layoutPtr->fix_up = _XmMenuBarFix;
+	    layoutPtr->space_above = 0;
+	    boxPtr += 2;
+	    layoutPtr++;
+	    break;
+	}
+    }
+
     newRow = False;
+    tookChild = False;
     if (MB_Symbol(_w) && XtIsManaged(MB_Symbol(_w)) &&
 	_XmGeoSetupKid(boxPtr, MB_Symbol(_w)))
     {
 	layoutPtr->even_width = 0;
 	layoutPtr->even_height = 1;
-	layoutPtr->stretch_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	layoutPtr->space_between = BB_MarginWidth(_w);
 	newRow = TRUE;
@@ -696,6 +819,16 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     }
     if (MB_Message(_w) && XtIsManaged(MB_Message(_w)) &&
 	_XmGeoSetupKid(boxPtr, MB_Message(_w)))
+    {
+	layoutPtr->even_width = 0;
+	layoutPtr->even_height = 1;
+	layoutPtr->space_above = BB_MarginHeight(_w);
+	layoutPtr->space_between = BB_MarginWidth(_w);
+	newRow = TRUE;
+	boxPtr++;
+    }
+    else if (newRow && nextras && XtIsManaged(extras[0]) &&
+	     _XmGeoSetupKid(boxPtr, extras[0]))
     {
 	layoutPtr->even_width = 0;
 	layoutPtr->stretch_height = 1;
@@ -703,6 +836,19 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	layoutPtr->space_between = BB_MarginWidth(_w);
 	newRow = TRUE;
+	tookChild = TRUE;
+	boxPtr++;
+    }
+
+    if (!newRow && nextras && XtIsManaged(extras[0]) &&
+	_XmGeoSetupKid(boxPtr, extras[0]))
+    {
+	layoutPtr->fill_mode = XmGEO_CENTER;
+	layoutPtr->even_width = 0;
+	layoutPtr->stretch_height = 1;
+	layoutPtr->space_above = BB_MarginHeight(_w);
+	newRow = TRUE;
+	tookChild = TRUE;
 	boxPtr++;
     }
 
@@ -712,6 +858,17 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	boxPtr++;
     }
 
+    if (!tookChild && nextras && XtIsManaged(extras[0]) &&
+	_XmGeoSetupKid(boxPtr, extras[0]))
+    {
+	layoutPtr->fill_mode = XmGEO_CENTER;
+	layoutPtr->even_width = 0;
+	layoutPtr->stretch_height = 1;
+	layoutPtr->space_above = BB_MarginHeight(_w);
+	boxPtr += 2;
+	layoutPtr++;
+    }
+
     if (MB_Separator(_w) && XtIsManaged(MB_Separator(_w)) &&
 	_XmGeoSetupKid(boxPtr, MB_Separator(_w)))
     {
@@ -725,16 +882,32 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     if (MB_OKButton(_w)     && XtIsManaged(MB_OKButton(_w)) &&
 	_XmGeoSetupKid(boxPtr++, MB_OKButton(_w))) {
 	layoutPtr->fill_mode = XmGEO_CENTER;
-	layoutPtr->fit_mode = XmGEO_AVERAGING;
+	layoutPtr->fit_mode = XmGEO_WRAP;
 	layoutPtr->even_width = 1;
 	layoutPtr->even_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
+    for (i = 1; i < nextras; i++)
+    {
+	if (extras[i] && XtIsManaged(extras[i]) &&
+	    (XmIsPushButton(extras[i]) || XmIsPushButtonGadget(extras[i])) &&
+	    _XmGeoSetupKid(boxPtr++, extras[i]))
+	{
+	    _XmBulletinBoardSetDefaultShadow(extras[i]);
+	    layoutPtr->fill_mode = XmGEO_CENTER;
+	    layoutPtr->fit_mode = XmGEO_WRAP;
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	    layoutPtr->space_above = BB_MarginHeight(_w);
+	    newRow = True;
+	}
+    }
+
     if (BB_CancelButton(_w) && XtIsManaged(BB_CancelButton(_w)) &&
 	_XmGeoSetupKid(boxPtr++, BB_CancelButton(_w))) {
 	layoutPtr->fill_mode = XmGEO_CENTER;
-	layoutPtr->fit_mode = XmGEO_AVERAGING;
+	layoutPtr->fit_mode = XmGEO_WRAP;
 	layoutPtr->even_width = 1;
 	layoutPtr->even_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
@@ -743,7 +916,7 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     if (MB_HelpButton(_w)   && XtIsManaged(MB_HelpButton(_w)) &&
 	_XmGeoSetupKid(boxPtr++, MB_HelpButton(_w))) {
 	layoutPtr->fill_mode = XmGEO_CENTER;
-	layoutPtr->fit_mode = XmGEO_AVERAGING;
+	layoutPtr->fit_mode = XmGEO_WRAP;
 	layoutPtr->even_width = 1;
 	layoutPtr->even_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
@@ -758,6 +931,8 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 
     layoutPtr->space_above = 0; /* BB_MarginHeight(_w); */
     layoutPtr->end = TRUE;
+    if (nextras)
+	XtFree((char *)extras);
     return(geoSpec);
 }
 
@@ -795,26 +970,22 @@ XmCreateErrorDialog(Widget parent,
     Arg		*al;
     int		nargs, i;
   
-    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
-    nargs=0;
-    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
-
     shell_name = _XmMakeDialogName(name);
 
-    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
-  
-    for (i=0; i<argcount; i++) {
-	al[i].name = arglist[i].name;
-	al[i].value = arglist[i].value;
+    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
+
+    nargs=0;
+    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
+    XtSetArg(al[nargs], XmNdialogType, XmDIALOG_ERROR); nargs++;
+
+    for (i = 0; i < argcount; i++) {
+	XtSetArg(al[nargs], arglist[i].name, arglist[i].value); nargs++;
     }
-    XtSetArg(al[i], XmNdialogType, XmDIALOG_ERROR); i++;
   
-#ifdef	DO_MANAGE_SHELLS
-    XtManageChild(shell);
-#endif
+    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
     XtFree(shell_name);
   
-    r =  XmCreateMessageBox(shell, name, al, i);
+    r =  XmCreateMessageBox(shell, name, al, nargs);
     XtFree((XtPointer)al);
   
     return r;
@@ -831,26 +1002,22 @@ XmCreateInformationDialog(Widget parent,
     Arg		*al;
     int		nargs, i;
     
-    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
-    nargs=0;
-    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
-   
     shell_name = _XmMakeDialogName(name);
 
-    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
-    
-    for (i=0; i<argcount; i++) {
-	al[i].name = arglist[i].name;
-	al[i].value = arglist[i].value;
+    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
+
+    nargs=0;
+    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
+    XtSetArg(al[nargs], XmNdialogType, XmDIALOG_INFORMATION); nargs++;
+
+    for (i = 0; i < argcount; i++) {
+	XtSetArg(al[nargs], arglist[i].name, arglist[i].value); nargs++;
     }
-    XtSetArg(al[i], XmNdialogType, XmDIALOG_INFORMATION); i++;
     
-#ifdef	DO_MANAGE_SHELLS
-    XtManageChild(shell);
-#endif
+    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
     XtFree(shell_name);
     
-    r =  XmCreateMessageBox(shell, name, al, i);
+    r =  XmCreateMessageBox(shell, name, al, nargs);
     XtFree((XtPointer)al);
     
     return r;
@@ -862,31 +1029,29 @@ XmCreateMessageDialog(Widget parent,
 		      Arg *arglist,
 		      Cardinal argcount)
 {
-    Widget	shell;
+    Widget	shell, r;
     char	*shell_name;
     Arg		*al;
     int		nargs, i;
 
-    al = (Arg *)XtCalloc(argcount+2, sizeof(Arg));
-    for (i=0; i<argcount; i++) {
-	al[i].name = arglist[i].name;
-	al[i].value = arglist[i].value;
-    }
-
-    nargs = argcount;
-    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
-
     shell_name = _XmMakeDialogName(name);
 
-    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
+    al = (Arg *)XtCalloc(argcount+2, sizeof(Arg));
 
-#ifdef	DO_MANAGE_SHELLS
-    XtManageChild(shell);
-#endif
+    nargs = 0;
+    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
+
+    for (i=0; i<argcount; i++) {
+	XtSetArg(al[nargs], arglist[i].name, arglist[i].value); nargs++;
+    }
+
+    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
     XtFree(shell_name);
+
+    r =  XmCreateMessageBox(shell, name, al, nargs);
     XtFree((XtPointer)al);
 
-    return XmCreateMessageBox(shell, name, arglist, argcount);
+    return r;
 }
 
 Widget
@@ -900,26 +1065,22 @@ XmCreateQuestionDialog(Widget parent,
     Arg		*al;
     int		nargs, i;
     
-    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
-    nargs=0;
-    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
-    
     shell_name = _XmMakeDialogName(name);
 
-    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
-    
-    for (i=0; i<argcount; i++) {
-	al[i].name = arglist[i].name;
-	al[i].value = arglist[i].value;
+    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
+
+    nargs=0;
+    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
+    XtSetArg(al[nargs], XmNdialogType, XmDIALOG_QUESTION); nargs++;
+
+    for (i = 0; i < argcount; i++) {
+	XtSetArg(al[nargs], arglist[i].name, arglist[i].value); nargs++;
     }
-    XtSetArg(al[i], XmNdialogType, XmDIALOG_QUESTION); i++;
-    
-#ifdef	DO_MANAGE_SHELLS
-    XtManageChild(shell);
-#endif
+
+    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
     XtFree(shell_name);
     
-    r =  XmCreateMessageBox(shell, name, al, i);
+    r =  XmCreateMessageBox(shell, name, al, nargs);
     XtFree((XtPointer)al);
 
     return r;
@@ -931,24 +1092,29 @@ XmCreateTemplateDialog(Widget parent,
 		       Arg *arglist,
 		       Cardinal argcount)
 {
-    Widget shell;
-    char *shell_name;
-    Arg		args[10];
-    int		nargs;
-    
-    nargs=0;
-    XtSetArg(args[nargs], XmNallowShellResize, True); nargs++;
-    
+    Widget	shell, r;
+    char	*shell_name;
+    Arg		*al;
+    int		nargs, i;
+ 
     shell_name = _XmMakeDialogName(name);
-    
-    shell = XmCreateDialogShell(parent, shell_name, args, nargs);
-    
-#ifdef	DO_MANAGE_SHELLS
-    XtManageChild(shell);
-#endif
+
+    al = (Arg *)XtCalloc(argcount+1, sizeof(Arg));
+
+    nargs=0;
+    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
+
+    for (i=0; i<argcount; i++) {
+	XtSetArg(al[nargs], arglist[i].name, arglist[i].value);
+	nargs++;
+    }
+
+    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
     XtFree(shell_name);
     
-    return XmCreateMessageBox(shell, name, arglist, argcount);
+    r =  XmCreateMessageBox(shell, name, al, nargs);
+    XtFree((char *)al);
+    return r;
 }
 
 Widget
@@ -961,29 +1127,25 @@ XmCreateWarningDialog(Widget parent,
     char		*shell_name;
     Arg		*al;
     int		nargs, i;
-    
-    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
-    nargs=0;
-    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
-    
+
     shell_name = _XmMakeDialogName(name);
 
-    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
-    
-    for (i=0; i<argcount; i++) {
-	al[i].name = arglist[i].name;
-	al[i].value = arglist[i].value;
+    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
+
+    nargs=0;
+    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
+    XtSetArg(al[nargs], XmNdialogType, XmDIALOG_WARNING); nargs++;
+ 
+    for (i = 0; i < argcount; i++) {
+	XtSetArg(al[nargs], arglist[i].name, arglist[i].value); nargs++;
     }
-    XtSetArg(al[i], XmNdialogType, XmDIALOG_WARNING); i++;
-    
-#ifdef	DO_MANAGE_SHELLS
-    XtManageChild(shell);
-#endif
+
+    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
     XtFree(shell_name);
-    
-    r =  XmCreateMessageBox(shell, name, al, i);
+ 
+    r =  XmCreateMessageBox(shell, name, al, nargs);
     XtFree((XtPointer)al);
-    
+ 
     return r;
 }
 
@@ -997,29 +1159,25 @@ XmCreateWorkingDialog(Widget parent,
     char		*shell_name;
     Arg		*al;
     int		nargs, i;
-    
-    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
-    nargs=0;
-    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
-    
+ 
     shell_name = _XmMakeDialogName(name);
 
-    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
-    
-    for (i=0; i<argcount; i++) {
-	al[i].name = arglist[i].name;
-	al[i].value = arglist[i].value;
+    al = (Arg *)XtCalloc(argcount + 2, sizeof(Arg));
+
+    nargs=0;
+    XtSetArg(al[nargs], XmNallowShellResize, True); nargs++;
+    XtSetArg(al[nargs], XmNdialogType, XmDIALOG_WORKING); nargs++;
+
+    for (i = 0; i < argcount; i++) {
+	XtSetArg(al[nargs], arglist[i].name, arglist[i].value); nargs++;
     }
-    XtSetArg(al[i], XmNdialogType, XmDIALOG_WORKING); i++;
-    
-#ifdef	DO_MANAGE_SHELLS
-    XtManageChild(shell);
-#endif
+
+    shell = XmCreateDialogShell(parent, shell_name, al, nargs);
     XtFree(shell_name);
     
-    r =  XmCreateMessageBox(shell, name, al, i);
+    r =  XmCreateMessageBox(shell, name, al, nargs);
     XtFree((XtPointer)al);
-    
+ 
     return r;
 }
 
@@ -1089,3 +1247,21 @@ static void _XmMbButton(Widget w, XtPointer client, XtPointer call)
 		XtCallCallbackList((Widget)sb, MGR_HelpCallbackList(sb), (XtPointer)&cbs);
 	}
 }
+
+static void
+DeleteChild(Widget w)
+{
+	Widget	mb = XtParent(w);
+
+	XdbDebug2(__FILE__, mb, w, "DeleteChild\n");
+
+#define superclass (&xmBulletinBoardClassRec)
+    (*superclass->composite_class.delete_child)(w);
+#undef superclass
+
+	if (w == MB_OKButton(mb))
+	    MB_OKButton(mb) = NULL;
+	if (w == MB_HelpButton(mb))
+	    MB_HelpButton(mb) = NULL;
+/* The Cancel Button is dealt with in BulletinBoard - that's where it belongs. */
+}
