@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Primitive.c,v 1.29 1996/04/20 05:55:40 toshok Exp $
+ * $Id: Primitive.c,v 1.8 1997/01/11 02:19:49 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,26 +22,27 @@
  *
  **/
 
-static char rcsid[] = "$Id: Primitive.c,v 1.29 1996/04/20 05:55:40 toshok Exp $";
+static char rcsid[] = "$Id: Primitive.c,v 1.8 1997/01/11 02:19:49 miers Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
 #include <Xm/BaseClassP.h>
 #include <Xm/PrimitiveP.h>
 #include <Xm/RepType.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/ManagerP.h>
-#include <stdio.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes);
 static void destroy(Widget w);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
-static void expose(Widget w, XEvent *event, Region region);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 
 static void focus_change(Widget w, XmFocusChange fevent);
 static XmNavigability widget_navigable(Widget w);
@@ -240,7 +241,13 @@ static XtActionsRec actions[] = {
     {"PrimitiveHelp",           _XmPrimitiveHelp},
     {"PrimitiveFocusIn",        _XmPrimitiveFocusIn},
     {"PrimitiveFocusOut",       _XmPrimitiveFocusOut},
-    {"PrimitiveUnmap",          _XmPrimitiveUnmap}
+    {"PrimitiveUnmap",          _XmPrimitiveUnmap},
+    {"unmap",          _XmPrimitiveUnmap},
+    {"Help",           _XmPrimitiveHelp},
+    {"enter",          _XmPrimitiveLeave},
+    {"leave",          _XmPrimitiveEnter},
+    {"PrevTabGroup",   _XmTraverseNextTabGroup},
+    {"NextTabGroup",   _XmTraversePrevTabGroup},
 };
 
 static XmBaseClassExtRec _XmPrimitiveCoreClassExtRec = {
@@ -302,7 +309,7 @@ XmPrimitiveClassRec xmPrimitiveClassRec = {
 	/* visible_interest      */ FALSE,
 	/* destroy               */ destroy,
 	/* resize                */ NULL,
-	/* expose                */ expose,
+	/* expose                */ NULL,
 	/* set_values            */ set_values,
 	/* set_values_hook       */ NULL,
 	/* set_values_almost     */ XtInheritSetValuesAlmost,
@@ -339,40 +346,22 @@ class_initialize()
 static void
 class_part_initialize(WidgetClass widget_class)
 {
-    XmPrimitiveWidgetClass pwc;
-    XmPrimitiveClassExt ext, *extptr;
+    XmPrimitiveWidgetClass pwc, swc;
 
     pwc = (XmPrimitiveWidgetClass)widget_class;
+    swc = (XmPrimitiveWidgetClass)widget_class->core_class.superclass;
 
     /* Handle the Primitive Widget class's inheritance stuff */
 
     if (pwc->primitive_class.border_highlight == XmInheritBorderHighlight)
-	pwc->primitive_class.border_highlight = primitive_border_highlight;
+	pwc->primitive_class.border_highlight =
+		swc->primitive_class.border_highlight;
     if (pwc->primitive_class.border_unhighlight == XmInheritBorderUnhighlight)
-	pwc->primitive_class.border_unhighlight = primitive_border_unhighlight;
+	pwc->primitive_class.border_unhighlight =
+		swc->primitive_class.border_unhighlight;
     if (pwc->primitive_class.translations == XtInheritTranslations)
-	pwc->primitive_class.translations = _XmPrimitive_defaultTranslations;
-
-    /* Make sure the extension record is in place if
-       not specified by a subclass */
-
-
-    extptr = (XmPrimitiveClassExt*)_XmGetClassExtensionPtr(
-		(XmGenericClassExt*)&(pwc->primitive_class.extension),
-		NULLQUARK);
-
-    if (extptr == NULL || *extptr == NULL)
-    {
-	ext = (XmPrimitiveClassExt) XtNew(XmPrimitiveClassExtRec);
-	if (ext != NULL)
-	{
-	    ext->next_extension = pwc->primitive_class.extension;
-	    ext->record_type = NULLQUARK;
-	    ext->version = XmPrimitiveClassExtVersion;
-	    ext->record_size = sizeof(XmPrimitiveClassExtRec);
-	    pwc->primitive_class.extension = (XtPointer) ext;
-	}
-    }
+	pwc->primitive_class.translations =
+		swc->primitive_class.translations;
 
     _XmBaseClassPartInitialize(widget_class);
     _XmFastSubclassInit(widget_class, XmPRIMITIVE_BIT);
@@ -452,8 +441,12 @@ CreateBottomShadowGC(XmPrimitiveWidget pw)
 	values.background = XtBackground(pw);
     }
 
-    mask |= GCLineWidth;
+    /* these GC's get used for shadow drawing, so set 'em up */
+    mask |= GCLineWidth | GCLineStyle | GCJoinStyle | GCCapStyle;
     values.line_width = 1;
+    values.line_style = LineSolid;
+    values.join_style = JoinMiter;
+    values.cap_style = CapButt;
 
     Prim_BottomShadowGC(pw) = XtGetGC((Widget)pw, mask, &values);    
 }
@@ -481,54 +474,56 @@ CreateTopShadowGC(XmPrimitiveWidget pw)
 	values.background = XtBackground(pw);
     }
 
-    mask |= GCLineWidth;
+    /* these GC's get used for shadow drawing, so set 'em up */
+    mask |= GCLineWidth | GCLineStyle | GCCapStyle | GCJoinStyle;
     values.line_width = 1;
+    values.line_style = LineSolid;
+    values.join_style = JoinMiter;
+    values.cap_style = CapButt;
 
     Prim_TopShadowGC(pw) = XtGetGC((Widget)pw, mask, &values);
 }
 
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XmPrimitiveWidget pw = (XmPrimitiveWidget) new;
-    XmPrimitiveWidgetClass pwc = (XmPrimitiveWidgetClass)XtClass(new);
+    XmPrimitiveWidget pw = (XmPrimitiveWidget) new_w;
+    XmPrimitiveWidgetClass pwc = (XmPrimitiveWidgetClass)XtClass(new_w);
 
     /* merge the traversal translations into the regular widget translations */
     if (pwc->primitive_class.translations)
-        XtOverrideTranslations(new, XtParseTranslationTable(pwc->primitive_class.translations));
+        XtOverrideTranslations(new_w, XtParseTranslationTable(pwc->primitive_class.translations));
     
-    Prim_HaveTraversal(new) = 
-	Prim_Highlighted(new) =
-        Prim_HighlightDrawn(new) = False;
+    Prim_HaveTraversal(new_w) = 
+	Prim_Highlighted(new_w) =
+        Prim_HighlightDrawn(new_w) = False;
 
     /* validate the entries in XmNnavigationType and XmNunitType 
-  
        Are these correct?  Or do we need to call a defaultProc? FIX ME*/
 
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRNavigationType),
 			     pw->primitive.navigation_type,
-			     new))
+			     new_w))
 	pw->primitive.navigation_type = XmNONE; 
 
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRUnitType),
 			     pw->primitive.unit_type,
-			     new))
+			     new_w))
 	pw->primitive.unit_type = XmPIXELS;
 
-    /* according to the OSF, primitives force this resource to 0 */
-    XtBorderWidth(pw) = 0;
+    _XmNavigInitialize(request, new_w, args, num_args);
 
     if (XtWidth(request) == (Dimension)0)
-	XtWidth(new) = (Prim_HighlightThickness(new) * 2
-			+ Prim_ShadowThickness(new) * 2);
+	XtWidth(new_w) = (Prim_HighlightThickness(new_w) * 2
+			+ Prim_ShadowThickness(new_w) * 2);
     if (XtHeight(request) == (Dimension)0)
-	XtHeight(new) = (Prim_HighlightThickness(new) * 2
-			 + Prim_ShadowThickness(new) * 2);
+	XtHeight(new_w) = (Prim_HighlightThickness(new_w) * 2
+			 + Prim_ShadowThickness(new_w) * 2);
 
-    _XmPrimitiveImportArgs(new, args, num_args);
+    _XmPrimitiveImportArgs(new_w, args, num_args);
 
     /* create all the initial GC's */
     CreateHighlightGC(pw);
@@ -545,7 +540,7 @@ destroy(Widget w)
     XtReleaseGC(w, Prim_BottomShadowGC(pw));
     XtReleaseGC(w, Prim_HighlightGC(pw));
 
-    /* deallocate traversal information held by this widget FIX ME */
+    _XmNavigDestroy(w);
 }
 
 static void 
@@ -555,6 +550,9 @@ realize(Widget w,
 {
    int i;
    
+   XdbDebug(__FILE__, w, "Realize wid before create %d ht %d bw %d\n",
+	    XtWidth(w), XtHeight(w), XtBorderWidth(w));
+
    XtCreateWindow(w, (unsigned int) InputOutput,
 		  (Visual *)CopyFromParent, *value_mask, attributes);
 
@@ -563,86 +561,77 @@ realize(Widget w,
    for (i=0; i < w->core.num_popups; i++)
      XtRealizeWidget(w->core.popup_list[i]);
 
-   XdbDebug(__FILE__, w, "Realize wid %d ht %d\n", XtWidth(w), XtHeight(w));
-}
-
-static void
-expose(Widget w, XEvent *event, Region region)
-{
+   XdbDebug(__FILE__, w, "Realize wid %d ht %d bw %d\n",
+	    XtWidth(w), XtHeight(w), XtBorderWidth(w));
 }
 
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     Boolean need_refresh = False;
 
-    /* validate the new values in XmNnavigationType and XmNunitType 
-  
-       Are these correct?  Or do we need to call a defaultProc? FIX ME*/
-
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRNavigationType),
-			     Prim_NavigationType(new),
-			     new))
-	Prim_NavigationType(new) = XmNONE; 
+			     Prim_NavigationType(new_w),
+			     new_w))
+	Prim_NavigationType(new_w) = Prim_NavigationType(old); 
 
-#if 0
-    /* FIXME FIXME FIXME -- this isn't working right */
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRUnitType),
-			     Prim_UnitType(new),
-			     new))
-	Prim_UnitType(new) = XmPIXELS;
-#endif
+			     Prim_UnitType(new_w),
+			     new_w))
+	Prim_UnitType(new_w) = Prim_UnitType(old);
 
-/* Debug print */
-    if (Prim_Foreground(old) != Prim_Foreground(new)) {
-	XdbDebug(__FILE__, new, "Primitive SetValues: Foreground changed\n");
+    need_refresh = _XmNavigSetValues(old, request, new_w, args, num_args);
+
+    /* Debug print */
+    if (Prim_Foreground(old) != Prim_Foreground(new_w)) {
+	XdbDebug(__FILE__, new_w, "Primitive SetValues: Foreground changed\n");
 	need_refresh = True;
     }
 
-    if (Prim_ShadowThickness(old) != Prim_ShadowThickness(new) 
-	|| Prim_HighlightThickness(old) != Prim_HighlightThickness(new)
-	|| Prim_Foreground(old) != Prim_Foreground(new))
+    if (Prim_ShadowThickness(old) != Prim_ShadowThickness(new_w) 
+	|| Prim_HighlightThickness(old) != Prim_HighlightThickness(new_w)
+	|| Prim_Foreground(old) != Prim_Foreground(new_w))
 	need_refresh = True;
 
-    if (Prim_HighlightColor(old) != Prim_HighlightColor(new)
-	|| Prim_HighlightPixmap(old) != Prim_HighlightPixmap(new)) 
+    if (Prim_HighlightColor(old) != Prim_HighlightColor(new_w)
+	|| Prim_HighlightPixmap(old) != Prim_HighlightPixmap(new_w)) 
     {
-	XtReleaseGC(new, Prim_HighlightGC(new));
-	CreateHighlightGC((XmPrimitiveWidget)new);
+	XtReleaseGC(new_w, Prim_HighlightGC(new_w));
+	CreateHighlightGC((XmPrimitiveWidget)new_w);
 	need_refresh = True;
     }
 
-    if (Prim_TopShadowColor(old) != Prim_TopShadowColor(new)
-	|| Prim_TopShadowPixmap(old) != Prim_TopShadowPixmap(new))
+    if (Prim_TopShadowColor(old) != Prim_TopShadowColor(new_w)
+	|| Prim_TopShadowPixmap(old) != Prim_TopShadowPixmap(new_w))
     {
-	XtReleaseGC(new, Prim_TopShadowGC(new));
-	CreateTopShadowGC((XmPrimitiveWidget)new);
+	XtReleaseGC(new_w, Prim_TopShadowGC(new_w));
+	CreateTopShadowGC((XmPrimitiveWidget)new_w);
 	need_refresh = True;
     }
 
-    if (Prim_BottomShadowColor(old) != Prim_BottomShadowColor(new)
-	|| Prim_BottomShadowPixmap(old) != Prim_BottomShadowPixmap(new))
+    if (Prim_BottomShadowColor(old) != Prim_BottomShadowColor(new_w)
+	|| Prim_BottomShadowPixmap(old) != Prim_BottomShadowPixmap(new_w))
     {
-	XtReleaseGC(new, Prim_BottomShadowGC(new));
-	CreateBottomShadowGC((XmPrimitiveWidget)new);
+	XtReleaseGC(new_w, Prim_BottomShadowGC(new_w));
+	CreateBottomShadowGC((XmPrimitiveWidget)new_w);
 	need_refresh = True;
     }
 
-    if ((Prim_HighlightDrawn(new) || ! XtSensitive(new))
-	&& _XmGetFocusPolicy(new) == XmPOINTER
+    if ((Prim_HighlightDrawn(new_w) || ! XtSensitive(new_w))
+	&& _XmGetFocusPolicy(new_w) == XmPOINTER
 	&& Prim_HighlightOnEnter(old) == True
-	&& Prim_HighlightOnEnter(new) == False)
+	&& Prim_HighlightOnEnter(new_w) == False)
     {
-	XmPrimitiveWidgetClass pwc = (XmPrimitiveWidgetClass)XtClass(new);
+	XmPrimitiveWidgetClass pwc = (XmPrimitiveWidgetClass)XtClass(new_w);
 	
-	(*pwc->primitive_class.border_unhighlight)(new);
+	(*pwc->primitive_class.border_unhighlight)(new_w);
     }
 
-    _XmPrimitiveImportArgs(new, args, num_args);
+    _XmPrimitiveImportArgs(new_w, args, num_args);
 
     return need_refresh;
 }
@@ -694,10 +683,13 @@ widget_navigable(Widget w)
 	     Prim_NavigationType(w) == XmEXCLUSIVE_TAB_GROUP ||
 	     Prim_NavigationType(w) == XmTAB_GROUP) && !_XmShellIsExclusive(w))
 	{
+	    XdbDebug(__FILE__, w, "WidgetNavigable => XmTAB_NAVIGABLE\n");
 	    return XmTAB_NAVIGABLE;
 	}
+	XdbDebug(__FILE__, w, "WidgetNavigable => XmCONTROL_NAVIGABLE\n");
 	return XmCONTROL_NAVIGABLE;
     }
+    XdbDebug(__FILE__, w, "WidgetNavigable => XmNOT_NAVIGABLE\n");
     return XmNOT_NAVIGABLE;
 }
 
@@ -709,13 +701,9 @@ primitive_border_unhighlight(Widget w)
 
     if (XmIsManager(XtParent(w)))
     {
-	XSetForeground(XtDisplay(w), Prim_HighlightGC(w), XmParentBackground(w));
-
-	_XmDrawHighlight(XtDisplay(w), XtWindow(w), Prim_HighlightGC(w),
+	_XmDrawHighlight(XtDisplay(w), XtWindow(w), XmParentBackgroundGC(w),
 			 0, 0, XtWidth(w), XtHeight(w),
 			 Prim_HighlightThickness(w), LineSolid);
-
-	XSetForeground(XtDisplay(w), Prim_HighlightGC(w), Prim_HighlightColor(w));
     }
     else
 	_XmClearBorder(XtDisplay(w),
@@ -759,6 +747,7 @@ _XmTraverseRight(Widget w,
 		 String *params, 
 		 Cardinal *num_params)
 {
+    XdbDebug(__FILE__, w, "XmProcessTraversal(TRAVERSE_RIGHT)\n");
     XmProcessTraversal(w, XmTRAVERSE_RIGHT);
 }
 
@@ -853,6 +842,19 @@ _XmPrimitiveHelp(Widget w,
 		 String *params, 
 		 Cardinal *num_params)
 {
+    Widget cur = w;
+    XmAnyCallbackStruct cbs;
+
+    cbs.reason = XmCR_HELP;
+    cbs.event = event;
+
+    while (cur != NULL) {
+	if (XtHasCallbacks(w, XmNhelpCallback) == XtCallbackHasSome) {
+	    XtCallCallbacks(w, XmNhelpCallback, (XtPointer)&cbs);
+	    return;
+	}
+	cur = XtParent(cur);
+    }
 }
 
 Boolean
@@ -867,21 +869,24 @@ _XmDifferentBackground(Widget w, Widget parent)
 
 /* Public Functions */
 
-Boolean
-XmWidgetGetBaselines(Widget w,
-		     Dimension **baselines,
-		     int *line_count)
-{
-    return False; /* FIX ME */
-}
-
-Boolean
-XmWidgetGetDisplayRect(Widget w,
-		       XRectangle *displayrect)
-{
-    return False; /* FIX ME */
-}
-
+/*
+ * The functions XmResolvePartOffsets and XmResolveAllPartOffsets were designed
+ * to be used by widgets subclassing OSF/Motif widgets. Therefore, LessTif must
+ * provide, but not (necessarily) use them.
+ *
+ * The big idea is that *binary* compatibility can be obtained for these subclasses,
+ * even if the Motif (LessTif) revisions add fields to the instance parts of the
+ * widgets.
+ *
+ * The functions below are used together with a macro called XmField which is in
+ * LESSTIF/include/Xm/XmP.h .
+ *
+ * All this is described very thoroughly in an article called "Achieving Binary
+ * Compatibility using the XmResolvePartOffset API" by Daniel DarDailler, who has
+ * been in the Motif team at OSF for as long as there was a Motif team at OSF :-)
+ *
+ * Look for the article at http://www.x.org/people/daniel/xmresolv
+ */
 void
 XmResolvePartOffsets(WidgetClass w_class, XmOffsetPtr *offset)
 {
@@ -892,5 +897,3 @@ XmResolveAllPartOffsets(WidgetClass w_class, XmOffsetPtr *offset,
 			XmOffsetPtr *constraint_offset)
 {
 }
-
-
