@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ResInd.c,v 1.23 1996/04/22 22:55:36 miers Exp $
+ * $Id: ResInd.c,v 1.9 1996/11/28 09:21:45 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -28,17 +28,20 @@
  *
  **/
 
-static char rcsid[] = "$Id: ResInd.c,v 1.23 1996/04/22 22:55:36 miers Exp $";
+static char rcsid[] = "$Id: ResInd.c,v 1.9 1996/11/28 09:21:45 u27113 Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
 #include <X11/Xfuncs.h>
 #include <Xm/ManagerP.h>
 #include <Xm/PrimitiveP.h>
 #include <Xm/GadgetP.h>
 #include <Xm/ExtObjectP.h>
-#include <stdio.h>
+#include <Xm/VendorSEP.h>
+
+#include <XmI/DebugUtil.h>
 
 #define MMsPerInch		(25.4)
 #define InchesPerMM		(1.0 / MMsPerInch)
@@ -47,13 +50,20 @@ static char rcsid[] = "$Id: ResInd.c,v 1.23 1996/04/22 22:55:36 miers Exp $";
 #define InchesPerPoint		(1.0 / 72.0)
 #define PointsPerInch		(72.0)
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #if XtSpecificationRelease >= 6
 extern void _XtCopyToArg(char *, XtArgVal **, int);
 #else
 extern void _XtCopyToArg(char *, XtArgVal *, int);
 #endif
 extern void _XtCopyFromArg(XtArgVal, char *, int);
-extern int _XmGetFontUnit(Screen *, unsigned char);
+
+#ifdef __cplusplus
+}
+#endif
 
 void 
 _XmFromHorizontalPixels(Widget widget, 
@@ -72,6 +82,15 @@ _XmFromHorizontalPixels(Widget widget,
     *value = converted_value;
 }
 
+/* Motif 2.* version of the above */
+void 
+XmeFromHorizontalPixels(Widget widget, 
+			int offset, 
+			XtArgVal *value)
+{
+	_XmFromHorizontalPixels(widget, offset, value);
+}
+
 void 
 _XmFromVerticalPixels(Widget widget, 
 		      int offset, 
@@ -89,6 +108,15 @@ _XmFromVerticalPixels(Widget widget,
     *value = converted_value;
 }
 
+/* Motif 2.* version of the above */
+void 
+XmeFromVerticalPixels(Widget widget, 
+		      int offset, 
+		      XtArgVal *value)
+{
+	_XmFromVerticalPixels(widget, offset, value);
+}
+
 XmImportOperator
 _XmToHorizontalPixels(Widget widget, 
 		      int offset, 
@@ -107,6 +135,15 @@ _XmToHorizontalPixels(Widget widget,
     return XmSYNTHETIC_LOAD;
 }
 
+/* Motif 2.* version of the above */
+XmImportOperator
+XmeToHorizontalPixels(Widget widget, 
+		      int offset, 
+		      XtArgVal *value)
+{
+	return _XmToHorizontalPixels(widget, offset, value);
+}
+
 XmImportOperator
 _XmToVerticalPixels(Widget widget, 
 		    int offset, 
@@ -125,11 +162,20 @@ _XmToVerticalPixels(Widget widget,
     return XmSYNTHETIC_LOAD;
 }
 
+/* Motif 2.* version of the above */
+XmImportOperator
+XmeToVerticalPixels(Widget widget, 
+		    int offset, 
+		    XtArgVal *value)
+{
+	return _XmToVerticalPixels(widget, offset, value);
+}
+
 /*
  * this function _does_ appear to combine all the synthetic resources an
  * object has with the synthetic resources the parent classes have.  Check
  * out testXm/misc/test6.c and testXm/misc/mot.compsyn.  Closer examinination
- * of mot.compsyn will show that duplicate synthetics are not found, event
+ * of mot.compsyn will show that duplicate synthetics are not found, even
  * when a subclass has the same synthetic as a superclass (e.g., Form
  * and BulletinBoard.  My assumption is that the subclass synthetic takes
  * precedence over the superclass, as that's the class initialization order,
@@ -232,7 +278,7 @@ _XmPrimitiveGetValuesHook(Widget w,
     XmPrimitiveWidgetClass pwc = (XmPrimitiveWidgetClass)XtClass(w);
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)XtClass(XtParent(w));
 
-    XdbDebug(__FILE__, NULL, "PrimitiveGetValuesHook\n");
+    XdbDebug(__FILE__, w, "PrimitiveGetValuesHook\n");
 
     if (!XmIsPrimitive(w))
 	return;
@@ -318,7 +364,7 @@ _XmGadgetGetValuesHook(Widget w,
     XmGadgetClass gwc = (XmGadgetClass)XtClass(w);
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)XtClass(XtParent(w));
 
-    XdbDebug(__FILE__, NULL, "GadgetGetValuesHook\n");
+    XdbDebug(__FILE__, w, "GadgetGetValuesHook\n");
 
     if (!XmIsGadget(w))
 	return;
@@ -403,7 +449,7 @@ _XmManagerGetValuesHook(Widget w,
     XmManagerWidgetClass pwc = (XmManagerWidgetClass)XtClass(w);
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)XtClass(XtParent(w));
 
-    XdbDebug(__FILE__, NULL, "ManagerGetValuesHook\n");
+    XdbDebug(__FILE__, w, "ManagerGetValuesHook\n");
 
     if (!XmIsManager(w))
 	return;
@@ -490,7 +536,7 @@ _XmExtGetValuesHook(Widget w,
     XrmQuark rq;
 
 
-    XdbDebug(__FILE__, NULL, "ExtObjectGetValuesHook\n");
+    XdbDebug(__FILE__, w, "ExtObjectGetValuesHook\n");
 
     if (!XmIsExtObject(w))
 	return;
@@ -533,7 +579,7 @@ _XmExtImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    XdbDebug(__FILE__, NULL, "ExtImportArgs\n");
+    XdbDebug(__FILE__, w, "ExtImportArgs\n");
 
     if (!XmIsExtObject(w))
 	return;
@@ -582,7 +628,7 @@ _XmPrimitiveImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    XdbDebug(__FILE__, NULL, "PrimitiveImportArgs\n");
+    XdbDebug(__FILE__, w, "PrimitiveImportArgs\n");
 
     if (!XmIsPrimitive(w))
 	return;
@@ -659,7 +705,7 @@ _XmGadgetImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    XdbDebug(__FILE__, NULL, "GadgetImportArgs\n");
+    XdbDebug(__FILE__, w, "GadgetImportArgs\n");
 
     if (!XmIsGadget(w))
 	return;
@@ -737,7 +783,7 @@ _XmGadgetImportSecondaryArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    XdbDebug(__FILE__, NULL, "GadgetImportSecondaryArgs\n");
+    XdbDebug(__FILE__, w, "GadgetImportSecondaryArgs\n");
 
     bce = (XmBaseClassExt *)_XmGetBaseClassExtPtr(XtClass(w), XmQmotif);
 
@@ -766,19 +812,10 @@ _XmGadgetImportSecondaryArgs(Widget w,
 		 * if this works.
 		 */
 		value = args[i].value;
-		if ((res->import_proc)(w, res->resource_offset, &value) ==
-		    XmSYNTHETIC_LOAD) {
-	    	    _XtCopyFromArg(value,
-				   (char *)w + res->resource_offset,
-				   res->resource_size);
-		}
-		else
-		    args[i].value = value;
+		(res->import_proc)(w, res->resource_offset, &value);
+		args[i].value = value;
 	    }
 	}
-	/*
-	 * Ext object will never have constraints.
-	 */
     }
 }
 
@@ -792,7 +829,7 @@ _XmManagerImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    XdbDebug(__FILE__, NULL, "ManagerImportArgs\n");
+    XdbDebug(__FILE__, w, "ManagerImportArgs\n");
 
     if (!XmIsManager(w))
 	return;
@@ -990,8 +1027,8 @@ _XmConvertUnits(Screen *screen,
 /*
  * MLM: In Motif, if this function is called with an uncompiled resource list,
  * it explodes messily.  However, if you look at the output of misc/test5.c
- * you will notice an interesting thing;  the XmNunitType resource, like shit,
- * floats to the top (check testXm/misc/test5.c + testXm/misc/mot.compres).
+ * you will notice an interesting thing;  the XmNunitType resource, floats
+ * to the top (check testXm/misc/test5.c + testXm/misc/mot.compres).
  * I can't find any other candidate for doing this other than
  * _XmSortResourceList; after passing in a LessTif widget (suitably
  * renamed) resource list into this function, I can verify that that does
@@ -1027,6 +1064,8 @@ _XmSortResourceList(XrmResource *list[],
 unsigned char
 _XmGetUnitType(Widget widget)
 {
+    Widget ve;
+
     /*
      * There used to be a call to XtGetValues here.  Unfortunately, with the
      * BaseClass stuff, that won't work, as that looks like a recursive call
@@ -1035,14 +1074,20 @@ _XmGetUnitType(Widget widget)
      * result that GetValuesLeafWrapper was called infinitely recursive.
      * Bad thing, that.
      */
-    if (XmIsGadget(widget))
-	return G_UnitType(widget);
+    if (XmIsVendorShell(widget)) {
+	if ((ve = _LtFindVendorExt(widget)) != NULL)
+	    return VSEP_UnitType(ve);
+	else
+	    return XmPIXELS;
+    }
+    if (XmIsManager(widget))
+	return MGR_UnitType(widget);
 
     if (XmIsPrimitive(widget))
         return Prim_UnitType(widget);
 
-    if (XmIsManager(widget))
-	return MGR_UnitType(widget);
+    if (XmIsGadget(widget))
+	return G_UnitType(widget);
 
     if (XmIsExtObject(widget))
 	return G_UnitType(ExtObj_LogicalParent(widget));
@@ -1050,6 +1095,57 @@ _XmGetUnitType(Widget widget)
     return XmPIXELS;
 }
 
+/*
+ * XmP.h says this is here
+ */
+void
+_XmUnitTypeDefault(Widget w,
+		   int offset,
+		   XrmValue *val)
+{
+    static unsigned char unit_type;
+    Widget ve;
+
+    unit_type = XmPIXELS;
+
+    if ((XmIsPrimitive(w) || XmIsGadget(w))) {
+	if (XmIsManager(XtParent(w)))
+	    unit_type = MGR_UnitType(XtParent(w));
+    }
+    else if (XmIsManager(w)) {
+	if (XmIsManager(XtParent(w)))
+	    unit_type = MGR_UnitType(XtParent(w));
+	else if (XmIsVendorShell(XtParent(w))) {
+	    if ((ve = _LtFindVendorExt(XtParent(w))) != NULL)
+		unit_type = VSEP_UnitType(ve);
+	}
+    }
+
+    val->addr = (XtPointer)&unit_type;
+}
+
+void
+_XmExportXmString(Widget w, int offset, XtArgVal *value)
+{
+    XmString str;
+
+    str = *(XmString *)(((char *)w)+offset);
+    if (str)
+	str = XmStringCopy(str);
+    *value = (XtArgVal)str;
+}
+
+void
+_XmExportString(Widget w, int offset, XtArgVal *value)
+{
+    String str;
+
+    str = *(String *)(((char *)w)+offset);
+    if (str)
+	str = XtNewString(str);
+    *value = (XtArgVal)str;
+}
+
 int
 XmConvertUnits(Widget widget,
 	       int orientation,
@@ -1109,37 +1205,12 @@ XmCvtFromVerticalPixels(Screen *screen, int from_val, int to_type)
     return _XmConvertUnits(screen, XmVERTICAL, XmPIXELS, from_val, to_type);
 }
 
-/*
- * XmP.h says this is here
- */
 void
-_XmUnitTypeDefault(Widget w,
-		   int offset,
-		   XrmValue *val)
-{
-    static unsigned char unit_type;
-
-    unit_type = XmPIXELS;
-
-    if (XmIsPrimitive(w) || XmIsGadget(w))
-    {
-	XmManagerWidget mw;
-	if (XmIsManager(XtParent(w)))
-	{
-	    mw = (XmManagerWidget)XtParent(w);
-
-	    unit_type = mw->manager.unit_type;
-	}
-    }
-}
-
-void
-XmSetFontUnits(void)
+XmSetFontUnits(Display *display, int h_value, int v_value)
 {
 }
 
 void
-XmSetFontUnit(void)
+XmSetFontUnit(Display *display, int value)
 {
 }
-
