@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Scale.c,v 1.25 1996/04/30 23:59:26 toshok Exp $
+ * $Id: Scale.c,v 1.8 1996/12/19 06:57:24 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,42 +22,57 @@
  *
  **/
 
-static char rcsid[] = "$Id: Scale.c,v 1.25 1996/04/30 23:59:26 toshok Exp $";
+static char rcsid[] = "$Id: Scale.c,v 1.8 1996/12/19 06:57:24 u27113 Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
+#include <Xm/XmP.h>
 #include <Xm/ScaleP.h>
-#include <Xm/DebugUtil.h>
-#include <Xm/LabelG.h>
+#include <Xm/LabelGP.h>
 #include <Xm/ScrollBarP.h>
 #include <Xm/BulletinBP.h>
 #include <Xm/MenuShellP.h>
 #include <Xm/VendorSP.h>
 #include <Xm/VendorSEP.h>
-#include <limits.h>
+
 #include <stdio.h>
-#include <Xm/XmI.h>
+#include <limits.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 #define CHILD_LAB	0
 #define CHILD_SB	1
 
+/* FIX ME/BUGS
+ */
+
 /*
  * these values come from behavior observation of M*tif.  Change at your own
  * risk
+ *
+ * Hmm, I've observed HSLIDERSIZE to be 30 (M*tif 1.2.2) -- PvH
  */
 #define SCB_MAX		1000000000
 #define SCB_MIN		0
-#define HSLIDERSIZE	40
+#define HSLIDERSIZE	/*40*/30
 #define VSLIDERSIZE	12
 #define MIN_SLIDE	4
 
+#define SB_TRANSVERSAL_SIZE	15
+#define SB_LONGITUDAL_SIZE	100
+
+#define _XmMax(a,b) ((a)>(b)?(a):(b))
+#define Scale_ShadowThickness(x) MGR_ShadowThickness(x)
+
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal * num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal * num_args);
 static void destroy(Widget w);
 static void realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal * num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal * num_args);
 static XtGeometryResult query_geometry(Widget w, XtWidgetGeometry * request, XtWidgetGeometry * reply);
 static XtGeometryResult geometry_manager(Widget w, XtWidgetGeometry * request, XtWidgetGeometry * reply);
 static void expose(Widget w, XEvent * event, Region region);
@@ -149,7 +164,7 @@ static XtResource resources[] =
     {
 	XmNhighlightThickness, XmCHighlightThickness, XmRHorizontalDimension,
 	sizeof(Dimension), Offset(highlight_thickness),
-	XtRImmediate, (XtPointer) 2
+	XmRImmediate, (XtPointer) 2
     },
     {
 	XmNhighlightOnEnter, XmCHighlightOnEnter, XmRBoolean,
@@ -164,7 +179,7 @@ static XtResource resources[] =
     {
 	XmNdragCallback, XmCCallback, XmRCallback,
 	sizeof(XtCallbackList), Offset(drag_callback),
-	XtRCallback, (XtPointer) NULL
+	XmRCallback, (XtPointer) NULL
     },
     {
 	XmNscaleMultiple, XmCScaleMultiple, XmRInt,
@@ -177,7 +192,7 @@ static XmSyntheticResource syn_resources[] = {
     {
 	XmNtitleString,
 	sizeof(XmString), Offset(title),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNscaleWidth,
@@ -221,8 +236,8 @@ static XmBaseClassExtRec _XmScaleCoreClassExtRec = {
     /* compiled_ext_resources    */ NULL, /* FIXME */
     /* num_ext_resources         */ 0, /* FIXME */
     /* use_sub_resources         */ FALSE, /* FIXME */
-    /* widget_navigable          */ NULL, /* FIXME */
-    /* focus_change              */ NULL, /* FIXME */
+    /* widget_navigable          */ XmInheritWidgetNavigable,
+    /* focus_change              */ XmInheritFocusChange,
     /* wrapper_data              */ NULL
 };
 
@@ -341,79 +356,95 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmSCALE_BIT);
 }
 
+static void
+CreateForegroundGC(Widget w)
+{
+    XtGCMask mask;
+    XGCValues values;
+
+    mask = GCForeground | GCBackground | GCFillStyle | GCFunction | GCFont |
+		GCSubwindowMode | GCGraphicsExposures | GCPlaneMask;
+    values.function = GXcopy;
+    values.plane_mask = -1;
+    values.subwindow_mode = ClipByChildren;
+    values.graphics_exposures = False;
+    values.foreground = MGR_Foreground(w);
+    values.background = XtBackground(w);
+    values.fill_style = FillSolid;
+    values.font = Scale_FontStruct(w)->fid;
+
+    Scale_ForegroundGC(w) = XtGetGC(w, mask, &values);
+}
+
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     Widget title;
     Widget sb;
-    Arg argl[12];
-    int argc, sm;
+    Arg argl[10];
+    int argc;
 
-    if (Scale_Value(new) == INT_MAX) {
-	if (Scale_Minimum(new) >= 0)
-	    Scale_Value(new) = Scale_Minimum(new);
+    if (Scale_Value(new_w) == INT_MAX) {
+	if (Scale_Minimum(new_w) >= 0)
+	    Scale_Value(new_w) = Scale_Minimum(new_w);
 	else 
-	    Scale_Value(new) = 0;
+	    Scale_Value(new_w) = 0;
     }
     
-    if (Scale_ScaleMultiple(new) == 0) {
-	Scale_ScaleMultiple(new) = (Scale_Maximum(new) - Scale_Minimum(new)) / 10;
+    if (Scale_ScaleMultiple(new_w) == 0) {
+	Scale_ScaleMultiple(new_w) = (Scale_Maximum(new_w) - Scale_Minimum(new_w)) / 10;
     }
 
-    Scale_LastValue(new) = Scale_Value(new);
-    Scale_ForegroundGC(new) = XtAllocateGC(new,
-					   0, 0,
-					   NULL,
-					   0, 0);
+    Scale_LastValue(new_w) = Scale_Value(new_w);
 
-    if (Scale_FontList(new) == NULL)
-	Scale_FontList(new) = _XmGetDefaultFontList(new, XmLABEL_FONTLIST);
+    if (Scale_FontList(new_w) == NULL)
+	Scale_FontList(new_w) = _XmGetDefaultFontList(new_w, XmLABEL_FONTLIST);
 
     /* MLM - FIXME -- Should the titleString get the FontList from the scale? */
     argc = 0;
-    XtSetArg(argl[argc], XmNlabelString, Scale_Title(new)); argc++;
+    if (Scale_Title(new_w) != NULL)
+	Scale_Title(new_w) = XmStringCopy(Scale_Title(new_w));
+    XtSetArg(argl[argc], XmNlabelString, Scale_Title(new_w)); argc++;
     XtSetArg(argl[argc], XmNalignment, XmALIGNMENT_BEGINNING); argc++;
-    title = XmCreateLabelGadget(new, "Title", argl, argc);
-    if (Scale_Title(new) != NULL)
+    title = XmCreateLabelGadget(new_w, "Title", argl, argc);
+    if (Scale_Title(new_w) != NULL)
 	XtManageChild(title);
 
-    Scale_SliderSize(new) = _XmScaleConvertWidthToSliderSize(new);
-
     argc = 0;
     XtSetArg(argl[argc], XmNshowArrows, FALSE); argc++;
     XtSetArg(argl[argc], XmNmaximum, SCB_MAX); argc++;
     XtSetArg(argl[argc], XmNminimum, SCB_MIN); argc++;
-    XtSetArg(argl[argc], XmNorientation, Scale_Orientation(new)); argc++;
-    XtSetArg(argl[argc], XmNprocessingDirection, Scale_ProcessingDirection(new)); argc++;
-    XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new)); argc++;
-    XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new)); argc++;
-    XtSetArg(argl[argc], XmNvalue, _XmScaleConvertScaleValueToSCBValue(new)); argc++;
-    sm = (int)((float)Scale_ScaleMultiple(new) /
-	       (float)(Scale_Maximum(new) - Scale_Minimum(new)) *
-	       (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(new)));
-    XtSetArg(argl[argc], XmNpageIncrement, sm); argc++;
-    XtSetArg(argl[argc], XmNhighlightThickness, Scale_HighlightThickness(new)); argc++;
-    XtSetArg(argl[argc], XmNhighlightOnEnter, Scale_HighlightOnEnter(new)); argc++;
-    XtSetArg(argl[argc], XmNsliderSize, Scale_SliderSize(new)); argc++;
-    sb = XmCreateScrollBar(new, "scrollBar", argl, argc);
+    XtSetArg(argl[argc], XmNorientation, Scale_Orientation(new_w)); argc++;
+    XtSetArg(argl[argc], XmNprocessingDirection, Scale_ProcessingDirection(new_w)); argc++;
+    XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new_w)); argc++;
+    XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new_w)); argc++;
+    XtSetArg(argl[argc], XmNshadowThickness, Scale_ShadowThickness(new_w)); argc++;
+    XtSetArg(argl[argc], XmNhighlightThickness, Scale_HighlightThickness(new_w)); argc++;
+    XtSetArg(argl[argc], XmNhighlightOnEnter, Scale_HighlightOnEnter(new_w)); argc++;
+    sb = XmCreateScrollBar(new_w, "Scrollbar", argl, argc);
     _XmSetEtchedSlider((XmScrollBarWidget)sb);
     XtAddCallback(sb, XmNdragCallback, _ScaleDrag, NULL);
-    if (Scale_Value(new) == INT_MAX)
-	Scale_Value(new) = Scale_Minimum(new);
+    if (Scale_Value(new_w) == INT_MAX)
+	Scale_Value(new_w) = Scale_Minimum(new_w);
     XtAddCallback(sb, XmNvalueChangedCallback, _ScaleValueChanged, NULL);
     XtManageChild(sb);
 
-    _XmFontListGetDefaultFont(Scale_FontList(new), &Scale_FontStruct(new));
+    _XmFontListGetDefaultFont(Scale_FontList(new_w), &Scale_FontStruct(new_w));
 
-    if (Scale_FontStruct(new) == NULL) {
-      Scale_FontList(new) = _XmFontListCreateDefault(XtDisplay(new));
-      _XmFontListGetDefaultFont(Scale_FontList(new), &Scale_FontStruct(new));
+    if (Scale_FontStruct(new_w) == NULL) {
+      Scale_FontList(new_w) = _XmFontListCreateDefault(XtDisplay(new_w));
+      _XmFontListGetDefaultFont(Scale_FontList(new_w), &Scale_FontStruct(new_w));
     }
 
-    _XmScaleLayout(new, True, NULL, False, NULL);
+    CreateForegroundGC(new_w);
+
+    /* M*tif doesn't do this */
+/*
+    _XmScaleLayout(new_w, True, NULL, False, NULL);
+*/
 }
 
 static void
@@ -428,63 +459,96 @@ destroy(Widget w)
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal * num_args)
 {
     Boolean		refresh = False;
-    Arg argl[11];
+    Arg argl[8];
     int argc;
     Widget sb, lab;
 
-    sb = ((XmScaleWidget)new)->composite.children[CHILD_SB];
-    lab = ((XmScaleWidget)new)->composite.children[CHILD_LAB];
+    sb = ((XmScaleWidget)new_w)->composite.children[CHILD_SB];
+    lab = ((XmScaleWidget)new_w)->composite.children[CHILD_LAB];
 
-    if (Scale_Title(new) != Scale_Title(old)) {
+    if (MGR_Foreground(new_w) != MGR_Foreground(old) ||
+	XtBackground(new_w) != XtBackground(old) ||
+	Scale_FontList(new_w) != Scale_FontList(old)) {
+	_XmFontListGetDefaultFont(Scale_FontList(new_w), &Scale_FontStruct(new_w));
+
+	if (Scale_FontStruct(new_w) == NULL) {
+	    Scale_FontList(new_w) = _XmFontListCreateDefault(XtDisplay(new_w));
+	    _XmFontListGetDefaultFont(Scale_FontList(new_w),
+				      &Scale_FontStruct(new_w));
+	}
+	XtReleaseGC(new_w, Scale_ForegroundGC(new_w));
+	CreateForegroundGC(new_w);
+	refresh = True;
+    }
+	
+    if (Scale_Title(new_w) != Scale_Title(old)) {
 	argc = 0;
-	XtSetArg(argl[argc], XmNlabelString, Scale_Title(new)); argc++;
+	if (Scale_Title(old))
+	    XmStringFree(Scale_Title(old));
+	Scale_Title(new_w) = XmStringCopy(Scale_Title(new_w));
+	XtSetArg(argl[argc], XmNlabelString, Scale_Title(new_w)); argc++;
 	XtSetValues(lab, argl, argc);
 	refresh = True;
     }
 
-    if (Scale_Maximum(new) != Scale_Maximum(old) ||
-        Scale_Minimum(new) != Scale_Minimum(old) ||
-        Scale_Orientation(new) != Scale_Orientation(old) ||
-        Scale_ProcessingDirection(new) != Scale_ProcessingDirection(old) ||
-        Scale_ScaleWidth(new) != Scale_ScaleWidth(old) ||
-        Scale_ScaleHeight(new) != Scale_ScaleHeight(old) ||
-        Scale_ScaleMultiple(new) != Scale_ScaleMultiple(old) ||
-        Scale_HighlightOnEnter(new) != Scale_HighlightOnEnter(old) ||
-        Scale_HighlightThickness(new) != Scale_HighlightThickness(old) ||
-	Scale_Value(new) != Scale_Value(old)) {
-	int sm;
-
-	Scale_SliderSize(new) = _XmScaleConvertWidthToSliderSize(new);
-
+/*
+ * This is taken out of the (large) if statement below,
+ * because somehow taking all of the values to scrollbar
+ * breaks scrollbar.
+ * Therefore we treat XmNvalue separately here.
+ * Yes this is a workaround in case you didn't know by now :-(
+ */
+    if (Scale_Value(new_w) != Scale_Value(old)) {
+	int	newval;
+	newval = _XmScaleConvertScaleValueToSCBValue(new_w);
 	argc = 0;
-	XtSetArg(argl[argc], XmNorientation, Scale_Orientation(new)); argc++;
-	XtSetArg(argl[argc], XmNprocessingDirection, Scale_ProcessingDirection(new)); argc++;
-	XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new)); argc++;
-	XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new)); argc++;
-	XtSetArg(argl[argc], XmNvalue, _XmScaleConvertScaleValueToSCBValue(new)); argc++;
-	sm = (int)((float)Scale_ScaleMultiple(new) /
-		   (float)(Scale_Maximum(new) - Scale_Minimum(new)) *
-		   (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(new)));
-	XtSetArg(argl[argc], XmNpageIncrement, sm); argc++;
-	XtSetArg(argl[argc], XmNhighlightThickness, Scale_HighlightThickness(new)); argc++;
-	XtSetArg(argl[argc], XmNhighlightOnEnter, Scale_HighlightOnEnter(new)); argc++;
+	XtSetArg(argl[argc], XmNvalue, newval); argc++;
 	XtSetValues(sb, argl, argc);
 
 	refresh = True;
     }
 
-    if (Scale_DecimalPoints(new) != Scale_DecimalPoints(old) ||
-	Scale_FontList(new) != Scale_FontList(old) ||
-	Scale_ShowValue(new) != Scale_ShowValue(old) ||
-	Scale_Value(new) != Scale_Value(old))
+    if (Scale_Maximum(new_w) != Scale_Maximum(old) ||
+        Scale_Minimum(new_w) != Scale_Minimum(old) ||
+        Scale_Orientation(new_w) != Scale_Orientation(old) ||
+        Scale_ProcessingDirection(new_w) != Scale_ProcessingDirection(old) ||
+        Scale_ScaleWidth(new_w) != Scale_ScaleWidth(old) ||
+        Scale_ScaleHeight(new_w) != Scale_ScaleHeight(old) ||
+        Scale_ScaleMultiple(new_w) != Scale_ScaleMultiple(old) ||
+        Scale_HighlightOnEnter(new_w) != Scale_HighlightOnEnter(old) ||
+        Scale_HighlightThickness(new_w) != Scale_HighlightThickness(old)) {
+
+	int	newval;
+
+	argc = 0;
+	XtSetArg(argl[argc], XmNorientation, Scale_Orientation(new_w)); argc++;
+	XtSetArg(argl[argc], XmNprocessingDirection, Scale_ProcessingDirection(new_w)); argc++;
+	XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new_w)); argc++;
+	XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new_w)); argc++;
+#if 0
+	newval = _XmScaleConvertScaleValueToSCBValue(new_w);
+	XtSetArg(argl[argc], XmNvalue, newval); argc++;
+#endif
+	XtSetArg(argl[argc], XmNshadowThickness, Scale_ShadowThickness(new_w)); argc++;
+	XtSetArg(argl[argc], XmNhighlightThickness, Scale_HighlightThickness(new_w)); argc++;
+	XtSetArg(argl[argc], XmNhighlightOnEnter, Scale_HighlightOnEnter(new_w)); argc++;
+	XtSetValues(sb, argl, argc);
+
+	refresh = True;
+    }
+
+    if (Scale_DecimalPoints(new_w) != Scale_DecimalPoints(old) ||
+	Scale_FontList(new_w) != Scale_FontList(old) ||
+	Scale_ShowValue(new_w) != Scale_ShowValue(old) ||
+	Scale_Value(new_w) != Scale_Value(old))
 	refresh = True;
 
-    _XmScaleLayout(new, True, NULL, False, NULL);
+    _XmScaleLayout(new_w, True, NULL, False, NULL);
 
     return refresh;
 }
@@ -610,6 +674,7 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes) {
         (*superclass->core_class.realize)(w, value_mask, attributes);
 #undef superclass
 
+    /* Redundant? XtRealizeWidget calls changed_managed ... -- PvH */
     _XmScaleLayout(w, True, NULL, False, NULL);
 }
 
@@ -713,13 +778,26 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
     Dimension *child_w, *child_h;
     Dimension curw, curh;
     Dimension incw, inch;
+    int maxw, maxh;
+    XtWidgetGeometry request;
+
     Widget *children;
     int num_children;
     XtGeometryResult result;
     int i;
+    int inc;
+    int p_inc;
+
+    /* FIX ME: signs are some of the XtIsManaged tests will have to go. */
+    /* FIX ME: if scaleWidth/Height is specified  scrollbar_width/height can
+     * get arbitrary small. If width/height is specified slider_width/height
+     * is bound to a minimum.
+     */
+    /* FIX ME: tick mark x/y seems to have a minimum value of -4 */
 
     /*
      * layout is normal processing as per the M*tif docs
+     * (Hmm.. tick marks and title get laid out on opposite sides of SB -- PvH)
      *  per me, this is as follows (FIXME -- this is not complete):
      *    if (orientation is HORIZONTAL) then
      *      layout title (if managed), and other children (if present)
@@ -831,14 +909,37 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
     curx = cury = 0;
     curw = curh = 0;
     incw = inch = 0;
+    maxw = maxh = 0;
+
     num_children = ((XmScaleWidget)scale)->composite.num_children;
     children = ((XmScaleWidget)scale)->composite.children;
+
     child_x = (Position*)XtMalloc(sizeof(Position) * num_children);
     child_y = (Position*)XtMalloc(sizeof(Position) * num_children);
     child_w = (Dimension*)XtMalloc(sizeof(Dimension) * num_children);
     child_h = (Dimension*)XtMalloc(sizeof(Dimension) * num_children);
+
+#define	FREE \
+	{XtFree((XtPointer)child_x); XtFree((XtPointer)child_y); \
+	 XtFree((XtPointer)child_w); XtFree((XtPointer)child_h);}
   
+    for (i = 2; i < num_children; i++) {
+	if (XtIsManaged(children[i])) {
+	    if (maxw < XtWidth(children[i]))
+		maxw = XtWidth(children[i]);
+	    if (maxh < XtHeight(children[i]))
+		maxh = XtHeight(children[i]);
+
+	    child_w[i] = XtWidth(children[i]);
+	    child_h[i] = XtHeight(children[i]);
+	}
+    }
+
     if (Scale_Orientation(scale) == XmHORIZONTAL) {
+
+	curh = maxh;
+	cury = maxh;
+
 	/*
 	 * do the value, if there is one
 	 */
@@ -846,8 +947,8 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 
 	    computeValueSize(scale);
 
-	    incw += Scale_ShowValueWidth(scale);
-	    inch += Scale_ShowValueHeight(scale);
+	    incw = Scale_ShowValueWidth(scale);
+	    inch = Scale_ShowValueHeight(scale);
 
 	    if (incw > curw)
 		curw = incw;
@@ -864,17 +965,28 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	    child_x[CHILD_SB] = curx;
 	    child_y[CHILD_SB] = cury;
 
-	    incw = Scale_ScaleWidth(scale);
-	    if (incw == 0)
-		incw = Scale_Maximum(scale) - Scale_Minimum(scale) +
-			2 * Prim_ShadowThickness(children[CHILD_SB]) +
-			2 * Scale_HighlightThickness(scale);
-
 	    inch = Scale_ScaleHeight(scale);
+
 	    if (inch == 0)
-		inch = VSLIDERSIZE +
-			2 * Prim_ShadowThickness(children[CHILD_SB]) +
-			2 * Scale_HighlightThickness(scale);
+		inch = SB_TRANSVERSAL_SIZE + 2 * Scale_HighlightThickness(scale);
+
+	    incw = Scale_ScaleWidth(scale);
+
+	    if (incw == 0)
+		incw = XtWidth(scale);
+
+	    if (incw == 0 && num_children >= 3) {
+		if (num_children == 3)
+		    incw = XtWidth(children[2]);
+		else
+		    incw = HSLIDERSIZE +
+			   2 * (Prim_HighlightThickness(children[CHILD_SB]) +
+				Prim_ShadowThickness(children[CHILD_SB])) +
+			   maxw * (num_children - 3);
+	    }
+
+	    if (incw == 0)
+		incw = SB_LONGITUDAL_SIZE + 2 * Scale_HighlightThickness(scale);
 
 	    child_w[CHILD_SB] = incw;
 	    child_h[CHILD_SB] = inch;
@@ -884,6 +996,7 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 		    childgeom->width = child_w[CHILD_SB];
 		if (childgeom->request_mode & CWHeight)
 		    childgeom->height = child_h[CHILD_SB];
+		FREE;
 		return;
 	    }
 
@@ -894,6 +1007,63 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	    cury += inch;
 	}
 
+	/* lay out the tick marks */
+	if (num_children > 2) {
+	    /* centres are nicely aligned */
+
+	    if (num_children > 3) {
+		int spacing;
+
+		if (curw == 0)
+		    spacing = maxw * (num_children - 3);
+		else
+		    spacing = (curw - 
+			       (HSLIDERSIZE +
+				2 * (Prim_HighlightThickness(children[CHILD_SB]) +
+				     Prim_ShadowThickness(children[CHILD_SB]))));
+
+		for (i = 2; i < num_children; i++) {
+		    if (XtIsManaged(children[i])) {
+			child_y[i] = (maxh - XtHeight(children[i])) / 2; 
+			child_x[i] = (HSLIDERSIZE - XtWidth(children[i])) / 2 + 
+				     Prim_HighlightThickness(children[CHILD_SB]) +
+				     Prim_ShadowThickness(children[CHILD_SB]) +
+				     (i - 2) * spacing / (num_children - 3);
+
+			child_h[i] = XtHeight(children[i]);
+			child_w[i] = XtWidth(children[i]);
+
+			if (children[i] == child) {
+			    if (childgeom->request_mode & CWWidth)
+				childgeom->width = child_w[i];
+			    if (childgeom->request_mode & CWHeight)
+				childgeom->height = child_h[i];
+			    FREE;
+			    return;
+			}
+		    }
+		}
+	    }
+	    else {
+		if (XtIsManaged(children[2])) {
+		    child_y[2] = 0;
+		    child_x[2] = (curw - XtWidth(children[2])) / 2;
+
+		    child_h[2] = XtHeight(children[2]);
+		    child_w[2] = XtWidth(children[2]);
+
+		    if (children[2] == child) {
+			if (childgeom->request_mode & CWWidth)
+			    childgeom->width = child_w[2];
+			if (childgeom->request_mode & CWHeight)
+			    childgeom->height = child_h[2];
+			FREE;
+			return;
+		    }
+		}
+	    }
+	}
+
 	/*
 	 * then the title, if there is one
 	 */
@@ -903,7 +1073,9 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	    child_x[CHILD_LAB] = curx;
 	    child_y[CHILD_LAB] = cury;
 
-	    _XmLabelDimensions(children[CHILD_LAB], &incw, &inch);
+	    _XmCalcLabelGDimensions(children[CHILD_LAB]);
+	    incw = XtWidth(children[CHILD_LAB]);
+	    inch = XtHeight(children[CHILD_LAB]);
 
 	    child_w[CHILD_LAB] = incw;
 	    child_h[CHILD_LAB] = inch;
@@ -913,6 +1085,7 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 		    childgeom->width = child_w[CHILD_LAB];
 		if (childgeom->request_mode & CWHeight)
 		    childgeom->height = child_h[CHILD_LAB];
+		FREE;
 		return;
 	    }
 
@@ -922,46 +1095,13 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 
 	    curx += incw;
 	}
-
-	/*
-	 * and finish with any remaining children.  These get laid out
-	 * alongside the title, if there is one.
-	 */
-	for (i = 2; i < num_children; i++) {
-
-	    if (XtIsManaged(children[i])) {
-		child_x[CHILD_SB] = curx;
-		child_y[CHILD_SB] = cury;
-
-		incw = XtWidth(children[i]);
-		inch = XtHeight(children[i]);
-
-		child_w[CHILD_SB] = incw;
-		child_h[CHILD_SB] = inch;
-
-		if ((curx + incw) > curw)
-		    curw = incw;
-		if ((cury + inch) > curh)
-		    curh = inch;
-
-	        curx += incw;
-	    }
-	    else {
-		child_w[CHILD_SB] = incw;
-		child_h[CHILD_SB] = inch;
-	    }
-
-	    if (children[i] == child) {
-		if (childgeom->request_mode & CWWidth)
-		    childgeom->width = child_w[i];
-		if (childgeom->request_mode & CWHeight)
-		    childgeom->height = child_h[i];
-		return;
-	    }
-	}
     }
     /* XmNorientation == XmVERTICAL */
     else {
+
+	curw = maxw;
+	curx = maxw;
+
 	/*
 	 * do the value, if there is one
 	 */
@@ -988,16 +1128,28 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	    child_y[CHILD_SB] = cury;
 
 	    incw = Scale_ScaleWidth(scale);
+
 	    if (incw == 0)
-		incw = VSLIDERSIZE +
-			2 * Prim_ShadowThickness(children[CHILD_SB]) +
-			2 * Scale_HighlightThickness(scale);
+		incw = SB_TRANSVERSAL_SIZE + 2 * Scale_HighlightThickness(scale);
 
 	    inch = Scale_ScaleHeight(scale);
+
 	    if (inch == 0)
-		inch = Scale_Maximum(scale) - Scale_Minimum(scale) +
-			2 * Prim_ShadowThickness(children[CHILD_SB]) +
-			2 * Scale_HighlightThickness(scale);
+		inch = XtHeight(scale);
+
+	    if (inch == 0 && num_children >= 3) {
+		if (num_children == 3)
+		    inch = XtHeight(children[2]);
+		else
+		    /* FIX ME: SrollBar's or Scale's? */
+		    inch = HSLIDERSIZE +
+			   2 * (Prim_HighlightThickness(children[CHILD_SB]) +
+				Prim_ShadowThickness(children[CHILD_SB])) +
+			   maxh * (num_children - 3);
+	    }
+
+	    if (inch == 0)
+		inch = SB_LONGITUDAL_SIZE + 2 * Scale_HighlightThickness(scale);
 
 	    child_w[CHILD_SB] = incw;
 	    child_h[CHILD_SB] = inch;
@@ -1007,6 +1159,7 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 		    childgeom->width = child_w[CHILD_SB];
 		if (childgeom->request_mode & CWHeight)
 		    childgeom->height = child_h[CHILD_SB];
+		FREE;
 		return;
 	    }
 
@@ -1017,6 +1170,63 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	    curx += incw;
 	}
 
+	/* lay out the tick marks */
+	if (num_children > 2) {
+	    /* centres are nicely aligned */
+
+	    if (num_children > 3) {
+		int spacing;
+
+		if (curh == 0)
+		    spacing = maxh * (num_children - 3);
+		else
+		    spacing = (curh - 
+			       (HSLIDERSIZE +
+				2 * (Prim_HighlightThickness(children[CHILD_SB]) +
+				     Prim_ShadowThickness(children[CHILD_SB]))));
+
+		for (i = 2; i < num_children; i++) {
+		    if (XtIsManaged(children[i])) {
+			child_x[i] = (maxw - XtWidth(children[i])) / 2; 
+			child_y[i] = (HSLIDERSIZE - XtHeight(children[i])) / 2 + 
+				     Prim_HighlightThickness(children[CHILD_SB]) +
+				     Prim_ShadowThickness(children[CHILD_SB]) +
+				     (i - 2) * spacing / (num_children - 3);
+
+			child_w[i] = XtWidth(children[i]);
+			child_h[i] = XtHeight(children[i]);
+
+			if (children[i] == child) {
+			    if (childgeom->request_mode & CWWidth)
+				childgeom->width = child_w[i];
+			    if (childgeom->request_mode & CWHeight)
+				childgeom->height = child_h[i];
+			    FREE;
+			    return;
+			}
+		    }
+		}
+	    }
+	    else {
+		if (XtIsManaged(children[2])) {
+		    child_x[2] = 0;
+		    child_y[2] = (curh - XtHeight(children[2])) / 2;
+
+		    child_w[2] = XtWidth(children[2]);
+		    child_h[2] = XtHeight(children[2]);
+
+		    if (children[2] == child) {
+			if (childgeom->request_mode & CWWidth)
+			    childgeom->width = child_w[2];
+			if (childgeom->request_mode & CWHeight)
+			    childgeom->height = child_h[2];
+			FREE;
+			return;
+		    }
+		}
+	    }
+	}
+
 	/*
 	 * then the title, if there is one
 	 */
@@ -1027,7 +1237,9 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	    child_y[CHILD_LAB] = cury;
 
 
-	    _XmLabelDimensions(children[CHILD_LAB], &incw, &inch);
+	    _XmCalcLabelGDimensions(children[CHILD_LAB]);
+	    incw = XtWidth(children[CHILD_LAB]);
+	    inch = XtHeight(children[CHILD_LAB]);
 
 	    child_w[CHILD_LAB] = incw;
 	    child_h[CHILD_LAB] = inch;
@@ -1037,6 +1249,7 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 		    childgeom->width = child_w[CHILD_LAB];
 		if (childgeom->request_mode & CWHeight)
 		    childgeom->height = child_h[CHILD_LAB];
+		FREE;
 		return;
 	    }
 
@@ -1046,43 +1259,6 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	    curw += incw;
 	    cury += inch;
 	}
-
-	/*
-	 * and finish with any remaining children.  These get laid out
-	 * alongside the title, if there is one.
-	 */
-	for (i = 2; i < ((XmScaleWidget)scale)->composite.num_children; i++) {
-
-	    if (XtIsManaged(children[i])) {
-		child_x[CHILD_SB] = curx;
-		child_y[CHILD_SB] = cury;
-
-		incw = XtWidth(children[i]);
-		inch = XtHeight(children[i]);
-
-		child_w[CHILD_SB] = incw;
-		child_h[CHILD_SB] = inch;
-
-		if ((curx + incw) > curw)
-		    curw = incw;
-		if ((cury + inch) > curh)
-		    curh = inch;
-
-	        curx += incw;
-	    }
-	    else {
-		child_w[CHILD_SB] = incw;
-		child_h[CHILD_SB] = inch;
-	    }
-
-	    if (children[i] == child) {
-		if (childgeom->request_mode & CWWidth)
-		    childgeom->width = child_w[i];
-		if (childgeom->request_mode & CWHeight)
-		    childgeom->height = child_h[i];
-		return;
-	    }
-	}
     }
 
     /*
@@ -1091,6 +1267,7 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
     if (child == scale) {
 	childgeom->width = curw;
 	childgeom->height = curh;
+	FREE;
 	return;
     }
 
@@ -1098,9 +1275,13 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
      * if we can, change our size to match
      */
     if (ParentResize) {
-	do {
-	    result = XtMakeResizeRequest(scale, curw, curh, &curw, &curh);
-	} while (result == XtGeometryAlmost);
+
+	request.request_mode = (CWWidth|CWHeight);
+	request.width = curw;
+	request.height = curh;
+	result = _XmMakeGeometryRequest(scale, &request);
+	curw = request.width;
+	curh = request.height;
 
 	if (result == XtGeometryYes) {
 	    XtWidth(scale) = curw;
@@ -1115,6 +1296,7 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
      * do the layout
      */
     if (Scale_Orientation(scale) == XmHORIZONTAL) {
+#if 0
 	if (Scale_ScaleWidth(scale) == 0)
 	    child_w[CHILD_SB] = XtWidth(scale);
 
@@ -1147,13 +1329,14 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 					 (float)XtWidth(scale));
 	    }
 	}
-
+#endif
 	if (XtHeight(scale) > curh) {
-	    for (i = 0; i < num_children; i++)
+	    for (i = 0; i < 2; i++)
 		child_y[i] = XtHeight(scale) - (curh - child_y[i]);
 	}
     }
     else if (Scale_Orientation(scale) == XmVERTICAL) {
+#if 0
 	if (Scale_ScaleHeight(scale) == 0)
 	    child_h[CHILD_SB] = XtHeight(scale);
 
@@ -1181,14 +1364,14 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	}
 
 	if (XtWidth(scale) > curw) {
-	    for (i = 2; i < num_children; i++){
+	    for (i = 2; i < num_children; i++)
 		child_h[i] = (Dimension)((float)child_h[i] / (float)curw *
 					 (float)XtWidth(scale));
-	    }
 	}
+#endif
 
 	if (XtWidth(scale) > curw) {
-	    for (i = 0; i < num_children; i++)
+	    for (i = 0; i < 2; i++)
 		child_x[i] = XtWidth(scale) - (curw - child_x[i]);
 	}
     }
@@ -1205,13 +1388,21 @@ _XmScaleLayout(Widget scale, Boolean ParentResize,
 	}
     }
 
-    XtVaSetValues(children[CHILD_SB],
-		  XmNsliderSize, Scale_SliderSize(scale),
-		  XmNvalue, _XmScaleConvertScaleValueToSCBValue(scale),
-		  XmNpageIncrement, (int)((float)Scale_ScaleMultiple(scale) /
-					  (float)(Scale_Maximum(scale) - Scale_Minimum(scale)) *
-					  (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(scale))),
-		  NULL);
+    Scale_SliderSize(scale) = _XmScaleConvertWidthToSliderSize(scale);
+    inc = (int)((float)(SCB_MAX - SCB_MIN - Scale_SliderSize(scale)) /
+		(float)(Scale_Maximum(scale) - Scale_Minimum(scale)) +
+		0.5);
+    inc = _XmMax(inc, 1); /* FIX ME */
+    p_inc = inc * Scale_ScaleMultiple(scale);
+
+    XmScrollBarSetValues(children[CHILD_SB],
+			 _XmScaleConvertScaleValueToSCBValue(scale),
+			 Scale_SliderSize(scale),
+			 inc, 
+			 p_inc,
+			 False);
+
+    FREE;
 }
   
 static void
@@ -1237,12 +1428,20 @@ showValue(Widget w, int scb_value, int scale_value) {
     sb = ((XmScaleWidget)w)->composite.children[CHILD_SB];
     valueToString(w, buf);
 
-    XSetForeground(XtDisplay(w), Scale_ForegroundGC(w), XtBackground(w));
+#if 1
+/* Fix from John Richardson <jrichard@zko.dec.com> 1/7/1996 */
     if (XtIsRealized(w)) {
-	XFillRectangle(XtDisplay(w), XtWindow(w), Scale_ForegroundGC(w),
+	XFillRectangle(XtDisplay(w), XtWindow(w), MGR_BackgroundGC(w),
 		       Scale_ShowValueX(w), Scale_ShowValueY(w),
 		       Scale_ShowValueWidth(w), Scale_ShowValueHeight(w));
     }
+#else
+    if (XtIsRealized(w)) {
+	XFillRectangle(XtDisplay(w), XtWindow(w), MGR_BackgroundGC(w),
+		       Scale_ShowValueX(w), Scale_ShowValueY(w) - Scale_FontStruct(w)->descent,
+		       Scale_ShowValueWidth(w), Scale_ShowValueHeight(w));
+    }
+#endif
 
     if (Scale_Orientation(w) == XmHORIZONTAL) {
 	Scale_ShowValueX(w) = _XmScrollBarValueToPos(sb, scb_value) +
@@ -1259,10 +1458,6 @@ showValue(Widget w, int scb_value, int scale_value) {
     else
 	_XmError(w, "Scale Orientation wrong.");
 
-    XSetForeground(XtDisplay(w), Scale_ForegroundGC(w), MGR_HighlightColor(w));
-    XSetBackground(XtDisplay(w), Scale_ForegroundGC(w), XtBackground(w));
-    XSetFont(XtDisplay(w), Scale_ForegroundGC(w), Scale_FontStruct(w)->fid);
-
     if (XtIsRealized(w)) {
 	int offset;
 
@@ -1278,7 +1473,7 @@ showValue(Widget w, int scb_value, int scale_value) {
 	else if (Scale_Orientation(w) == XmVERTICAL) {
 	    XDrawString(XtDisplay(w), XtWindow(w),
 			Scale_ForegroundGC(w),
-			Scale_ShowValueX(w) +  offset,
+			Scale_ShowValueX(w) + offset,
 			Scale_ShowValueY(w) + Scale_FontStruct(w)->ascent,
 			buf, strlen(buf));
 	}
@@ -1345,51 +1540,57 @@ LoseFocus(Widget w, XEvent *event, String *params, Cardinal *num_params) {
 static int
 _XmScaleConvertWidthToSliderSize(Widget w) {
     int ret, last_ret;
+    XmScrollBarWidget sb;
+    Dimension ht;
+    Dimension st;
+
+    /* FIX ME: the next two statements are a kludge */
+    sb = (XmScrollBarWidget)(((XmScaleWidget)w)->composite.children[CHILD_SB]);
+
+    /* FIX ME: Can't we use scale's values here? I.o.w. what shouls scale do
+     * when these values are set explicitly and thus differ from scale's ?
+     */
+    XtVaGetValues((Widget)sb,
+		  XmNhighlightThickness, &ht,
+		  XmNshadowThickness, &st,
+		  NULL);
 
     ret = Scale_SliderSize(w);
     do {
+        /* FIX ME: rework the if's */
 	last_ret = ret;
 	if (Scale_Orientation(w) == XmHORIZONTAL) {
-	    if (Scale_ScaleWidth(w) == 0) {
-		if (XtWidth(w) == 0) {
-		    ret = (int)((float)(SCB_MAX - SCB_MIN) /
-				(float)(Scale_Maximum(w) - Scale_Minimum(w)) *
-				(float)HSLIDERSIZE);
-		}
-		else if (XtWidth(w) < HSLIDERSIZE) {
-		    ret = SCB_MAX - SCB_MIN;
-		}
-		else {
-		    ret = (int)((float)(SCB_MAX - SCB_MIN) /
-				(float)XtWidth(w) *
-				(float)HSLIDERSIZE);
-		}
-	    }
-	    else
-		ret = (int)((float)(SCB_MAX - SCB_MIN) /
-			    (float)Scale_ScaleWidth(w) *
+    	    if (XtWidth(sb) == 2 * (ht + st) ) {
+		/* Note: M*tif 1.2.2 SIGFPE's on this -- PvH */
+	        ret = (int)((float)(SCB_MAX - SCB_MIN) /
+			    (float)(Scale_Maximum(w) - Scale_Minimum(w)) *
 			    (float)HSLIDERSIZE);
+	    }
+	    else if (XtWidth(sb) < HSLIDERSIZE) {
+	        ret = SCB_MAX - SCB_MIN;
+	    }
+	    else {
+		/* Note: M*tif doesn't use float arithmetic here -- PvH */
+	        ret = (int)((SCB_MAX - SCB_MIN) /
+			    (XtWidth(sb) - 2 * (ht + st)) *
+			    HSLIDERSIZE);
+	    }
 	}
 	else {
-	    if (Scale_ScaleHeight(w) == 0) {
-		if (XtHeight(w) == 0) {
-		    ret = (int)((float)(SCB_MAX - SCB_MIN) /
-				(float)(Scale_Maximum(w) - Scale_Minimum(w)) *
-				(float)HSLIDERSIZE);
-		}
-		else if (XtHeight(w) < HSLIDERSIZE) {
-		    ret = SCB_MAX - SCB_MIN;
-		}
-		else {
-		    ret = (int)((float)(SCB_MAX - SCB_MIN) /
-				(float)XtHeight(w) *
-				(float)HSLIDERSIZE);
-		}
-	    }
-	    else
+	    if (XtHeight(sb) == 2 * (ht + st) ) {
 		ret = (int)((float)(SCB_MAX - SCB_MIN) /
-			    (float)Scale_ScaleHeight(w) *
+			    (float)(Scale_Maximum(w) - Scale_Minimum(w)) *
 			    (float)HSLIDERSIZE);
+	    }
+	    else if (XtHeight(sb) < HSLIDERSIZE) {
+		ret = SCB_MAX - SCB_MIN;
+	    }
+	    else {
+		/* Note: M*tif doesn't use float arithmetic here -- PvH */
+		ret = (int)((SCB_MAX - SCB_MIN) /
+			    (XtHeight(sb) - 2 * (ht + st)) *
+			    HSLIDERSIZE);
+	    }
 	}
     } while (last_ret != ret);
 
