@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ScrollBar.c,v 1.23 1996/04/23 17:40:39 miers Exp $
+ * $Id: ScrollBar.c,v 1.5 1996/12/30 07:36:31 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,20 +22,22 @@
  *
  **/
 
-static char rcsid[] = "$Id: ScrollBar.c,v 1.23 1996/04/23 17:40:39 miers Exp $";
+static char rcsid[] = "$Id: ScrollBar.c,v 1.5 1996/12/30 07:36:31 u27113 Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
 #include <Xm/ScrollBarP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/ScaleP.h>
 #include <limits.h>
-#include <stdio.h>
-#include <Xm/XmI.h>
 #include <Xm/ScreenP.h>
 #include <stdlib.h>
 
+#include <XmI/DebugUtil.h>
+
 #define _XmMin(a,b) (((a)<(b))?(a):(b))
+#define _XmMax(a,b) (((a)>(b))?(a):(b))
 
 #define DEF_MIN			0
 #define DEF_MAX			100
@@ -49,13 +51,10 @@ static char rcsid[] = "$Id: ScrollBar.c,v 1.23 1996/04/23 17:40:39 miers Exp $";
 
 /* BUGS/FIX ME:
  *
- * Motif 1.2.2 handles initial_x and initial_y differently.
+ * M*tif 1.2.2 handles initial_x and initial_y differently.
  *
- * Arghhhh... in Motif 1.2.2 value (and hence saved_value, I guess) private
+ * Arghhhh... in M*tif 1.2.2 value (and hence saved_value, I guess) private
  * vars are independent of processing direction.
- *
- * If shadowThickness == 0 arrow shadow thickness == 0, otherwise arrow shadow
- * thickness == 2.
  */
 
 /*
@@ -110,13 +109,18 @@ static char rcsid[] = "$Id: ScrollBar.c,v 1.23 1996/04/23 17:40:39 miers Exp $";
  * arrow and the slider, regardless of size.
  */
 
+/*
+ * pick a GC, any GC
+ */
+#define SCB_GC(w)	(XtSensitive(w) \
+				? SCB_ForegroundGC(w) : SCB_UnavailableGC(w))
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static XtGeometryResult query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply);
 static void expose(Widget w, XEvent *event, Region region);
 static void resize(Widget w);
@@ -140,7 +144,8 @@ static void decrementByPage(Widget sw, XEvent *event);
 
 /* misc scrollbar routines */
 
-static Dimension _XmScrollBarArrowSize(Widget sw);
+static int _XmScrollBarArrowWidth(Widget sw);
+static int _XmScrollBarArrowHeight(Widget sw);
 static int _XmScrollBarPosToValue(Widget sw, Position pos);
 
 static void draw_slider(Widget sw);
@@ -275,7 +280,7 @@ static XtResource resources[] = {
     {
 	XmNtraversalOn, XmCTraversalOn, XmRBoolean,
 	sizeof(Boolean), Prim_Offset(traversal_on),
-	XmRCallProc, (XtPointer)NULL /* FIXME: NEED PROC HERE */
+	XmRCallProc, (XtPointer)_XmScrollBarTraversalOnDefault,
     },
     {
 	XmNhighlightThickness, XmCHighlightThickness, XmRHorizontalDimension,
@@ -505,89 +510,118 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmSCROLL_BAR_BIT);
 }
 
+static void
+CreateForegroundGC(Widget w)
+{
+    XtGCMask mask;
+    XGCValues values;
+
+    mask = GCFunction | GCPlaneMask | GCSubwindowMode | GCGraphicsExposures |
+	   GCForeground | GCBackground | GCFillStyle;
+    values.function = GXcopy;
+    values.plane_mask = -1;
+    values.subwindow_mode = ClipByChildren;
+    values.graphics_exposures = False;
+    values.foreground = XtBackground(w);
+    values.background = Prim_Foreground(w);
+    values.fill_style = FillStippled;
+
+    SCB_ForegroundGC(w) = XtGetGC(w, mask, &values);
+}
+
+static void
+CreateUnavailableGC(Widget w)
+{
+    XtGCMask mask;
+    XGCValues values;
+
+    mask = GCFunction | GCPlaneMask | GCSubwindowMode | GCGraphicsExposures |
+	   GCForeground | GCBackground | GCStipple | GCFillStyle |
+	   GCTileStipXOrigin | GCTileStipYOrigin;
+    values.function = GXcopy;
+    values.plane_mask = -1;
+    values.subwindow_mode = ClipByChildren;
+    values.graphics_exposures = False;
+    values.foreground = XtBackground(w);
+    values.background = Prim_Foreground(w);
+    values.fill_style = FillStippled;
+    values.ts_x_origin = values.ts_y_origin = 0;
+    values.stipple = XmGetPixmapByDepth(XtScreen(w),
+					XmEVEN_STIPPLE_IMAGE,
+					WhitePixelOfScreen(XtScreen(w)),
+					BlackPixelOfScreen(XtScreen(w)),
+					1);
+
+    SCB_UnavailableGC(w) = XtGetGC(w, mask, &values);
+}
+
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    int add_space;
-
     /*
      * we override the Primitive width/height if the user didn't specify one
      */
-    SCB_ArrowWidth(new) = SCB_ArrowHeight(new) = _XmScrollBarArrowSize(request);
-    if (SCB_ShowArrows(new))
-	add_space = ARROW_SPACING;
-    else
-	add_space = 0;
 
-    if (SCB_Orientation(new) == XmHORIZONTAL)
+    if (SCB_Orientation(new_w) == XmHORIZONTAL)
     {
 	if (XtWidth(request) == 0) {
-	    XtWidth(new) = DEF_SB_LEN +
-			2 * Prim_ShadowThickness(new) +
-			2 * Prim_HighlightThickness(new);
+	    XtWidth(new_w) = DEF_SB_LEN +
+			2 * Prim_ShadowThickness(new_w) +
+			2 * Prim_HighlightThickness(new_w);
 	}
 
 	if (XtHeight(request) == 0) {
-	    XtHeight(new) = DEF_SB_DIM +
-			2 * Prim_ShadowThickness(new) +
-			2 * Prim_HighlightThickness(new);
+	    XtHeight(new_w) = DEF_SB_DIM +
+			2 * Prim_ShadowThickness(new_w) +
+			2 * Prim_HighlightThickness(new_w);
 	}
 
-	SCB_Arrow1Orientation(new) = XmARROW_LEFT;
-	SCB_Arrow2Orientation(new) = XmARROW_RIGHT;
+	SCB_Arrow1Orientation(new_w) = XmARROW_LEFT;
+	SCB_Arrow2Orientation(new_w) = XmARROW_RIGHT;
     }
     else
     {
 	if (XtHeight(request) == 0) {
-	    XtHeight(new) = DEF_SB_LEN +
-			2 * Prim_ShadowThickness(new) +
-			2 * Prim_HighlightThickness(new);
+	    XtHeight(new_w) = DEF_SB_LEN +
+			2 * Prim_ShadowThickness(new_w) +
+			2 * Prim_HighlightThickness(new_w);
 	}
 
 	if (XtWidth(request) == 0) {
-	    XtWidth(new) = DEF_SB_DIM +
-			2 * Prim_ShadowThickness(new) +
-			2 * Prim_HighlightThickness(new);
+	    XtWidth(new_w) = DEF_SB_DIM +
+			2 * Prim_ShadowThickness(new_w) +
+			2 * Prim_HighlightThickness(new_w);
 	}
 
-	SCB_Arrow1Orientation(new) = XmARROW_UP;
-	SCB_Arrow2Orientation(new) = XmARROW_DOWN;
+	SCB_Arrow1Orientation(new_w) = XmARROW_UP;
+	SCB_Arrow2Orientation(new_w) = XmARROW_DOWN;
     }
 
-    SCB_Arrow1Selected(new) = False;
-    SCB_Arrow2Selected(new) = False;
-    SCB_Flags(new) = 0;
-    SCB_EtchedSlider(new) = False;
+    SCB_Arrow1Selected(new_w) = False;
+    SCB_Arrow2Selected(new_w) = False;
+    SCB_Flags(new_w) = 0;
+    SCB_EtchedSlider(new_w) = False;
 
-    SCB_Pixmap(new) = _XmAllocScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(new)),
-					    DefaultDepthOfScreen(XtScreen(new)),
-					    XtWidth(new), XtHeight(new));
+    SCB_Pixmap(new_w) = _XmAllocScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(new_w)),
+					    DefaultDepthOfScreen(XtScreen(new_w)),
+					    XtWidth(new_w), XtHeight(new_w));
 
-    SCB_Timer(new) = 0;
+    SCB_Timer(new_w) = 0;
 
-    SCB_UnavailableGC(new) = XtAllocateGC((Widget)new,
-						0,
-						0,
-						NULL,
-						GCForeground, ~(GCForeground));
+    CreateForegroundGC(new_w);
+    CreateUnavailableGC(new_w);
 
-    SCB_ForegroundGC(new) = XtAllocateGC((Widget)new,
-						0,
-						0,
-						NULL,
-						GCForeground, ~(GCForeground));
+    if (XtIsSubclass(XtParent(new_w), xmScaleWidgetClass))
+	SCB_ChangeType(new_w) |= CHANGE_TYPE_SCALE;
 
-    if (XtIsSubclass(XtParent(new), xmScaleWidgetClass))
-	SCB_ChangeType(new) |= CHANGE_TYPE_SCALE;
+    check_constraints(new_w, True);
 
-    check_constraints(new, True);
+    SCB_SavedValue(new_w) = SCB_Value(new_w);
 
-    SCB_SavedValue(new) = SCB_Value(new);
-
-    DoLayout((Widget)new, True, False);
+    DoLayout((Widget)new_w, True, False);
 }
 
 /* FIX ME: split this up into two functions, one for initialize and one for
@@ -620,7 +654,8 @@ check_constraints(Widget sw, Boolean init)
 	    SCB_SliderSize(sw) = 1;
 	}
 
-	if (SCB_SliderSize(sw) > SCB_Maximum(sw) - SCB_Minimum(sw))
+	if (SCB_SliderSize(sw) > SCB_Maximum(sw) - SCB_Minimum(sw)
+	 && SCB_Maximum(sw) != SCB_Minimum(sw))
 	{
 	    _XmWarning(sw, MSG_SLSZ_GT_MAXMIN);
 	    SCB_SliderSize(sw) = SCB_Maximum(sw) - SCB_Minimum(sw);
@@ -637,7 +672,8 @@ check_constraints(Widget sw, Boolean init)
 	    SCB_Value(sw) = SCB_Minimum(sw);
 	}
 
-	if (SCB_Value(sw) > SCB_Maximum(sw) - SCB_SliderSize(sw))
+	if (SCB_Value(sw) > SCB_Maximum(sw) - SCB_SliderSize(sw)
+	 && SCB_Maximum(sw) != SCB_SliderSize(sw))
 	{
 	    _XmWarning(sw, MSG_VAL_GT_MAXSLSZ);
 	    SCB_Value(sw) = SCB_Minimum(sw);
@@ -651,7 +687,8 @@ check_constraints(Widget sw, Boolean init)
 	    SCB_SliderSize(sw) = (SCB_Maximum(sw) - SCB_Minimum(sw)) / 10;
 	}
 
-	if (SCB_SliderSize(sw) > SCB_Maximum(sw) - SCB_Minimum(sw))
+	if (SCB_SliderSize(sw) > SCB_Maximum(sw) - SCB_Minimum(sw)
+	 && SCB_Maximum(sw) != SCB_Minimum(sw))
 	{
 	    _XmWarning(sw, MSG_SLSZ_GT_MAXMIN);
 	    SCB_SliderSize(sw) = (SCB_Maximum(sw) - SCB_Minimum(sw)) / 10;
@@ -663,7 +700,8 @@ check_constraints(Widget sw, Boolean init)
 	    SCB_Value(sw) = SCB_Minimum(sw);
 	}
 
-	if (SCB_Value(sw) > SCB_Maximum(sw) - SCB_SliderSize(sw))
+	if (SCB_Value(sw) > SCB_Maximum(sw) - SCB_SliderSize(sw)
+	 && SCB_Maximum(sw) != SCB_SliderSize(sw))
 	{
 	    _XmWarning(sw, MSG_VAL_GT_MAXSLSZ);
 	    SCB_Value(sw) = SCB_Maximum(sw) - SCB_SliderSize(sw);
@@ -724,22 +762,31 @@ destroy(Widget w)
 }
 
 static Boolean 
-set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args)
+set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 {
     Boolean		r = False;
 
-#define	CNE(x)	(x(new) != x(current))
+#define	CNE(x)	(x(new_w) != x(current))
+
+    if (SCB_TroughColor(new_w) != SCB_TroughColor(current) ||
+        Prim_Foreground(new_w) != Prim_Foreground(current)) {
+	XtReleaseGC(new_w, SCB_ForegroundGC(new_w));
+	XtReleaseGC(new_w, SCB_UnavailableGC(new_w));
+	CreateUnavailableGC(new_w);
+	CreateForegroundGC(new_w);
+	r = True;
+    }
 
     /* FIX ME: test too restrictive? */
     if (CNE(SCB_Orientation) || CNE(SCB_Minimum) || CNE(SCB_Maximum) || CNE(SCB_Value)
 			|| CNE(XtX) || CNE(XtY) || CNE(SCB_SliderSize)) {
 	r = True;
 	/* Ensure slidersize and value are within constraints */
-	check_constraints(new, False);
+	check_constraints(new_w, False);
 
-	erase_slider(new);
+	erase_slider(new_w);
 
-	DoLayout(new, False, True);
+	DoLayout(new_w, False, True);
     }
 
     return r;
@@ -748,15 +795,14 @@ set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *n
 static void
 redraw_inset(Widget sw)
 {
-    XSetForeground(XtDisplay(sw),
-		   SCB_UnavailableGC(sw),
-		   SCB_TroughColor(sw));
+    XSetForeground(XtDisplay(sw), SCB_GC(sw), SCB_TroughColor(sw));
 
     XFillRectangle(XtDisplay(sw),
 		   SCB_Pixmap(sw),
-		   SCB_UnavailableGC(sw),
+		   SCB_GC(sw),
 		   0, 0,
 		   XtWidth(sw), XtHeight(sw));
+    XSetForeground(XtDisplay(sw), SCB_GC(sw), XtBackground(sw));
 
     _XmDrawShadows(XtDisplay((Widget)sw),
 		   (Window)SCB_Pixmap(sw),
@@ -776,22 +822,18 @@ redraw_arrows(Widget sw)
     if (!SCB_ShowArrows(sw))
 	return;
 
-    XSetForeground(XtDisplay(sw),
-		   SCB_ForegroundGC(sw),
-		   XtBackground(sw));
-
     if (SCB_Arrow1Selected(sw))
     {
 	_XmDrawArrow(XtDisplay(sw),
 		     (Window)SCB_Pixmap(sw),
 		     Prim_TopShadowGC(sw),
 		     Prim_BottomShadowGC(sw),
-		     SCB_ForegroundGC(sw),
+		     SCB_GC(sw),
 		     SCB_Arrow1X(sw),
 		     SCB_Arrow1Y(sw),
 		     SCB_ArrowWidth(sw),
 		     SCB_ArrowHeight(sw),
-		     _XmMin(2, Prim_ShadowThickness(sw)),
+		     Prim_ShadowThickness(sw) == 0 ? 0 : 2,
 		     SCB_Arrow1Orientation(sw));
     }
     else
@@ -800,42 +842,42 @@ redraw_arrows(Widget sw)
 		     (Window)SCB_Pixmap(sw),
 		     Prim_BottomShadowGC(sw),
 		     Prim_TopShadowGC(sw),
-		     SCB_ForegroundGC(sw),
+		     SCB_GC(sw),
 		     SCB_Arrow1X(sw),
 		     SCB_Arrow1Y(sw),
 		     SCB_ArrowWidth(sw),
 		     SCB_ArrowHeight(sw),
-		     _XmMin(2, Prim_ShadowThickness(sw)),
+		     Prim_ShadowThickness(sw) == 0 ? 0 : 2,
 		     SCB_Arrow1Orientation(sw));
     }
 
     if (SCB_Arrow2Selected(sw))
     {
-	    _XmDrawArrow(XtDisplay(sw),
-			 (Window)SCB_Pixmap(sw),
-			 Prim_TopShadowGC(sw),
-			 Prim_BottomShadowGC(sw),
-			 SCB_ForegroundGC(sw),
-			 SCB_Arrow2X(sw),
-			 SCB_Arrow2Y(sw),
-			 SCB_ArrowWidth(sw),
-			 SCB_ArrowHeight(sw),
-			 _XmMin(2, Prim_ShadowThickness(sw)),
-			 SCB_Arrow2Orientation(sw));
+	_XmDrawArrow(XtDisplay(sw),
+		     (Window)SCB_Pixmap(sw),
+		     Prim_TopShadowGC(sw),
+		     Prim_BottomShadowGC(sw),
+		     SCB_GC(sw),
+		     SCB_Arrow2X(sw),
+		     SCB_Arrow2Y(sw),
+		     SCB_ArrowWidth(sw),
+		     SCB_ArrowHeight(sw),
+		     Prim_ShadowThickness(sw) == 0 ? 0 : 2,
+		     SCB_Arrow2Orientation(sw));
     }
     else
     {
-	    _XmDrawArrow(XtDisplay(sw),
-			 (Window)SCB_Pixmap(sw),
-			 Prim_BottomShadowGC(sw),
-			 Prim_TopShadowGC(sw),
-			 SCB_ForegroundGC(sw),
-			 SCB_Arrow2X(sw),
-			 SCB_Arrow2Y(sw),
-			 SCB_ArrowWidth(sw),
-			 SCB_ArrowHeight(sw),
-			 _XmMin(2, Prim_ShadowThickness(sw)),
-			 SCB_Arrow2Orientation(sw));
+	_XmDrawArrow(XtDisplay(sw),
+		     (Window)SCB_Pixmap(sw),
+		     Prim_BottomShadowGC(sw),
+		     Prim_TopShadowGC(sw),
+		     SCB_GC(sw),
+		     SCB_Arrow2X(sw),
+		     SCB_Arrow2Y(sw),
+		     SCB_ArrowWidth(sw),
+		     SCB_ArrowHeight(sw),
+		     Prim_ShadowThickness(sw) == 0 ? 0 : 2,
+		     SCB_Arrow2Orientation(sw));
     }
 }
 
@@ -845,18 +887,15 @@ expose(Widget w,
        Region region)
 {
     if (XtIsRealized(w))
+
 	XCopyArea(XtDisplay(w),
 		  SCB_Pixmap(w),
 		  XtWindow(w),
-		  SCB_ForegroundGC(w),
+		  SCB_GC(w),
 		  0,0,
 		  XtWidth(w),
 		  XtHeight(w),
 		  0,0);
-
-#define superclass (&xmPrimitiveClassRec)
-    (*superclass->core_class.expose)(w, event, region);
-#undef superclass
 }
 
 static void 
@@ -904,7 +943,8 @@ check_pixel_constraints(Widget sw)
 static void 
 DoLayout(Widget sw, Boolean first, Boolean resize)
 {
-    int add_space;
+    int arrow_width;
+    int arrow_height;
 
     if (resize) {
 	_XmFreeScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(sw)),
@@ -916,39 +956,57 @@ DoLayout(Widget sw, Boolean first, Boolean resize)
 					       XtHeight(sw) ? XtHeight(sw) : 1);
     }
 
-    SCB_ArrowWidth(sw) = SCB_ArrowHeight(sw) = _XmScrollBarArrowSize(sw);
+    arrow_width = _XmScrollBarArrowWidth(sw);
+    arrow_height = _XmScrollBarArrowHeight(sw);
 
-    if (SCB_ShowArrows(sw))
-	add_space = ARROW_SPACING;
-    else
-	add_space = 0;
+    SCB_ArrowWidth(sw) = (Dimension) _XmMax(1, arrow_width);
+    SCB_ArrowHeight(sw) = (Dimension) _XmMax(1, arrow_height);
 
     if (SCB_Orientation(sw) == XmHORIZONTAL)
     {
-	SCB_SliderAreaX(sw) = SCB_ArrowWidth(sw) +
-				Prim_ShadowThickness(sw) +
-				Prim_HighlightThickness(sw) +
-				add_space;
-	SCB_SliderAreaY(sw) = Prim_ShadowThickness(sw) +
-				Prim_HighlightThickness(sw);
+	if (SCB_ShowArrows(sw)) {
+	    SCB_SliderAreaX(sw) = arrow_width +
+				  Prim_ShadowThickness(sw) +
+				  Prim_HighlightThickness(sw) +
+				  ARROW_SPACING;
+	    SCB_SliderAreaY(sw) = (XtHeight(sw) - SCB_ArrowHeight(sw) + 1) / 2;
 
-	SCB_SliderAreaWidth(sw) = XtWidth(sw) -
-					2 * (SCB_ArrowWidth(sw) +
-					     Prim_ShadowThickness(sw) +
-					     Prim_HighlightThickness(sw) +
-					     add_space);
-	SCB_SliderAreaHeight(sw) = XtHeight(sw) -
+	    SCB_SliderAreaWidth(sw) = XtWidth(sw) -
+					    2 * (arrow_width +
+						 Prim_ShadowThickness(sw) +
+						 Prim_HighlightThickness(sw) +
+						 ARROW_SPACING);
+	    SCB_SliderAreaHeight(sw) = SCB_ArrowHeight(sw);
+
+	    SCB_Arrow1Y(sw) = SCB_Arrow2Y(sw) = (XtHeight(sw) - arrow_height) / 2;
+	    SCB_Arrow1X(sw) = Prim_ShadowThickness(sw) +
+			      Prim_HighlightThickness(sw);
+
+	    SCB_Arrow2X(sw) = XtWidth(sw) -
+				    (Prim_ShadowThickness(sw) +
+				     Prim_HighlightThickness(sw) +
+				     arrow_width);
+        }
+	else /* ! showArrows */ {
+	    SCB_ArrowWidth(sw) = 0;
+	    SCB_ArrowHeight(sw) = 0;
+
+	    SCB_SliderAreaX(sw) = Prim_HighlightThickness(sw) +
+				  Prim_ShadowThickness(sw);
+	    SCB_SliderAreaY(sw) = (XtHeight(sw) - SCB_SliderAreaHeight(sw) + 1) / 2;
+
+	    SCB_SliderAreaWidth(sw) = _XmMax(1, XtWidth(sw) -
 					2 * (Prim_ShadowThickness(sw) +
-					     Prim_HighlightThickness(sw));
+					     Prim_HighlightThickness(sw)));
+	    SCB_SliderAreaHeight(sw) = _XmMax(1, XtHeight(sw) -
+					2 * (Prim_ShadowThickness(sw) +
+					     Prim_HighlightThickness(sw)));
 
-	SCB_Arrow1Y(sw) = SCB_Arrow2Y(sw) =
-			 (XtHeight(sw) - SCB_ArrowHeight(sw)) / 2;
-	SCB_Arrow1X(sw) = Prim_ShadowThickness(sw) +
-			  Prim_HighlightThickness(sw);
-	SCB_Arrow2X(sw) = XtWidth(sw) -
-				(Prim_ShadowThickness(sw) +
-				 Prim_HighlightThickness(sw) +
-				 SCB_ArrowWidth(sw));
+	    SCB_Arrow1X(sw) = 0;
+	    SCB_Arrow1Y(sw) = 0;
+	    SCB_Arrow2X(sw) = 0;
+	    SCB_Arrow2Y(sw) = 0;
+	}
 
 	SCB_SliderHeight(sw) = SCB_SliderAreaHeight(sw);
 	SCB_SliderWidth(sw) = _XmScrollBarSliderPixSize(sw);
@@ -959,32 +1017,51 @@ DoLayout(Widget sw, Boolean first, Boolean resize)
 	SCB_SliderX(sw) = _XmScrollBarValueToPos(sw, SCB_Value(sw));
 	SCB_SliderY(sw) = SCB_SliderAreaY(sw);
     }
-    else
+    else /* vertical */
     {
-	SCB_SliderAreaY(sw) = SCB_ArrowHeight(sw) +
-				Prim_ShadowThickness(sw) +
-				Prim_HighlightThickness(sw) +
-				add_space;
-	SCB_SliderAreaX(sw) = Prim_ShadowThickness(sw) +
+	if (SCB_ShowArrows(sw)) {
+	    SCB_SliderAreaY(sw) = arrow_height +
+				  Prim_ShadowThickness(sw) +
+				  Prim_HighlightThickness(sw) +
+				  ARROW_SPACING;
+	    SCB_SliderAreaX(sw) = (XtWidth(sw) - SCB_ArrowWidth(sw) + 1) / 2;
+
+	    SCB_SliderAreaHeight(sw) = XtHeight(sw) -
+					    2 * (arrow_height +
+						 Prim_ShadowThickness(sw) +
+						 Prim_HighlightThickness(sw) +
+						 ARROW_SPACING);
+	    SCB_SliderAreaWidth(sw) = SCB_ArrowWidth(sw);
+
+	    SCB_Arrow1X(sw) = SCB_Arrow2X(sw) = (XtWidth(sw) - arrow_width) / 2;
+	    SCB_Arrow1Y(sw) = Prim_ShadowThickness(sw) +
 			      Prim_HighlightThickness(sw);
+	    SCB_Arrow2Y(sw) = XtHeight(sw) -
+				    (Prim_ShadowThickness(sw) +
+				     Prim_HighlightThickness(sw) +
+				     arrow_height);
 
-	SCB_SliderAreaHeight(sw) = XtHeight(sw) -
-					2 * (SCB_ArrowHeight(sw) +
-					     Prim_ShadowThickness(sw) +
-					     Prim_HighlightThickness(sw) +
-					     add_space);
-	SCB_SliderAreaWidth(sw) = XtWidth(sw) -
+        }
+	else { /* ! showArrows */
+	    SCB_ArrowHeight(sw) = 0;
+	    SCB_ArrowWidth(sw) = 0;
+
+	    SCB_SliderAreaY(sw) = Prim_HighlightThickness(sw) +
+				  Prim_ShadowThickness(sw);
+	    SCB_SliderAreaX(sw) = (XtWidth(sw) - SCB_SliderAreaWidth(sw) + 1) / 2;
+
+	    SCB_SliderAreaHeight(sw) = _XmMax(1, XtHeight(sw) -
 					2 * (Prim_ShadowThickness(sw) +
-					     Prim_HighlightThickness(sw));
+					     Prim_HighlightThickness(sw)));
+	    SCB_SliderAreaWidth(sw) = _XmMax(1, XtWidth(sw) -
+					2 * (Prim_ShadowThickness(sw) +
+					     Prim_HighlightThickness(sw)));
 
-	SCB_Arrow1X(sw) = SCB_Arrow2X(sw) =
-			 (XtWidth(sw) - SCB_ArrowWidth(sw)) / 2;
-	SCB_Arrow1Y(sw) = Prim_ShadowThickness(sw) +
-			  Prim_HighlightThickness(sw);
-	SCB_Arrow2Y(sw) = XtHeight(sw) -
-				(Prim_ShadowThickness(sw) +
-				 Prim_HighlightThickness(sw) +
-				 SCB_ArrowHeight(sw));
+	    SCB_Arrow1Y(sw) = 0;
+	    SCB_Arrow1X(sw) = 0;
+	    SCB_Arrow2Y(sw) = 0;
+	    SCB_Arrow2X(sw) = 0;
+	}
 
 	SCB_SliderWidth(sw) = SCB_SliderAreaWidth(sw);
 	SCB_SliderHeight(sw) = _XmScrollBarSliderPixSize(sw);
@@ -1125,29 +1202,25 @@ XmScrollBarSetValues(Widget sw,
 static void
 erase_slider(Widget sw)
 {
-    XSetForeground(XtDisplay(sw),
-		   SCB_UnavailableGC(sw),
-		   SCB_TroughColor(sw));
+    XSetForeground(XtDisplay(sw), SCB_GC(sw), SCB_TroughColor(sw));
 
     XFillRectangle(XtDisplay(sw),
 		   SCB_Pixmap(sw),
-		   SCB_UnavailableGC(sw),
-		   SCB_SliderX(sw) , 
-		   SCB_SliderY(sw) , 
+		   SCB_GC(sw),
+		   SCB_SliderX(sw), 
+		   SCB_SliderY(sw), 
 		   SCB_SliderWidth(sw),
 		   SCB_SliderHeight(sw));
+
+    XSetForeground(XtDisplay(sw), SCB_GC(sw), XtBackground(sw));
 }
 
 static void
 draw_slider(Widget sw)
 {
-    XSetForeground(XtDisplay(sw),
-		   SCB_ForegroundGC(sw),
-		   XtBackground(sw));
-
     XFillRectangle(XtDisplay(sw),
 		   SCB_Pixmap(sw),
-		   SCB_ForegroundGC(sw),
+		   SCB_GC(sw),
 		   SCB_SliderX(sw),
 		   SCB_SliderY(sw), 
 		   SCB_SliderWidth(sw) , 
@@ -1177,7 +1250,7 @@ draw_slider(Widget sw)
 	    sep_y = SCB_SliderY(sw) + Prim_ShadowThickness(sw) / 2;
 	    _XmDrawSeparator(XtDisplay(sw), (Window)SCB_Pixmap(sw),
 			     Prim_TopShadowGC(sw), Prim_BottomShadowGC(sw),
-			     SCB_ForegroundGC(sw),
+			     SCB_GC(sw),
 			     sep_x, sep_y,
 			     Prim_ShadowThickness(sw),
 			     SCB_SliderHeight(sw) - Prim_ShadowThickness(sw),
@@ -1191,7 +1264,7 @@ draw_slider(Widget sw)
 	    sep_x = SCB_SliderX(sw) + Prim_ShadowThickness(sw) / 2;
 	    _XmDrawSeparator(XtDisplay(sw), (Window)SCB_Pixmap(sw),
 			     Prim_TopShadowGC(sw), Prim_BottomShadowGC(sw),
-			     SCB_ForegroundGC(sw),
+			     SCB_GC(sw),
 			     sep_x, sep_y,
 			     SCB_SliderWidth(sw) - Prim_ShadowThickness(sw),
 			     Prim_ShadowThickness(sw),
@@ -1300,7 +1373,7 @@ Select(Widget sw,
     }
 
     /* Testing for the button defeats the translation scheme, yet 
-       this is how Motif seems to do it.  hep@mart.nl 
+       this is how Motif seems to do it.  -- PvH 
      */
 
     else if (ev->button != 1)
@@ -1589,7 +1662,7 @@ TopOrBottom(Widget sw,
 
 	/* This is per Motif: osfBeginLine results in a toTopCallback
 	 * whereas all oher keys bound to TopOrBottom() result in a
-	 * toBottomCallback -- hep@mart.nl
+	 * toBottomCallback -- PvH
 	 */
 
 	toTopCallback = (keysym == XStringToKeysym(&_XmStrings[12304]));
@@ -1902,7 +1975,7 @@ IncrementUpOrLeft(Widget w,
 /* The FLG_DRAG test lines in the next functions were introduced to make
  * CancelDrag work. They prevent key (page)?(inc/dec)rementing to
  * work. Well, they seem to be redundant anyway because of the canceled() test
- * in the button timer callbacks. -- hep@mart.nl
+ * in the button timer callbacks. -- PvH
  */
 static void
 increment(Widget sw, XEvent *event)
@@ -2257,8 +2330,9 @@ _XmScrollBarPosToValue(Widget sw, Position pos)
 	SCB_ProcessingDirection(sw) == XmMAX_ON_RIGHT) {
 	switch (SCB_Orientation(sw)) {
 	case XmHORIZONTAL:
-	    if ((SCB_Maximum(sw) - SCB_Minimum(sw) - SCB_SliderSize(sw)) == 0)
-		return 0;
+	    if (SCB_SliderAreaWidth(sw) <= SCB_SliderWidth(sw))
+		return SCB_Maximum(sw) - SCB_SliderSize(sw); /* FIX ME: return what? == instead of <= ?? */
+
 	    return (int)((float)(pos - SCB_SliderAreaX(sw)) /
 			 (float)(SCB_SliderAreaWidth(sw) -
 				  SCB_SliderWidth(sw)) *
@@ -2266,8 +2340,9 @@ _XmScrollBarPosToValue(Widget sw, Position pos)
 				 SCB_SliderSize (sw)) +
 			 (float)SCB_Minimum(sw) + 0.5);
 	case XmVERTICAL:
-	    if ((SCB_Maximum(sw) - SCB_Minimum(sw) - SCB_SliderSize(sw)) == 0)
-		return 0;
+	    if (SCB_SliderAreaHeight(sw) <= SCB_SliderHeight(sw))
+		return SCB_Maximum(sw) - SCB_SliderSize(sw); /* FIX ME: return what? == instead of <= ?? */
+
 	    return (int)((float)(pos - SCB_SliderAreaY(sw)) /
 			 (float)(SCB_SliderAreaHeight(sw) -
 				 SCB_SliderHeight(sw)) *
@@ -2327,13 +2402,15 @@ _XmScrollBarSliderPixSize(Widget sw) {
 	if ((minmax - SCB_SliderSize(sw)) == 0)
 	    return SCB_SliderAreaWidth(sw);
 	ret = (Dimension)(((float)SCB_SliderSize(sw) / (float)minmax) *
-			   (float)SCB_SliderAreaWidth(sw));
+			   (float)SCB_SliderAreaWidth(sw) +
+			  0.5);
 	return ret;
     case XmVERTICAL:
 	if ((minmax - SCB_SliderSize(sw)) == 0)
 	    return SCB_SliderAreaHeight(sw);
 	ret = (Dimension)(((float)SCB_SliderSize(sw) / (float)minmax) *
-			   (float)SCB_SliderAreaHeight(sw));
+			   (float)SCB_SliderAreaHeight(sw) +
+			  0.5);
 	return ret;
     default:
 	_XmError((Widget)sw,
@@ -2343,51 +2420,80 @@ _XmScrollBarSliderPixSize(Widget sw) {
     return 0;
 }
 
-static Dimension
-_XmScrollBarArrowSize(Widget w)
+static int
+_XmScrollBarArrowWidth(Widget w)
 {
-    int size = 0, oversize;
-
-    if (SCB_ShowArrows(w) == False)
-	return 0;
+    int size = 0, hor_size;
 
     if (SCB_Orientation(w) == XmHORIZONTAL) {
 	if (XtHeight(w) == 0)
-	    return (Dimension)DEF_SB_DIM;
+	    return DEF_SB_DIM;
 
+	/*
+	 * Don't change these three lines lightly. Remember that a + b/2 need
+	 * not equal (2*a + b)/2, depending on the rounding in case of negative
+	 * dividends. -- PvH
+	 */
 	size = (int)XtHeight(w) -
 		2 * (int)(Prim_ShadowThickness(w) + Prim_HighlightThickness(w));
-	oversize = (int)(XtWidth(w) -
-			(int)(2 * size + DEF_SB_DIM +
+
+	hor_size = (int)(XtWidth(w) -
+			(int)(MIN_SLIDER_LENGTH + 
+			      2 * ARROW_SPACING +
 			      2 * Prim_ShadowThickness(w) +
 			      2 * Prim_HighlightThickness(w))) / 2;
 
-	if (oversize < 0)
-	    size += oversize;
-
-	if (size < DEF_SB_DIM)
-	    size = DEF_SB_DIM;
-
+	if (hor_size < size)
+	    size = hor_size;
     }
     else if (SCB_Orientation(w) == XmVERTICAL) {
 	if (XtWidth(w) == 0)
-	    return (Dimension)DEF_SB_DIM;
+	    return DEF_SB_DIM;
 
 	size = (int)XtWidth(w) -
-		2 * (int)(Prim_ShadowThickness(w) + Prim_HighlightThickness(w));
-	oversize = (int)(XtHeight(w) -
-			(int)(2 * size + DEF_SB_DIM +
-			      2 * Prim_ShadowThickness(w) +
-			      2 * Prim_HighlightThickness(w))) / 2;
-
-	if (oversize < 0)
-	    size += oversize;
-
-	if (size < DEF_SB_DIM)
-	    size = DEF_SB_DIM;
+	        2 * (int)(Prim_ShadowThickness(w) + Prim_HighlightThickness(w));
     }
     else
 	_XmError((Widget)w, "ScrollBar Orientation is incorrect");
 
-    return (Dimension)size;
+    return size;
+}
+
+static int
+_XmScrollBarArrowHeight(Widget w)
+{
+    int size = 0, ver_size;
+
+    if (SCB_Orientation(w) == XmHORIZONTAL) {
+	if (XtHeight(w) == 0)
+	    return DEF_SB_DIM;
+
+	size = (int)XtHeight(w) -
+		2 * (int)(Prim_ShadowThickness(w) + Prim_HighlightThickness(w));
+    }
+    else if (SCB_Orientation(w) == XmVERTICAL) {
+	if (XtWidth(w) == 0)
+	    return DEF_SB_DIM;
+
+	/*
+	 * Don't change these three lines lightly. Remember that a + b/2 need
+	 * not equal (2*a + b)/2, depending on the rounding in case of negative
+	 * dividends. -- PvH
+	 */
+	size = (int)XtWidth(w) -
+		2 * (int)(Prim_ShadowThickness(w) + Prim_HighlightThickness(w));
+
+	ver_size = (int)(XtHeight(w) -
+			(int)(MIN_SLIDER_LENGTH + 
+			      2 * ARROW_SPACING +
+			      2 * Prim_ShadowThickness(w) +
+			      2 * Prim_HighlightThickness(w))) / 2;
+
+	if (ver_size < size)
+	    size = ver_size;
+    }
+    else
+	_XmError((Widget)w, "ScrollBar Orientation is incorrect");
+
+    return size;
 }
