@@ -1,6 +1,6 @@
 /**
  *
- * $Id: SelectionBox.c,v 1.31 1996/04/23 05:52:09 u27113 Exp $
+ * $Id: SelectionBox.c,v 1.13 1997/01/11 02:19:52 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,13 +22,18 @@
  *
  **/
 
-static char rcsid[] = "$Id: SelectionBox.c,v 1.31 1996/04/23 05:52:09 u27113 Exp $";
+static char rcsid[] = "$Id: SelectionBox.c,v 1.13 1997/01/11 02:19:52 miers Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
 #include <Xm/SelectioBP.h>
-#include <Xm/DebugUtil.h>
+#include <Xm/CommandP.h>
 #include <Xm/List.h>
+#include <Xm/PushBP.h>
+#include <Xm/PushBGP.h>
+#include <Xm/RowColumnP.h>
 #ifdef USE_WIDGETS
 #include <Xm/Separator.h>
 #else
@@ -36,21 +41,26 @@ static char rcsid[] = "$Id: SelectionBox.c,v 1.31 1996/04/23 05:52:09 u27113 Exp
 #endif
 #include <Xm/DialogS.h>
 #include <Xm/TextF.h>
-#include <Xm/XmI.h>
-#include <stdio.h>
+
+#include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
-static XtGeometryResult geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static void InsertChild(Widget w);
 
 XmGeoMatrix _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref);
 Boolean _XmSelectionBoxNoGeoRequest(XmGeoMatrix _geoSpec);
 
+void _XmSelectionBoxGetListItems(Widget wid, int resource_offset, XtArgVal *value);
+void _XmSelectionBoxGetListItemCount(Widget wid, int resource_offset, XtArgVal *value);
+void _XmSelectionBoxGetTextColumns(Widget wid, int resource_offset, XtArgVal *value);
+void _XmSelectionBoxGetTextString(Widget wid, int resource_offset, XtArgVal *value);
+
 /*
  * Resources for the Selection Box class
  */
@@ -162,32 +172,32 @@ static XmSyntheticResource syn_resources[] = {
     {
 	XmNselectionLabelString,
 	sizeof(XmString), Offset(selection_label_string),
-	NULL /* FIXME */, NULL /* FIXME */
+	_XmExportXmString, NULL /* FIXME */
     },
     {
 	XmNlistLabelString,
 	sizeof(XmString), Offset(list_label_string),
-	NULL /* FIXME */, NULL /* FIXME */
+	_XmExportXmString, NULL /* FIXME */
     },
     {
 	XmNtextColumns,
 	sizeof(short), Offset(text_columns),
-	NULL /* FIXME */, NULL
+	_XmSelectionBoxGetTextColumns, NULL
     },
     {
 	XmNtextString,
 	sizeof(XmString), Offset(text_string),
-	NULL /* FIXME */, NULL /* FIXME */
+	_XmSelectionBoxGetTextString, NULL /* FIXME */
     },
     {
 	XmNlistItems,
 	sizeof(XmStringTable), Offset(list_items),
-	NULL /* FIXME */, NULL /* FIXME */
+	_XmSelectionBoxGetListItems, NULL /* FIXME */
     },
     {
 	XmNlistItemCount,
 	sizeof(int), Offset(list_item_count),
-	NULL /* FIXME */, NULL /* FIXME */
+	_XmSelectionBoxGetListItemCount, NULL /* FIXME */
     },
     {
 	XmNlistVisibleItemCount,
@@ -197,22 +207,22 @@ static XmSyntheticResource syn_resources[] = {
     {
 	XmNokLabelString,
 	sizeof(XmString), Offset(ok_label_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNapplyLabelString,
 	sizeof(XmString), Offset(apply_label_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNcancelLabelString,
 	sizeof(XmString), Offset(cancel_label_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     },
     {
 	XmNhelpLabelString,
 	sizeof(XmString), Offset(help_label_string),
-	NULL /* FIXME */, NULL
+	_XmExportXmString, NULL
     }
 };
 
@@ -250,8 +260,8 @@ static XmBaseClassExtRec _XmSelectionBCoreClassExtRec = {
     /* compiled_ext_resources    */ NULL, /* FIXME */
     /* num_ext_resources         */ 0, /* FIXME */
     /* use_sub_resources         */ FALSE, /* FIXME */
-    /* widget_navigable          */ NULL, /* FIXME */
-    /* focus_change              */ NULL, /* FIXME */
+    /* widget_navigable          */ XmInheritWidgetNavigable,
+    /* focus_change              */ XmInheritFocusChange,
     /* wrapper_data              */ NULL
 };
 
@@ -281,7 +291,7 @@ XmSelectionBoxClassRec xmSelectionBoxClassRec = {
 	/* num_resources         */ XtNumber(resources),
 	/* xrm_class             */ NULLQUARK,
 	/* compress_motion       */ TRUE,
-	/* compress_exposure     */ XtExposeCompressMultiple,
+	/* compress_exposure     */ XtExposeCompressMaximal,
 	/* compress_enterleave   */ TRUE,
 	/* visible_interest      */ FALSE,
 	/* destroy               */ destroy,
@@ -295,16 +305,16 @@ XmSelectionBoxClassRec xmSelectionBoxClassRec = {
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ XtInheritTranslations,
-	/* query_geometry        */ NULL,
+	/* query_geometry        */ XtInheritQueryGeometry,
 	/* display_accelerator   */ NULL,
 	/* extension             */ (XtPointer)&_XmSelectionBCoreClassExtRec
     },
     /* Composite class part */
     {
-	/* geometry manager */ geometry_manager, 
+	/* geometry manager */ XtInheritGeometryManager, 
         /* change_managed   */ XtInheritChangeManaged, 
-        /* insert_child     */ XtInheritInsertChild,
-        /* delete_child     */ XtInheritInsertChild,
+        /* insert_child     */ InsertChild,
+        /* delete_child     */ XtInheritDeleteChild,
         /* extension        */ NULL,
     },
     /* Constraint class part */
@@ -359,7 +369,7 @@ class_part_initialize(WidgetClass widget_class)
  * Determine if the selection is a match with one of the
  * items in the list
  */
-static Boolean
+Boolean
 _XmSelectionBoxMatch(XmSelectionBoxWidget w)
 {
     char		*s;
@@ -397,6 +407,11 @@ ListSingleSelect(Widget w, XtPointer client, XtPointer call)
     XmListCallbackStruct	*lp = (XmListCallbackStruct *)call;
 
     if (SB_Text(sb) == NULL) {
+	XdbDebug(__FILE__, w, "ListSingleSelect (No TextField)\n");
+	return;
+    }
+
+    if (lp == NULL || lp->item == NULL) {
 	XdbDebug(__FILE__, w, "ListSingleSelect (NULL)\n");
 	return;
     }
@@ -422,6 +437,7 @@ static void _XmSbButton(Widget w, XtPointer client, XtPointer call)
 	XmSelectionBoxWidget	sb = (XmSelectionBoxWidget) XtParent(w);
 	XmAnyCallbackStruct		*a = (XmAnyCallbackStruct *) call;
 	XmSelectionBoxCallbackStruct	cbs;
+	int				au = 0;
 
 	XdbDebug(__FILE__, w, "_XmSbButton\n");
 
@@ -429,12 +445,14 @@ static void _XmSbButton(Widget w, XtPointer client, XtPointer call)
 	cbs.value = (XmString)0;
 	cbs.length = 0;
 
-	if (w == SB_OkButton(sb) || w == SB_List(sb)) {
-	    if (w == SB_OkButton(sb)) {
+	if (w == SB_OkButton(sb) || w == SB_List(sb) || XmIsTextField(w)) {
+	    if (w == SB_OkButton(sb) || XmIsTextField(w)) {
 		char	*s = XmTextFieldGetString(SB_Text(sb));
 
 		cbs.value = XmStringCreateSimple(s);
 		XtFree(s);
+
+		au++;
 	    } else {
 		XmListCallbackStruct	*ls = (XmListCallbackStruct *) call;
 		cbs.value = ls->item;
@@ -445,6 +463,8 @@ static void _XmSbButton(Widget w, XtPointer client, XtPointer call)
 		if (_XmSelectionBoxMatch(sb)) {
 		    cbs.reason = XmCR_OK;
 		    XtCallCallbackList((Widget)sb, SB_OkCallback(sb), &cbs);
+
+		    au++;
 		} else {
 		    cbs.reason = XmCR_NO_MATCH;
 		    XtCallCallbackList((Widget)sb, SB_NoMatchCallback(sb), &cbs);
@@ -452,6 +472,8 @@ static void _XmSbButton(Widget w, XtPointer client, XtPointer call)
 	    } else {
 		cbs.reason = XmCR_OK;
 		XtCallCallbackList((Widget)sb, SB_OkCallback(sb), &cbs);
+
+		au++;
 	    }
 	} else if (w == BB_CancelButton(sb)) {
 		cbs.reason = XmCR_CANCEL;
@@ -464,6 +486,19 @@ static void _XmSbButton(Widget w, XtPointer client, XtPointer call)
 		cbs.reason = XmCR_HELP;
 		XtCallCallbackList((Widget)sb, sb->manager.help_callback, &cbs);
 	}
+
+    if (BB_AutoUnmanage(sb) && au && XmIsTextField(w)) {
+        Widget  s = XtParent((Widget)sb);
+        
+        if (XtIsSubclass(s, xmDialogShellWidgetClass)) {
+            XtUnmanageChild((Widget)sb);
+            XdbDebug2(__FILE__, (Widget)sb, w, "AutoUnmanage\n");
+        
+            /* XtNpopdownCallback */  
+            XtCallCallbackList(s, Shell_PopdownCallback(s), NULL);
+        }
+    }   
+
 }
 
 /*
@@ -477,13 +512,13 @@ static void _XmSbButton(Widget w, XtPointer client, XtPointer call)
  */
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XmSelectionBoxWidget sb = (XmSelectionBoxWidget)new;
+    XmSelectionBoxWidget sb = (XmSelectionBoxWidget)new_w;
 
-    XdbDebug(__FILE__, new, "XmSelectionBox initialize\n");
+    XdbDebug(__FILE__, new_w, "XmSelectionBox initialize\n");
 
     /*
      * Dynamically determine XmNdialogType
@@ -503,6 +538,9 @@ initialize(Widget request,
 	_XmSelectionBoxCreateList(sb);
     }
 
+    _XmSelectionBoxCreateSelectionLabel(sb);
+    _XmSelectionBoxCreateText(sb);
+
     SB_Separator(sb) = NULL;
     if (SB_DialogType(sb) != XmDIALOG_COMMAND) {
 
@@ -525,13 +563,12 @@ initialize(Widget request,
 	SB_HelpLabelString(sb) = NULL;
     }
 
-    _XmSelectionBoxCreateSelectionLabel(sb);
-    _XmSelectionBoxCreateText(sb);
-
+#ifdef	DO_FLUSH
     XFlush(XtDisplay(SB_Text(sb)));
+#endif
 
     if (SB_DialogType(sb) != XmDIALOG_COMMAND)
-	XtVaSetValues(new, XmNdefaultButton, SB_OkButton(sb), NULL);
+	XtVaSetValues(new_w, XmNdefaultButton, SB_OkButton(sb), NULL);
 }
 
 static void
@@ -543,41 +580,29 @@ destroy(Widget w)
  *	that we've allocated, such as for XmString fields and such more.
  * Also GC's etc.
  */
-#ifdef	notdef
-	XmSelectionBoxWidget sb = (XmSelectionBoxWidget)w;
-
-/* Must destroy all existing child widgets here */
-#define	WD(x)	 if (x(sb))	XtDestroyWidget(x(sb))
-
-	if (SB_List(sb)) XtDestroyWidget(XtParent(SB_List(sb)));
-	WD(SB_ListLabel);
-	WD(SB_Separator);
-	WD(SB_OkButton);
-	if (XtIsManaged(BB_CancelButton(sb)))
-		XtDestroyWidget(BB_CancelButton(sb));
-	WD(SB_ApplyButton);
-	WD(SB_HelpButton);
-	WD(SB_Text);
-	WD(SB_SelectionLabel);
-#endif
 }
 
+/*
+ * handle element updates
+ */
+#define	NE(x)	(x(o) != x(sb))
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XmSelectionBoxWidget	sb = (XmSelectionBoxWidget)new;
+    XmSelectionBoxWidget	sb = (XmSelectionBoxWidget)new_w;
     XmSelectionBoxWidget	o = (XmSelectionBoxWidget)old;
     Boolean			r = False;		/* whether to redisplay */
     Arg				al[10];
     int				ac;
 
-#define	NE(x)	(x(o) != x(sb))
+    /* this is required for every BB subclass */
+    BB_InSetValues(new_w) = False;
 
-/* XmList */
+    /* XmList */
     ac = 0;
     if (NE(SB_ListItems) || NE(SB_ListItemCount)) {
 	XtSetArg(al[ac], XmNitems, SB_ListItems(sb)); ac++;
@@ -587,7 +612,7 @@ set_values(Widget old,
     if (ac && SB_List(sb))
 	XtSetValues(SB_List(sb), al, ac);
 
-/* Labels : List Label */
+    /* Labels : List Label */
     ac = 0;
     if (NE(SB_ListLabelString)) {
 	XtSetArg(al[ac], XmNlabelString, SB_ListLabelString(sb)); ac++;
@@ -596,7 +621,7 @@ set_values(Widget old,
     if (ac && SB_ListLabel(sb))
 	XtSetValues(SB_ListLabel(sb), al, ac);
 
-/* Labels : OK Button Label */
+    /* Labels : OK Button Label */
     ac = 0;
     if (NE(SB_OkLabelString)) {
 	XtSetArg(al[ac], XmNlabelString, SB_OkLabelString(sb)); ac++;
@@ -605,7 +630,7 @@ set_values(Widget old,
     if (ac && SB_OkButton(sb))
 	XtSetValues(SB_OkButton(sb), al, ac);
 
-/* Labels : Cancel Button Label */
+    /* Labels : Cancel Button Label */
     ac = 0;
     if (NE(SB_CancelLabelString)) {
 	XtSetArg(al[ac], XmNlabelString, SB_CancelLabelString(sb)); ac++;
@@ -614,7 +639,7 @@ set_values(Widget old,
     if (ac && SB_CancelButton(sb))
 	XtSetValues(BB_CancelButton(sb), al, ac);
 
-/* Labels : Apply Button Label */
+    /* Labels : Apply Button Label */
     ac = 0;
     if (NE(SB_ApplyLabelString)) {
 	XtSetArg(al[ac], XmNlabelString, SB_ApplyLabelString(sb)); ac++;
@@ -623,7 +648,7 @@ set_values(Widget old,
     if (ac && SB_ApplyButton(sb))
 	XtSetValues(SB_ApplyButton(sb), al, ac);
 
-/* Labels : Help Button Label */
+    /* Labels : Help Button Label */
     ac = 0;
     if (NE(SB_HelpLabelString)) {
 	XtSetArg(al[ac], XmNlabelString, SB_HelpLabelString(sb)); ac++;
@@ -632,40 +657,17 @@ set_values(Widget old,
     if (ac && SB_HelpButton(sb))
 	XtSetValues(SB_HelpButton(sb), al, ac);
 
+    BB_InSetValues(new_w) = False;
+
+    if (r && (XtClass(new_w) == xmSelectionBoxWidgetClass))
+    {
+	_XmBulletinBoardSizeUpdate(new_w);
+	return False;
+    }
+
     return r;
 }
 
-static XtGeometryResult
-geometry_manager(Widget w,
-		 XtWidgetGeometry *request,
-		 XtWidgetGeometry *reply)
-{
-	XmSelectionBoxWidget	sb = (XmSelectionBoxWidget) XtParent(w);
-	XtGeometryResult	r = XtGeometryYes;
-
-	if (reply) {
-		*reply = *request;
-		reply->request_mode &= CWWidth | CWHeight;
-	}
-
-#define	Wants(x)	(request->request_mode & x)
-
-	if (SB_List(sb) && w == XtParent(SB_List(sb))) {
-		Dimension	W = XtWidth(sb) - 2 * BB_MarginWidth(sb);
-
-		if (Wants(CWWidth) && request->width != W) {
-			reply->width = W;
-			r = XtGeometryAlmost;
-		}
-	}
-
-	XdbDebug2(__FILE__, (Widget)sb, w, "GeometryManager [%d %d => %d %d, %s]\n",
-		request->width, request->height, reply->width, reply->height,
-		XdbGeometryResult2String(r));
-
-	return r;
-}
-
 /*
  * MLM - I rather suspect that the following functions are an attempt to
  * foil the user that changes the dialog type of an existing selbox
@@ -712,22 +714,22 @@ void
 _XmSelectionBoxCreateList(XmSelectionBoxWidget sb)
 {
     int ac;
-    Arg al[2];
+    Arg al[4];
 
     ac = 0;
-    XtSetArg(al[ac],
-	     XmNvisibleItemCount, SB_ListVisibleItemCount(sb)); ac++;
-    XtSetArg(al[ac],
-	     XmNselectionPolicy, XmSINGLE_SELECT); ac++;
+    XtSetArg(al[ac], XmNvisibleItemCount, SB_ListVisibleItemCount(sb)); ac++;
+    XtSetArg(al[ac], XmNselectionPolicy, XmBROWSE_SELECT); ac++;
+    XtSetArg(al[ac], XmNlistSizePolicy, XmCONSTANT); ac++;
+    XtSetArg(al[ac], XmNscrollBarDisplayPolicy, XmAS_NEEDED); ac++;
     SB_List(sb) = XmCreateScrolledList((Widget)sb, "itemsList", al, ac);
     XtManageChild(SB_List(sb));
-    XtManageChild(XtParent(SB_List(sb)));	/* FIX ME */
-    /* The above should be handled in XmScrolledWindow's manage_child */
 
     XtAddCallback(SB_List(sb),
 	 	  XmNdefaultActionCallback, _XmSbButton, NULL);
     XtAddCallback(SB_List(sb),
 		  XmNsingleSelectionCallback, ListSingleSelect, sb);
+    XtAddCallback(SB_List(sb),
+		  XmNbrowseSelectionCallback, ListSingleSelect, sb);
 }
 
 void
@@ -775,7 +777,7 @@ _XmSelectionBoxCreateApplyButton(XmSelectionBoxWidget sb)
     SB_ApplyButton(sb) = _XmBB_CreateButtonG((Widget)sb,
 					     SB_ApplyLabelString(sb),
 					     "Apply");
-	/* get rid of auto_unmanage */
+    /* get rid of auto_unmanage */
     XtRemoveAllCallbacks(SB_ApplyButton(sb), XmNactivateCallback);
     XtAddCallback(SB_ApplyButton(sb),
 		  XmNactivateCallback, _XmSbButton, NULL);
@@ -828,24 +830,66 @@ void
 _XmSelectionBoxGetTextColumns(Widget wid, int resource_offset,
 			      XtArgVal *value)
 {
+	Arg		a;
+	int		n;
+
+	XdbDebug(__FILE__, wid, "XmSelectionBoxGettTextColumns\n");
+
+	XtSetArg(a, XmNtextColumns, &n);
+	n = -1;
+	XtGetValues(SB_Text(wid), &a, 1);
+/* Should this be a copy or the real thing ?? */
+	*value = (XtArgVal)n;
 }
 
 void
 _XmSelectionBoxGetTextString(Widget wid, int resource_offset,
 			     XtArgVal *value)
 {
+	XdbDebug(__FILE__, wid, "_XmSelectionBoxGetTextString\n");
+	*value = (XtArgVal) XmStringCreateSimple(XmTextFieldGetString(SB_Text(wid)));
 }
 
 void
 _XmSelectionBoxGetListItems(Widget wid, int resource_offset,
 			    XtArgVal *value)
 {
+	XmStringTable	l;
+	Arg		a;
+
+	XdbDebug(__FILE__, wid, "XmSelectionBoxGetListItems(_, %d, _)\n", resource_offset);
+
+	XtSetArg(a, XmNitems, &l);
+	l = NULL;
+	XtGetValues(SB_List(wid), &a, 1);
+/* Should this be a copy or the real thing ?? */
+/* MLM: This (I think) should be a copy, as the user might free it. Unless the
+ * List doesn't make a copy when you do the GetValues call.  You probably could
+ * use the access macros from ListP.h and make a copy of that, for better
+ * code efficiency.  Also, remember that if the List has a Synthetic resource
+ * for ListItems, and it makes a copy, then you've got a copy already if you've
+ * used XtGetValues()...
+ */
+	*value = (XtArgVal)l;
 }
 
 void
 _XmSelectionBoxGetListItemCount(Widget wid, int resource_offset,
 				XtArgVal *value)
 {
+	Arg		a;
+	int		n;
+
+	XdbDebug(__FILE__, wid, "XmSelectionBoxGetListItemCount\n");
+
+	XtSetArg(a, XmNitemCount, &n);
+	n = -1;
+	XtGetValues(SB_List(wid), &a, 1);
+/* Should this be a copy or the real thing ?? */
+/* MLM: I don't think this matters here.  Except that you may want to use
+ * the access macros from ListP.h, and avoid the GetValues overhead.
+ */
+	*value = (XtArgVal)n;
 }
 
 void
@@ -899,11 +943,71 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     register XmKidGeometry boxPtr;
     Cardinal numKids;
     Boolean newRow;
-    int nrows;
+    int nrows, i, nextras;
+    Widget *extras;
+
+    numKids = MGR_NumChildren(_w);
+
+    nextras = 0;
+    extras = NULL;
+    for (i = 0; i < numKids; i++)
+    {
+	/*
+	 * There seems to be no way to add a button to the button area
+	 * without first managing a child in the work area. Motif allows
+	 * me to create an "unmanaged" child in the work area so that extra
+	 * buttons will be in the proper place. -- rws
+	 */
+	if (MGR_Children(_w)[i] != SB_ListLabel(_w) &&
+	    (SB_List(_w)
+		? MGR_Children(_w)[i] != XtParent(SB_List(_w))
+		: True) &&
+	    MGR_Children(_w)[i] != SB_SelectionLabel(_w) &&
+	    MGR_Children(_w)[i] != SB_Text(_w) &&
+	    MGR_Children(_w)[i] != SB_Separator(_w) &&
+	    MGR_Children(_w)[i] != SB_OkButton(_w) &&
+	    MGR_Children(_w)[i] != SB_ApplyButton(_w) &&
+	    MGR_Children(_w)[i] != SB_HelpButton(_w) &&
+	    MGR_Children(_w)[i] != BB_CancelButton(_w))
+	{
+	    nextras++;
+	}
+    }
+
+    if (nextras)
+	extras = (Widget *)XtMalloc(sizeof(Widget) * nextras);
+
+    nextras = 0;
+    for (i = 0; i < numKids; i++)
+    {
+	if (MGR_Children(_w)[i] != SB_ListLabel(_w) &&
+	    (SB_List(_w)
+		? MGR_Children(_w)[i] != XtParent(SB_List(_w))
+		: True) &&
+	    MGR_Children(_w)[i] != SB_SelectionLabel(_w) &&
+	    MGR_Children(_w)[i] != SB_Text(_w) &&
+	    MGR_Children(_w)[i] != SB_Separator(_w) &&
+	    MGR_Children(_w)[i] != SB_OkButton(_w) &&
+	    MGR_Children(_w)[i] != SB_ApplyButton(_w) &&
+	    MGR_Children(_w)[i] != SB_HelpButton(_w) &&
+	    MGR_Children(_w)[i] != BB_CancelButton(_w))
+	{
+	    extras[nextras] = MGR_Children(_w)[i];
+	    nextras++;
+	}
+    }
 
     nrows = 0;
 
-    numKids = MGR_NumChildren(_w);
+    /* note the starting from one.  The zero'th child is the "work area" */
+    if (nextras > 0) {
+	for (i = 1; i < nextras; i++) {
+	    if (XmIsMenuBar(extras[i]) && XtIsManaged(extras[i]))
+		nrows++;
+	}
+	if (extras[0] && XtIsManaged(extras[0]))
+	    nrows++;
+    }
 
     if (SB_ListLabel(_w) && XtIsManaged(SB_ListLabel(_w)))
 	nrows++;
@@ -925,6 +1029,16 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
         (SB_ApplyButton(_w)  && XtIsManaged(SB_ApplyButton(_w))) ||
         (SB_HelpButton(_w)   && XtIsManaged(SB_HelpButton(_w))))
 	nrows++;
+    else {
+	for (i = 1; i < nextras; i++) {
+	    if (extras[i] && XtIsManaged(extras[i]) &&
+		(XmIsPushButton(extras[i]) || XmIsPushButtonGadget(extras[i])))
+	    {
+		nrows++;
+		break;
+	    }
+	}
+    }
 
     geoSpec = _XmGeoMatrixAlloc(nrows, numKids, 0);
     geoSpec->composite = (Widget)_w;
@@ -938,6 +1052,46 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     layoutPtr = &(geoSpec->layouts->row);
     boxPtr = geoSpec->boxes;
 
+    for (i = 1; i < nextras; i++) {
+	if (XmIsMenuBar(extras[i]) && XtIsManaged(extras[i]))
+	{
+	    layoutPtr->fix_up = _XmMenuBarFix;
+	    layoutPtr->space_above = 0;
+	    boxPtr += 2;
+	    layoutPtr++;
+	    break;
+	}
+    }
+
+    if (SB_ChildPlacement(_w) == XmPLACE_TOP && nextras &&
+	extras[0] && XtIsManaged(extras[0]) &&
+	_XmGeoSetupKid(boxPtr, extras[0]))
+    {
+	    layoutPtr->stretch_height = 1;
+	    layoutPtr->fill_mode = XmGEO_EXPAND;
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	    layoutPtr->space_above = BB_MarginHeight(_w);
+	    layoutPtr++;
+	    boxPtr += 2;
+	    nrows++;
+    }
+
+    if (SB_DialogType(_w) == XmDIALOG_PROMPT &&
+	SB_ChildPlacement(_w) == XmPLACE_ABOVE_SELECTION && nextras &&
+	extras[0] && XtIsManaged(extras[0]) &&
+	_XmGeoSetupKid(boxPtr, extras[0]))
+    {
+	    layoutPtr->stretch_height = 1;
+	    layoutPtr->fill_mode = XmGEO_EXPAND;
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	    layoutPtr->space_above = BB_MarginHeight(_w);
+	    layoutPtr++;
+	    boxPtr += 2;
+	    nrows++;
+    }
+
     newRow = False;
     if (SB_ListLabel(_w) && XtIsManaged(SB_ListLabel(_w)) &&
 	_XmGeoSetupKid(boxPtr, SB_ListLabel(_w)))
@@ -958,11 +1112,27 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	boxPtr++;
     }
 
+    if (SB_DialogType(_w) == XmDIALOG_COMMAND &&
+	SB_ChildPlacement(_w) == XmPLACE_ABOVE_SELECTION && nextras &&
+	extras[0] && XtIsManaged(extras[0]) &&
+	_XmGeoSetupKid(boxPtr, extras[0]))
+    {
+	    layoutPtr->stretch_height = 1;
+	    layoutPtr->fill_mode = XmGEO_EXPAND;
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	    layoutPtr->space_above = BB_MarginHeight(_w);
+	    layoutPtr++;
+	    boxPtr += 2;
+	    nrows++;
+    }
+
     newRow = FALSE;
     if (SB_List(_w) && XtIsManaged(SB_List(_w)) &&
 	_XmGeoSetupKid(boxPtr, XtParent(SB_List(_w))))
     {
 	layoutPtr->stretch_height = 1;
+	layoutPtr->min_height = 40;
 	layoutPtr->fill_mode = XmGEO_EXPAND;
 	layoutPtr->fit_mode = XmGEO_PROPORTIONAL;
 	layoutPtr->even_width = 1;
@@ -979,6 +1149,22 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	boxPtr++;
     }
 
+    if (SB_DialogType(_w) != XmDIALOG_COMMAND &&
+	SB_DialogType(_w) != XmDIALOG_PROMPT &&
+	SB_ChildPlacement(_w) == XmPLACE_ABOVE_SELECTION && nextras &&
+	extras[0] && XtIsManaged(extras[0]) &&
+	_XmGeoSetupKid(boxPtr, extras[0]))
+    {
+	    layoutPtr->stretch_height = 1;
+	    layoutPtr->fill_mode = XmGEO_EXPAND;
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	    layoutPtr->space_above = BB_MarginHeight(_w);
+	    layoutPtr++;
+	    boxPtr += 2;
+	    nrows++;
+    }
+
     if (SB_SelectionLabel(_w) && XtIsManaged(SB_SelectionLabel(_w)) &&
 	_XmGeoSetupKid(boxPtr, SB_SelectionLabel(_w)))
     {
@@ -994,7 +1180,6 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	_XmGeoSetupKid(boxPtr, SB_Text(_w)))
     {
 	layoutPtr->fill_mode = XmGEO_EXPAND;
-	layoutPtr->stretch_height = 0;
 	layoutPtr->even_height = 1;
 	layoutPtr->even_width = 0;
 	layoutPtr->space_above = 0; /* BB_MarginHeight(_w); */
@@ -1002,6 +1187,20 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	layoutPtr++;
     }
 
+    if (SB_ChildPlacement(_w) == XmPLACE_BELOW_SELECTION && nextras &&
+	extras[0] && XtIsManaged(extras[0]) &&
+	_XmGeoSetupKid(boxPtr, extras[0]))
+    {
+	    layoutPtr->stretch_height = 1;
+	    layoutPtr->fill_mode = XmGEO_EXPAND;
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	    layoutPtr->space_above = BB_MarginHeight(_w);
+	    layoutPtr++;
+	    boxPtr += 2;
+	    nrows++;
+    }
+
     if (SB_Separator(_w) && XtIsManaged(SB_Separator(_w)) &&
 	_XmGeoSetupKid( boxPtr, SB_Separator(_w)))
     {
@@ -1015,16 +1214,32 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     if (SB_OkButton(_w)     && XtIsManaged(SB_OkButton(_w)) &&
 	_XmGeoSetupKid(boxPtr++, SB_OkButton(_w))) {
 	layoutPtr->fill_mode = XmGEO_CENTER;
-	layoutPtr->fit_mode = XmGEO_AVERAGING;
+	layoutPtr->fit_mode = XmGEO_WRAP;
 	layoutPtr->even_width = 1;
 	layoutPtr->even_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
+    for (i = 1; i < nextras; i++)
+    {
+	if (extras[i] && XtIsManaged(extras[i]) &&
+	    (XmIsPushButton(extras[i]) || XmIsPushButtonGadget(extras[i])) &&
+	    _XmGeoSetupKid(boxPtr++, extras[i]))
+	{
+	    _XmBulletinBoardSetDefaultShadow(extras[i]);
+	    layoutPtr->fill_mode = XmGEO_CENTER;
+	    layoutPtr->fit_mode = XmGEO_WRAP;
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	    layoutPtr->space_above = BB_MarginHeight(_w);
+	    newRow = True;
+	}
+    }
+
     if (SB_ApplyButton(_w)  && XtIsManaged(SB_ApplyButton(_w)) &&
 	_XmGeoSetupKid(boxPtr++, SB_ApplyButton(_w))) {
 	layoutPtr->fill_mode = XmGEO_CENTER;
-	layoutPtr->fit_mode = XmGEO_AVERAGING;
+	layoutPtr->fit_mode = XmGEO_WRAP;
 	layoutPtr->even_width = 1;
 	layoutPtr->even_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
@@ -1033,7 +1248,7 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     if (BB_CancelButton(_w) && XtIsManaged(BB_CancelButton(_w)) &&
 	_XmGeoSetupKid(boxPtr++, BB_CancelButton(_w))) {
 	layoutPtr->fill_mode = XmGEO_CENTER;
-	layoutPtr->fit_mode = XmGEO_AVERAGING;
+	layoutPtr->fit_mode = XmGEO_WRAP;
 	layoutPtr->even_width = 1;
 	layoutPtr->even_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
@@ -1042,7 +1257,7 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     if (SB_HelpButton(_w)   && XtIsManaged(SB_HelpButton(_w)) &&
 	_XmGeoSetupKid(boxPtr++, SB_HelpButton(_w))) {
 	layoutPtr->fill_mode = XmGEO_CENTER;
-	layoutPtr->fit_mode = XmGEO_AVERAGING;
+	layoutPtr->fit_mode = XmGEO_WRAP;
 	layoutPtr->even_width = 1;
 	layoutPtr->even_height = 1;
 	layoutPtr->space_above = BB_MarginHeight(_w);
@@ -1057,17 +1272,20 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 
     layoutPtr->space_above = 0; /* BB_MarginHeight(_w); */
     layoutPtr->end = TRUE;
+    if (nextras)
+	XtFree((char *)extras);
     return(geoSpec);
 }
 
 Boolean
 _XmSelectionBoxNoGeoRequest(XmGeoMatrix _geoSpec)
 {
-	if(BB_InSetValues(_geoSpec->composite) && 
-		(XtClass(_geoSpec->composite) == xmSelectionBoxWidgetClass))
-			return(TRUE);
+    if (BB_InSetValues(_geoSpec->composite) && 
+	(XtClass(_geoSpec->composite) == xmSelectionBoxWidgetClass ||
+	 XtClass(_geoSpec->composite) == xmCommandWidgetClass))
+	return TRUE;
 
-	return( FALSE);
+    return FALSE;
 }
 
 Widget
@@ -1089,18 +1307,25 @@ XmCreateSelectionDialog(Widget parent,
 			Arg *argList,
 			Cardinal argcount)
 {
-    Widget	d;
+    Widget	d, r;
     char	*s;
-    Arg		a;
+    Arg		*al = (Arg *)XtCalloc(argcount + 1, sizeof(Arg));
+    int		ac, i;
 
     s = _XmMakeDialogName(name);
 
-    d = XtCreateWidget(s, xmDialogShellWidgetClass, parent, argList, argcount);
-    XtFree(s);
-    XtSetArg(a, XmNallowShellResize, True);
-    XtSetValues(d, &a, 1);
+    ac = 0;
+    XtSetArg(al[ac], XmNallowShellResize, True); ac++;
+    for (i=0; i<argcount; i++) {
+	XtSetArg(al[ac], argList[i].name, argList[i].value); ac++;
+    }
 
-    return XtCreateWidget(name, xmSelectionBoxWidgetClass, d, argList, argcount);
+    d = XtCreateWidget(s, xmDialogShellWidgetClass, parent, al, ac);
+    XtFree(s);
+
+    r =  XtCreateWidget(name, xmSelectionBoxWidgetClass, d, al, ac);
+    XtFree((XtPointer)al);
+    return r;
 }
 
 Widget 
@@ -1117,12 +1342,14 @@ XmCreatePromptDialog(Widget parent, char *name, Arg *argList, Cardinal argcount)
     for (i=0; i<argcount; i++)
 	al[i] = argList[i];
 
-    XtSetArg(al[argcount], XmNallowShellResize, True);
-    d = XtCreateWidget(s, xmDialogShellWidgetClass, parent, al, argcount + 1);
+    XtSetArg(al[argcount], XmNallowShellResize, True); argcount ++;
+    d = XtCreateWidget(s, xmDialogShellWidgetClass, parent, al, argcount);
     XtFree(s);
 
-    XtSetArg(al[argcount], XmNdialogType, XmDIALOG_PROMPT);
-    ret = XtCreateWidget(name, xmSelectionBoxWidgetClass, d, al, argcount + 1);
+/* This resource goes to the selectionBox, the previous one goes to Shell.
+ * Use the same spot in the array for both. */
+    XtSetArg(al[argcount - 1], XmNdialogType, XmDIALOG_PROMPT);
+    ret = XtCreateWidget(name, xmSelectionBoxWidgetClass, d, al, argcount);
 
     XtFree((XtPointer)al);
     return ret;
@@ -1175,3 +1402,20 @@ XmSelectionBoxGetChild(Widget w, unsigned char child)
 		return NULL;
 	}
 }
+
+/*      
+ * Keep track of button children
+ */     
+static void 
+InsertChild(Widget w)
+{       
+#define superclass      (&xmBulletinBoardClassRec)
+    (*superclass->composite_class.insert_child)(w);
+#undef  superclass
+
+    if (_XmIsFastSubclass(XtClass(w), XmTEXT_FIELD_BIT))
+    {
+        XtAddCallback(w, XmNactivateCallback, _XmSbButton, NULL);
+    }
+}
+
