@@ -1,6 +1,6 @@
 /**
  *
- * $Id: SeparatoG.c,v 1.23 1996/04/22 22:55:52 miers Exp $
+ * $Id: SeparatoG.c,v 1.4 1996/11/28 09:21:56 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,39 +22,41 @@
  *
  **/
 
-static char rcsid[] = "$Id: SeparatoG.c,v 1.23 1996/04/22 22:55:52 miers Exp $";
+static char rcsid[] = "$Id: SeparatoG.c,v 1.4 1996/11/28 09:21:56 u27113 Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/ManagerP.h>
 #include <Xm/SeparatoGP.h>
 #include <Xm/ExtObjectP.h>
 #include <Xm/CacheP.h>
-#include <stdio.h>
 #include <X11/Xfuncs.h>
 
+#include <XmI/DebugUtil.h>
+
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
 static void expose(Widget w, XEvent *event, Region region);
 
 static XtGeometryResult query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer);
 
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void get_values_hook(Widget w, ArgList args, Cardinal *num_args);
 
-static void secondary_object_create(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void secondary_object_create(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 
-static void initialize_prehook(Widget request, Widget new, ArgList args, Cardinal *num_args);
-static void initialize_posthook(Widget request, Widget new, ArgList args, Cardinal *num_args);
-static Boolean set_values_prehook(Widget old, Widget request, Widget new, ArgList args, Cardinal *num_args);
-static Boolean set_values_posthook(Widget old, Widget request, Widget new, ArgList args, Cardinal *num_args);
-static void get_values_prehook(Widget new, ArgList args, Cardinal *num_args);
-static void get_values_posthook(Widget new, ArgList args, Cardinal *num_args);
+static void initialize_prehook(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static void initialize_posthook(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static Boolean set_values_prehook(Widget old, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static Boolean set_values_posthook(Widget old, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static void get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args);
+static void get_values_posthook(Widget new_w, ArgList args, Cardinal *num_args);
 static Cardinal get_sec_res_data(WidgetClass wc, XmSecondaryResourceData **data);
 
 /*
@@ -258,7 +260,7 @@ WidgetClass xmSeparatorGadgetClass = (WidgetClass)&xmSeparatorGadgetClassRec;
 
 static void
 secondary_object_create(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
@@ -267,10 +269,10 @@ secondary_object_create(Widget request,
     XmWidgetExtData ed;
     int size;
 
-    XdbDebug(__FILE__, new, "SeparatorGCacheRec %s being initialized.\n",
-	     XtName(new));
+    XdbDebug(__FILE__, new_w, "SeparatorGCacheRec %s being initialized.\n",
+	     XtName(new_w));
 
-    bce = _XmGetBaseClassExtPtr(XtClass(new), XmQmotif);
+    bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
 
     size = (*bce)->secondaryObjectClass->core_class.widget_size;
     nsec = _XmExtObjAlloc(size);
@@ -278,16 +280,16 @@ secondary_object_create(Widget request,
 
     ((XmExtRec *)nsec)->object.self = (Widget)nsec;
     ((XmExtRec *)nsec)->object.widget_class = (*bce)->secondaryObjectClass;
-    ((XmExtRec *)nsec)->object.parent = XtParent(new);
-    ((XmExtRec *)nsec)->object.xrm_name = new->core.xrm_name;
+    ((XmExtRec *)nsec)->object.parent = XtParent(new_w);
+    ((XmExtRec *)nsec)->object.xrm_name = new_w->core.xrm_name;
     ((XmExtRec *)nsec)->object.being_destroyed = False;
     ((XmExtRec *)nsec)->object.destroy_callbacks = NULL;
     ((XmExtRec *)nsec)->object.constraints = NULL;
 
-    ExtObj_LogicalParent(nsec) = new;
+    ExtObj_LogicalParent(nsec) = new_w;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    XtGetSubresources(new, nsec, NULL, NULL,
+    XtGetSubresources(new_w, nsec, NULL, NULL,
     		      (*bce)->secondaryObjectClass->core_class.resources,
     		      (*bce)->secondaryObjectClass->core_class.num_resources,
 		      args, *num_args);
@@ -299,18 +301,18 @@ secondary_object_create(Widget request,
     bcopy(nsec, rsec, size);
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
-    _XmPushWidgetExtData(new, ed, XmCACHE_EXTENSION);
+    _XmPushWidgetExtData(new_w, ed, XmCACHE_EXTENSION);
 
-    SEPG_Cache(new) = &(((XmSeparatorGCacheObject)nsec)->separator_cache);
+    SEPG_Cache(new_w) = &(((XmSeparatorGCacheObject)nsec)->separator_cache);
     SEPG_Cache(request) = &(((XmSeparatorGCacheObject)rsec)->separator_cache);
 }
 
 int
 _XmSeparatorCacheCompare(XtPointer A, XtPointer B)
 {
-    return !memcmp(((XmSeparatorGCacheObjPart *)A),
-		   ((XmSeparatorGCacheObjPart *)B),
-		   sizeof(XmSeparatorGCacheObjPart));
+    return !bcmp(((XmSeparatorGCacheObjPart *)A),
+		 ((XmSeparatorGCacheObjPart *)B),
+		 sizeof(XmSeparatorGCacheObjPart));
 }
 
 /***************************** GADGET PART *******************************/
@@ -336,11 +338,11 @@ class_part_initialize(WidgetClass widget_class)
 
 static void
 initialize_prehook(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XdbDebug(__FILE__, new, "SeparatoG InitializePrehook\n");
+    XdbDebug(__FILE__, new_w, "SeparatoG InitializePrehook\n");
 
     /*
      * We should allocate secondary object here, using _XmExtObjAlloc (to create
@@ -359,54 +361,119 @@ initialize_prehook(Widget request,
 
 static void
 initialize_posthook(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     XmWidgetExtData ext;
 
-    XdbDebug(__FILE__, new, "SeparatoG InitializePosthook\n");
+    XdbDebug(__FILE__, new_w, "SeparatoG InitializePosthook\n");
 
     /* don't let the null fool you */
-    SEPG_Cache(new) = _XmCachePart(SEPG_ClassCachePart(NULL),
-				   (XtPointer)SEPG_Cache(new),
+    SEPG_Cache(new_w) = _XmCachePart(SEPG_ClassCachePart(NULL),
+				   (XtPointer)SEPG_Cache(new_w),
 				   sizeof(XmSeparatorGCacheObjPart));
 
-    _XmPopWidgetExtData(new, &ext, XmCACHE_EXTENSION);
+    _XmPopWidgetExtData(new_w, &ext, XmCACHE_EXTENSION);
     _XmExtObjFree((XtPointer)ext->widget);
     _XmExtObjFree((XtPointer)ext->reqWidget);
     XtFree((char *)ext);
 }
 
+static void
+CreateSeparatorGC(Widget w)
+{
+    XtGCMask valueMask;
+    XGCValues values;
+
+    valueMask =  GCBackground | GCForeground | GCLineStyle | GCJoinStyle |
+			GCCapStyle;
+
+    switch (SEPG_SeparatorType(w)) {
+    case XmSINGLE_LINE:
+	values.line_style = LineSolid;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+
+    case XmSINGLE_DASHED_LINE:
+	values.line_style = LineDoubleDash;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+
+    case XmDOUBLE_LINE:
+	values.line_style = LineSolid;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+
+    case XmDOUBLE_DASHED_LINE:
+	values.line_style = LineDoubleDash;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+
+    case XmSHADOW_ETCHED_IN:
+	/* doesn't really matter */
+	values.line_style = LineSolid;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+
+    case XmSHADOW_ETCHED_OUT:
+	/* doesn't really matter */
+	values.line_style = LineSolid;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+
+    case XmSHADOW_ETCHED_IN_DASH:
+    case XmSHADOW_ETCHED_OUT_DASH:
+    default:
+	SEPG_SeparatorType(w) = XmSHADOW_ETCHED_IN;
+	values.line_style = LineSolid;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+    }
+
+    SEPG_SeparatorGC(w) = XtGetGC(w, valueMask, &values);
+}
+
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XtGCMask valueMask;
+    if (! XtIsSubclass(XtParent(new_w), xmManagerWidgetClass))
+        _XmError(new_w, "parent should be manager.");
 
-    if (! XtIsSubclass(XtParent(new), xmManagerWidgetClass))
-        _XmError(new, "parent should be manager.");
-
-    valueMask =  GCBackground | GCForeground | GCLineStyle;
-
-    /*
-     * note the shared GC.  This requires that the three fields above be set
-     * before each use of the GC.  This one GC is used for ALL separator
-     * Gadgets, so don't forget this!
-     */
-    SEPG_SeparatorGC(new) = XtAllocateGC(new, 0, 0, NULL,
-					 valueMask, ~valueMask);
-
-    if (SEPG_Orientation(new) == XmVERTICAL) {
-	XtWidth(new) = G_ShadowThickness(new);
+    if (SEPG_Orientation(new_w) == XmVERTICAL) {
+	XtWidth(new_w) = G_ShadowThickness(new_w);
     }
-    if (SEPG_Orientation(new) == XmHORIZONTAL) {
-	XtHeight(new) = G_ShadowThickness(new);
+    if (SEPG_Orientation(new_w) == XmHORIZONTAL) {
+	XtHeight(new_w) = G_ShadowThickness(new_w);
     }
 
-    G_EventMask(new) = 0;
+    CreateSeparatorGC(new_w);
+
+    G_EventMask(new_w) = 0;
 }
 
 static void
@@ -420,7 +487,7 @@ destroy(Widget w)
 static Boolean
 set_values_prehook(Widget old,
 		   Widget request,
-		   Widget new,
+		   Widget new_w,
 		   ArgList args,
 		   Cardinal *num_args)
 {
@@ -429,7 +496,7 @@ set_values_prehook(Widget old,
     int size;
     XtPointer nsec, rsec;
 
-    bce = _XmGetBaseClassExtPtr(XtClass(new), XmQmotif);
+    bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
     size = (*bce)->secondaryObjectClass->core_class.widget_size;
 
     nsec = _XmExtObjAlloc(size);
@@ -437,16 +504,16 @@ set_values_prehook(Widget old,
 
     ((XmExtRec *)nsec)->object.self = (Widget)nsec;
     ((XmExtRec *)nsec)->object.widget_class = (*bce)->secondaryObjectClass;
-    ((XmExtRec *)nsec)->object.parent = XtParent(new);
-    ((XmExtRec *)nsec)->object.xrm_name = new->core.xrm_name;
+    ((XmExtRec *)nsec)->object.parent = XtParent(new_w);
+    ((XmExtRec *)nsec)->object.xrm_name = new_w->core.xrm_name;
     ((XmExtRec *)nsec)->object.being_destroyed = False;
     ((XmExtRec *)nsec)->object.destroy_callbacks = NULL;
     ((XmExtRec *)nsec)->object.constraints = NULL;
 
-    ExtObj_LogicalParent(nsec) = new;
+    ExtObj_LogicalParent(nsec) = new_w;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(SEPG_Cache(new),
+    bcopy(SEPG_Cache(new_w),
 	  &((XmSeparatorGCacheObject)nsec)->separator_cache,
 	  sizeof(XmSeparatorGCacheObjPart));
 
@@ -454,9 +521,9 @@ set_values_prehook(Widget old,
     ed->widget = (Widget)nsec;
     ed->reqWidget = (Widget)rsec;
 
-    _XmPushWidgetExtData(new, ed, XmCACHE_EXTENSION);
+    _XmPushWidgetExtData(new_w, ed, XmCACHE_EXTENSION);
 
-    _XmGadgetImportSecondaryArgs(new, args, num_args);
+    _XmGadgetImportSecondaryArgs(new_w, args, num_args);
 
     XtSetSubvalues((XtPointer)nsec,
                     (*bce)->secondaryObjectClass->core_class.resources,
@@ -466,7 +533,7 @@ set_values_prehook(Widget old,
     bcopy(nsec, rsec, size);
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
-    SEPG_Cache(new) = &(((XmSeparatorGCacheObject)nsec)->separator_cache);
+    SEPG_Cache(new_w) = &(((XmSeparatorGCacheObject)nsec)->separator_cache);
     SEPG_Cache(request) = &(((XmSeparatorGCacheObject)rsec)->separator_cache);
 
     _XmExtImportArgs((Widget)nsec, args, num_args);
@@ -477,24 +544,24 @@ set_values_prehook(Widget old,
 static Boolean
 set_values_posthook(Widget old,
 		   Widget request,
-		   Widget new,
+		   Widget new_w,
 		   ArgList args,
 		   Cardinal *num_args)
 {
     XmWidgetExtData ext;
 
-    if (!_XmSeparatorCacheCompare(SEPG_Cache(new), SEPG_Cache(old))) {
+    if (!_XmSeparatorCacheCompare(SEPG_Cache(new_w), SEPG_Cache(old))) {
 
 	_XmCacheDelete((XtPointer)SEPG_Cache(old));
 
-	SEPG_Cache(new) = _XmCachePart(SEPG_ClassCachePart(NULL),
-				       (XtPointer)SEPG_Cache(new),
+	SEPG_Cache(new_w) = _XmCachePart(SEPG_ClassCachePart(NULL),
+				       (XtPointer)SEPG_Cache(new_w),
 				       sizeof(XmSeparatorGCacheObjPart));
     }
     else
-	SEPG_Cache(new) = SEPG_Cache(old);
+	SEPG_Cache(new_w) = SEPG_Cache(old);
 
-    _XmPopWidgetExtData(new, &ext, XmCACHE_EXTENSION);
+    _XmPopWidgetExtData(new_w, &ext, XmCACHE_EXTENSION);
 
     _XmExtObjFree((XtPointer)ext->widget);
     _XmExtObjFree((XtPointer)ext->reqWidget);
@@ -507,12 +574,19 @@ set_values_posthook(Widget old,
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
     Boolean refresh_needed = False;
 
+    if (XmParentForeground(new_w) != XmParentForeground(old) ||
+	SEPG_SeparatorType(new_w) != SEPG_SeparatorType(old)) {
+	XtReleaseGC(new_w, SEPG_SeparatorGC(new_w));
+	CreateSeparatorGC(new_w);
+	refresh_needed = False;
+    }
+
     return refresh_needed;
 }
 
@@ -523,37 +597,37 @@ get_values_hook(Widget w, ArgList args, Cardinal *num_args)
 }
 
 static void
-get_values_prehook(Widget new, ArgList args, Cardinal *num_args)
+get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
     int size;
     XtPointer nsec;
 
-    bce = _XmGetBaseClassExtPtr(XtClass(new), XmQmotif);
+    bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
     size = (*bce)->secondaryObjectClass->core_class.widget_size;
 
     nsec = _XmExtObjAlloc(size);
 
     ((XmExtRec *)nsec)->object.self = (Widget)nsec;
     ((XmExtRec *)nsec)->object.widget_class = (*bce)->secondaryObjectClass;
-    ((XmExtRec *)nsec)->object.parent = XtParent(new);
-    ((XmExtRec *)nsec)->object.xrm_name = new->core.xrm_name;
+    ((XmExtRec *)nsec)->object.parent = XtParent(new_w);
+    ((XmExtRec *)nsec)->object.xrm_name = new_w->core.xrm_name;
     ((XmExtRec *)nsec)->object.being_destroyed = False;
     ((XmExtRec *)nsec)->object.destroy_callbacks = NULL;
     ((XmExtRec *)nsec)->object.constraints = NULL;
 
-    ExtObj_LogicalParent(nsec) = new;
+    ExtObj_LogicalParent(nsec) = new_w;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(SEPG_Cache(new),
+    bcopy(SEPG_Cache(new_w),
 	  &((XmSeparatorGCacheObject)nsec)->separator_cache,
 	  sizeof(XmSeparatorGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
 
-    _XmPushWidgetExtData(new, ed, XmCACHE_EXTENSION);
+    _XmPushWidgetExtData(new_w, ed, XmCACHE_EXTENSION);
 
     XtGetSubvalues((XtPointer)nsec,
                     (*bce)->secondaryObjectClass->core_class.resources,
@@ -564,11 +638,11 @@ get_values_prehook(Widget new, ArgList args, Cardinal *num_args)
 }
 
 static void
-get_values_posthook(Widget new, ArgList args, Cardinal *num_args)
+get_values_posthook(Widget new_w, ArgList args, Cardinal *num_args)
 {
      XmWidgetExtData ext;
 
-     _XmPopWidgetExtData(new, &ext, XmCACHE_EXTENSION);
+     _XmPopWidgetExtData(new_w, &ext, XmCACHE_EXTENSION);
 
      _XmExtObjFree((XtPointer)ext->widget);
 
@@ -583,13 +657,6 @@ expose(Widget w,
     XdbDebug(__FILE__, w, "XmSeparatorGadget %s Expose (shadowThickness %d)\n",
 	     XtName(w), G_ShadowThickness(w));
 
-    XSetBackground(XtDisplayOfObject(w),
-		   SEPG_SeparatorGC(w),
-		   XmParentBackground(w));
-    XSetForeground(XtDisplayOfObject(w),
-		   SEPG_SeparatorGC(w),
-		   XmParentForeground(w));
-
     _XmDrawSeparator(XtDisplayOfObject(w), 
 		     XtWindowOfObject(w),
 		     XmParentTopShadowGC(w), XmParentBottomShadowGC(w),
