@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Shadow.c,v 1.20 1996/04/22 22:55:55 miers Exp $
+ * $Id: Shadow.c,v 1.5 1996/12/11 04:18:34 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,14 +22,16 @@
  *
  **/
 
-static char rcsid[] = "$Id: Shadow.c,v 1.20 1996/04/22 22:55:55 miers Exp $";
+static char rcsid[] = "$Id: Shadow.c,v 1.5 1996/12/11 04:18:34 miers Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
 #include <Xm/PrimitiveP.h>
 #include <Xm/ManagerP.h>
-#include <stdio.h>
+
+#include <XmI/DebugUtil.h>
 
 void
 _XmClearBorder(Display *display,
@@ -61,6 +63,19 @@ _XmClearBorder(Display *display,
 	       False);
 }
 
+/* Motif 2.* version of the above */
+void
+XmeClearBorder(Display *display,
+	       Window win,
+	       Position x,
+	       Position y,
+	       Dimension width,
+	       Dimension height,
+	       Dimension shadow_thick)
+{
+	_XmClearBorder(display, win, x, y, width, height, shadow_thick);
+}
+
 void
 _XmDrawShadows(Display *display,
 	       Window win,
@@ -232,7 +247,23 @@ _XmDrawShadows(Display *display,
     }
 }
 
-void _XmDrawShadow(Display *display,
+/* Motif 2.* version of the above */
+void
+XmeDrawShadows(Display *display,
+	       Window win,
+	       GC topShadowGC,
+	       GC bottomShadowGC,
+	       Position x, Position y, 
+	       Dimension width, Dimension height,
+	       Dimension shadowThickness,
+	       unsigned int shadowType)
+{
+	_XmDrawShadows(display, win, topShadowGC, bottomShadowGC, x, y,
+		width, height, shadowThickness, shadowType);
+}
+
+void
+_XmDrawShadow(Display *display,
 		   Drawable d,
 		   GC top_gc,
 		   GC bottom_gc,
@@ -388,20 +419,6 @@ _XmDrawArrow(Display *display,
  	return;
     }
  
-    XSetLineAttributes(display,
- 		       topGC,
- 		       1,
- 		       LineSolid,
- 		       CapButt,
- 		       JoinMiter);
- 
-    XSetLineAttributes(display,
- 		       bottomGC,
- 		       1,
- 		       LineSolid,
- 		       CapButt,
- 		       JoinMiter);
- 
     /* normally dark */
     switch(direction){
     case XmARROW_UP:
@@ -467,6 +484,22 @@ _XmDrawArrow(Display *display,
     XSetClipMask(display,bottomGC,None);
 }
 
+/* Motif 2.* version of the above */
+void
+XmeDrawArrow(Display *display,
+	     Window win,
+	     GC bottomGC,
+	     GC topGC,
+	     GC fillGC,
+	     Position x, Position y,
+	     Dimension width, Dimension height,
+	     Dimension shadowThickness,
+	     unsigned char direction)
+{
+	_XmDrawArrow(display, win, bottomGC, topGC, fillGC,
+		x, y, width, height, shadowThickness, direction);
+}
+
 void
 _XmDrawDiamond(Display *display,
 	       Window win,
@@ -516,6 +549,22 @@ _XmDrawDiamond(Display *display,
     }
 }
 
+/* Motif 2.* version of the above */
+void
+XmeDrawDiamond(Display *display,
+	       Window win,
+	       GC bottomGC,
+	       GC topGC,
+	       GC selectGC,
+	       Position x, Position y,
+	       Dimension width, Dimension height,
+	       Dimension shadowThickness,
+	       Dimension fill)
+{
+	_XmDrawDiamond(display, win, bottomGC, topGC, selectGC, x, y, width, height,
+	       shadowThickness, fill);
+}
+
 void 
 _XmDrawSeparator(Display *display, 
 		 Drawable win, 
@@ -535,7 +584,9 @@ _XmDrawSeparator(Display *display,
     int x1 = 0,y1 = 0,x2 = 0,y2 = 0,x3 = 0,y3 = 0,x4 = 0,y4 = 0;
     int width, height, i;
 
+#ifdef	DO_FLUSH
     XFlush(display);
+#endif
 
     if (orientation == XmHORIZONTAL)
     {
@@ -601,26 +652,16 @@ _XmDrawSeparator(Display *display,
     switch(separator_type)
     {
     case XmSINGLE_LINE:
-	XSetLineAttributes(display, separator_GC, 0, LineSolid,
-			   CapButt,JoinMiter);
     	XDrawLine(display, win, separator_GC, x1,y1,x2,y2);
         break;
     case XmSINGLE_DASHED_LINE:
-	XSetLineAttributes(display, separator_GC, 0, LineDoubleDash,
-			   CapButt,JoinMiter);
     	XDrawLine(display, win, separator_GC, x1,y1,x2,y2);
 	break;
     case XmDOUBLE_LINE:
-	XSetLineAttributes(display, separator_GC, 0, LineSolid,
-			   CapButt,JoinMiter);
     	XDrawLine(display, win, separator_GC, x1,y1,x2,y2);
     	XDrawLine(display, win, separator_GC, x3,y3,x4,y4);
 	break;
     case XmDOUBLE_DASHED_LINE:
-	XSetLineAttributes(display, separator_GC, 0, LineSolid,
-			   CapButt,JoinMiter);
-	XSetLineAttributes(display, separator_GC, 0, LineDoubleDash,
-			   CapButt,JoinMiter);
     	XDrawLine(display, win, separator_GC, x1,y1,x2,y2);
     	XDrawLine(display, win, separator_GC, x3,y3,x4,y4);
 	break;
@@ -703,10 +744,6 @@ _XmDrawSeparator(Display *display,
 	}
 	break;
     case XmSHADOW_ETCHED_OUT_DASH:
-	XSetLineAttributes(display, top_gc, 0, LineDoubleDash,
-			   CapButt,JoinMiter);
-	XSetLineAttributes(display, bottom_gc, 0, LineDoubleDash,
-			   CapButt,JoinMiter);
 	if (orientation == XmVERTICAL) {
 	    for (i = 0; i < shadowThickness/2; i++)
 		XDrawLine(display, win, top_gc,
@@ -736,6 +773,26 @@ _XmDrawSeparator(Display *display,
     }
 }
 
+/* Motif 2.* version of the above */
+void 
+XmeDrawSeparator(Display *display, 
+		 Drawable win, 
+		 GC top_gc, 
+		 GC bottom_gc,	
+		 GC separator_GC, 
+		 Position wx, 
+		 Position wy, 
+		 Dimension wwidth,	
+		 Dimension wheight, 
+		 Dimension shadowThickness, 
+		 Dimension margin,	
+		 unsigned char orientation, 
+		 unsigned char separator_type)
+{
+	_XmDrawSeparator(display, win, top_gc, bottom_gc, separator_GC, 
+		 wx, wy, wwidth, wheight, shadowThickness, margin, orientation, separator_type);
+}
+
 void
 _XmDrawSimpleHighlight(Display *display,
                        Drawable d,
@@ -760,13 +817,52 @@ _XmDrawHighlight(Display *display,
                  int line_style)
 {
     int i;
+    static      char dash_list[] = { 8, 8 };
 
     /* this is dangerous, but the same as the Manger and Primitive */
-    XSetLineAttributes(display, gc, 1, line_style, CapButt, JoinMiter);
+    XSetLineAttributes(display, gc, 0, line_style, CapButt, JoinMiter);
 
-    for (i = 0; i < highlight_thick; i++) {
-        XDrawRectangle(display, (Window)d, gc,
+    switch (line_style) {
+    case LineOnOffDash:
+    case LineDoubleDash:
+        XSetDashes(display, gc, 0, dash_list, 2);
+        for (i = 0; i < highlight_thick; i++) {
+            XDrawLine(display, (Window)d, gc, 
+                  x, y+i, x+width-1, y+i);
+        
+            XDrawLine(display, (Window)d, gc, 
+                  x, y+height-1-i, x+width-1, y+height-1-i);
+        
+            XDrawLine(display, (Window)d, gc, 
+                  x+i, y, x+i, y+height-1);
+        
+            XDrawLine(display, (Window)d, gc, 
+                  x+width-1-i, y, x+width-1-i, y+height-1);
+        }
+	break;
+
+    case LineSolid:
+    default:
+        for (i = 0; i < highlight_thick; i++) {
+            XDrawRectangle(display, (Window)d, gc,
                        x + i, y + i,
                        width - (2*i) - 1, height - (2*i) - 1);
+        }
+        break;
     }
 }
+
+/* Motif 2.* version of the above */
+void
+XmeDrawHighlight(Display *display,
+                 Drawable d,
+                 GC gc,
+                 Position x,
+                 Position y,
+                 Dimension width,
+                 Dimension height,
+                 Dimension highlight_thick,
+                 int line_style)
+{
+	_XmDrawHighlight(display, d, gc, x, y, width, height, highlight_thick, line_style);
+}
