@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ToggleB.c,v 1.17 1996/04/22 22:56:03 miers Exp $
+ * $Id: ToggleB.c,v 1.10 1996/12/18 00:45:27 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,27 +22,34 @@
  *
  **/
 
-static char rcsid[] = "$Id: ToggleB.c,v 1.17 1996/04/22 22:56:03 miers Exp $";
+static char rcsid[] = "$Id: ToggleB.c,v 1.10 1996/12/18 00:45:27 miers Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
-#include <Xm/XmI.h>
 #include <Xm/ToggleBP.h>
 #include <Xm/RowColumnP.h>
 #include <Xm/MenuUtilP.h>
 #include <Xm/MenuShell.h>
 #include <Xm/CascadeB.h>
 #include <Xm/CascadeBG.h>
-#include <stdio.h>
+#include <X11/ShellP.h>
+
+#include <XmI/DebugUtil.h>
+
+#include <Xm/ToggleBGP.h>
 
 /* Forward Declarations */
 
 static void class_initialize();
-static void class_part_initialize(WidgetClass class);
-static void initialize(Widget request, Widget new, ArgList args, Cardinal *num_args);
+static void class_part_initialize(WidgetClass w_class);
+static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static void initialize_prehook(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
+static void initialize_posthook(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
 static void expose(Widget w, XEvent *event, Region region);
-static Boolean set_values(Widget current, Widget request, Widget new, ArgList args, Cardinal *num_args);
+static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 
 /*
  * Resources for the toggleButton class
@@ -174,6 +181,9 @@ char _XmToggleB_menuTranslations[] =
      ~s ~m ~a <Key>Return:ArmAndActivate()\n\
      ~s ~m ~a <Key>space:ArmAndActivate()";
 
+static XtTranslations default_trans = NULL;
+static XtTranslations menu_trans = NULL;
+
 static XtActionsRec actions[] = {
     {"Arm", Arm},
     {"ArmAndActivate", ArmAndActivate},
@@ -191,16 +201,16 @@ static XmBaseClassExtRec _XmToggleBCoreClassExtRec = {
     /* record_type               */ NULLQUARK,                             
     /* version                   */ XmBaseClassExtVersion,
     /* size                      */ sizeof(XmBaseClassExtRec),
-    /* initialize_prehook        */ NULL, /* FIXME */
-    /* set_values_prehook        */ NULL, /* FIXME */
-    /* initialize_posthook       */ NULL, /* FIXME */
-    /* set_values_posthook       */ NULL, /* FIXME */
-    /* secondary_object_class    */ NULL, /* FIXME */
-    /* secondary_object_create   */ NULL, /* FIXME */
-    /* get_secondary_resources   */ NULL, /* FIXME */
-    /* fast_subclass             */ { 0 }, /* FIXME */
-    /* get_values_prehook        */ NULL, /* FIXME */
-    /* get_values_posthook       */ NULL, /* FIXME */
+    /* initialize_prehook        */ initialize_prehook,
+    /* set_values_prehook        */ XmInheritSetValuesPrehook,
+    /* initialize_posthook       */ initialize_posthook,
+    /* set_values_posthook       */ XmInheritSetValuesPosthook,
+    /* secondary_object_class    */ XmInheritClass,
+    /* secondary_object_create   */ XmInheritSecObjectCreate,
+    /* get_secondary_resources   */ XmInheritGetSecResData,
+    /* fast_subclass             */ { 0 },
+    /* get_values_prehook        */ XmInheritGetValuesPrehook,
+    /* get_values_posthook       */ XmInheritGetValuesPosthook,
     /* class_part_init_prehook   */ NULL,
     /* class_part_init_posthook  */ NULL,
     /* ext_resources             */ NULL,
@@ -217,9 +227,9 @@ XmPrimitiveClassExtRec _XmToggleBPrimClassExtRec = {
     /* record_type         */ NULLQUARK,
     /* version             */ XmPrimitiveClassExtVersion,
     /* record_size         */ sizeof(XmPrimitiveClassExtRec),
-    /* widget_baseline     */ NULL, /* FIX ME */
-    /* widget_display_rect */ NULL, /* FIX ME */
-    /* widget_margins      */ NULL  /* FIX ME */
+    /* widget_baseline     */ XmInheritBaselineProc,
+    /* widget_display_rect */ XmInheritDisplayRectProc,
+    /* widget_margins      */ NULL
 };
 
 XmToggleButtonClassRec xmToggleButtonClassRec = {
@@ -240,7 +250,7 @@ XmToggleButtonClassRec xmToggleButtonClassRec = {
 	/* num_resources         */ XtNumber(resources),
 	/* xrm_class             */ NULLQUARK,
 	/* compress_motion       */ TRUE,
-	/* compress_exposure     */ XtExposeCompressMultiple,
+	/* compress_exposure     */ XtExposeCompressMaximal,
 	/* compress_enterleave   */ TRUE,
 	/* visible_interest      */ FALSE,
 	/* destroy               */ destroy,
@@ -253,9 +263,9 @@ XmToggleButtonClassRec xmToggleButtonClassRec = {
 	/* accept_focus          */ NULL,
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
-	/* tm_table              */ _XmToggleB_defaultTranslations,
+	/* tm_table              */ NULL,
 	/* query_geometry        */ XtInheritQueryGeometry,
-	/* display_accelerator   */ XtInheritDisplayAccelerator,
+	/* display_accelerator   */ NULL,
 	/* extension             */ (XtPointer)&_XmToggleBCoreClassExtRec
     },
     /* Primitive Class part */
@@ -288,15 +298,15 @@ WidgetClass xmToggleButtonWidgetClass = (WidgetClass)&xmToggleButtonClassRec;
   Some #defines to make the code below more readable
  */
 
-#define IN_MENU(w) (XmIsRowColumn(XtParent(w)) \
-                    && (RC_Type(XtParent(w)) == XmMENU_POPUP \
-                          || (RC_Type(XtParent(w)) == XmMENU_PULLDOWN)))
-#define IN_POPUP(w) (XmIsRowColumn(XtParent(w)) \
-                    && (RC_Type(XtParent(w)) == XmMENU_POPUP)
+#define IN_MENU(w) (Lab_MenuType(w) == XmMENU_POPUP || \
+                    Lab_MenuType(w) == XmMENU_PULLDOWN)
 
 static void
 class_initialize()
 {
+    menu_trans = XtParseTranslationTable(_XmToggleB_menuTranslations);
+    default_trans = XtParseTranslationTable(_XmToggleB_defaultTranslations);
+
     _XmToggleBCoreClassExtRec.record_type = XmQmotif;
 }
 
@@ -306,44 +316,115 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmTOGGLE_BUTTON_BIT);
 }
 
+static void
+CreateSelectGC(Widget w)
+{
+    XGCValues values;
+    XtGCMask mask;
+
+    mask = GCForeground | GCBackground | GCFillStyle | GCFunction |
+		GCSubwindowMode | GCGraphicsExposures | GCPlaneMask;
+    values.function = GXcopy;
+    values.plane_mask = -1;
+    values.subwindow_mode = ClipByChildren;
+    values.graphics_exposures = False;
+    values.foreground = TB_SelectColor(w);
+    values.background = XtBackground(w);
+    values.fill_style = FillSolid;
+
+    TB_SelectGC(w) = XtGetGC(w, mask, &values);
+}
+
+static void
+CreateBackgroundGC(Widget w)
+{
+    XGCValues values;
+    XtGCMask mask;
+
+    mask = GCForeground | GCBackground | GCFillStyle | GCFunction |
+		GCSubwindowMode | GCGraphicsExposures | GCPlaneMask;
+    values.function = GXcopy;
+    values.plane_mask = -1;
+    values.subwindow_mode = ClipByChildren;
+    values.graphics_exposures = False;
+    values.foreground = XtBackground(w);
+    values.background = Prim_Foreground(w);
+    values.fill_style = FillSolid;
+    TB_BackgroundGC(w) = XtGetGC(w, mask, &values);
+}
+
+static void
+initialize_prehook(Widget request,
+		   Widget new_w,
+		   ArgList args,
+		   Cardinal *num_args)
+{
+    _XmSaveCoreClassTranslations(new_w);
+
+    if (XmIsRowColumn(XtParent(new_w)) &&
+	(RC_Type(XtParent(new_w)) == XmMENU_PULLDOWN ||
+	 RC_Type(XtParent(new_w)) == XmMENU_POPUP))
+	new_w->core.widget_class->core_class.tm_table = (String)menu_trans;
+    else 
+	new_w->core.widget_class->core_class.tm_table = (String)default_trans;
+}
+
+static void
+initialize_posthook(Widget request,
+		    Widget new_w,
+		    ArgList args,
+		    Cardinal *num_args)
+{
+    _XmRestoreCoreClassTranslations(new_w);
+}
+
 static void
 initialize(Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    Dimension string_width, string_height;
+    TB_Armed(new_w) = False;
 
-    TB_Armed(new) = False;
+    TB_VisualSet(new_w) = TB_IndicatorSet(new_w) = TB_Set(new_w);
 
-    TB_Set(new) = !(!TB_Set(new));
-    TB_VisualSet(new) = TB_IndicatorSet(new) = TB_Set(new);
-
-    XmStringExtent(Lab_Font(new),
-                   Lab_Label(new),
-                   &string_width,
-                   &string_height);
-
-    if (TB_IndOn(new))
-        TB_IndicatorDim(new) = XmDEFAULT_INDICATOR_DIM;
+    if (TB_IndOn(new_w))
+        TB_IndicatorDim(new_w) = XmDEFAULT_INDICATOR_DIM;
     else
-        TB_IndicatorDim(new) = 0;
+        TB_IndicatorDim(new_w) = 0;
  
-    TB_SelectGC(new) = XtAllocateGC(new,
-                                    0,0,
-                                    NULL,
-                                    GCForeground,~GCForeground);
+    CreateSelectGC(new_w);
+    CreateBackgroundGC(new_w);
 
-    Lab_MarginLeft(new) = TB_IndicatorDim(new) + (TB_IndOn(new)
-                               ? 2 * TB_Spacing(new)
+    if (TB_IndType(new_w) == (unsigned char)XmUNSPECIFIED) {
+	if (XmIsRowColumn(XtParent(new_w)) && RC_RadioBehavior(XtParent(new_w)))
+	    TB_IndType(new_w) = XmONE_OF_MANY;
+	else
+	    TB_IndType(new_w) = XmN_OF_MANY;
+    }
+
+    /* allocate the normal and insensitive GC's */
+    Lab_MarginLeft(new_w) = TB_IndicatorDim(new_w) + (TB_IndOn(new_w)
+                               ? 2 * TB_Spacing(new_w)
                                : 0);
 
-    _XmLabelRecomputeSize(new);
+    if (Lab_MarginLeft(new_w) != Lab_MarginLeft(request)) {
 
-    if (IN_MENU(new))
-    {
-	XtOverrideTranslations(new, XtParseTranslationTable(_XmToggleB_menuTranslations));
-	Lab_Highlight(new) = 0;
+	if (XtWidth(request) == 0)
+	    XtWidth(new_w) = 0;
+	if (XtHeight(request) == 0)
+	    XtHeight(new_w) = 0;
+
+	_XmCalcLabelDimensions(new_w);
+
+	(*xmLabelClassRec.core_class.resize)(new_w);
+    }
+
+    if (IN_MENU(new_w)) {
+	Lab_Highlight(new_w) = 0;
+	if (Prim_ShadowThickness(new_w) == 0)
+	    Prim_ShadowThickness(new_w) = 2;
+	TB_Visible(new_w) = False;
     }
 }
 
@@ -351,42 +432,206 @@ static void
 destroy(Widget w)
 {
     XtReleaseGC(w, TB_SelectGC(w));
+    XtReleaseGC(w, TB_BackgroundGC(w));
 }
 
 static Boolean
 set_values(Widget old,
 	   Widget request,
-	   Widget new,
+	   Widget new_w,
 	   ArgList args,
 	   Cardinal *num_args)
 {
-    XmToggleButtonWidget lw = (XmToggleButtonWidget) new;
-    XmToggleButtonWidget rw = (XmToggleButtonWidget) request;
-    Boolean refresh_needed = True;
+    Boolean refresh_needed = False;
 
-    if (XtWidth(rw) != XtWidth(lw)
-	   || XtHeight(rw) != XtHeight(lw))
-    {
-	  XtWidth(lw) = XtWidth(rw);
-	  XtHeight(lw) = XtHeight(rw);
-	  refresh_needed = False;
+    if (TB_SelectColor(new_w) != TB_SelectColor(old)) {
+	XtReleaseGC(new_w, TB_SelectGC(new_w));
+	CreateSelectGC(new_w);
+	refresh_needed = True;
+    }
+    if (XtBackground(new_w) != XtBackground(old)) {
+	XtReleaseGC(new_w, TB_BackgroundGC(new_w));
+	CreateBackgroundGC(new_w);
+	refresh_needed = True;
     }
 
-    TB_VisualSet(lw) = TB_IndicatorSet(lw) = TB_Set(lw);
-    if (TB_Set(rw) != TB_Set(lw))
-   	expose(new, NULL, (Region)NULL);
+    TB_VisualSet(new_w) = TB_IndicatorSet(new_w) = TB_Set(new_w);
+    if (TB_Set(old) != TB_Set(new_w))
+   	refresh_needed = True;
 
-    if (TB_IndType(rw) != TB_IndType(lw))
+    if (TB_IndType(old) != TB_IndType(new_w))
     {
-      if ((TB_IndType(lw) != XmN_OF_MANY) && (TB_IndType(lw) != XmONE_OF_MANY))
+      if ((TB_IndType(new_w) != XmN_OF_MANY) && (TB_IndType(new_w) != XmONE_OF_MANY))
       {
-         TB_IndType(lw) = TB_IndType(rw);
+         TB_IndType(new_w) = TB_IndType(old);
       }
     }
 
     return refresh_needed;
 }
 
+static void
+expose(Widget w,
+       XEvent *event,
+       Region region)
+{
+    Boolean   State = TB_VisualSet(w);
+    Pixmap tmp_pix = XmUNSPECIFIED_PIXMAP, tmp2_pix = XmUNSPECIFIED_PIXMAP;
+    
+    if (!TB_IndOn(w)) {
+	if (TB_FillOnSelect(w)) {
+	    XFillRectangle(XtDisplay(w), XtWindow(w),
+                           State ? TB_SelectGC(w) : TB_BackgroundGC(w),
+                           Lab_Highlight(w) + Lab_Shadow(w),
+                           Lab_Highlight(w) + Lab_Shadow(w),
+                           XtWidth(w)
+				 - 2 * (Lab_Highlight(w) + Lab_Shadow(w)),
+                           XtHeight(w)
+				 - 2 * (Lab_Highlight(w) + Lab_Shadow(w)));
+	}
+
+	if (Lab_IsPixmap(w) && State &&
+	    TB_OnPixmap(w) != XmUNSPECIFIED_PIXMAP) {
+	    tmp_pix = Lab_Pixmap(w);
+	    tmp2_pix = Lab_PixmapInsensitive(w);
+	    Lab_Pixmap(w) = TB_OnPixmap(w);
+	}
+#define superclass (&xmLabelClassRec)
+	(*superclass->core_class.expose)(w, event, region);
+#undef superclass
+	if (Lab_IsPixmap(w) && State &&
+	    TB_OnPixmap(w) != XmUNSPECIFIED_PIXMAP) {
+	    Lab_Pixmap(w) = tmp_pix;
+	    Lab_PixmapInsensitive(w) = tmp2_pix;
+	}
+
+	if (!IN_MENU(w)) {
+	    _XmDrawShadows(XtDisplay(w), XtWindow(w),
+			   Prim_TopShadowGC(w),
+			   Prim_BottomShadowGC(w),
+			   Lab_Highlight(w), Lab_Highlight(w),
+			   XtWidth(w) - 2 * Lab_Highlight(w),
+			   XtHeight(w) - 2 * Lab_Highlight(w),
+			   Lab_Shadow(w),
+			   State ? XmSHADOW_IN : XmSHADOW_OUT);
+	}
+	else {
+	    _XmDrawShadows(XtDisplay(w), XtWindow(w),
+			   Prim_TopShadowGC(w),
+			   Prim_BottomShadowGC(w),
+			   0, 0,
+			   XtWidth(w), XtHeight(w),
+			   Lab_Shadow(w),
+			   TB_Armed(w) ? (int)XmSHADOW_OUT : (int)XmNO_LINE);
+	}
+    }
+    else if (TB_IndType(w) == XmN_OF_MANY){
+	if (Lab_IsPixmap(w) && State &&
+	    TB_OnPixmap(w) != XmUNSPECIFIED_PIXMAP) {
+	    tmp_pix = Lab_Pixmap(w);
+	    tmp2_pix = Lab_PixmapInsensitive(w);
+	    Lab_Pixmap(w) = TB_OnPixmap(w);
+	}
+#define superclass (&xmLabelClassRec)
+	(*superclass->core_class.expose)(w, event, region);
+#undef superclass
+	if (Lab_IsPixmap(w) && State &&
+	    TB_OnPixmap(w) != XmUNSPECIFIED_PIXMAP) {
+	    Lab_Pixmap(w) = tmp_pix;
+	    Lab_PixmapInsensitive(w) = tmp2_pix;
+	}
+
+        XFillRectangle(XtDisplay(w), XtWindow(w),
+                       State ? TB_SelectGC(w) : TB_BackgroundGC(w),
+                       TB_Spacing(w) + Lab_Highlight(w) + Lab_Shadow(w),
+                       (XtHeight(w) - TB_IndicatorDim(w)) / 2,
+                       TB_IndicatorDim(w),
+                       TB_IndicatorDim(w));
+
+	if (TB_Visible(w) || State) {
+	    _XmDrawShadows(XtDisplay(w),XtWindow(w),
+		   Prim_TopShadowGC(w),
+		   Prim_BottomShadowGC(w),
+		   TB_Spacing(w) + Lab_Highlight(w) + Lab_Shadow(w),
+		   (XtHeight(w) - TB_IndicatorDim(w)) / 2,
+		   TB_IndicatorDim(w), TB_IndicatorDim(w),
+		   2, State ? XmSHADOW_IN : XmSHADOW_OUT);
+	}
+	if (!IN_MENU(w)) {
+	    _XmDrawShadows(XtDisplay(w),XtWindow(w),
+			   Prim_TopShadowGC(w),
+			   Prim_BottomShadowGC(w),
+			   Prim_HighlightThickness(w),
+			   Prim_HighlightThickness(w),
+			   XtWidth(w) - 2 * Prim_HighlightThickness(w),
+			   XtHeight(w) - 2 * Prim_HighlightThickness(w),
+			   Lab_Shadow(w),
+			   XmSHADOW_OUT);
+	}
+	else {
+	    _XmDrawShadows(XtDisplay(w), XtWindow(w),
+			   Prim_TopShadowGC(w),
+			   Prim_BottomShadowGC(w),
+			   0, 0,
+			   XtWidth(w), XtHeight(w),
+			   Lab_Shadow(w),
+			   TB_Armed(w) ? (int)XmSHADOW_OUT : (int)XmNO_LINE);
+	}
+    }
+    else 
+    {
+	if (Lab_IsPixmap(w) && State &&
+	    TB_OnPixmap(w) != XmUNSPECIFIED_PIXMAP) {
+	    tmp_pix = Lab_Pixmap(w);
+	    tmp2_pix = Lab_PixmapInsensitive(w);
+	    Lab_Pixmap(w) = TB_OnPixmap(w);
+	}
+#define superclass (&xmLabelClassRec)
+	(*superclass->core_class.expose)(w, event, region);
+#undef superclass
+	if (Lab_IsPixmap(w) && State &&
+	    TB_OnPixmap(w) != XmUNSPECIFIED_PIXMAP) {
+	    Lab_Pixmap(w) = tmp_pix;
+	    Lab_PixmapInsensitive(w) = tmp2_pix;
+	}
+
+	if (TB_Visible(w) || State) {
+	    _XmDrawDiamond(XtDisplay(w), XtWindow(w),
+	                   State
+				? Prim_TopShadowGC(w)
+				: Prim_BottomShadowGC(w),
+			   State
+				? Prim_BottomShadowGC(w)
+				: Prim_TopShadowGC(w),
+	                   State ? TB_SelectGC(w) : TB_BackgroundGC(w),
+			   TB_Spacing(w) + Lab_Highlight(w),
+			   (XtHeight(w) - TB_IndicatorDim(w)) / 2,
+			   TB_IndicatorDim(w), TB_IndicatorDim(w),
+			   2, True);
+	}
+	if (!IN_MENU(w)) {
+	    _XmDrawShadows(XtDisplay(w),XtWindow(w),
+			   Prim_TopShadowGC(w),
+			   Prim_BottomShadowGC(w),
+			   Prim_HighlightThickness(w),
+			   Prim_HighlightThickness(w),
+			   XtWidth(w) - 2 * Prim_HighlightThickness(w),
+			   XtHeight(w) - 2 * Prim_HighlightThickness(w),
+			   Lab_Shadow(w),
+			   XmSHADOW_OUT);
+	}
+	else {
+	    _XmDrawShadows(XtDisplay(w), XtWindow(w),
+			   Prim_TopShadowGC(w),
+			   Prim_BottomShadowGC(w),
+			   0, 0,
+			   XtWidth(w), XtHeight(w),
+			   Lab_Shadow(w),
+			   TB_Armed(w) ? (int)XmSHADOW_OUT : (int)XmNO_LINE);
+	}
+    }
+}
+
 static void
 Arm(Widget w,
     XEvent *event,
@@ -394,21 +639,26 @@ Arm(Widget w,
     Cardinal *num_params)
 {
     XmToggleButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
+
+    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 
     TB_Armed(w) = True;
     TB_VisualSet(w) = TB_IndicatorSet(w) = !TB_Set(w);
 
-    cbs.reason = XmCR_ARM;
-    cbs.event = event;
-    cbs.set = TB_Set(w);
+    (exp)(w, event, (Region)NULL);
 
-    XFlush(XtDisplay(w));
+    if (TB_ArmCallback(w)) {
+	cbs.reason = XmCR_ARM;
+	cbs.event = event;
+	cbs.set = TB_Set(w);
 
-    XtCallCallbackList(w,
-		       TB_ArmCallback(w),
-		       (XtPointer)&cbs);
+	XFlush(XtDisplay(w));
 
-    expose(w, NULL, (Region)NULL);
+	XtCallCallbackList(w,
+			   TB_ArmCallback(w),
+			   (XtPointer)&cbs);
+    }
 }
 
 static void
@@ -420,19 +670,33 @@ Select(Widget w,
     XmToggleButtonCallbackStruct cbs;
     XButtonEvent *ev = (XButtonEvent *)event;
 
-    if ((ev->x >= 0 && ev->x < XtWidth(w)) &&
-	(ev->y >= 0 && ev->y < XtHeight(w))) {
+    if (ev->type == KeyPress || ev->type == KeyRelease ||
+	((ev->x >= 0 && ev->x < XtWidth(w)) &&
+	 (ev->y >= 0 && ev->y < XtHeight(w)))) {
+
+#if 0
+      /* Why is this here?  We're not in a menu here. -- Chris */
+	if (XmIsRowColumn(XtParent(w)))
+            (*xmLabelClassRec.label_class.menuProcs)(XmMENU_CALLBACK,
+                                                     XtParent(w), False,
+                                                     w, &cbs);
+#endif
+
 	if (TB_VisualSet(w) == TB_IndicatorSet(w)) {
+
 	    TB_Set(w) = TB_VisualSet(w);
-	    cbs.reason = XmCR_VALUE_CHANGED;
-	    cbs.event = event;
-	    cbs.set = TB_Set(w);
 
-	    XFlush(XtDisplay(w));
+	    if (!Lab_SkipCallback(w) && TB_ValueChangedCallback(w)) {
+		cbs.reason = XmCR_VALUE_CHANGED;
+		cbs.event = event;
+		cbs.set = TB_Set(w);
 
-	    XtCallCallbackList(w,
-			       TB_ValueChangedCallback(w),
-			       (XtPointer)&cbs);
+		XFlush(XtDisplay(w));
+
+		XtCallCallbackList(w,
+				   TB_ValueChangedCallback(w),
+				   (XtPointer)&cbs);
+	    }
 	}
 	else
 	    TB_IndicatorSet(w) = TB_Set(w);
@@ -448,19 +712,19 @@ Disarm(Widget w,
 {
     XmToggleButtonCallbackStruct cbs;
 
-    /* expose(w, NULL, (Region)NULL); */
-
     TB_Set(w) = TB_IndicatorSet(w) = TB_VisualSet(w);
 
-    cbs.reason = XmCR_DISARM;
-    cbs.event = event;
-    cbs.set = TB_Set(w);
+    if (TB_DisarmCallback(w)) {
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.set = TB_Set(w);
 
-    XFlush(XtDisplay(w));
+	XFlush(XtDisplay(w));
 
-    XtCallCallbackList(w,
-		       TB_DisarmCallback(w),
-		       (XtPointer)&cbs);
+	XtCallCallbackList(w,
+			   TB_DisarmCallback(w),
+			   (XtPointer)&cbs);
+    }
 
     TB_Armed(w) = False;
 }
@@ -486,6 +750,11 @@ ButtonUp(Widget w,
     Boolean validButton, poppedUp;
     XmToggleButtonCallbackStruct cbs;
 
+    XdbDebug(__FILE__, w, "ToggleB ButtonUp()\n");
+
+    /* queue events until the next button event */
+    XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
+
     parent = XtParent(w);
 
     shell = parent;
@@ -493,15 +762,18 @@ ButtonUp(Widget w,
 	shell = XtParent(shell);
 
     if (event && (event->type == ButtonRelease))
-	(* xmLabelClassRec.label_class.menuProcs) (XmMENU_BUTTON, parent, NULL, event, &validButton);
+	(*xmLabelClassRec.label_class.menuProcs)(XmMENU_BUTTON, parent,
+						 NULL, event, &validButton);
 
     if (!validButton)
 	return;
 
     if (IN_MENU(w) && !XmIsMenuShell(shell))
-	(* xmLabelClassRec.label_class.menuProcs) (XmMENU_POPDOWN, w, NULL, event, &poppedUp);
+	(*xmLabelClassRec.label_class.menuProcs)(XmMENU_POPDOWN, w,
+						 NULL, event, &poppedUp);
     else
-	(* xmLabelClassRec.label_class.menuProcs) (XmMENU_BUTTON_POPDOWN, w , NULL, event, &poppedUp);
+	(*xmLabelClassRec.label_class.menuProcs)(XmMENU_BUTTON_POPDOWN, w,
+						 NULL, event, &poppedUp);
 
     _XmRecordEvent(event);
     
@@ -509,18 +781,31 @@ ButtonUp(Widget w,
 	return;
 
     TB_Armed(w) = False;
-    
+
     TB_Set(w) = TB_VisualSet(w);
-    cbs.reason = XmCR_VALUE_CHANGED;
-    cbs.event = event;
-    cbs.set = TB_Set(w);
 
-    XFlush(XtDisplay(w));
+    if (!Lab_SkipCallback(w) && TB_ValueChangedCallback(w)) {
+	cbs.reason = XmCR_VALUE_CHANGED;
+	cbs.event = event;
+	cbs.set = TB_Set(w);
 
-    XtCallCallbackList(w, 
-		       TB_ValueChangedCallback(w), 
-		       (XtPointer)&cbs);
+	XFlush(XtDisplay(w));
 
+	XtCallCallbackList(w, 
+			   TB_ValueChangedCallback(w), 
+			   (XtPointer)&cbs);
+    }
+    if (TB_DisarmCallback(w)) {
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.set = TB_Set(w);
+
+	XFlush(XtDisplay(w));
+
+	XtCallCallbackList(w,
+			   TB_DisarmCallback(w),
+			   (XtPointer)&cbs);
+    }
 
     _XmSetInDragMode(w, False);
 }
@@ -534,12 +819,17 @@ ButtonDown(Widget w,
     ShellWidget popupShell;
     int validButton;
     Widget child;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
     
+    /* queue events until the next button event. */
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
-    
+
+    XdbDebug(__FILE__, w, "ToggleB ButtonDown()\n");
+
     if (event && (event->type == ButtonPress))
     {
-	(* xmLabelClassRec.label_class.menuProcs) (XmMENU_BUTTON, XtParent(w), NULL, event, &validButton);
+	(*xmLabelClassRec.label_class.menuProcs)(XmMENU_BUTTON, XtParent(w),
+						 NULL, event, &validButton);
 	if (!validButton)
 	    return;
     }
@@ -547,16 +837,20 @@ ButtonDown(Widget w,
     _XmSetInDragMode(w, TRUE);
 
     TB_Armed(w) = True;
+    TB_VisualSet(w) = TB_IndicatorSet(w) = !TB_Set(w);
+    (*exp)(w, event, NULL);
 
     popupShell = (ShellWidget)_XmGetRC_PopupPosted(XtParent(w));
     if  (popupShell)
     {
 	if (popupShell->shell.popped_up)
-	    (* xmLabelClassRec.label_class.menuProcs)(XmMENU_SHELL_POPDOWN, (Widget)popupShell, NULL, event, NULL);
+	    (*xmLabelClassRec.label_class.menuProcs)(XmMENU_SHELL_POPDOWN,
+						     (Widget)popupShell,
+						     NULL, event, NULL);
 	
 	child = ((XmManagerWidget)XtParent(w))->manager.active_child;
 	if (child && (XmIsCascadeButton(child) || XmIsCascadeButtonGadget(child)))
-	    XmCascadeButtonHighlight (child, FALSE);
+	    XmCascadeButtonHighlight(child, FALSE);
     }
     
     _XmSetInDragMode(w, FALSE);
@@ -570,6 +864,7 @@ Help(Widget w,
      String *params,
      Cardinal *num_params)
 {
+    XtCallActionProc(w, "PrimitiveHelp", event, params, *num_params);
 }
 
 static void
@@ -578,10 +873,55 @@ EnterWindow(Widget w,
 	    String *params,
 	    Cardinal *num_params)
 {
-    if (TB_Armed(w))
+    XmToggleButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
+
+    XdbDebug(__FILE__, w, "ToggleB Enter\n");
+
+    if (!IN_MENU(w))
     {
-        TB_VisualSet(w) = TB_IndicatorSet(w);
-	expose(w, NULL, (Region)NULL);
+	_XmPrimitiveLeave(w, event, NULL, NULL);
+	if (TB_Armed(w))
+	{
+            TB_VisualSet(w) = TB_IndicatorSet(w);
+	    (*exp)(w, NULL, (Region)NULL);
+	}
+    }
+    else /* In menu */
+    {
+	if (_XmGetInDragMode(w))
+	{
+	    ShellWidget popupShell;
+
+	    popupShell = (ShellWidget)_XmGetRC_PopupPosted(XtParent(w));
+	    if  (popupShell)
+	    {
+		if (popupShell->shell.popped_up)
+		    (*xmLabelClassRec.label_class.menuProcs)(XmMENU_SHELL_POPDOWN, 
+							     (Widget)popupShell, 
+							     NULL, 
+							     event, 
+							     NULL);
+	    }
+
+	    TB_Armed(w) = True; 
+
+	    TB_VisualSet(w) = TB_VisualSet(w) = !TB_Set(w);
+
+	    (*exp)(w, event, NULL);
+
+	    if (TB_ArmCallback(w)) {
+		cbs.reason = XmCR_ARM;
+		cbs.event = event;
+		cbs.set = TB_Set(w);
+
+		XFlush(XtDisplay(w));
+
+		XtCallCallbackList(w,
+				   TB_ArmCallback(w),
+				   (XtPointer)&cbs);
+	    }
+	}
     }
 }
 
@@ -591,108 +931,41 @@ LeaveWindow(Widget w,
 	    String *params,
 	    Cardinal *num_params)
 {
-    if (TB_Armed(w))
+    XmToggleButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
+
+    if (!IN_MENU(w))
     {
-        TB_VisualSet(w) = TB_Set(w);
-	expose(w, NULL, (Region)NULL);
-    }
-}
-
-static void
-expose(Widget w,
-       XEvent *event,
-       Region region)
-{
-    Boolean   State = TB_VisualSet(w);
-    GC myGC;
-
-    if (XtSensitive(w))
-        myGC = TB_SelectGC(w);
-    else
-        myGC = Lab_InsensitiveGC(w);
-
-    if (TB_FillOnSelect(w) && State)
-        XSetForeground(XtDisplay(w),
-                       myGC,
-                       TB_SelectColor(w));
-    else
-        XSetForeground(XtDisplay(w),
-                       myGC,
-                       XtBackground(w));
-
-    
-    if (!TB_IndOn(w)) {
-	if (TB_FillOnSelect(w)) {
-	    XFillRectangle(XtDisplay(w), XtWindow(w),
-                           myGC,
-                           Lab_Highlight(w) + Lab_Shadow(w),
-                           Lab_Highlight(w) + Lab_Shadow(w),
-                           XtWidth(w)
-				 - 2 * (Lab_Highlight(w) + Lab_Shadow(w)),
-                           XtHeight(w)
-				 - 2 * (Lab_Highlight(w) + Lab_Shadow(w)));
+	_XmPrimitiveLeave(w, event, NULL, NULL);
+	if (TB_Armed(w))
+	{
+            TB_VisualSet(w) = TB_Set(w);
+	    (*exp)(w, NULL, (Region)NULL);
 	}
-        _XmDrawShadows(XtDisplay(w), XtWindow(w),
-                       Prim_TopShadowGC(w),
-                       Prim_BottomShadowGC(w),
-		       Lab_Highlight(w), Lab_Highlight(w),
-                       XtWidth(w) - 2 * Lab_Highlight(w),
-		       XtHeight(w) - 2 * Lab_Highlight(w),
-                       Lab_Shadow(w),
-                       State ? XmSHADOW_IN : XmSHADOW_OUT);
     }
-    else if (TB_IndType(w) == XmN_OF_MANY){
-        XFillRectangle(XtDisplay(w), XtWindow(w),
-                       myGC,
-                       TB_Spacing(w) + Lab_Highlight(w) + Lab_Shadow(w),
-                       (XtHeight(w) - TB_IndicatorDim(w)) / 2,
-                       TB_IndicatorDim(w),
-                       TB_IndicatorDim(w));
-	_XmDrawShadows(XtDisplay(w),XtWindow(w),
-		       Prim_TopShadowGC(w),
-		       Prim_BottomShadowGC(w),
-		       TB_Spacing(w) + Lab_Highlight(w) + Lab_Shadow(w),
-		       (XtHeight(w) - TB_IndicatorDim(w)) / 2,
-		       TB_IndicatorDim(w), TB_IndicatorDim(w),
-		       2, State ? XmSHADOW_IN : XmSHADOW_OUT);
-	_XmDrawShadows(XtDisplay(w),XtWindow(w),
-		       Prim_TopShadowGC(w),
-		       Prim_BottomShadowGC(w),
-		       Prim_HighlightThickness(w),
-		       Prim_HighlightThickness(w),
-		       XtWidth(w) - 2 * Prim_HighlightThickness(w),
-		       XtHeight(w) - 2 * Prim_HighlightThickness(w),
-		       Lab_Shadow(w),
-		       XmSHADOW_OUT);
-    }
-    else 
+    else /* we're in a menu */
     {
-	_XmDrawDiamond(XtDisplay(w), XtWindow(w),
-                       State
-                           ? Prim_TopShadowGC(w)
-                           : Prim_BottomShadowGC(w),
-                       State
-                           ? Prim_BottomShadowGC(w)
-                           : Prim_TopShadowGC(w),
-                       myGC,
-		       TB_Spacing(w) + Lab_Highlight(w),
-		       (XtHeight(w) - TB_IndicatorDim(w)) / 2,
-		       TB_IndicatorDim(w), TB_IndicatorDim(w),
-		       2, True);
-	_XmDrawShadows(XtDisplay(w),XtWindow(w),
-		       Prim_TopShadowGC(w),
-		       Prim_BottomShadowGC(w),
-		       Prim_HighlightThickness(w),
-		       Prim_HighlightThickness(w),
-		       XtWidth(w) - 2 * Prim_HighlightThickness(w),
-		       XtHeight(w) - 2 * Prim_HighlightThickness(w),
-		       Lab_Shadow(w),
-		       XmSHADOW_OUT);
-    }
+	if (_XmGetInDragMode(w))
+	{
+            TB_VisualSet(w) = TB_Set(w);
 
-#define superclass (&xmLabelClassRec)
-    (*superclass->core_class.expose)(w, event, region);
-#undef superclass
+	    TB_Armed(w) = False;
+
+	    (*exp)(w, event, NULL);
+
+	    if (TB_DisarmCallback(w)) {
+		cbs.reason = XmCR_DISARM;
+		cbs.event = event;
+		cbs.set = TB_Set(w);
+
+		XFlush(XtDisplay(w));
+    
+		XtCallCallbackList(w,
+				   TB_DisarmCallback(w),
+				   (XtPointer)&cbs);
+	    }
+	}
+    }
 }
 
 Widget
@@ -711,7 +984,11 @@ XmCreateToggleButton(Widget parent,
 Boolean
 XmToggleButtonGetState(Widget widget)
 {
-    return TB_Set(widget);
+    if (XmIsToggleButtonGadget(widget))
+	return XmToggleButtonGadgetGetState(widget);
+    else if (XmIsToggleButton(widget))
+	return TB_Set(widget);
+    return False;
 }
 
 void
@@ -720,24 +997,32 @@ XmToggleButtonSetState(Widget widget,
 		       Boolean notify)
 {
     XmToggleButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(widget)->core_class.expose;
+
+    if (XmIsGadget(widget)) {
+	XmToggleButtonGadgetSetState(widget, state, notify);
+	return;
+    }
+    if (!XmIsToggleButton(widget))
+	return;
 
     if ((TB_Set(widget) && !state) || (!TB_Set(widget) && state)) 
     {
        TB_VisualSet(widget) = TB_Set(widget) = state ? True : False;
-        if (notify) 
+        if (notify && TB_ValueChangedCallback(widget)) 
         {
-            cbs.reason = XmCR_VALUE_CHANGED;
-            cbs.event = NULL; /* FIXME -- IS THIS RIGHT ? */
-            cbs.set = state;
+	    cbs.reason = XmCR_VALUE_CHANGED;
+	    cbs.event = NULL;
+	    cbs.set = state;
 
 	    XFlush(XtDisplay(widget));
 
-            XtCallCallbackList(widget,
+	    XtCallCallbackList(widget,
 			       TB_ValueChangedCallback(widget),
 			       (XtPointer)&cbs);
         }
 
-        if (XtIsRealized(widget))
-            expose(widget, NULL, (Region)NULL);
+	if (XtIsRealized(widget))
+	    (*exp)(widget, NULL, (Region)NULL);
     }
 }
