@@ -1,6 +1,6 @@
 /**
  *
- * $Id: VaSimple.c,v 1.12 1996/04/30 23:59:32 toshok Exp $
+ * $Id: VaSimple.c,v 1.6 1996/11/28 09:22:20 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -49,9 +49,11 @@
  *
  **/
 
-static char rcsid[] = "$Id: VaSimple.c,v 1.12 1996/04/30 23:59:32 toshok Exp $";
+static char rcsid[] = "$Id: VaSimple.c,v 1.6 1996/11/28 09:22:20 u27113 Exp $";
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
 #include <Xm/ToggleBGP.h>
 #include <Xm/PushBGP.h>
@@ -59,9 +61,9 @@ static char rcsid[] = "$Id: VaSimple.c,v 1.12 1996/04/30 23:59:32 toshok Exp $";
 #include <Xm/SeparatoGP.h>
 #include <Xm/RowColumnP.h>
 #include <Xm/VaSimpleP.h>
-#include <Xm/DebugUtil.h>
 #include <X11/Xfuncs.h>
-#include <stdio.h>
+
+#include <XmI/DebugUtil.h>
 
 /* this seems realistic to me */
 #define MAX_EMBEDDED_BUTTONS	512
@@ -71,35 +73,6 @@ static KeySym button_mnemonics[MAX_EMBEDDED_BUTTONS];
 static String button_accel[MAX_EMBEDDED_BUTTONS];
 static XmString button_acc_text[MAX_EMBEDDED_BUTTONS];
 
-#define Offset(field) XtOffsetOf(XmSimpleMenuRec, field)
-static XtResource simple_resources[] = {
-    {XmNbuttonAccelerators, XmCButtonAccelerators, XmRStringTable, sizeof(String *),
-     Offset(accelerator), XmRImmediate, (XtPointer)NULL},
-    {XmNbuttonAcceleratorText, XmCButtonAcceleratorText, XmRXmStringTable, sizeof(XmStringTable),
-     Offset(accelerator_text), XmRImmediate, (XtPointer)NULL},
-    {XmNbuttonCount, XmCButtonCount, XmRInt, sizeof(int),
-     Offset(count), XmRImmediate, (XtPointer)0},
-    {XmNbuttonMnemonicCharSets, XmCButtonMnemonicCharSets, XmRStringTable, sizeof(String *),
-     Offset(mnemonic_charset), XmRImmediate, (XtPointer)NULL},
-    {XmNbuttonMnemonics, XmCButtonMnemonics, XmRKeySymTable, sizeof(XmKeySymTable),
-     Offset(mnemonic), XmRImmediate, (XtPointer)NULL},
-    {XmNbuttons, XmCButtons, XmRXmStringTable, sizeof(XmStringTable),
-     Offset(label_string), XmRImmediate, (XtPointer)NULL},
-    {XmNbuttonSet, XmCButtonSet, XmRInt, sizeof(int),
-     Offset(button_set), XmRImmediate, (XtPointer)-1},
-    {XmNbuttonType, XmCButtonType, XmRPointer, sizeof(XtPointer),
-     Offset(button_type), XmRImmediate, (XtPointer)NULL},
-    {XmNoptionLabel, XmCOptionLabel, XmRXmString, sizeof(XmString),
-     Offset(option_label), XmRImmediate, (XtPointer)NULL},
-    {XmNoptionLabel, XmCOptionLabel, XmRXmString, sizeof(XmString),
-     Offset(option_label), XmRImmediate, (XtPointer)NULL},
-    {XmNoptionMnemonic, XmCOptionMnemonic, XmRKeySym, sizeof(KeySym),
-     Offset(option_mnemonic), XmRImmediate, (XtPointer)NULL},
-    {XmNpostFromButton, XmCPostFromButton, XmRInt, sizeof(int),
-     Offset(option_mnemonic), XmRImmediate, (XtPointer)NULL},
-    {XmNsimpleCallback, XmCCallback, XmRCallback, sizeof(XtCallbackList),
-     Offset(callback), XmRImmediate, (XtPointer)NULL},
-};
 
 extern Widget _XtCreateWidget(String,
 			      WidgetClass,
@@ -280,7 +253,7 @@ _XmTypedArgToArg(Widget widget, XtTypedArgList typed_arg, ArgList arg_return,
     }
 
     arg_return->name = typed_arg->name;
-    memory_return->value = (XtArgVal) NULL;
+    memory_return->value = (XtArgVal)NULL;
 
     if (strcmp(to_type, XtRString) == 0) {
 	arg_return->value = (XtArgVal) to_val.addr;
@@ -295,9 +268,9 @@ _XmTypedArgToArg(Widget widget, XtTypedArgList typed_arg, ArgList arg_return,
 	else if (to_val.size == sizeof(XtArgVal))
 	    arg_return->value = *(XtArgVal *)to_val.addr;
 	else if (to_val.size > sizeof(XtArgVal)) {
-	    arg_return->value = (XtArgVal) XtMalloc(to_val.size);
-	    memory_return->value = (XtArgVal)
-		memcpy((void *)arg_return->value, to_val.addr, to_val.size);
+	    arg_return->value = (XtArgVal)XtMalloc(to_val.size);
+	    memory_return->value = (XtArgVal)arg_return->value;
+	    bcopy(to_val.addr, (void *)arg_return->value, to_val.size);
 	}
     }
        
@@ -602,149 +575,141 @@ _XmVaToTypedArgList(va_list var, int max_count,
     *num_args_return = count;
 }
 
+/*
+ * 
+ * note: The Va version of the "Simple" routines wipe out any
+ * "Simple" creation resources the caller specified in the va_list.
+ * This is poorly documented, but is Motif 1.2 behavior.
+ */
+
 Widget
 XmVaCreateSimpleCheckBox(Widget parent,
 			 String name,
 			 XtCallbackProc callback,
 			 ...)
 {
+    ArgList arglist;
+    Arg myarglist[8];
+    Cardinal n = 0;
+    int total_count = 0;
+    int args_count = 0;
+    int button_count = 0;
+    int typed_count = 0;
     va_list var;
     Widget rc;
-    Arg myArgList[5];
-    char buf[32];
-    int i, n, total_count, typed_count, args_count, button_count;
-    ArgList args;
-    XmSimpleMenuRec data;
-    XtTypedArgList typed_args = NULL;
-    Cardinal num_args;
 
     XdbDebug(__FILE__, parent, "XmVaCreateSimpleCheckBox();\n");
 
     while (parent && !XtIsComposite(parent))
 	parent = XtParent(parent);
 
-    memset((void *)&data, 0, sizeof(XmSimpleMenuRec));
-
-    n = 0;
-#if 0
-    XtSetArg(myArgList[n], XmNrowColumnType, XmWORK_AREA); n++;
-#endif
-    XtSetArg(myArgList[n], XmNradioAlwaysOne, False); n++;
-    XtSetArg(myArgList[n], XmNisHomogeneous, True); n++;
-    XtSetArg(myArgList[n], XmNentryClass, xmToggleButtonGadgetClass); n++;
-
+    /* find out how many buttons, how many args, etc... we have 
+     * (this also sets the button_strings, button_mnemonics, etc
+     * arrays)
+     */
     Va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToTypedArgList(var, total_count, &typed_args, &num_args);
+    /* set Simple resources */
+    XtSetArg(myarglist[n], XmNsimpleCallback, callback); n++;
+    XtSetArg(myarglist[n], XmNbuttonCount, button_count); n++;
+    XtSetArg(myarglist[n], XmNbuttons, button_strings); n++;
 
-    rc = _XtCreateWidget(name,
-			 xmRowColumnWidgetClass,
-			 parent,
-			 myArgList, n,
-			 typed_args, num_args);
-    if (typed_args != NULL) {
-	XtFree((XtPointer)typed_args);
-    }
+#if 0
+    /* the XmVaCreateSimpleCheckBox man page says these Simple
+     * resources aren't used in the 1.2 release of Motif 
+     */
+    XtSetArg(myarglist[n], XmNbuttonMnemonics, button_mnemonics); n++;
+    XtSetArg(myarglist[n], XmNbuttonAccelerators, button_accel); n++;
+    XtSetArg(myarglist[n], XmNbuttonAcceleratorText, button_acc_text); n++;
+#endif
+
+    rc = XmCreateSimpleCheckBox(parent, name, myarglist, n);
+
+    /* I'm not sure if this (the callers valist) list should be merged
+     * into the list passed to the above XmCreateSimpleCheckBox() or
+     * if it can be set afterward, but it is easier to set it
+     * afterward
+     */
+    Va_start(var, callback);
+    _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToArgList(rc, var, total_count, &args, &num_args);
-    XtGetApplicationResources(rc, (XtPointer)&data, 
-			      simple_resources, XtNumber(simple_resources),
-			      args, num_args);
-    if (data.count == 0)
-	data.count = button_count;
-    data.callback = callback;
-    data.button_type = button_types;
-    data.label_string = button_strings;
-    data.mnemonic = button_mnemonics;
-    data.accelerator = button_accel;
-    data.accelerator_text = button_acc_text;
-
-    for (i = 0; i < data.count; i++) {
-	sprintf(buf, "button_%d", i);
-        _XmCreateSimpleGadget(buf, rc, XmTOGGLEBUTTON, &data, i, args, num_args);
-    }
-    va_end(var);
-    if (num_args != 0) {
-	_XmFreeArgList(args, total_count, typed_count);
-    }
+    XtSetValues(rc, arglist, n);
+    
+    _XmFreeArgList(arglist, total_count, typed_count);
 
     return rc;
 }
 
+/*
+ * I'm not 100% sure the XmVaCreateSimple*Menu* calls are correct,
+ * since I can't really test them.
+ * 
+ * note: The Va version of the "Simple" routines wipe out any
+ * "Simple" creation resources the caller specified in the va_list.
+ * This is poorly documented, but is Motif 1.2 behavior.
+ */
 Widget
 XmVaCreateSimpleMenuBar(Widget parent,
 			String name,
 			...)
 {
+    ArgList arglist;
+    Arg myarglist[8];
+    Cardinal n = 0;
+    int total_count = 0;
+    int args_count = 0;
+    int button_count = 0;
+    int typed_count = 0;
     va_list var;
     Widget rc;
-    Arg myArgList[5];
-    char buf[32];
-    int i, n, total_count, typed_count, args_count, button_count;
-    ArgList args;
-    XmSimpleMenuRec data;
-    XtTypedArgList typed_args = NULL;
-    Cardinal num_args;
 
     XdbDebug(__FILE__, parent, "XmVaCreateSimpleMenuBar();\n");
 
     while (parent && !XtIsComposite(parent))
 	parent = XtParent(parent);
 
-    memset((void *)&data, 0, sizeof(XmSimpleMenuRec));
-
-    n = 0;
-    XtSetArg(myArgList[n], XmNrowColumnType, XmMENU_BAR); n++;
-    XtSetArg(myArgList[n], XmNentryClass, xmCascadeButtonGadgetClass); n++;
-    XtSetArg(myArgList[n], XmNisHomogeneous, True); n++;
-
+    /* find out how many buttons, how many args, etc... we have 
+     * (this also sets the button_strings, button_mnemonics, etc
+     * arrays)
+     */
     Va_start(var, name);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
-    Va_start(var, name);
-    _XmVaToTypedArgList(var, total_count, &typed_args, &num_args);
+    /* set Simple resources */
+    XtSetArg(myarglist[n], XmNbuttonCount, button_count); n++;
+    XtSetArg(myarglist[n], XmNbuttons, button_strings); n++;
+    XtSetArg(myarglist[n], XmNbuttonMnemonics, button_mnemonics); n++;
 
-    rc = _XtCreateWidget(name,
-			 xmRowColumnWidgetClass,
-			 parent,
-			 myArgList, n,
-			 typed_args, num_args);
-    if (typed_args != NULL) {
-	XtFree((XtPointer)typed_args);
-    }
+    rc = XmCreateSimpleMenuBar(parent, name, myarglist, n);
+
+    /* I'm not sure if this (the callers valist) list should be merged
+     * into the list passed to the above XmCreateSimpleMenuBar() or
+     * if it can be set afterward, but it is easier to set it
+     * afterward
+     */
+    Va_start(var, name);
+    _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
-    Va_start(var, name);
-    _XmVaToArgList(rc, var, total_count, &args, &num_args);
-    XtGetApplicationResources(rc, (XtPointer)&data, 
-			      simple_resources, XtNumber(simple_resources),
-			      args, num_args);
-    if (data.count == 0)
-	data.count = button_count;
-    data.button_type = button_types;
-    data.label_string = button_strings;
-    data.mnemonic = button_mnemonics;
-    data.accelerator = button_accel;
-    data.accelerator_text = button_acc_text;
-
-    for (i = 0; i < data.count; i++) {
-	sprintf(buf, "button_%d", i);
-        _XmCreateSimpleGadget(buf, rc, XmCASCADEBUTTON, &data, i, args, num_args);
-    }
-    va_end(var);
-    if (num_args != 0) {
-	_XmFreeArgList(args, total_count, typed_count);
-    }
+    XtSetValues(rc, arglist, n);
+    
+    _XmFreeArgList(arglist, total_count, typed_count);
 
     return rc;
 }
 
+/*
+ * This sucker can be (and has been) tested with Xinvest, also with
+ * testXm/vasimple/test4 .
+ *
+ * note: The Va version of the "Simple" routines wipe out any
+ * "Simple" creation resources the caller specified in the va_list.
+ * This is poorly documented, but is Motif 1.2 behavior.
+ */
 Widget
 XmVaCreateSimpleOptionMenu(Widget parent,
 			   String name,
@@ -754,181 +719,122 @@ XmVaCreateSimpleOptionMenu(Widget parent,
 			   XtCallbackProc callback,
 			   ...)
 {
+    ArgList arglist, combined;
+    Arg myarglist[11];
+    Cardinal n = 0;
+    int total_count = 0, args_count = 0, button_count = 0, typed_count = 0, mine;
     va_list var;
-    Widget rc, label;
-    Arg myArgList[5];
-    ArgList args;
-    char buf[32];
-    int i, n, total_count, typed_count, args_count, button_count;
-    XmSimpleMenuRec data;
-    char *oname;
-    XtTypedArgList typed_args = NULL;
-    Cardinal num_args = 0;
-    int mt = 0;
+    Widget rc = NULL;
 
     XdbDebug(__FILE__, parent, "XmVaCreateSimpleOptionMenu();\n");
 
     while (parent && !XtIsComposite(parent))
 	parent = XtParent(parent);
 
-    memset((void *)&data, 0, sizeof(XmSimpleMenuRec));
-
-    n = 0;
-    XtSetArg(myArgList[n], XmNrowColumnType, XmMENU_OPTION); n++;
-
+    /* find out how many buttons, how many args, etc... we have 
+     * (this also sets the button_strings, button_mnemonics, etc
+     * arrays)
+     */
     Va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
     Va_start(var, callback);
-    _XmVaToTypedArgList(var, total_count, &typed_args, &num_args);
-
-    rc = _XtCreateWidget(name,
-			 xmRowColumnWidgetClass,
-			 parent,
-			 myArgList, n,
-			 typed_args, num_args);
-    if (typed_args != NULL) {
-	XtFree((XtPointer)typed_args);
-    }
+    _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
-    XtVaGetValues(rc, XmNrowColumnType, &mt, NULL);
-    XdbDebug(__FILE__, rc, "RCTYPE: %d\n", mt);
+    /* set Simple resources */
+    mine = 0;
+    XtSetArg(myarglist[mine], XmNbuttonSet, button_set); mine++;
+    XtSetArg(myarglist[mine], XmNoptionLabel, option_label); mine++;
+    XtSetArg(myarglist[mine], XmNoptionMnemonic, option_mnemonic); mine++;
+    XtSetArg(myarglist[mine], XmNsimpleCallback, callback); mine++;
+    XtSetArg(myarglist[mine], XmNbuttonCount, button_count); mine++;
+    XtSetArg(myarglist[mine], XmNbuttonType, button_types); mine++;
+    XtSetArg(myarglist[mine], XmNbuttons, button_strings); mine++;
+    XtSetArg(myarglist[mine], XmNbuttonMnemonics, button_mnemonics); mine++;
+    XtSetArg(myarglist[mine], XmNbuttonAccelerators, button_accel); mine++;
+    XtSetArg(myarglist[mine], XmNbuttonAcceleratorText, button_acc_text); mine++;
 
-    Va_start(var, callback);
-    _XmVaToArgList(rc, var, total_count, &args, &num_args);
-    XtGetApplicationResources(rc, (XtPointer)&data, 
-			      simple_resources, XtNumber(simple_resources),
-			      args, num_args);
-    data.option_label = option_label;
-    data.option_mnemonic = option_mnemonic;
-    data.button_set = button_set;
-    if (data.count == 0)
-	data.count = button_count;
-    data.callback = callback;
-    data.button_type = button_types;
-    data.label_string = button_strings;
-    data.mnemonic = button_mnemonics;
-    data.accelerator = button_accel;
-    data.accelerator_text = button_acc_text;
-
-    oname = XtMalloc(strlen(name) + strlen("_label") + 1);
-    strcpy(oname, name);
-    strcat(oname, "_label");
-    n = 0;
-    RC_OptionLabel(rc) = XmStringCopy(option_label);
-    XtSetArg(myArgList[n], XmNlabelString, RC_OptionLabel(rc)); n++;
-    label = XmCreateLabelGadget(rc, oname, myArgList, n);
-    XtManageChild(label);
-    XtFree(oname);
-
-    oname = XtMalloc(strlen(name) + strlen("_cascade") + 1);
-    strcpy(oname, name);
-    strcat(oname, "_cascade");
-    n = 0;
-    XtSetArg(myArgList[n], XmNmnemonic, option_mnemonic); n++;
-    RC_CascadeBtn(rc) = XmCreateCascadeButtonGadget(rc, oname, myArgList, n);
-    XtManageChild(RC_CascadeBtn(rc));
-    XtFree(oname);
-
-#if 0
-    /* This isn't right.  This should be passed in by the user, and we should
-     * check to see if the subMenuId is not set.  If it isn't we should fail
-     * the call.
-     */
-    oname = XtMalloc(strlen(name) + strlen("_option") + 1);
-    strcpy(oname, name);
-    strcat(oname, "_option");
-    n = 0;
-    XtSetArg(myArgList[n], XmNrowColumnType, XmMENU_PULLDOWN); n++;
-    RC_OptionSubMenu(rc) = XmCreateRowColumn(rc, oname, myArgList, n);
-    XtVaSetValues(RC_CascadeBtn(rc), XmNsubMenuId, RC_OptionSubMenu(rc), NULL);
-    XtFree(oname);
-#endif
-
-    for (i = 0; i < data.count; i++) {
-	sprintf(buf, "button_%d", i);
-        _XmCreateSimpleGadget(buf, RC_OptionSubMenu(rc), XmPUSHBUTTON, &data, i, args, num_args);
-    }
-    va_end(var);
-
-    if (num_args != 0) { 
-	_XmFreeArgList(args, total_count, typed_count);
-    }
+    combined = XtMergeArgLists(myarglist, mine, arglist, n);
+    rc = XmCreateSimpleOptionMenu(parent, name, combined, n + mine);
+    XtFree((XtPointer)combined);
 
     return rc;
 }
 
+/*
+ * I'm not 100% sure the XmVaCreateSimple*Menu* calls are correct,
+ * since I can't really test them.
+ * 
+ * note: The Va version of the "Simple" routines wipe out any
+ * "Simple" creation resources the caller specified in the va_list.
+ * This is poorly documented, but is Motif 1.2 behavior.
+ */
 Widget
 XmVaCreateSimplePopupMenu(Widget parent,
 			  String name,
 			  XtCallbackProc callback,
 			  ...)
 {
+    ArgList arglist;
+    Arg myarglist[11];
+    Cardinal n = 0;
+    int total_count = 0;
+    int args_count = 0;
+    int button_count = 0;
+    int typed_count = 0;
     va_list var;
     Widget rc;
-    Arg myArgList[5];
-    char buf[32];
-    int i, n, total_count, typed_count, args_count, button_count;
-    ArgList args;
-    XmSimpleMenuRec data;
-    XtTypedArgList typed_args = NULL;
-    Cardinal num_args;
 
     XdbDebug(__FILE__, parent, "XmVaCreateSimplePopupMenu();\n");
 
     while (parent && !XtIsComposite(parent))
 	parent = XtParent(parent);
 
-    memset((void *)&data, 0, sizeof(XmSimpleMenuRec));
-
-    n = 0;
-    XtSetArg(myArgList[n], XmNrowColumnType, XmMENU_POPUP); n++;
-
+    /* find out how many buttons, how many args, etc... we have 
+     * (this also sets the button_strings, button_mnemonics, etc
+     * arrays)
+     */
     Va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToTypedArgList(var, total_count, &typed_args, &num_args);
+    /* set Simple resources */
+    XtSetArg(myarglist[n], XmNsimpleCallback, callback); n++;
+    XtSetArg(myarglist[n], XmNbuttonCount, button_count); n++;
+    XtSetArg(myarglist[n], XmNbuttonType, button_types); n++;
+    XtSetArg(myarglist[n], XmNbuttons, button_strings); n++;
+    XtSetArg(myarglist[n], XmNbuttonMnemonics, button_mnemonics); n++;
+    XtSetArg(myarglist[n], XmNbuttonAccelerators, button_accel); n++;
+    XtSetArg(myarglist[n], XmNbuttonAcceleratorText, button_acc_text); n++;
 
-    rc = _XtCreateWidget(name,
-			 xmRowColumnWidgetClass,
-			 parent,
-			 myArgList, n,
-			 typed_args, num_args);
-    if (typed_args != NULL) {
-	XtFree((XtPointer)typed_args);
-    }
+    rc = XmCreateSimplePopupMenu(parent, name, myarglist, n);
+
+    /* I'm not sure if this (the callers valist) list should be merged
+     * into the list passed to the above XmCreateSimplePopupMenu() or
+     * if it can be set afterward, but it is easier to set it
+     * afterward
+     */
+    Va_start(var, callback);
+    _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToArgList(rc, var, total_count, &args, &num_args);
-    XtGetApplicationResources(rc, (XtPointer)&data, 
-			      simple_resources, XtNumber(simple_resources),
-			      args, num_args);
-    if (data.count == 0)
-	data.count = button_count;
-    data.callback = callback;
-    data.button_type = button_types;
-    data.label_string = button_strings;
-    data.mnemonic = button_mnemonics;
-    data.accelerator = button_accel;
-    data.accelerator_text = button_acc_text;
-
-    for (i = 0; i < data.count; i++) {
-	sprintf(buf, "button_%d", i);
-        _XmCreateSimpleGadget(buf, rc, XmTOGGLEBUTTON, &data, i, args, num_args);
-    }
-    va_end(var);
-    if (num_args != 0) {
-	_XmFreeArgList(args, total_count, typed_count);
-    }
+    XtSetValues(rc, arglist, n);
+    
+    _XmFreeArgList(arglist, total_count, typed_count);
 
     return rc;
 }
 
+/*
+ * I'm not 100% sure the XmVaCreateSimple*Menu* calls are correct,
+ * since I can't really test them.
+ * 
+ * note: The Va version of the "Simple" routines wipe out any
+ * "Simple" creation resources the caller specified in the va_list.
+ * This is poorly documented, but is Motif 1.2 behavior.
+ */
 Widget
 XmVaCreateSimplePulldownMenu(Widget parent,
 			     String name,
@@ -936,70 +842,65 @@ XmVaCreateSimplePulldownMenu(Widget parent,
 			     XtCallbackProc callback,
 			     ...)
 {
+    ArgList arglist;
+    Arg myarglist[11];
+    Cardinal n = 0;
+    int total_count = 0;
+    int args_count = 0;
+    int button_count = 0;
+    int typed_count = 0;
     va_list var;
     Widget rc;
-    Arg myArgList[5];
-    char buf[32];
-    int i, n, total_count, typed_count, args_count, button_count;
-    ArgList args;
-    XmSimpleMenuRec data;
-    XtTypedArgList typed_args = NULL;
-    Cardinal num_args;
+
 
     XdbDebug(__FILE__, parent, "XmVaCreateSimplePulldownMenu();\n");
 
     while (parent && !XtIsComposite(parent))
 	parent = XtParent(parent);
 
-    memset((void *)&data, 0, sizeof(XmSimpleMenuRec));
-
-    n = 0;
-    XtSetArg(myArgList[n], XmNrowColumnType, XmMENU_PULLDOWN); n++;
 
+    /* find out how many buttons, how many args, etc... we have 
+     * (this also sets the button_strings, button_mnemonics, etc
+     * arrays)
+     */
     Va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToTypedArgList(var, total_count, &typed_args, &num_args);
+    /* set Simple resources */
+    XtSetArg(myarglist[n], XmNpostFromButton, post_from_button); n++;
+    XtSetArg(myarglist[n], XmNsimpleCallback, callback); n++;
+    XtSetArg(myarglist[n], XmNbuttonCount, button_count); n++;
+    XtSetArg(myarglist[n], XmNbuttonType, button_types); n++;
+    XtSetArg(myarglist[n], XmNbuttons, button_strings); n++;
+    XtSetArg(myarglist[n], XmNbuttonMnemonics, button_mnemonics); n++;
+    XtSetArg(myarglist[n], XmNbuttonAccelerators, button_accel); n++;
+    XtSetArg(myarglist[n], XmNbuttonAcceleratorText, button_acc_text); n++;
 
-    rc = _XtCreateWidget(name,
-			 xmRowColumnWidgetClass,
-			 parent,
-			 myArgList, n,
-			 typed_args, num_args);
-    if (typed_args != NULL) {
-	XtFree((XtPointer)typed_args);
-    }
+    rc = XmCreateSimplePulldownMenu(parent, name, myarglist, n);
+
+    /* I'm not sure if this (the callers valist) list should be merged
+     * into the list passed to the above XmCreateSimplePulldownMenu() or
+     * if it can be set afterward, but it is easier to set it
+     * afterward
+     */
+    Va_start(var, callback);
+    _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToArgList(rc, var, total_count, &args, &num_args);
-    XtGetApplicationResources(rc, (XtPointer)&data, 
-			      simple_resources, XtNumber(simple_resources),
-			      args, num_args);
-    data.post_from_button = post_from_button;
-    if (data.count == 0)
-	data.count = button_count;
-    data.callback = callback;
-    data.button_type = button_types;
-    data.label_string = button_strings;
-    data.mnemonic = button_mnemonics;
-    data.accelerator = button_accel;
-    data.accelerator_text = button_acc_text;
-
-    for (i = 0; i < data.count; i++) {
-	sprintf(buf, "button_%d", i);
-        _XmCreateSimpleGadget(buf, rc, XmPUSHBUTTON, &data, i, args, num_args);
-    }
-    va_end(var);
-    if (num_args != 0) {
-	_XmFreeArgList(args, total_count, typed_count);
-    }
+    XtSetValues(rc, arglist, n);
+    
+    _XmFreeArgList(arglist, total_count, typed_count);
 
     return rc;
 }
 
+/*
+ * 
+ * note: The Va version of the "Simple" routines wipe out any
+ * "Simple" creation resources the caller specified in the va_list.
+ * This is poorly documented, but is Motif 1.2 behavior.
+ */
 Widget
 XmVaCreateSimpleRadioBox(Widget parent,
 			 String name,
@@ -1007,81 +908,58 @@ XmVaCreateSimpleRadioBox(Widget parent,
 			 XtCallbackProc callback,
 			 ...)
 {
+    ArgList arglist;
+    Arg myarglist[8];
+    Cardinal n = 0;
+    int total_count = 0;
+    int args_count = 0;
+    int button_count = 0;
+    int typed_count = 0;
     va_list var;
-    Widget rc, tgl;
-    Arg myArgList[5];
-    char buf[32];
-    int i, n, total_count, typed_count, args_count, button_count;
-    ArgList args;
-    XmSimpleMenuRec data;
-    XtTypedArgList typed_args = NULL;
-    Cardinal num_args;
+    Widget rc;
 
     XdbDebug(__FILE__, parent, "XmVaCreateSimpleRadioBox();\n");
 
     while (parent && !XtIsComposite(parent))
 	parent = XtParent(parent);
 
-    memset((void *)&data, 0, sizeof(XmSimpleMenuRec));
-
-    n = 0;
-#if 0
-    XtSetArg(myArgList[n], XmNrowColumnType, XmWORK_AREA); n++;
-#endif
-    XtSetArg(myArgList[n], XmNradioAlwaysOne, True); n++;
-    XtSetArg(myArgList[n], XmNisHomogeneous, True); n++;
-    XtSetArg(myArgList[n], XmNentryClass, xmToggleButtonGadgetClass); n++;
-
+    /* find out how many buttons, how many args, etc... we have 
+     * (this also sets the button_strings, button_mnemonics, etc
+     * arrays)
+     */
     Va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToTypedArgList(var, total_count, &typed_args, &num_args);
+    /* set Simple resources */
+    XtSetArg(myarglist[n], XmNbuttonSet, button_set); n++;
+    XtSetArg(myarglist[n], XmNsimpleCallback, callback); n++;
+    XtSetArg(myarglist[n], XmNbuttonCount, button_count); n++;
+    XtSetArg(myarglist[n], XmNbuttons, button_strings); n++;
 
-    rc = _XtCreateWidget(name,
-			 xmRowColumnWidgetClass,
-			 parent,
-			 myArgList, n,
-			 typed_args, num_args);
-    if (typed_args != NULL) {
-	XtFree((XtPointer)typed_args);
-    }
+#if 0
+    /* the XmVaCreateSimpleRadioButton man page says these Simple
+     * resources aren't used in the 1.2 release of Motif 
+     */
+    XtSetArg(myarglist[n], XmNbuttonMnemonics, button_mnemonics); n++;
+    XtSetArg(myarglist[n], XmNbuttonAccelerators, button_accel); n++;
+    XtSetArg(myarglist[n], XmNbuttonAcceleratorText, button_acc_text); n++;
+#endif
+
+    rc = XmCreateSimpleRadioBox(parent, name, myarglist, n);
+
+    /* I'm not sure if this (the callers valist) list should be merged
+     * into the list passed to the above XmCreateSimpleRadioBox() or
+     * if it can be set afterward, but it is easier to set it
+     * afterward
+     */
+    Va_start(var, callback);
+    _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
-    Va_start(var, callback);
-    _XmVaToArgList(rc, var, total_count, &args, &num_args);
-    XtGetApplicationResources(rc, (XtPointer)&data, 
-			      simple_resources, XtNumber(simple_resources),
-			      args, num_args);
-    data.button_set = button_set;
-    data.callback = callback;
-    if (data.count == 0)
-	data.count = button_count;
-    data.button_type = button_types;
-    data.label_string = button_strings;
-    data.mnemonic = button_mnemonics;
-    data.accelerator = button_accel;
-    data.accelerator_text = button_acc_text;
+    XtSetValues(rc, arglist, n);
+    
+    _XmFreeArgList(arglist, total_count, typed_count);
 
-    for (i = 0; i < data.count; i++) {
-	sprintf(buf, "button_%d", i);
-       tgl = _XmCreateSimpleGadget(buf, rc, XmRADIOBUTTON, &data, i,
-                                   args, num_args);
-       if ((data.button_set > 0) &&
-           XmIsToggleButton(tgl) &&
-           (i == data.button_set))
-           XmToggleButtonSetState(tgl, True, False);
-       else if ((data.button_set > 0) &&
-                XmIsToggleButtonGadget(tgl) &&
-                (i == data.button_set))
-           XmToggleButtonGadgetSetState(tgl, True, False);
-    }
-    va_end(var);
-    if (num_args != 0) {
-	_XmFreeArgList(args, total_count, typed_count);
-    }
- 
     return rc;
 }
-
