@@ -1,6 +1,6 @@
 /**
  *
- * $Id: VirtKeys.c,v 1.14 1996/04/23 17:40:50 miers Exp $
+ * $Id: VirtKeys.c,v 1.6 1996/12/03 03:45:21 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,24 +22,30 @@
  *
  **/
 
-static char rcsid[] = "$Id: VirtKeys.c,v 1.14 1996/04/23 17:40:50 miers Exp $";
+static char rcsid[] = "$Id: VirtKeys.c,v 1.6 1996/12/03 03:45:21 miers Exp $";
 
 #include <LTconfig.h>
-#include <Xm/XmP.h>
-#include <Xm/DebugUtil.h>
-#include <Xm/AtomMgr.h>
-#include <Xm/DisplayP.h>
-#include <Xm/MwmUtil.h>
-#include <Xm/VirtKeysP.h>
-#include <X11/keysym.h>
+
 #include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
+#define ANSI_STRINGS
 #endif
 #include <stdlib.h>
 
+#include <XmI/XmI.h>
+#include <Xm/XmP.h>
+#include <Xm/AtomMgr.h>
+#include <Xm/DisplayP.h>
+#include <Xm/MwmUtil.h>
+#include <Xm/VirtKeysP.h>
+#include <X11/keysym.h>
+#include <X11/Xfuncs.h>
+
+#include <XmI/DebugUtil.h>
+
 /*
  * Order of binding precedences:
  *  1) defaultVirtualBindings resource
@@ -274,9 +280,11 @@ static XmVirtualKeysymRec VirtualKeysyms[] = {
  * Armed with a virtual keysym (one of the osfXK_xxxx keysyms) return the
  * keysym and modifiers, which trigger this virtual keysym.
  */
-void _XmVirtualToActualKeysym(Display *Dsp, KeySym VirtualKeysym,
-                              KeySym *RealKeysymReturn,
-                              Modifiers *ModifierReturn)
+void
+_XmVirtualToActualKeysym(Display *Dsp,
+			KeySym VirtualKeysym,
+			KeySym *RealKeysymReturn,
+			Modifiers *ModifierReturn)
 {
     XmDisplayRec *wd;
     int           i;
@@ -299,6 +307,16 @@ void _XmVirtualToActualKeysym(Display *Dsp, KeySym VirtualKeysym,
     *ModifierReturn   = (Modifiers) 0;
 } /* _XmVirtualToActualKeysym */
 
+/* Motif 2.* version of the above */
+void
+XmeVirtualToActualKeysym(Display *Dsp,
+			KeySym VirtualKeysym,
+			KeySym *RealKeysymReturn,
+			Modifiers *ModifierReturn)
+{
+	_XmVirtualToActualKeysym(Dsp, VirtualKeysym, RealKeysymReturn, ModifierReturn);
+}
+
 /*
  * GetModifierMapping -- returns the modifier masks for the ALT and META
  * modifier keys. If there is no such mapping, then a mask of zero will
@@ -1007,7 +1025,7 @@ bindings on a widget which is not a subclass of XmDisplay.");
     /*
      * Initialize some fields for compatibility reasons...
      */
-    memset(((XmDisplayRec *) w)->display.keycode_tag, 0, XmKEYCODE_TAG_SIZE);
+    bzero(((XmDisplayRec *) w)->display.keycode_tag, XmKEYCODE_TAG_SIZE);
     ((XmDisplayRec *) w)->display.lastKeyEvent = 
         (XKeyEvent *) XtMalloc(sizeof(XKeyEvent));
 } /* XmVirtKeysInitialize */
@@ -1089,6 +1107,11 @@ void XmTranslateKey(Display *Dsp, KeyCode Keycode,
                        modifiers_return, Keysym_return);
 } /* XmTranslateKey */
 
+void
+_XmVirtKeysStoreBindings(Widget shell, String binding)
+{
+}
+
 /*
  * This handler just collects any key presses and stores the least one
  * with the appropiate display widget object. This one is in mainly for
@@ -1102,6 +1125,6 @@ void _XmVirtKeysHandler(Widget w, XtPointer Data, XEvent *Event,
     
     if ( !w->core.being_destroyed && (Event->xany.type == KeyPress) ) {
         wd = (XmDisplayRec *) XmGetXmDisplay(XtDisplay(w));
-        memcpy(wd->display.lastKeyEvent, Event, sizeof(XKeyEvent));
+        bcopy(Event, wd->display.lastKeyEvent, sizeof(XKeyEvent));
     }
 } /* _XmVirtKeysHandler */
