@@ -1,6 +1,6 @@
 /**
  *
- * $Id: misc.c,v 1.7 1996/04/23 21:22:43 miers Exp $
+ * $Id: misc.c,v 1.4 1996/11/08 04:46:31 miers Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,15 +22,84 @@
  *
  **/
 
-/* This file contains routines that just don't belong anywhere
-   else */
+/*
+ * This file contains routines that just don't belong anywhere else
+ */
 
 #include <LTconfig.h>
+#include <XmI/XmI.h>
+
 #include <Xm/XmP.h>
 #if XtSpecificationRelease < 6
 #include <X11/IntrinsicP.h>
 #endif
 
+#ifndef HAVE_STRCASECMP
+int
+strcasecmp(s1, s2)
+    char *s1, *s2;
+{
+    int c1, c2;
+
+    while (*s1 && *s2) {
+	c1 = tolower(*s1);
+	c2 = tolower(*s2);
+	if (c1 != c2)
+	    return (c1 - c2);
+	s1++;
+	s2++;
+    }
+    return (int) (*s1 - *s2);
+}
+#endif
+
+#ifndef HAVE_STRNCASECMP
+int
+strncasecmp(s1, s2, count)
+    char *s1, *s2;
+    int count;
+{
+    int c1, c2;
+    int i = 0;
+
+    while (*s1 && *s2 && i < count) {
+	c1 = tolower(*s1);
+	c2 = tolower(*s2);
+	if (c1 != c2)
+	    return (c1 - c2);
+	s1++;
+	s2++;
+	i++;
+    }
+    return (int) (*s1 - *s2);
+}
+#endif
+
+#ifndef HAVE_STRSTR
+char *
+strstr(s1, s2)
+    char *s1;
+    char *s2;
+{
+    int i;
+    char *p1;
+    char *p2;
+    char *s = s1;
+
+    for (p2 = s2, i = 0; *s; p2 = s2, i++, s++)
+    {
+	for (p1 = s; *p1 && *p2 && *p1 == *p2; p1++, p2++)
+	    ;
+	if (!*p2)
+	    break;
+    }
+    if (!*p2)
+	return s1 + i;
+
+    return 0;
+}
+#endif
+
 void
 XmUpdateDisplay(Widget w)
 {
@@ -51,11 +120,34 @@ XmUpdateDisplay(Widget w)
 	XtDispatchEvent(&ev);
 }
 
+#define even_stipple_width 8
+#define even_stipple_height 8
+static unsigned char even_stipple_bits[] = {
+   0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55 };
+#define odd_stipple_width 8
+#define odd_stipple_height 8
+static unsigned char odd_stipple_bits[] = {
+   0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa };
+
+void
+_XmInstallStippleImages(Widget w) {
+    XImage *even_stipple_image;
+    XImage *odd_stipple_image;
+
+    _XmCreateImage(even_stipple_image, XtDisplay(w), (char *)even_stipple_bits,
+		   even_stipple_width, even_stipple_height, LSBFirst);
+    XmInstallImage(even_stipple_image, XmEVEN_STIPPLE_IMAGE);
+
+    _XmCreateImage(odd_stipple_image, XtDisplay(w), (char *)odd_stipple_bits,
+		   odd_stipple_width, odd_stipple_height, LSBFirst);
+    XmInstallImage(odd_stipple_image, XmODD_STIPPLE_IMAGE);
+}
+
 /*
  * The missing commas are intentional.  Let the compiler
  * concatenate the strings
  */
-const char _XmStrings[] =
+_XmConst char _XmStrings[] =
 /* 0     */ "\0"
 /* 1     */ "Accelerator\0"
 /* 13    */ "AcceleratorText\0"
@@ -919,4 +1011,6 @@ const char _XmStrings[] =
 /* 12544 */ "FONTLIST_DEFAULT_TAG_STRING\0"
 /* 12572 */ "XmFONTLIST_DEFAULT_TAG_STRING\0"
 /* 12602 */ "TopItemPosition\0"
+/* 12618 */ "operatingCursorIcon\0"
+/* ????? */ 
 "\0";
