@@ -1,6 +1,7 @@
 
 #include <Xm/ArrowBG.h>
 #include <Xm/BulletinB.h>
+#include <stdio.h>
 
 void
 cb(Widget w, XtPointer userData, XtPointer cbs) {
@@ -13,7 +14,6 @@ int
 main(int argc, char **argv)
 {
   Widget toplevel, one,two;
-  Widget gadget;
   XtAppContext app;
 
   XtSetLanguageProc(NULL, NULL, NULL);
