@@ -10,6 +10,15 @@
 #include <Xm/PushBG.h>
 #include <Xm/Form.h>
 
+char *fallback[] = {
+	"*XmForm.marginWidth:	45",
+	"*XmForm.marginHeight:	30",
+	"*XmForm.background:	dark slate blue",
+	"*XmForm.?.background:	sea green",
+	"*foreground:		yellow",
+	NULL
+};
+
 int
 main(int argc, char **argv)
 {
@@ -18,7 +27,7 @@ main(int argc, char **argv)
 
   XtSetLanguageProc(NULL, NULL, NULL);
 
-  toplevel = XtVaAppInitialize(&app, "Form1", NULL, 0, &argc, argv, NULL, NULL);
+  toplevel = XtVaAppInitialize(&app, "Form1", NULL, 0, &argc, argv, fallback, NULL);
 
   one = XtVaCreateManagedWidget("form", xmFormWidgetClass, toplevel,
 				XmNfractionBase, 4,
