@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <Xm/Xm.h>
 #include <Xm/PushB.h>
+#include <Xm/PushBG.h>
 #include <Xm/Form.h>
 
 int
@@ -29,7 +30,7 @@ main(int argc, char **argv)
 				XmNrightAttachment, XmATTACH_FORM,
 				NULL);
 
-  three = XtVaCreateManagedWidget("three", xmPushButtonWidgetClass, one,
+  three = XtVaCreateManagedWidget("three", xmPushButtonGadgetClass, one,
 				  XmNtopAttachment, XmATTACH_FORM,
 				  XmNbottomAttachment, XmATTACH_FORM,
 				  XmNleftAttachment, XmATTACH_FORM,
