@@ -2,6 +2,7 @@
 
 #include <Xm/Label.h>
 #include <Xm/LabelP.h>
+#include <stdio.h>
 
 int
 main(int argc, char **argv)
@@ -9,8 +10,17 @@ main(int argc, char **argv)
   Widget toplevel, one;
   XtAppContext app;
   XmFontList fontlist;
-  XmString xmstr1 = XmStringCreate("Here is a ", "MY_FONT1");
+#if 1
+  XmString xmstr1 = XmStringCreateLtoR("\n\nHere is a\n\n", "MY_FONT1");
+#else
+  XmString xmstr1 = XmStringCreateLtoR("Here is a", "MY_FONT1");
+#endif
   XmString xmstr2 = XmStringCreate("different font", "MY_FONT");
+  XmStringContext context;
+  char *text;
+  XmStringCharSet tag;
+  XmStringDirection dir;
+  Boolean sep;
 
   XmString xmstr = XmStringConcat(xmstr1, xmstr2);
 
@@ -18,6 +28,12 @@ main(int argc, char **argv)
 
   toplevel = XtVaAppInitialize(&app, "Label", NULL, 0, &argc, argv, NULL, NULL);
 
+  XmStringInitContext(&context, xmstr);
+  while (XmStringGetNextSegment(context, &text, &tag, &dir, &sep)) {
+	printf("Text: %s Tag: %s Dir: %d Sep: %d\n", text, tag, dir, sep);
+	fflush(stdout);
+  }
+
   fontlist = XmFontListAppendEntry(NULL,
 			   XmFontListEntryCreate("MY_FONT",
 						 XmFONT_IS_FONT,
@@ -33,6 +49,7 @@ main(int argc, char **argv)
   one = XtVaCreateManagedWidget("One", xmLabelWidgetClass, toplevel, XmNfontList, fontlist, 
 				XmNlabelString, xmstr, NULL);
 
+  printf("xmstr: %p %p\n", xmstr, ((XmLabelRec*)one)->label._label);
   XtRealizeWidget(toplevel);
   XtAppMainLoop(app);
 
