@@ -1,4 +1,4 @@
-/* $Id: MegaB.c,v 1.2 1996/01/16 16:12:40 miers Exp $ */
+/* $Id: MegaB.c,v 1.3 1996/08/25 01:32:55 miers Exp $ */
 /*
  * Copyright 1994 John L. Cwikla
  *
@@ -42,97 +42,98 @@
 #include <X11/Xfuncs.h>
 
 #if HAVE_SOURCE || defined(LESSTIF_VERSION)
-#include "Xm/TraversalI.h"
+#include "XmI/TraversalI.h"
 #endif /* HAVE_SOURCE */
 
 #include "MegaBP.h"
 
 #include <stdio.h>
+#include <ctype.h>
 
 #define SCROLL_TIME 100
 #define MINI_SEP_HEIGHT 3
 
 static XtWidgetClassProc classPartInit(WidgetClass _wc);
 static XtProc classInit();
-static XtInitProc initialize(XmMegaButtonWidget _request, XmMegaButtonWidget _new, String *_args, 
+static XtInitProc initialize(Widget _request, Widget _new, String *_args, 
 		Cardinal *_numArg); 
-static XtWidgetProc destroy(XmMegaButtonWidget _mbw);
-static void expose(XmMegaButtonWidget _mbw, XEvent *_event, Region _region);
-static Boolean setValues( Widget _current, Widget _request, Widget _new);
-static void borderHighlight(XmMegaButtonWidget _mbw);
-static void borderUnhighlight(XmMegaButtonWidget _mbw);
+static XtWidgetProc destroy(Widget _mbw);
+static void expose(Widget _mbw, XEvent *_event, Region _region);
+static Boolean setValues(Widget _current, Widget _request, Widget _new, ArgList args, Cardinal *num_args);
+static void borderHighlight(Widget _mbw);
+static void borderUnhighlight(Widget _mbw);
 
-static void getMaxWidthPos(XmMegaButtonWidget _mbw, int *_maxWidthPos, Dimension *_height);
-static void drawMiniSeparator(XmMegaButtonWidget _mbw, Position _y);
+static void getMaxWidthPos(Widget _mbw, int *_maxWidthPos, Dimension *_height);
+static void drawMiniSeparator(Widget _mbw, Position _y);
 
-static void enterWidget(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void leaveWidget(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void buttonMotion(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void up(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void down(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void buttonUp(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void buttonDown(XmMegaButtonWidget _mbw, XEvent *_event, String *_params,
+static void enterWidget(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void leaveWidget(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void buttonMotion(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void up(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void down(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void buttonUp(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void buttonDown(Widget _mbw, XEvent *_event, String *_params,
 		Cardinal *_numParms);
-static void armAndActivate(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void focusIn(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
-static void focusOut(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void armAndActivate(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void focusIn(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
+static void focusOut(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams);
 static void timedScroll(XtPointer _closure, XtIntervalId *_id);
 static Boolean cvtStringToMegaButtonMode(Display *_display, XrmValuePtr _args,
 	Cardinal *_numArgs, XrmValuePtr _from, XrmValuePtr _to, XtPointer *_data);
 
-static void drawShadowedItem(XmMegaButtonWidget _mbw, int _pos, Boolean _clear);
-static void drawArrow(XmMegaButtonWidget _mbw, int _pos, int _direction, Boolean _sensitive);
-static void clearArrow(XmMegaButtonWidget _mbw, int _pos);
-static void scroll(XmMegaButtonWidget _mbw, int _direction, Boolean _addTimeout, int _delay);
-static void findGoodShowPos(XmMegaButtonWidget _mbw);
+static void drawShadowedItem(Widget _mbw, int _pos, Boolean _clear);
+static void drawArrow(Widget _mbw, int _pos, int _direction, Boolean _sensitive);
+static void clearArrow(Widget _mbw, int _pos);
+static void scroll(Widget _mbw, int _direction, Boolean _addTimeout, int _delay);
+static void findGoodShowPos(Widget _mbw);
 
-static void toggleDrawProc(XmMegaButtonWidget _mbw, Position _x, Position _y, Boolean _on);
-static void toggleSpaceProc(XmMegaButtonWidget _mbw, Dimension *_width, Dimension *_height);
+static void toggleDrawProc(Widget _mbw, Position _x, Position _y, Boolean _on);
+static void toggleSpaceProc(Widget _mbw, Dimension *_width, Dimension *_height);
 
-static void recheckSizesAtPos(XmMegaButtonWidget _mbw, int _pos);
+static void recheckSizesAtPos(Widget _mbw, int _pos);
 
 static Widget XtGetShell(Widget _w);
 
-#define CORE(w) ((w)->core)
-#define PRIM(w) ((w)->primitive)
-#define LABEL(w) ((w)->label)
-#define PUSH(w) ((w)->push_button)
-#define MB(w) ((w)->mega_button)
+#define CORE(w) (((XmMegaButtonWidget)(w))->core)
+#define PRIM(w) (((XmMegaButtonWidget)(w))->primitive)
+#define LABEL(w) (((XmMegaButtonWidget)(w))->label)
+#define PUSH(w) (((XmMegaButtonWidget)(w))->push_button)
+#define MB(w) (((XmMegaButtonWidget)(w))->mega_button)
 
-#define WIDTH(w) ((w)->core.width)
-#define HEIGHT(w) ((w)->core.height)
+#define WIDTH(w) (((XmMegaButtonWidget)(w))->core.width)
+#define HEIGHT(w) (((XmMegaButtonWidget)(w))->core.height)
 
-#define VIS_COUNT(w) ((w)->mega_button.visibleItemCount)
-#define ICOUNT(w) ((w)->mega_button.itemCount)
-#define ITEMS(w) ((w)->mega_button._items)
-#define EXITEMS(w) ((w)->mega_button.exitems)
-#define VIS_POS(w) ((w)->mega_button.visiblePos)
-#define FPOS(w) ((w)->mega_button.firstVisibleItem)
-#define EL_HEIGHT(w) ((w)->mega_button.elementHeight)
-#define YOFFSET(w) ((w)->mega_button.yOffset)
-#define XOFFSET(w) ((w)->mega_button.xOffset)
-#define HAS_ARROWS(w) ((w)->mega_button.hasArrows)
-#define SET_POS(w) ((w)->mega_button.setPosition)
-#define COPY_GC(w) ((w)->mega_button.copyGC)
-#define TIMER(w) ((w)->mega_button.timer)
-#define TDIR(w) ((w)->mega_button.timedScrollDirection)
-#define SFOS(w) ((w)->mega_button.savedFillOnSelect)
-#define IDELAY(w) ((w)->mega_button.initialDelay)
-#define RDELAY(w) ((w)->mega_button.repeatDelay)
-#define FAKE_ITEM(w) ((w)->mega_button.fakeItem)
-#define CUR_SIZE(w) ((w)->mega_button.currentSize)
-#define DATA(w) ((w)->mega_button.data)
-#define CB_DATA(w) ((w)->mega_button.cbData)
-#define MAX_STRING_POS(w) ((w)->mega_button.maxStringPos)
-#define MAX_STRING_WIDTH(w) ((w)->mega_button.maxStringWidth)
-#define MODE(w) ((w)->mega_button.mode)
-#define TOG_DATA(w) ((w)->mega_button.toggleData)
-#define FILL_ON_SELECT(w) ((w)->mega_button.fillOnSelect)
-#define SELECT_COLOR(w) ((w)->mega_button.selectColor)
-#define SELECT_GC(w) ((w)->mega_button.selectGC)
-#define CHUNK_SIZE(w) ((w)->mega_button.chunkSize)
-#define VIS_W_OFF(w) ((w)->mega_button.visibleWhenOff)
-#define STR_HEIGHT(w) ((w)->mega_button.stringHeight)
+#define VIS_COUNT(w) (((XmMegaButtonWidget)(w))->mega_button.visibleItemCount)
+#define ICOUNT(w) (((XmMegaButtonWidget)(w))->mega_button.itemCount)
+#define ITEMS(w) (((XmMegaButtonWidget)(w))->mega_button._items)
+#define EXITEMS(w) (((XmMegaButtonWidget)(w))->mega_button.exitems)
+#define VIS_POS(w) (((XmMegaButtonWidget)(w))->mega_button.visiblePos)
+#define FPOS(w) (((XmMegaButtonWidget)(w))->mega_button.firstVisibleItem)
+#define EL_HEIGHT(w) (((XmMegaButtonWidget)(w))->mega_button.elementHeight)
+#define YOFFSET(w) (((XmMegaButtonWidget)(w))->mega_button.yOffset)
+#define XOFFSET(w) (((XmMegaButtonWidget)(w))->mega_button.xOffset)
+#define HAS_ARROWS(w) (((XmMegaButtonWidget)(w))->mega_button.hasArrows)
+#define SET_POS(w) (((XmMegaButtonWidget)(w))->mega_button.setPosition)
+#define COPY_GC(w) (((XmMegaButtonWidget)(w))->mega_button.copyGC)
+#define TIMER(w) (((XmMegaButtonWidget)(w))->mega_button.timer)
+#define TDIR(w) (((XmMegaButtonWidget)(w))->mega_button.timedScrollDirection)
+#define SFOS(w) (((XmMegaButtonWidget)(w))->mega_button.savedFillOnSelect)
+#define IDELAY(w) (((XmMegaButtonWidget)(w))->mega_button.initialDelay)
+#define RDELAY(w) (((XmMegaButtonWidget)(w))->mega_button.repeatDelay)
+#define FAKE_ITEM(w) (((XmMegaButtonWidget)(w))->mega_button.fakeItem)
+#define CUR_SIZE(w) (((XmMegaButtonWidget)(w))->mega_button.currentSize)
+#define DATA(w) (((XmMegaButtonWidget)(w))->mega_button.data)
+#define CB_DATA(w) (((XmMegaButtonWidget)(w))->mega_button.cbData)
+#define MAX_STRING_POS(w) (((XmMegaButtonWidget)(w))->mega_button.maxStringPos)
+#define MAX_STRING_WIDTH(w) (((XmMegaButtonWidget)(w))->mega_button.maxStringWidth)
+#define MODE(w) (((XmMegaButtonWidget)(w))->mega_button.mode)
+#define TOG_DATA(w) (((XmMegaButtonWidget)(w))->mega_button.toggleData)
+#define FILL_ON_SELECT(w) (((XmMegaButtonWidget)(w))->mega_button.fillOnSelect)
+#define SELECT_COLOR(w) (((XmMegaButtonWidget)(w))->mega_button.selectColor)
+#define SELECT_GC(w) (((XmMegaButtonWidget)(w))->mega_button.selectGC)
+#define CHUNK_SIZE(w) (((XmMegaButtonWidget)(w))->mega_button.chunkSize)
+#define VIS_W_OFF(w) (((XmMegaButtonWidget)(w))->mega_button.visibleWhenOff)
+#define STR_HEIGHT(w) (((XmMegaButtonWidget)(w))->mega_button.stringHeight)
 
 #define DEFAULT_CHUNK_SIZE 50
 
@@ -282,11 +283,11 @@ XmMegaButtonClassRec xmMegaButtonClassRec =
 		(XtArgsFunc)NULL,								/* set_values_hook */
 		XtInheritSetValuesAlmost,					/* set_values_almost */
 		(XtArgsProc)NULL,								/* get_values_hook */
-		NULL, /* XtInheritAcceptFocus,						/* accept_focus */
+		NULL, 						/* accept_focus */
 		XtVersion,									/* version */
 		(XtPointer)NULL,								/* callback_private */
 		XtInheritTranslations,
-		(XtGeometryHandler)NULL, /*XtInheritQueryGeometry,		  /* query_geometry */
+		(XtGeometryHandler)NULL,		  /* query_geometry */
 		XtInheritDisplayAccelerator,				 /* display_accelerator */
 		(XtPointer)&megaBBaseClassExtRec,										/* extension */
 	},
@@ -343,7 +344,7 @@ static XtProc classInit()
 	return (XtProc)NULL;
 }
 
-static XtInitProc initialize(XmMegaButtonWidget _request, XmMegaButtonWidget _new, String *_args, Cardinal *_numArgs)
+static XtInitProc initialize(Widget _request, Widget _new, String *_args, Cardinal *_numArgs)
 {
 	int i;
 	XmString xmstring;
@@ -462,7 +463,7 @@ static XtInitProc initialize(XmMegaButtonWidget _request, XmMegaButtonWidget _ne
 	return (XtInitProc)NULL;
 }
 
-static XtWidgetProc destroy(XmMegaButtonWidget _mbw)
+static XtWidgetProc destroy(Widget _mbw)
 {
 	register int i;
 
@@ -478,7 +479,7 @@ static XtWidgetProc destroy(XmMegaButtonWidget _mbw)
 	return (XtWidgetProc)NULL;
 }
 
-static void expose(XmMegaButtonWidget _mbw, XEvent *_event, Region _region)
+static void expose(Widget _mbw, XEvent *_event, Region _region)
 {
 	GC gc;
 	int i, first;
@@ -537,12 +538,12 @@ static void expose(XmMegaButtonWidget _mbw, XEvent *_event, Region _region)
 	/* drawShadowedItem(_mbw, VIS_POS(_mbw), FALSE); */
 }
 
-static Boolean setValues( Widget _current, Widget _request, Widget _new)
+static Boolean setValues( Widget _current, Widget _request, Widget _new, ArgList args, Cardinal *num_args)
 {
 	return FALSE;
 }
 
-static void getMaxWidthPos(XmMegaButtonWidget _mbw, int *_maxWidthPos, Dimension *_height)
+static void getMaxWidthPos(Widget _mbw, int *_maxWidthPos, Dimension *_height)
 {
 	int first, i;
 	Dimension maxHeight, maxWidth;
@@ -579,7 +580,7 @@ static void getMaxWidthPos(XmMegaButtonWidget _mbw, int *_maxWidthPos, Dimension
 }
 		
 
-static void drawShadowedItem(XmMegaButtonWidget _mbw, int _pos, Boolean _clear)
+static void drawShadowedItem(Widget _mbw, int _pos, Boolean _clear)
 {
 	Position y;
 	
@@ -618,7 +619,7 @@ static void drawShadowedItem(XmMegaButtonWidget _mbw, int _pos, Boolean _clear)
 	}
 }
 
-static void clearArrow(XmMegaButtonWidget _mbw, int _pos)
+static void clearArrow(Widget _mbw, int _pos)
 {
 	Position y = 0;
 	if (HAS_ARROWS(_mbw))
@@ -638,7 +639,7 @@ static void clearArrow(XmMegaButtonWidget _mbw, int _pos)
 		EL_HEIGHT(_mbw) - 2 * PRIM(_mbw).shadow_thickness, FALSE);
 }
 
-static int yToPos(XmMegaButtonWidget _mbw, Position _y)
+static int yToPos(Widget _mbw, Position _y)
 {
 	Dimension topOffset, bottomOffset;
 
@@ -667,7 +668,7 @@ static int yToPos(XmMegaButtonWidget _mbw, Position _y)
 }
 
 
-static void enterWidget(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void enterWidget(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	if (ICOUNT(_mbw) == 0)
 		return;
@@ -703,7 +704,7 @@ static void enterWidget(XmMegaButtonWidget _mbw, XEvent *_event, String *_params
 	} 
 } 
 
-static void leaveWidget(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void leaveWidget(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	if (ICOUNT(_mbw) == 0)
 		return;
@@ -724,7 +725,7 @@ static void leaveWidget(XmMegaButtonWidget _mbw, XEvent *_event, String *_params
 	drawShadowedItem(_mbw, VIS_POS(_mbw), TRUE);
 } 
 
-static void buttonMotion(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void buttonMotion(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	if (ICOUNT(_mbw) == 0)
 		return;
@@ -757,7 +758,7 @@ static void buttonMotion(XmMegaButtonWidget _mbw, XEvent *_event, String *_param
 	}
 } 
 
-static void up(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void up(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	int vpos;
 
@@ -810,7 +811,7 @@ static void up(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardina
 
 }
 
-static void down(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void down(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	int vpos;
 
@@ -873,7 +874,7 @@ static void down(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardi
 
 }
 
-static void buttonUp(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void buttonUp(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	XmMegaButtonCallbackStruct mtbcs;
 	Widget parent, shell;
@@ -956,11 +957,11 @@ static void buttonUp(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, C
 	_XmSetInDragMode((Widget)_mbw, False);
 }
 
-static void buttonDown(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, 
+static void buttonDown(Widget _mbw, XEvent *_event, String *_params, 
 		Cardinal *_numParms)
 {
 	ShellWidget popupShell;
-	int validButton, pos;
+	int validButton, pos = 0;
 	Widget child;
 
 	if (ICOUNT(_mbw) == 0)
@@ -1022,7 +1023,7 @@ static void buttonDown(XmMegaButtonWidget _mbw, XEvent *_event, String *_params,
 	_XmRecordEvent(_event);
 }
 
-static void armAndActivate(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void armAndActivate(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	XmMegaButtonCallbackStruct mtbcs;
 
@@ -1080,7 +1081,7 @@ struct _XmFocusDataRec
 ** into the RowCol's children, and go from there.
 */
 
-static void focusIn(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void focusIn(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	int labelCount;
 	Widget oldFocus;
@@ -1129,7 +1130,7 @@ static void focusIn(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Ca
 	drawShadowedItem(_mbw, VIS_POS(_mbw), FALSE);
 }
 
-static void focusOut(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
+static void focusOut(Widget _mbw, XEvent *_event, String *_params, Cardinal *_numParams)
 {
 	if (ICOUNT(_mbw) == 0)
 		return;
@@ -1144,7 +1145,7 @@ static void focusOut(XmMegaButtonWidget _mbw, XEvent *_event, String *_params, C
 	VIS_POS(_mbw) = 0;
 }
 
-static void borderHighlight(XmMegaButtonWidget _mbw)
+static void borderHighlight(Widget _mbw)
 {
 	if (ICOUNT(_mbw) == 0)
 		return;
@@ -1156,7 +1157,7 @@ static void borderHighlight(XmMegaButtonWidget _mbw)
 	}
 }
 
-static void borderUnhighlight(XmMegaButtonWidget _mbw)
+static void borderUnhighlight(Widget _mbw)
 {
 	if (ICOUNT(_mbw) == 0)
 		return;
@@ -1166,7 +1167,7 @@ static void borderUnhighlight(XmMegaButtonWidget _mbw)
 		drawShadowedItem(_mbw, VIS_POS(_mbw), TRUE);
 }
 
-static void drawMiniSeparator(XmMegaButtonWidget _mbw, Position _y)
+static void drawMiniSeparator(Widget _mbw, Position _y)
 {
 	int i, firstHalf, secondHalf;
 
@@ -1186,7 +1187,7 @@ static void drawMiniSeparator(XmMegaButtonWidget _mbw, Position _y)
 }
 
 
-static void drawArrow(XmMegaButtonWidget _mbw, int _pos, int _direction, Boolean _sensitive)
+static void drawArrow(Widget _mbw, int _pos, int _direction, Boolean _sensitive)
 {
 	Dimension arrowWidth;
 	Position y = 0;
@@ -1221,10 +1222,10 @@ static void timedScroll(XtPointer _closure, XtIntervalId *_id)
 {
 	XmMegaButtonWidget mbw = (XmMegaButtonWidget)_closure;
 
-	scroll(mbw, TDIR(mbw), TRUE, RDELAY(mbw));
+	scroll((Widget)mbw, TDIR(mbw), TRUE, RDELAY(mbw));
 }
 
-static void scroll(XmMegaButtonWidget _mbw, int _direction, Boolean _addTimeout, int _delay)
+static void scroll(Widget _mbw, int _direction, Boolean _addTimeout, int _delay)
 {
 	Position sy, dy;
 
@@ -1363,7 +1364,7 @@ static void scroll(XmMegaButtonWidget _mbw, int _direction, Boolean _addTimeout,
 	}
 }
 
-static void toggleDrawProc(XmMegaButtonWidget _mbw, Position _x, Position _y, Boolean _on)
+static void toggleDrawProc(Widget _mbw, Position _x, Position _y, Boolean _on)
 {
 	int ecalc;
 
@@ -1386,7 +1387,7 @@ static void toggleDrawProc(XmMegaButtonWidget _mbw, Position _x, Position _y, Bo
 			TRUE);
 }
 
-static void toggleSpaceProc(XmMegaButtonWidget _mbw, Dimension *_width, Dimension *_height)
+static void toggleSpaceProc(Widget _mbw, Dimension *_width, Dimension *_height)
 {
 	int ecalc = STR_HEIGHT(_mbw) - 2 * PRIM(_mbw).shadow_thickness;
 	if (ecalc < 5)
@@ -1396,7 +1397,7 @@ static void toggleSpaceProc(XmMegaButtonWidget _mbw, Dimension *_width, Dimensio
 	*_width += ecalc + LABEL(_mbw).margin_width;
 }
 
-static void findGoodShowPos(XmMegaButtonWidget _mbw)
+static void findGoodShowPos(Widget _mbw)
 {
 	int topDiff, bottomDiff;
 
@@ -1523,10 +1524,10 @@ void XmMegaButtonAddItem(Widget _w, XmString _item, int _pos, XtPointer _cbData)
 	DATA(mbw)[_pos] = _cbData;
 	ICOUNT(mbw)++;
 
-	recheckSizesAtPos(mbw, _pos);
+	recheckSizesAtPos((Widget)mbw, _pos);
 }
 
-static void recheckSizesAtPos(XmMegaButtonWidget _mbw, int _pos)
+static void recheckSizesAtPos(Widget _mbw, int _pos)
 {
 	ShellWidget popupShell;
 	XtGeometryResult result;
@@ -1590,9 +1591,6 @@ XtPointer XmMegaButtonRemoveItem(Widget _w, int _pos)
 {
 	XtPointer data;
 	XmMegaButtonWidget mbw;
-	XtGeometryResult result;
-	Dimension width, height;
-	Dimension replyWidth, replyHeight, oldWidth;
 
 	mbw = (XmMegaButtonWidget)_w;
 
@@ -1607,7 +1605,7 @@ XtPointer XmMegaButtonRemoveItem(Widget _w, int _pos)
 	bcopy(DATA(mbw)+_pos+1, DATA(mbw)+_pos, (ICOUNT(mbw)-_pos) * sizeof(XtPointer));
 
 	if (ICOUNT(mbw) != 0)
-		recheckSizesAtPos(mbw, 0);
+		recheckSizesAtPos((Widget)mbw, 0);
 
 	return data;
 }
@@ -1668,6 +1666,6 @@ extern void XmMegaButtonSetXmStringAtPos(Widget _w, XmString _xmstring, int _pos
 	{
 		_XmStringFree(ITEMS(mbw)[_pos]);
 		ITEMS(mbw)[_pos] = _XmStringCreate(_xmstring);
-		recheckSizesAtPos(mbw, _pos);
+		recheckSizesAtPos((Widget)mbw, _pos);
 	}
 }
