@@ -6,6 +6,7 @@
 Widget button1, button;
 Widget toplevel, field, pane;
 
+#define SKIP_ADJUST
 void activate_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
   XmTextFieldSetString(field, "Hello");
@@ -29,16 +30,25 @@ main(int argc, char **argv)
   pane = XtVaCreateManagedWidget("pane", xmPanedWindowWidgetClass,
                                  toplevel, NULL);
 
-  field = XtVaCreateManagedWidget("field",xmTextFieldWidgetClass,
-                                  pane, 
-                                  NULL);  
-
   button = XtVaCreateManagedWidget("button", xmPushButtonWidgetClass,
 	                            pane, 
+#ifdef SKIP_ADJUST
+				  XmNskipAdjust, True,
+#endif
 				    NULL);
 
+  field = XtVaCreateManagedWidget("field",xmTextFieldWidgetClass,
+                                  pane, 
+#ifdef SKIP_ADJUST
+				  XmNskipAdjust, True,
+#endif
+                                  NULL);  
+
   button1 = XtVaCreateManagedWidget("button2", xmPushButtonWidgetClass,
                                    pane, 
+#ifdef SKIP_ADJUST
+				  XmNskipAdjust, True,
+#endif
                                    NULL);
 
   XtAddCallback(button, XmNactivateCallback, activate_callback, NULL);
