@@ -15,7 +15,6 @@ mytest(Widget w, XtPointer p, XtPointer s)
 {
     Pixel fg, bg;
     int height, width;
-    XmPushButtonCallbackStruct *cbs = s;
     Pixmap pixmap;
 
     printf("hi\n");
@@ -53,7 +52,6 @@ main(int argc, char **argv)
 {
     Widget toplevel, button;
     XtAppContext app;
-    Arg args[2];
 
     toplevel = XtVaAppInitialize(&app, "Hello", NULL, 0,
 				 &argc, argv, fallbacks, NULL);
@@ -66,4 +64,5 @@ main(int argc, char **argv)
 
     XtRealizeWidget(toplevel);
     XtAppMainLoop(app);
+    exit(0);
 }
