@@ -7,9 +7,9 @@
 int
 main(int argc, char **argv)
 {
-  Widget toplevel, one, two;
+  Widget toplevel, one;
   XtAppContext app;
-  Dimension w,i,s,l,t,h;
+  Dimension w,h;
 
   XtSetLanguageProc(NULL, NULL, NULL);
 
