@@ -18,7 +18,7 @@ main(int argc, char **argv)
 {
     XtAppContext theApp;
     Widget toplevel, rc;
-    Widget button, button1, button2, button3, button4, button5;
+    Widget button1, button2, button3, button4, button5;
 
     toplevel = XtVaAppInitialize(&theApp, "rc-test4", NULL, 0,
 				 &argc, argv, NULL, NULL);
@@ -60,6 +60,7 @@ main(int argc, char **argv)
     XtRealizeWidget(toplevel);
 
     XtAppMainLoop(theApp);    
+    exit(0);
 }
 
 
