@@ -1,4 +1,4 @@
-/* $Id: SmartMB.c,v 1.3 1996/03/19 05:07:47 miers Exp $ */
+/* $Id: SmartMB.c,v 1.2 1996/08/25 01:35:11 miers Exp $ */
 /*
  * Copyright 1994 John L. Cwikla
  *
@@ -45,25 +45,27 @@
 #include <Xm/VendorSP.h>
 #include <Xm/BulletinBP.h>
 #include <Xm/LabelG.h>
+#include <stdio.h>
+#include <ctype.h>
 #include "SmartMBP.h"
 
 #define managerTranslations _XmBulletinB_defaultTranslations
 extern void _XmBulletinBoardSetDefaultShadow(); /* Why rewrite? */
 
-#define CORE(a) ((a)->core)
-#define WIDTH(a) ((a)->core.width)
-#define HEIGHT(a) ((a)->core.height)
-#define COMP(a) ((a)->composite)
-#define MANAGER(a) ((a)->manager)
-#define SMB(a) ((a)->smart_message_box)
-#define LABEL(a) ((a)->smart_message_box.labelW)
-#define CONTROL(a) ((a)->smart_message_box.controlW)
-#define SEP(a) ((a)->smart_message_box.separatorW)
-#define DTYPE(a) ((a)->smart_message_box.dialogType)
-#define PIX(a) ((a)->smart_message_box.pixmap)
-#define DPOS(a) ((a)->smart_message_box.dialogPositioning)
+#define CORE(a) (((XmSmartMessageBoxWidget)(a))->core)
+#define WIDTH(a) (((XmSmartMessageBoxWidget)(a))->core.width)
+#define HEIGHT(a) (((XmSmartMessageBoxWidget)(a))->core.height)
+#define COMP(a) (((XmSmartMessageBoxWidget)(a))->composite)
+#define MANAGER(a) (((XmSmartMessageBoxWidget)(a))->manager)
+#define SMB(a) (((XmSmartMessageBoxWidget)(a))->smart_message_box)
+#define LABEL(a) (((XmSmartMessageBoxWidget)(a))->smart_message_box.labelW)
+#define CONTROL(a) (((XmSmartMessageBoxWidget)(a))->smart_message_box.controlW)
+#define SEP(a) (((XmSmartMessageBoxWidget)(a))->smart_message_box.separatorW)
+#define DTYPE(a) (((XmSmartMessageBoxWidget)(a))->smart_message_box.dialogType)
+#define PIX(a) (((XmSmartMessageBoxWidget)(a))->smart_message_box.pixmap)
+#define DPOS(a) (((XmSmartMessageBoxWidget)(a))->smart_message_box.dialogPositioning)
 
-#define MCLASS(a) ((a)->smart_message_box_class)
+#define MCLASS(a) ((XmSmartMessageBoxWidget)(a)->smart_message_box_class)
 
 #define GOOD(a) ((a) && XtIsManaged(a))
 
@@ -90,18 +92,20 @@ static Boolean cvtStringToSMBChildType();
 static Boolean cvtStringToDialogPositioning();
 static void addLabel();
 static void removeLabel();
+#if XPM
 static String findFile();
+#endif
 static void addPositioningToShell();
 static void removePositioningFromShell();
 
 #else
 static XtProc classInit();
-static XtInitProc initialize(XmSmartMessageBoxWidget _request, XmSmartMessageBoxWidget _new, String *_args, Cardinal *_numArgs);
-static Boolean setValues(XmSmartMessageBoxWidget _current, XmSmartMessageBoxWidget _request, XmSmartMessageBoxWidget _new);
+static XtInitProc initialize(Widget _request, Widget _new, String *_args, Cardinal *_numArgs);
+static Boolean setValues(Widget _current, Widget _request, Widget _new, ArgList args, Cardinal *num_args);
 static Boolean constraintSetValues(Widget _current, Widget _request, Widget _new);
 static XtWidgetProc insertChild(Widget _w);
 static XtWidgetProc deleteChild(Widget _w);
-static XtWidgetProc destroy(XmSmartMessageBoxWidget _smbw);
+static XtWidgetProc destroy(Widget _smbw);
 
 static Boolean noGeoRequest(XmGeoMatrix _geoSpec);
 static XmGeoMatrix geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref);
@@ -109,11 +113,13 @@ static Boolean cvtStringToSMBChildType(Display *_display, XrmValuePtr _args,
 	Cardinal *_numArgs, XrmValuePtr _from, XrmValuePtr _to, XtPointer *_data);
 static Boolean cvtStringToDialogPositioning(Display *_display, XrmValuePtr _args,
 	Cardinal *_numArgs, XrmValuePtr _from, XrmValuePtr _to, XtPointer *_data);
-static void addLabel(XmSmartMessageBoxWidget _smbw);
-static void removeLabel(XmSmartMessageBoxWidget _smbw);
-static String findFile(XmSmartMessageBoxWidget, char *_name);
-static void addPositioningToShell(XmSmartMessageBoxWidget _smb);
-static void removePositioningFromShell(XmSmartMessageBoxWidget _smb);
+static void addLabel(Widget _smbw);
+static void removeLabel(Widget _smbw);
+#if XPM
+static String findFile(Widget, char *_name);
+#endif
+static void addPositioningToShell(Widget _smb);
+static void removePositioningFromShell(Widget _smb);
 
 #endif /* _NO_PROTO */
 
@@ -167,8 +173,8 @@ XmSmartMessageBoxClassRec xmSmartMessageBoxClassRec =
 		(XtInitProc)initialize,									/* initialize */
 		(XtArgsProc)NULL,										/* init_hook */
 		XtInheritRealize,							/* realize */
-		(XtActionList)NULL, /* smartMBActions,										/* actions */
-		(Cardinal)0, /* XtNumber(smartMBActions),											 /* num_actions */
+		(XtActionList)NULL, 										/* actions */
+		(Cardinal)0, 										 /* num_actions */
 		(XtResourceList)resources,									 /* resources */
 		(Cardinal)XtNumber(resources),						 /* num_resources */
 		NULLQUARK,									 /* xrm_class */
@@ -322,12 +328,12 @@ static XtProc classInit()
 
 #ifdef _NO_PROTO
 static XtInitProc initialize(_request, _new, _args, _numArgs)
-XmSmartMessageBoxWidget _request;
-XmSmartMessageBoxWidget _new;
+Widget _request;
+Widget _new;
 String *_args;
 Cardinal *_numArgs;
 #else
-static XtInitProc initialize(XmSmartMessageBoxWidget _request, XmSmartMessageBoxWidget _new, String *_args, Cardinal *_numArgs)
+static XtInitProc initialize(Widget _request, Widget _new, String *_args, Cardinal *_numArgs)
 #endif /* _NO_PROTO */
 {
 	LABEL(_new) = (Widget)NULL;
@@ -418,9 +424,9 @@ static XtWidgetProc insertChild(Widget _w)
 
 #ifdef _NO_PROTO
 static XtWidgetProc destroy(_smbw)
-XmSmartMessageBoxWidget _smbw;
+Widget _smbw;
 #else
-static XtWidgetProc destroy(XmSmartMessageBoxWidget _smbw)
+static XtWidgetProc destroy(Widget _smbw)
 #endif /* _NO_PROTO */
 {
 	if (PIX(_smbw) != XmUNSPECIFIED_PIXMAP)
@@ -455,15 +461,16 @@ static XtWidgetProc deleteChild(Widget _w)
 }
 
 #ifdef _NO_PROTO
-static Boolean setValues(_current, _request, _new)
-XmSmartMessageBoxWidget _current;
-XmSmartMessageBoxWidget _request;
-XmSmartMessageBoxWidget _new;
+static Boolean setValues(_current, _request, _new, args, num_args)
+Widget _current;
+Widget _request;
+Widget _new;
+ArgList args;
+Cardinal *num_args;
 #else
-static Boolean setValues(XmSmartMessageBoxWidget _current, XmSmartMessageBoxWidget _request, XmSmartMessageBoxWidget _new)
+static Boolean setValues(Widget _current, Widget _request, Widget _new, ArgList args, Cardinal *num_args)
 #endif /* NO_PROTO */
 {
-	Widget parent;
 	Boolean redo = FALSE;
 
 	BB_InSetValues(_current) = TRUE;
@@ -682,9 +689,9 @@ static Boolean noGeoRequest(XmGeoMatrix _geoSpec)
 
 #ifdef _NO_PROTO
 static void addLabel(_smbw)
-XmSmartMessageBoxWidget _smbw;
+Widget _smbw;
 #else
-static void addLabel(XmSmartMessageBoxWidget _smbw)
+static void addLabel(Widget _smbw)
 #endif /* _NO_PROTO */
 {
 	Arg warg[2];
@@ -787,9 +794,9 @@ printf("LABEL IS NULL\n");
 
 #ifdef _NO_PROTO
 static void removeLabel(_smbw)
-XmSmartMessageBoxWidget _smbw;
+Widget _smbw;
 #else
-static void removeLabel(XmSmartMessageBoxWidget _smbw)
+static void removeLabel(Widget _smbw)
 #endif /* _NO_PROTO */
 {
 	if (GOOD(LABEL(_smbw)))
@@ -802,12 +809,13 @@ static void removeLabel(XmSmartMessageBoxWidget _smbw)
 	}
 }
 
+#if XPM
 #ifdef _NO_PROTO
 static String findFile(_smbw, _name)
-XmSmartMessageBoxWidget _smbw;
+Widget _smbw;
 char *_name;
 #else
-static String findFile(XmSmartMessageBoxWidget _smbw, char *_name)
+static String findFile(Widget _smbw, char *_name)
 #endif /* _NO_PROTO */
 {
 	char *retFile;
@@ -822,6 +830,7 @@ static String findFile(XmSmartMessageBoxWidget _smbw, char *_name)
 
 	return retFile;
 } 
+#endif
 	
 
 #ifdef _NO_PROTO
@@ -1102,9 +1111,9 @@ static XtCallbackProc shellMoveToPointer(Widget _shell, XtPointer _moveToDefault
 
 #ifdef _NO_PROTO
 static void addPositioningToShell(_smb)
-XmSmartMessageBoxWidget _smb;
+Widget _smb;
 #else
-static void addPositioningToShell(XmSmartMessageBoxWidget _smb)
+static void addPositioningToShell(Widget _smb)
 #endif /* _NO_PROTO */
 {
 	Widget parent;
@@ -1138,9 +1147,9 @@ static void addPositioningToShell(XmSmartMessageBoxWidget _smb)
 
 #ifdef _NO_PROTO
 static void removePositioningFromShell(_smb)
-XmSmartMessageBoxWidget _smb;
+Widget _smb;
 #else
-static void removePositioningFromShell(XmSmartMessageBoxWidget _smb)
+static void removePositioningFromShell(Widget _smb)
 #endif /* _NO_PROTO */
 {
     Widget parent;
