@@ -13,9 +13,6 @@ main(int argc, char **argv)
   XtAppContext theApp;
   Widget toplevel;
   Widget butt;
-  Dimension height;
-  Arg args[10];
-  int n;
 
   toplevel = XtVaAppInitialize(&theApp, "toggle1", NULL, 0,
 			       &argc, argv, NULL, NULL);
