@@ -1,11 +1,13 @@
 #include <Xm/PushB.h>
 #include <Xm/RowColumn.h>
+#include <stdio.h>
 
 void
 doit(Widget w, XtPointer data, XtPointer cbs) {
     XmProcessTraversal(w, XmTRAVERSE_HOME);
 }
 
+int
 main(int argc, char **argv) {
     Widget toplevel, rowcol, pb;
     XtAppContext app;
@@ -29,4 +31,5 @@ main(int argc, char **argv) {
 
     XtRealizeWidget(toplevel);
     XtAppMainLoop(app);
+    exit(0);
 }
