@@ -14,11 +14,12 @@ main(int argc,
 {
 	char dir[256], pat[256];
 	String quald, qualpat, *entries;
-	int numents, numalloc, i;
+	unsigned int numents, numalloc, i;
 
 	entries = NULL;
-	numents = numalloc = 0;
 	for (;;) {
+	  numents = numalloc = 0;
+	  
 		printf("Enter dir: ");
 		gets(dir);
 		if (feof(stdin))
@@ -35,4 +36,5 @@ main(int argc,
 		for (i = 0; i < numents; i++)
 			printf("Entry %d: '%s'\n", i, entries[i]);
 	}
+	exit(0);
 }
